@@ -102,199 +102,70 @@ draw_corners(MwmWindow *t, int i, GC hor, GC vert)
     XSegment seg[2];
     int n = 0;
 
-    if (t->boundary_width < 3)
+    switch (i)
     {
-	switch (i)
-	{
-	case 0:
-	    seg[0].x1 = t->boundary_width - 1;
-	    seg[0].x2 = t->corner_width;
-	    seg[0].y1 = t->boundary_width - 1;
-	    seg[0].y2 = t->boundary_width - 1;
-	    n = 1;
-	    break;
-	case 1:
-	    seg[0].x1 = 0;
-	    seg[0].x2 = t->corner_width - t->boundary_width /* -1 */ ;
-	    seg[0].y1 = t->boundary_width - 1;
-	    seg[0].y2 = t->boundary_width - 1;
-	    n = 1;
-	    break;
-	case 2:
-	    seg[0].x1 = t->boundary_width - 1;
-	    seg[0].x2 = t->corner_width - 2;
-	    seg[0].y1 = t->corner_width - t->boundary_width + t->bw;
-	    seg[0].y2 = t->corner_width - t->boundary_width + t->bw;
-	    n = 1;
-	    break;
-	case 3:
-	    seg[0].x1 = 0;
-	    seg[0].x2 = t->corner_width - t->boundary_width;
-	    seg[0].y1 = t->corner_width - t->boundary_width + t->bw;
-	    seg[0].y2 = t->corner_width - t->boundary_width + t->bw;
-	    n = 1;
-	    break;
-	}
-	XDrawSegments(dpy, t->corners[i], hor, seg, n);
-	switch (i)
-	{
-	case 0:
-	    seg[0].y1 = t->boundary_width - 1;
-	    seg[0].y2 = t->corner_width;
-	    seg[0].x1 = t->boundary_width - 1;
-	    seg[0].x2 = t->boundary_width - 1;
-	    n = 1;
-	    break;
-	case 1:
-	    seg[0].y1 = t->boundary_width - 1;
-	    seg[0].y2 = t->corner_width - 2;
-	    seg[0].x1 = t->corner_width - t->boundary_width;
-	    seg[0].x2 = t->corner_width - t->boundary_width;
-	    n = 1;
-	    break;
-	case 2:
-	    seg[0].y1 = 0;
-	    seg[0].y2 = t->corner_width - t->boundary_width;
-	    seg[0].x1 = t->boundary_width - 1;
-	    seg[0].x2 = t->boundary_width - 1;
-	    n = 1;
-	    break;
-	case 3:
-	    seg[0].y1 = 0;
-	    seg[0].y2 = t->corner_width - t->boundary_width + t->bw;
-	    seg[0].x1 = t->corner_width - t->boundary_width;
-	    seg[0].x2 = t->corner_width - t->boundary_width;
-	    n = 1;
-	    break;
-	}
-	XDrawSegments(dpy, t->corners[i], vert, seg, 1);
+    case 0:
+	seg[0].x1 = t->boundary_width - 1;
+	seg[0].x2 = t->corner_width;
+	seg[0].y1 = t->boundary_width - 1;
+	seg[0].y2 = t->boundary_width - 1;
+	n = 1;
+	break;
+    case 1:
+	seg[0].x1 = 0;
+	seg[0].x2 = t->corner_width - t->boundary_width;
+	seg[0].y1 = t->boundary_width - 1;
+	seg[0].y2 = t->boundary_width - 1;
+	n = 1;
+	break;
+    case 2:
+	seg[0].x1 = t->boundary_width - 1;
+	seg[0].x2 = t->corner_width - 2;
+	seg[0].y1 = t->corner_width - t->boundary_width + t->bw;
+	seg[0].y2 = t->corner_width - t->boundary_width + t->bw;
+	n = 1;
+	break;
+    case 3:
+	seg[0].x1 = 0;
+	seg[0].x2 = t->corner_width - t->boundary_width;
+	seg[0].y1 = t->corner_width - t->boundary_width + t->bw;
+	seg[0].y2 = t->corner_width - t->boundary_width + t->bw;
+	n = 1;
+	break;
     }
-    else
+    XDrawSegments(dpy, t->corners[i], hor, seg, n);
+    switch (i)
     {
-	switch (i)
-	{
-	case 0:
-	    seg[0].x1 = t->boundary_width - 2;
-	    seg[0].x2 = t->corner_width;
-	    seg[0].y1 = t->boundary_width - 2;
-	    seg[0].y2 = t->boundary_width - 2;
-
-	    seg[1].x1 = t->boundary_width - 2;
-	    seg[1].x2 = t->corner_width;
-	    seg[1].y1 = t->boundary_width - 1;
-	    seg[1].y2 = t->boundary_width - 1;
-	    n = 2;
-	    break;
-	case 1:
-	    seg[0].x1 = 1;
-	    seg[0].x2 = t->corner_width - t->boundary_width;
-	    seg[0].y1 = t->boundary_width - 2;
-	    seg[0].y2 = t->boundary_width - 2;
-
-	    seg[1].x1 = 0;
-	    seg[1].x2 = t->corner_width - t->boundary_width - 1;
-	    seg[1].y1 = t->boundary_width - 1;
-	    seg[1].y2 = t->boundary_width - 1;
-	    n = 2;
-	    break;
-	case 2:
-	    seg[0].x1 = t->boundary_width - 1;
-	    seg[0].x2 = t->corner_width - 2;
-	    seg[0].y1 = t->corner_width - t->boundary_width + 1;
-	    seg[0].y2 = t->corner_width - t->boundary_width + 1;
-	    n = 1;
-	    if (t->boundary_width > 3)
-	    {
-		seg[1].x1 = t->boundary_width - 2;
-		seg[1].x2 = t->corner_width - 3;
-		seg[1].y1 = t->corner_width - t->boundary_width + 2;
-		seg[1].y2 = t->corner_width - t->boundary_width + 2;
-		n = 2;
-	    }
-	    break;
-	case 3:
-	    seg[0].x1 = 0;
-	    seg[0].x2 = t->corner_width - t->boundary_width;
-	    seg[0].y1 = t->corner_width - t->boundary_width + 1;
-	    seg[0].y2 = t->corner_width - t->boundary_width + 1;
-	    n = 1;
-	    if (t->boundary_width > 3)
-	    {
-		seg[0].x2 = t->corner_width - t->boundary_width + 1;
-
-		seg[1].x1 = 0;
-		seg[1].x2 = t->corner_width - t->boundary_width + 1;
-		seg[1].y1 = t->corner_width - t->boundary_width + 2;
-		seg[1].y2 = t->corner_width - t->boundary_width + 2;
-		n = 2;
-	    }
-	    break;
-	}
-	XDrawSegments(dpy, t->corners[i], hor, seg, n);
-	switch (i)
-	{
-	case 0:
-	    seg[0].y1 = t->boundary_width - 2;
-	    seg[0].y2 = t->corner_width;
-	    seg[0].x1 = t->boundary_width - 2;
-	    seg[0].x2 = t->boundary_width - 2;
-
-	    seg[1].y1 = t->boundary_width - 2;
-	    seg[1].y2 = t->corner_width;
-	    seg[1].x1 = t->boundary_width - 1;
-	    seg[1].x2 = t->boundary_width - 1;
-	    n = 2;
-	    break;
-	case 1:
-	    seg[0].y1 = t->boundary_width - 1;
-	    seg[0].y2 = t->corner_width - 2;
-	    seg[0].x1 = t->corner_width - t->boundary_width;
-	    seg[0].x2 = t->corner_width - t->boundary_width;
-	    n = 1;
-	    if (t->boundary_width > 3)
-	    {
-		seg[1].y1 = t->boundary_width - 2;
-		seg[1].y2 = t->corner_width - 3;
-		seg[1].x1 = t->corner_width - t->boundary_width + 1;
-		seg[1].x2 = t->corner_width - t->boundary_width + 1;
-		n = 2;
-	    }
-	    break;
-	case 2:
-	    seg[0].y1 = 1;
-	    seg[0].y2 = t->corner_width - t->boundary_width + 1;
-	    seg[0].x1 = t->boundary_width - 2;
-	    seg[0].x2 = t->boundary_width - 2;
-	    n = 1;
-
-	    if (t->boundary_width > 3)
-	    {
-		seg[1].y1 = 0;
-		seg[1].y2 = t->corner_width - t->boundary_width;
-		seg[1].x1 = t->boundary_width - 1;
-		seg[1].x2 = t->boundary_width - 1;
-	    }
-	    break;
-	case 3:
-	    seg[0].y1 = 0;
-	    seg[0].y2 = t->corner_width - t->boundary_width + 1;
-	    seg[0].x1 = t->corner_width - t->boundary_width;
-	    seg[0].x2 = t->corner_width - t->boundary_width;
-	    n = 1;
-
-	    if (t->boundary_width > 3)
-	    {
-		seg[0].y2 = t->corner_width - t->boundary_width + 2;
-		seg[1].y1 = 0;
-		seg[1].y2 = t->corner_width - t->boundary_width + 2;
-		seg[1].x1 = t->corner_width - t->boundary_width + 1;
-		seg[1].x2 = t->corner_width - t->boundary_width + 1;
-		n = 2;
-	    }
-	    break;
-	}
-	XDrawSegments(dpy, t->corners[i], vert, seg, n);
+    case 0:
+	seg[0].y1 = t->boundary_width - 1;
+	seg[0].y2 = t->corner_width;
+	seg[0].x1 = t->boundary_width - 1;
+	seg[0].x2 = t->boundary_width - 1;
+	n = 1;
+	break;
+    case 1:
+	seg[0].y1 = t->boundary_width - 1;
+	seg[0].y2 = t->corner_width - 2;
+	seg[0].x1 = t->corner_width - t->boundary_width;
+	seg[0].x2 = t->corner_width - t->boundary_width;
+	n = 1;
+	break;
+    case 2:
+	seg[0].y1 = 0;
+	seg[0].y2 = t->corner_width - t->boundary_width;
+	seg[0].x1 = t->boundary_width - 1;
+	seg[0].x2 = t->boundary_width - 1;
+	n = 1;
+	break;
+    case 3:
+	seg[0].y1 = 0;
+	seg[0].y2 = t->corner_width - t->boundary_width + t->bw;
+	seg[0].x1 = t->corner_width - t->boundary_width;
+	seg[0].x2 = t->corner_width - t->boundary_width;
+	n = 1;
+	break;
     }
+    XDrawSegments(dpy, t->corners[i], vert, seg, 1);
 }
 
 /*
@@ -1804,13 +1675,13 @@ DEC_DrawTitleBar(ScreenInfo *scr, MwmWindow *t, Bool onoroff, Bool NewTitle)
 		       hor_off - 2, 0, w + 4, t->title_height, False);
 	}
 
-	XDrawString(dpy, t->title,
-		    onoroff
-		    ? scr->components[MWM_TITLE_A].active_GC
-		    : scr->components[MWM_TITLE_A].normal_GC,
-		    hor_off,
-		    scr->components[MWM_TITLE_A].f_y + 1,
-		    t->name, strlen(t->name));
+	XDrawImageString(dpy, t->title,
+			 onoroff
+			 ? scr->components[MWM_TITLE_A].active_GC
+			 : scr->components[MWM_TITLE_A].normal_GC,
+			 hor_off,
+			 scr->components[MWM_TITLE_A].f_y + 2,
+			 t->name, strlen(t->name));
 
 	DEC_DrawShadows(t, t->title, 0, 0, t->title_width, t->title_height,
 			ReliefGC, ShadowGC);
