@@ -1,5 +1,5 @@
 /*
- * $Id: events.c,v 1.5 1998/05/28 22:14:17 u27113 Exp $
+ * $Id: events.c,v 1.8 1998/10/17 06:23:09 gritton Exp $
  */
 /*****************************************************************************/
 /**       Copyright 1988 by Evans & Sutherland Computer Corporation,        **/
@@ -95,7 +95,7 @@ is_click(ScreenInfo *scr, int x, int y, XEvent *d)
 	   (y - ycurrent > -Mwm.move_threshold))
     {
 
-	MISC_usleep(10000);
+	_XmMicroSleep(10000);
 	total += 10;
 
 	if (XCheckMaskEvent(dpy, ButtonPressMask | ButtonReleaseMask, &trash))
@@ -207,6 +207,30 @@ focus_in(ScreenInfo *scr, MwmWindow *win, XEvent *event)
 	COLOR_InstallWindowColorMap(scr, win);
 }
 
+/*
+ * this function deals with bad focus tracks
+ * MLM: 6/??/98. Deal with what seems is a race condition between the
+ * toolkit and the window manager.  When we set the input focus in
+ * the unmap_notify() function, it can sometimes implicitly fail without
+ * our knowing it.  I believe this is a race condition where the toolkit
+ * calls XSetInputFocus() when dealing with the menu system.  The net
+ * result is that we sometimes do not receive the FocusIn event we expect;
+ * therefore the application terminates, and focus reverts to None.
+ * We find out about that here.
+ */
+static void
+focus_out(ScreenInfo *scr, MwmWindow *win, XEvent *event)
+{
+    Window focusBug;
+    int rt;
+
+    XGetInputFocus(dpy, &focusBug, &rt);
+    if (win == NULL && scr->mwm_focus != NULL && focusBug == None)
+    {
+        XSetInputFocus(dpy, scr->mwm_focus->w, RevertToParent, CurrentTime);
+    }
+}
+
 /*
  * key press event handler
  */
@@ -520,7 +544,6 @@ expose(ScreenInfo *scr, MwmWindow *win, XEvent *event)
 	if (WIN_WindowToStruct(scr, event->xany.window))
 	    PAGER_Redraw(scr);
     }
-    return;
 }
 
 /*
@@ -653,11 +676,15 @@ unmap_notify(ScreenInfo *scr, MwmWindow *win, XEvent *event)
 	    WIN_SetFocus(scr, win->ancestor->w, win->ancestor);
 	}
 	else
+        {
 	    WIN_SetFocus(scr, scr->no_focus_win, NULL);
+        }
     }
 
     if (scr->mwm_focus == win)
+    {
 	WIN_SetFocus(scr, scr->no_focus_win, NULL);
+    }
 
     if (win == scr->mwm_pushed)
 	scr->mwm_pushed = NULL;
@@ -674,8 +701,15 @@ unmap_notify(ScreenInfo *scr, MwmWindow *win, XEvent *event)
     if (XCheckTypedWindowEvent(dpy, event->xunmap.window,
 			       DestroyNotify, &dummy))
     {
+        /* here was the problem with the menu focus.  If the
+         * destroy notify happened later, we need to reset the focus,
+         * as the application has had a window of opportunity to steal it
+         * back.
+         */
 	WIN_DestroyWindow(scr, win);
+
 	XUngrabServer(dpy);
+
 	return;
     }
 
@@ -988,7 +1022,7 @@ button_press(ScreenInfo *scr, MwmWindow *win, XEvent *event)
 #endif
 		{
 		    Action = MouseEntry->item ? MouseEntry->item->action : NULL;
-		    /* win->custom_menu = NULL;	/* FIX ME */
+		    /* win->custom_menu = NULL;	*/ /* FIX ME */
 		    if (MouseEntry->menu == NULL)
 			MouseEntry->menu = (MenuRoot *)DEFAULT_WIN_MENU_NAME;
 		    FUNC_Execute(scr, F_W_POPUP, Action,
@@ -1179,8 +1213,6 @@ enter_notify(ScreenInfo *scr, MwmWindow *win, XEvent *event)
 	else
 	    COLOR_InstallWindowColorMap(scr, NULL);
     }
-
-    return;
 }
 
 /*
@@ -1618,6 +1650,10 @@ EVENT_Dispatch(XEvent *event)
 	focus_in(scr, win, event);
 	break;
 
+    case FocusOut:
+        focus_out(scr, win, event);
+        break;
+        
     case ConfigureRequest:
 	configure_request(scr, win, event);
 	break;
@@ -1647,6 +1683,4 @@ EVENT_Dispatch(XEvent *event)
 	    shape_notify(scr, win, event);
 	break;
     }
-
-    return;
 }
