@@ -76,7 +76,7 @@
 #endif
 
 extern int menuFromFrameOrWindowOrTitlebar;
-extern DoHandlePageing;
+extern Bool DoHandlePageing;
 extern XtAppContext app;
 extern volatile int alarmed;
 
@@ -512,8 +512,6 @@ move(ScreenInfo *scr, XEvent *eventp, Window w, MwmWindow *tmp_win,
     }
 
     PAGER_Clear(scr);
-
-    return;
 }
 
 static void
@@ -1239,10 +1237,15 @@ FUNC_Execute(ScreenInfo *scr, int func, char *action, Window in_w,
 	break;
 
     case F_RESTART:
-	if (Mwm.show_feedback & MWM_FEEDBACK_RESTART)
+	if ((Mwm.show_feedback & MWM_FEEDBACK_RESTART) && !action)
 	    restart(scr);
-	else
+	else {
+	  if (!action) {
 	    MWM_Done(1, g_argv[0]);
+	  }  else {
+	    MWM_Done(1, action);
+	  }
+	}
 	break;
 
     case F_RESTORE_AND_RAISE:
@@ -1393,6 +1396,4 @@ FUNC_Execute(ScreenInfo *scr, int func, char *action, Window in_w,
 #if 0
     MISC_WaitForButtonsUp(scr);
 #endif
-
-    return;
 }
