@@ -165,7 +165,6 @@ grab_icon_buttons(ScreenInfo *scr, MwmWindow *tmp_win, Window w)
 
 	MouseEntry = MouseEntry->next;
     }
-    return;
 }
 
 /*
@@ -182,7 +181,6 @@ grab_icon_keys(ScreenInfo *scr, MwmWindow *tmp_win, Window w)
 	    XGrabKey(dpy, tmp->keycode, tmp->mods, w, True,
 		     GrabModeAsync, GrabModeAsync);
     }
-    return;
 }
 
 /*
@@ -328,6 +326,7 @@ get_icon_bitmap(MwmWindow *tmp_win)
     }
 }
 
+#if 0
 /*
  * draw icon border windows
  */
@@ -456,6 +455,8 @@ draw_icon_border(MwmWindow *t, Window win, int x, int y, int w, int h,
     }
 }
 
+#endif
+
 /*
  * Creates an icon window as needed
  */
@@ -599,7 +600,7 @@ ICON_CreateWindow(ScreenInfo *scr, MwmWindow *tmp_win, int def_x, int def_y)
 
     if (Mwm.icon_decoration & XmICON_IMAGE)
     {
-	int i;
+        /* int i; */
 
 #if 0
 	attributes.event_mask = KeyPressMask | ButtonPressMask;
@@ -723,7 +724,6 @@ ICON_CreateWindow(ScreenInfo *scr, MwmWindow *tmp_win, int def_x, int def_y)
 	grab_icon_buttons(scr, tmp_win, tmp_win->icon_pixmap_w);
 	grab_icon_keys(scr, tmp_win, tmp_win->icon_pixmap_w);
     }
-    return;
 }
 
 /*
@@ -991,7 +991,6 @@ ICON_UpdateWindow(ScreenInfo *scr, MwmWindow *tmp, Boolean force)
     /* clear the icon window, and trigger a re-draw via an expose event */
     if (tmp->flags & ICONIFIED)
 	XClearArea(dpy, tmp->icon_w, 0, 0, 0, 0, True);
-    return;
 }
 
 /*
@@ -1203,12 +1202,13 @@ ICON_Iconify(ScreenInfo *scr, MwmWindow *tmp_win, int def_x, int def_y)
 	}
     }
     if (tmp_win->icon_w == None)
+    {
 	if (tmp_win->flags & ICON_MOVED)
 	    ICON_CreateWindow(scr, tmp_win,
 			      tmp_win->icon_x_loc, tmp_win->icon_y_loc);
 	else
 	    ICON_CreateWindow(scr, tmp_win, def_x, def_y);
-
+    }
     if (Mwm.icon_auto_place)
 	ICON_AutoPlace(scr, tmp_win);
     tmp_win->flags |= ICONIFIED;
@@ -1243,8 +1243,6 @@ ICON_Iconify(ScreenInfo *scr, MwmWindow *tmp_win, int def_x, int def_y)
     }
     if (Mwm.lower_on_iconify)
 	WIN_Lower(scr, tmp_win);
-
-    return;
 }
 
 /*
@@ -1333,5 +1331,4 @@ ICON_DeIconify(ScreenInfo *scr, MwmWindow *tmp_win)
 	WIN_ChangeFocus(scr, tmp_win, 1);
 
     PAGER_UpdateView(scr, tmp_win);
-    return;
 }
