@@ -50,6 +50,9 @@
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
+#ifdef __EMX__
+#include <netdb.h>
+#endif
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -58,6 +61,11 @@
 #include <Xm/DisplayP.h>
 #include <Xm/ScreenP.h>
 
+#ifdef	HAVE_GETRLIMIT
+#include <sys/types.h>
+#include <sys/resource.h>
+#endif
+
 #include <XmI/DebugUtil.h>
 
 extern int putenv();
@@ -110,9 +118,7 @@ static RETSIGTYPE
 sig_restart(int sig)
 {
     MWM_Done(1, *g_argv);
-#if RETSIGTYPE == void
-    return;
-#else
+#if RETSIGTYPE != void
     return (RETSIGTYPE) 0;
 #endif
 }
@@ -124,9 +130,7 @@ static RETSIGTYPE
 sig_done(int nonsense)
 {
     MWM_Done(0, NULL);
-#if RETSIGTYPE == void
-    return;
-#else
+#if RETSIGTYPE != void
     return (RETSIGTYPE) 0;
 #endif
 }
@@ -140,9 +144,7 @@ sig_alarm(int nonsense)
     alarmed = True;
     signal(SIGALRM, sig_alarm);
 
-#if RETSIGTYPE == void
-    return;
-#else
+#if RETSIGTYPE != void
     return (RETSIGTYPE) 0;
 #endif
 }
@@ -308,7 +310,7 @@ MWM_Done(int restart, char *command)
 
 	/* really need to destroy all windows, explicitly,
 	 * not sleep, but this is adequate for now */
-	sleep(1);
+	_XmSleep(1);
 	ReapChildren();
 
 	execvp(command, g_argv);
@@ -331,7 +333,7 @@ MWM_Done(int restart, char *command)
 /*
  * main - start of mwm
  */
-void
+int
 main(int argc, char **argv)
 {
     int i;
@@ -393,11 +395,19 @@ main(int argc, char **argv)
 #ifdef HAVE_SYSCONF
     fd_width = sysconf(_SC_OPEN_MAX);
 #else
-#ifdef HAVE_GETDTABLESIZE
+# ifdef HAVE_GETDTABLESIZE
     fd_width = getdtablesize();
-#else
-#error You lose
-#endif
+# else
+#  ifdef	HAVE_GETRLIMIT
+   {
+	struct rlimit rl;
+	getrlimit(RLIMIT_NOFILE, &rl);
+	fd_width = rl.rlim_cur;
+   }
+#  else
+#   error You lose
+#  endif
+# endif
 #endif
     x_fd = XConnectionNumber(dpy);
 
@@ -482,5 +492,5 @@ main(int argc, char **argv)
 	}
     }
 
-    exit(0);
+    return(0);
 }
