@@ -1,6 +1,6 @@
 /**
  *
- * $Id: resources.c,v 1.5 1997/10/12 01:36:49 miers Exp $
+ * $Id: resources.c,v 1.7 1998/10/12 06:11:11 gritton Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -394,7 +394,7 @@ static XtResource mwm_resources[] =
     },
     {
 	XmNiconPlacementMargin, XmCIconPlacementMargin, XmRDimension,
-	sizeof(Dimension), Offset(icon_placement),
+	sizeof(Dimension), Offset(icon_placement_margin),
 	XmRImmediate, (XtPointer)1
     },
     {
@@ -473,7 +473,7 @@ static XtResource mwm_resources[] =
 	XmRImmediate, (XtPointer)False
     },
     {
-	XmNresizeBorderWidth, XmCFrameBorderWidth, XmRInt,
+	XmNresizeBorderWidth, XmCFrameBorderWidth, XmRDimension,
 	sizeof(Dimension), Offset(resize_border_width),
 	XmRCallProc, (XtPointer)_WmDefaultResizeBorderWidth
     },
@@ -700,6 +700,11 @@ static XtResource mwm_component_resources[] =
 	sizeof(Pixel), Offset(background),
 	XmRCallProc, (XtPointer)_WmBackgroundDefault
     },
+    {
+	XmNforeground, XmCForeground, XmRPixel,
+	sizeof(XFontStruct *), Offset(foreground),
+	XmRCallProc, (XtPointer)_WmForegroundDefault
+    },
     {
 	XmNbackgroundPixmap, XmCBackgroundPixmap, XmRPixmap,
 	sizeof(Pixmap), Offset(background_pixmap),
@@ -720,11 +725,6 @@ static XtResource mwm_component_resources[] =
 	sizeof(XmFontList), Offset(font_list),
 	XmRString, (XtPointer)"fixed"
     },
-    {
-	XmNforeground, XmCForeground, XmRPixel,
-	sizeof(XFontStruct *), Offset(foreground),
-	XmRCallProc, (XtPointer)_WmForegroundDefault
-    },
     {
 	XmNsaveUnder, XmCSaveUnder, XmRBoolean,
 	sizeof(Boolean), Offset(save_under),
