@@ -171,7 +171,6 @@ get_gravity_offsets(MwmWindow *tmp, int *xp, int *yp)
 	*xp = (int)gravity_offsets[g].x;
 	*yp = (int)gravity_offsets[g].y;
     }
-    return;
 }
 
 /*
@@ -246,7 +245,6 @@ grab_buttons(ScreenInfo *scr, MwmWindow *tmp_win)
 	}
 	MouseEntry = MouseEntry->next;
     }
-    return;
 }
 
 /*
@@ -271,7 +269,6 @@ grab_keys(ScreenInfo *scr, MwmWindow *tmp_win)
 	    }
 	}
     }
-    return;
 }
 
 /*
@@ -343,7 +340,7 @@ smart_placement(ScreenInfo *scr, MwmWindow *t,
     int test_x = 0, test_y = 0;
     int loc_ok = False, tw, tx, ty, th;
     MwmWindow *test_window;
-    static last_x = 0, last_y = 0, begin_x = INC, begin_y = INC;
+    static int last_x = 0, last_y = 0, begin_x = INC, begin_y = INC;
 
     temp_h = height;
     temp_w = width;
@@ -1155,7 +1152,9 @@ WIN_SetFocus(ScreenInfo *scr, Window w, MwmWindow *Fw)
 	    /* Without the if statement below, we can crash here.
 	     * Danny 16/4/97 */
 	    if (Fw)
+            {
 		w = Fw->w;
+            }
 	}
     }
 
@@ -1235,7 +1234,22 @@ WIN_SetFocus(ScreenInfo *scr, Window w, MwmWindow *Fw)
     if (!((Fw) && (Fw->wmhints) && (Fw->wmhints->flags & InputHint) &&
 	  (Fw->wmhints->input == False)))
     {
+        /* Window rw; */
+        /* int rt; */
+
 	/* Window will accept input focus */
+        /* MLM: 6/??/98.  A very weird thing can happen here.  We set
+         * the focus, but if you add a call just below here to XGetInputFocus(),
+         * you'll see that the focus has reverted to the old value.  I *think*
+         * this is due to the XSetInputFocus calls in the menu system -- I
+         * think there is a race condition where a dying application with
+         * a posted menu can reclaim the focus *after* their window manager
+         * info is gone (matte and decor), but before the application
+         * really terminates.  This manifests itself as a "missing" FocusIn
+         * event, but we receive the FocusOut event when the window finally
+         * goes away.  The (hack) fix for this is in events.c (see FocusOut
+         * in the event handler).
+         */
 	XSetInputFocus(dpy, w, RevertToParent, MISC_FetchEventTime());
 	scr->mwm_focus = Fw;
     }
@@ -1696,8 +1710,6 @@ WIN_FreeNames(MwmWindow *tmp, Bool nukename, Bool nukeicon)
 	    XFree(tmp->icon_active_label);
 	tmp->icon_active_label = NULL;
     }
-
-    return;
 }
 
 /*
@@ -1802,13 +1814,19 @@ WIN_DestroyWindow(ScreenInfo *scr, MwmWindow *tmp)
     XSync(dpy, 0);
 
     if (tmp == scr->mwm_highlight)
+    {
 	scr->mwm_highlight = NULL;
+    }
 
     if (scr->mwm_last_focus == tmp)
+    {
 	scr->mwm_last_focus = NULL;
+    }
 
     if (scr->mwm_event == tmp)
+    {
 	scr->mwm_event = NULL;
+    }
 
     if (tmp == scr->mwm_focus && Mwm.keyboard_focus_policy == XmEXPLICIT &&
 	Mwm.auto_key_focus)
@@ -1824,13 +1842,17 @@ WIN_DestroyWindow(ScreenInfo *scr, MwmWindow *tmp)
 	    WIN_SetFocus(scr, tmp->ancestor->w, tmp->ancestor);
 	}
 	else
+        {
 	    WIN_SetFocus(scr, scr->no_focus_win, NULL);
+        }
     }
 
     MISC_RemoveFromTree(scr, tmp);
 
     if (scr->mwm_focus == tmp)
+    {
 	WIN_SetFocus(scr, scr->no_focus_win, NULL);
+    }
 
     MISC_SetFocusSequence(scr);
 
@@ -1915,5 +1937,4 @@ WIN_DestroyWindow(ScreenInfo *scr, MwmWindow *tmp)
 
     PAGER_Clear(scr);
     XSync(dpy, 0);
-    return;
 }
