@@ -1,6 +1,6 @@
 /**
  *
- * $Id: MacrosI.h,v 1.26 1998/04/22 19:47:19 u27113 Exp $
+ * $Id: MacrosI.h,v 1.31 1998/09/17 10:23:24 kbwj Exp $
  * 
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -50,6 +50,9 @@
 #define CoreColormap(w) \
     (((Widget)(w))->core.colormap)
 
+#define CoreAccelerators(w) \
+    (((Widget)(w))->core.accelerators)
+
 /* can't use XtWindow because we want to be sure which window
  * gets set. */
 #define CoreWindow(w) \
@@ -269,6 +272,9 @@
 #define MGR_SelectedGadget(m) \
 	(((XmManagerWidget)(m))->manager.selected_gadget)
 
+#define MGR_EligibleForMultiButtonEvent(m) \
+	(((XmManagerWidget)(m))->manager.eligible_for_multi_button_event)
+
 #define MGR_NavigationType(m) \
 	(((XmManagerWidget)(m))->manager.navigation_type)
 
@@ -289,6 +295,9 @@
 #define MGR_Children(m) \
 	(((XmManagerWidget)(m))->composite.children)
 
+#define MGR_NumSlots(m) \
+	(((XmManagerWidget)(m))->composite.num_slots)
+
 /*
  * GADGET ACCESS MACROS
  */
@@ -1922,6 +1931,15 @@
 /*
  * ROWCOLUMN WIDGET ACCESS MACROS
  */
+#define RC_OldWidth(m) \
+    (((XmRowColumnWidget)(m))->row_column.old_width)
+
+#define RC_OldHeight(m) \
+    (((XmRowColumnWidget)(m))->row_column.old_height)
+
+#define RC_OldShadowThickness(m) \
+    (((XmRowColumnWidget)(m))->row_column.old_shadow_thickness)
+
 #define RC_LastSelectToplevel(m) \
     (((XmRowColumnWidget)(m))->row_column.lastSelectToplevel)
 
@@ -2480,6 +2498,12 @@
 #define SW_CWY(w) \
     (((XmScrolledWindowWidget)w)->swindow.YOffset)
 
+#define SW_XOffset(w) \
+    (((XmScrolledWindowWidget)w)->swindow.XOffset)
+
+#define SW_YOffset(w) \
+    (((XmScrolledWindowWidget)w)->swindow.YOffset)
+
 #define SW_CWWidth(w) \
     (((XmScrolledWindowWidget)w)->swindow.AreaWidth)
 
@@ -2697,4 +2721,10 @@
 	? CoreColormap(((XmGadget)(w))->object.parent) \
 	: CoreColormap(w))
 
+/*
+ * Sometimes XmStrings are NULL and sometimes they are (XmString)XmUNSPECIFIED,
+ * both mean stay away.
+ */
+#define _XmStringIsSpecified(x) ((x) != NULL && (x) != (XmString)XmUNSPECIFIED)
+
 #endif /* MacrosI.h */
