@@ -1,6 +1,6 @@
 /**
  *
- * $Id: XmI.h,v 1.13 1998/03/24 23:00:39 rwscott Exp $
+ * $Id: XmI.h,v 1.20 1998/10/21 22:46:33 danny Exp $
  * 
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -113,17 +113,6 @@ struct _XmtStringContextRec {
 #define XmSTRING_TAG                    0xDFU
 #define XmSTRING_LENGTH                 0x80U
 
-/*
- * ArmTimeout record
- */
-typedef struct
-{
-    Widget w;
-    XEvent ev;
-    int has_event; /* boolean to determine if event is valid */
-}
-XmArmTimeoutRec;
-
 /*
  * XmIm stuff
  */
@@ -176,27 +165,9 @@ XmMWValues;
 /* default procs */
 void _XmCascadePixmapDefault(Widget w, int offset, XrmValue *val);
 
-/* resource converters */
-extern Boolean _XmCvtStringToXmString(Display *, XrmValue *, Cardinal *,
-				      XrmValue *, XrmValue *, XtPointer *);
-extern Boolean _XmCvtStringToFontlist(Display *, XrmValue *, Cardinal *,
-				      XrmValue *, XrmValue *, XtPointer *);
-extern Boolean _XmCvtStringToDimension(Display *, XrmValue *, Cardinal *,
-				       XrmValue *, XrmValue *, XtPointer *);
-extern Boolean _XmCvtStringToPosition(Display *, XrmValue *, Cardinal *,
-				      XrmValue *, XrmValue *, XtPointer *);
-extern Boolean _XmCvtStringToKeySym(Display *, XrmValue *, Cardinal *,
-				      XrmValue *, XrmValue *, XtPointer *);
-extern Boolean _XmCvtStringToStringTable (Display*, XrmValue*, Cardinal*,
-					  XrmValue*, XrmValue*, XtPointer*);
-
 /* Find the VendorShell Extension Object */
 extern Widget _LtFindVendorExt(Widget);
 
-/* resource destroyers */
-void _XmDestroyStringTable (XtAppContext, XrmValue*, XtPointer, XrmValue*,
-			    Cardinal*);
-
 /*
  * fontlist prototypes
  */
@@ -307,6 +278,10 @@ extern void _XmManagerUninstallMnemonic(Widget m, Widget w);
 extern Position _XmScrollBarValueToPos(Widget sw, int value);
 extern Dimension _XmScrollBarSliderPixSize(Widget sw);
 
+/* for ScrolledW */
+extern void _XmScrolledWindowPreferredSize(Widget w, XtWidgetGeometry *request,
+				      XtWidgetGeometry *answer);
+
 /* for TearOff */
 extern void _XmPushButtonSetTranslation(Widget, int);
 
@@ -343,6 +318,13 @@ extern XmModifierMaskSetReference _XmGetModifierMappingsForDisplay(Display *Dsp)
 extern void _XmInvalidateModifierMappingsForDisplay(Display *Dsp);
 extern void _XmRefreshVirtKeys(Widget w);
 
+/* Visual */
+
+extern void _XmAddBackgroundToColorCache(Screen *screen, Colormap color_map,
+					 String background_spec,
+					 int rgb_fallback, XrmValue *val);
+extern void _XmInvalidateColorCache(Boolean default_only);
+
 /*
  * Behaviour control for XmFormLayout, XmRowColumnLayout.
  * This is their second parameter.
@@ -357,6 +339,16 @@ extern void _XmRefreshVirtKeys(Widget w);
 extern Pixel _XmWhitePixelOfObject(Widget);
 extern Pixel _XmBlackPixelOfObject(Widget);
 
+/*
+ * Thread stuff (lib/Xm/misc.c).
+ */
+extern void _XmAppLock(XtAppContext appc);
+extern void _XmAppUnlock(XtAppContext appc);
+extern void _XmProcessLock(void);
+extern void _XmProcessUnlock(void);
+extern void _XmObjectLock(Widget w);
+extern void _XmObjectUnlock(Widget w);
+
 #ifdef __cplusplus
 }
 #endif
