@@ -1,6 +1,6 @@
 /**
  *
- * $Id: XmP.h,v 1.3 1997/10/16 00:24:47 rwscott Exp $
+ * $Id: XmP.h,v 1.5 1998/10/03 06:30:00 gritton Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -1176,6 +1176,7 @@ typedef struct _XmColorData {
 void _XmRegisterPixmapConverters(void);
 char *_XmGetBGPixmapName(void);
 void _XmClearBGPixmap(void);
+void XmeGetDefaultPixel(Widget widget, int type, int offset, XrmValue *value);
 void _XmForegroundColorDefault(Widget widget, int offset, XrmValue *value);
 void _XmHighlightColorDefault(Widget widget, int offset, XrmValue *value);
 void _XmBackgroundColorDefault(Widget widget, int offset, XrmValue *value);
@@ -1189,8 +1190,7 @@ void _XmPrimitiveHighlightPixmapDefault(Widget widget, int offset,
 					XrmValue *value);
 void _XmManagerHighlightPixmapDefault(Widget widget, int offset,
 				      XrmValue *value);
-void _XmGetDefaultThresholdsForScreen(Widget widget, int offset,
-				      XrmValue *value);
+void _XmGetDefaultThresholdsForScreen(Screen *screen);
 String _XmGetDefaultBackgroundColorSpec(Screen *screen);
 void _XmSetDefaultBackgroundColorSpec(Screen *screen, String new_color_spec);
 XmColorData *_XmGetDefaultColors(Screen *screen, Colormap color_map);
