@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Mrm.c,v 1.7 1998/03/11 21:23:14 u27113 Exp $
+ * $Id: Mrm.c,v 1.8 1998/06/19 17:05:41 jon Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -381,10 +381,10 @@ MakePixmap(Display *d, Pixmap *pixmap, PixmapType * data, Pixel fg,
     GCTable = (GC *)malloc(sizeof(GC) * NumColors);
     if (NULL == ColorTable[0].StringXPM)
     {
-	long theColor;
-
 	for (i = 0; i < NumColors; i++)
 	{
+	    long theColor = 0;
+
 	    value = ColorTable[i].Color;
 	    if (ColorTable[i].IsAddress)
 	    {
