@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/GrabShell.c,v 1.11 1998/01/12 02:12:26 pgw Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/GrabShell.c,v 1.13 1998/09/24 15:37:37 pgw Exp $
  *
  * Copyright (C) 1997 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/GrabShell.c,v 1.11 1998/01/12 02:12:26 pgw Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/GrabShell.c,v 1.13 1998/09/24 15:37:37 pgw Exp $";
 
 
 #include <LTconfig.h>
@@ -292,7 +292,8 @@ BtnDown(
    XButtonEvent *bev;
    Position rootx_return, rooty_return;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "BtnDown\n"));
+   DEBUGOUT(XdbDebug(__FILE__, w, "%s:BtnDown\n",
+   	__FILE__));
 
    /* Check to see if mapped yet */
    if(GS_Mapped(w) == False)
@@ -302,7 +303,7 @@ BtnDown(
 
    bev = (XButtonEvent *) xev;
 
-   XtTranslateCoords(w, XtX(w), XtY(w), &rootx_return, &rooty_return);
+   XtTranslateCoords(w, 0, 0, &rootx_return, &rooty_return);
 
    /* if within grab shell window then return */
    if ((bev->x_root >= rootx_return 
