@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Notebook.c,v 1.10 1998/01/12 02:12:28 pgw Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Notebook.c,v 1.14 1998/09/24 15:37:38 pgw Exp $
  *
  * Copyright (C) 1997-1998 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Notebook.c,v 1.10 1998/01/12 02:12:28 pgw Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Notebook.c,v 1.14 1998/09/24 15:37:38 pgw Exp $";
 
 /*
  * Also recognise 1.2 widgets
@@ -851,10 +851,57 @@ ConstraintInitialize(Widget request, Widget nw,
 		_XmWarning(nw, "has no XmQTnavigator trait\n");
 	}
     } else if (NBC_ChildType(nw) == XmPAGE) {
-	if (NB_FirstPageNumber(nb) > NBC_PageNumber(nw))
+	/*
+	 * When a page without a page number is managed, Notebook assigns
+	 * it the smallest unallocated page number that is not less than
+	 * the first page number and greater than the last allocated page
+	 * number.
+	 */
+	if (NBC_PageNumber(nw) == 0) {
+	    int	i, pn;
+
+	    pn = 0;
+	    for (i=0; i<MGR_NumChildren(nb); i++) {
+		Widget	c = MGR_Children(nb)[i];
+
+		if (NBC_ChildType(c) == XmPAGE && pn < NBC_PageNumber(c))
+			pn = NBC_PageNumber(c);
+	    }
+
+	    if (pn && pn >= NB_FirstPageNumber(nb))
+		NBC_PageNumber(nw) = pn + 1;
+	    else
+		NBC_PageNumber(nw) = NB_FirstPageNumber(nb);
+	} else {
+	    /* FIX ME */
+
+	    if (NB_FirstPageNumber(nb) > NBC_PageNumber(nw))
 		NB_FirstPageNumber(nb) = NBC_PageNumber(nw);
-	if (NB_LastPageNumber(nb) < NBC_PageNumber(nw))
+	    if (NB_LastPageNumber(nb) < NBC_PageNumber(nw))
 		NB_LastPageNumber(nb) = NBC_PageNumber(nw);
+	}
+
+	NB_LastAllocNum(nb) = NBC_PageNumber(nw);
+
+	DEBUGOUT(XdbDebug2(__FILE__, nb, nw, "Child has page no %d\n",
+		NBC_PageNumber(nw)));
+
+    }
+
+    if (NBC_ChildType(nw) == XmMAJOR_TAB || NBC_ChildType(nw) == XmMINOR_TAB ||
+	NBC_ChildType(nw) == XmSTATUS_AREA) {
+	/*
+	 * When a tab or a status area without a page number is managed,
+	 * the newly managed widget is assigned the page number of the
+	 * most recently managed page, unless the page already has the
+	 * same type of child. If the page does have the same type of child,
+	 * Notebook assigns the newly managed widget a page number one
+	 * greater than the most recently managed page; this new page number
+	 * is now occupied.
+	 */
+
+	if (NBC_PageNumber(nw) == 0)
+	    NBC_PageNumber(nw) = NB_LastAllocNum(nb);
     }
 
     /*
@@ -1337,11 +1384,16 @@ _XmNBBackPageBackgroundDefault(Widget w, int offset, XrmValue *val)
     back.green = color.green * 9 / 10;
     back.blue = color.blue * 9 / 10;
 
-    back.flags = DoRed | DoGreen | DoBlue;
+    /* If the color allocation fails,
+     * just use the same as the frame background
+     */
 
-    XAllocColor(XtDisplayOfObject(w),
-		DefaultColormapOfScreen(XtScreenOfObject(w)),
-		&back);
+    if (!XAllocColor(XtDisplayOfObject(w),
+		     DefaultColormapOfScreen(XtScreenOfObject(w)),
+		     &back))
+    {
+	back.pixel = color.pixel;
+    }
 
     DEBUGOUT(XdbDebug(__FILE__, w,
 	"_XmNBBackPageBackgroundDefault %d (%d,%d,%d) -> %d (%d,%d,%d)\n",
@@ -1365,6 +1417,9 @@ _XmNBTraitGetInfo(Widget sfw,
 		Cardinal *num_nav_list)
 {
 	DEBUGOUT(XdbDebug(__FILE__, sfw, "_XmNBTraitGetInfo\n"));
+
+	/* FIX ME unimplemented */
+	return False;
 }
 
 static void
