@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/PanedW.c,v 1.5 1998/05/13 21:06:36 u27113 Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/PanedW.c,v 1.7 1998/09/24 15:37:38 pgw Exp $
  *
  * Copyright (C) 1995-1998 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: PanedW.c,v 1.5 1998/05/13 21:06:36 u27113 Exp $";
+static const char rcsid[] = "$Id: PanedW.c,v 1.7 1998/09/24 15:37:38 pgw Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -1661,10 +1661,13 @@ _XmPanedWHLayout(Widget w, Widget rchild,
 		XmKidGeometry *boxptr, XmKidGeometry *sashptr,
 		XmKidGeometry *sepptr)
 {
+#define SEXT(x) \
+       ((int)((short)(x)))
     int i, save_mode;
     Dimension width, nw, curw, curh;
     Widget child;
     XmKidGeometry boxes, sashes, seps;
+    int total_delta ;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "Pane Horizontal Layout\n"));
 
@@ -1679,21 +1682,74 @@ _XmPanedWHLayout(Widget w, Widget rchild,
     seps = (XmKidGeometry)XtCalloc(PW_PaneCount(w) + 1,
 				   sizeof(XmKidGeometryRec));
 
-    /* compute the optimimum layout */
-    width = PW_MarginWidth(w);
-
+    /* ------- New ------------- */
+    /* Pass 1 - each pane can be its desired width but within its PaneMaximum and PaneMinimum */
+    width = 2 * PW_MarginWidth(w);     /* margins on either side */
     for (i = 0; i < PW_PaneCount(w); i++)
     {
 	child = PW_ManagedChildren(w)[i];
 
-	if (i == PW_PaneCount(w) - 1)
+       boxes[i].box.width = PWC_DWidth(child);
+       if (SEXT(boxes[i].box.width) > SEXT(PWC_PaneMaximum(child)))
+           boxes[i].box.width = PWC_PaneMaximum(child) ;
+       else if (SEXT(boxes[i].box.width) < SEXT(PWC_PaneMinimum(child)))
+           boxes[i].box.width = PWC_PaneMinimum(child) ;
+       width += boxes[i].box.width + PW_Spacing(w) ;
+    }
+    width -= PW_Spacing(w) ;  /* no need for last spacing */
+    total_delta = curw - width ;       /* adjustment needed to fill out to current width */
+
+    /* pass 2 - expand or contract each pane to make fit within curw */
+    for (i = PW_PaneCount(w) - 1; i >= 0; i--)
+    {
+       child = PW_ManagedChildren(w)[i];
+
+       if (PWC_SkipAdjust(child) )     /* we can't change the size of this one */
+           continue;
+       if (total_delta > 0)    /* need to make some pane(s) bigger */
 	{
-	    nw = curw - width - PW_MarginWidth(w);
+           if (SEXT(boxes[i].box.width) < SEXT(PWC_PaneMaximum(child)))        /* can grow */
+           {
+               if (SEXT(boxes[i].box.width) + total_delta <= SEXT(PWC_PaneMaximum(child)))
+               {
+                   boxes[i].box.width += total_delta;  /* can absorb full delta, so done */
+                   break;
+               }
+               else
+               {
+                   /* make pane maximum, but this leaves some delta left so continue */
+                   total_delta -= SEXT(PWC_PaneMaximum(child)) - SEXT(boxes[i].box.width);
+                   boxes[i].box.width = PWC_PaneMaximum(child) ;
+               }
+           }
 	}
-	else
+       else if (total_delta < 0)       /* need to make some pane(s) smaller */
 	{
-	    nw = PWC_DWidth(child);
+           if (SEXT(boxes[i].box.width) > SEXT(PWC_PaneMinimum(child)))        /* allowed to shrink */
+           {
+               if (SEXT(boxes[i].box.width) + total_delta >= SEXT(PWC_PaneMinimum(child)))
+               {
+                   boxes[i].box.width += total_delta;  /* can contract by full delta, so done */
+                   break;
+               }
+               else
+               {
+                   /* make pane minimum, but this leaves some delta left so continue */
+                   total_delta -= SEXT(PWC_PaneMinimum(child)) - SEXT(boxes[i].box.width);
+                   boxes[i].box.width = PWC_PaneMinimum(child) ;
+               }
+           }
 	}
+    }
+    /* ------- End New ------------- */
+
+    /* now we have the sizes of the panes determined and we can position
+       them along with their sashes and Separators */
+    width = PW_MarginWidth(w);
+    for (i = 0; i < PW_PaneCount(w); i++)
+    {
+       child = PW_ManagedChildren(w)[i];
+       nw = boxes[i].box.width ;
 	boxes[i].kid = child;
 	boxes[i].box.y = PW_MarginHeight(w);
 	boxes[i].box.height = curh - 2 * PW_MarginHeight(w);
@@ -1702,7 +1758,7 @@ _XmPanedWHLayout(Widget w, Widget rchild,
 	boxes[i].box.border_width = XtBorderWidth(child);
 
 	PWC_DX(child) = width;
-	width += PWC_DWidth(child);
+       width += nw;
 
 	if (i != (PW_PaneCount(w) - 1))
 	{
@@ -1742,119 +1798,7 @@ _XmPanedWHLayout(Widget w, Widget rchild,
 	}
     }
 
-#define SEXT(x) \
-	((int)((short)(x)))
-
-    /* now make it fit, by rippling the geom differences */
-    for (i = PW_PaneCount(w) - 1; i >= 0; i--)
-    {
-	int delta;
-	int j;
-
-	child = PW_ManagedChildren(w)[i];
-
-	if (SEXT(boxes[i].box.width) > SEXT(PWC_PaneMaximum(child)))
-	{
-	    delta = SEXT(boxes[i].box.width) - SEXT(PWC_PaneMaximum(child));
-
-	    boxes[i].box.width = PWC_PaneMaximum(child);
-
-	    for (j = i - 1; j >= 0; j--)
-	    {
-		int k;
-
-		if (SEXT(boxes[j].box.width) <
-		    SEXT(PWC_PaneMaximum(PW_ManagedChildren(w)[j])))
-		{
-		    if (delta <
-			(SEXT(PWC_PaneMaximum(PW_ManagedChildren(w)[j])) -
-			 SEXT(boxes[j].box.width)))
-		    {
-			boxes[j].box.width = SEXT(boxes[j].box.width) + delta;
-
-			/* move lower boxes down */
-			for (k = j + 1; k < PW_PaneCount(w); k++)
-			{
-			    boxes[k].box.x += delta;
-			    sashes[k - 1].box.x += delta;
-			    seps[k - 1].box.x += delta;
-			}
-
-			break;	/* out of j-loop */
-		    }
-		    else
-		    {
-			int ddt;
-
-			ddt = SEXT(PWC_PaneMaximum(PW_ManagedChildren(w)[j])) -
-			    SEXT(boxes[j].box.width);
-
-			boxes[j].box.width = SEXT(boxes[j].box.width) + ddt;
-
-			delta -= ddt;
-
-			/* move lower boxes down */
-			for (k = j + 1; k < PW_PaneCount(w); k++)
-			{
-			    boxes[k].box.x += ddt;
-			    sashes[k - 1].box.x += ddt;
-			    seps[k - 1].box.x += ddt;
-			}
-		    }
-		}
-	    }			/* end j loop */
-	}
-	else if (SEXT(boxes[i].box.width) < SEXT(PWC_PaneMinimum(child)))
-	{
-	    delta = SEXT(PWC_PaneMinimum(child)) - SEXT(boxes[i].box.width);
-
-	    boxes[i].box.width = PWC_PaneMinimum(child);
-
-	    for (j = i - 1; j >= 0; j--)
-	    {
-		int k;
-
-		if (SEXT(boxes[j].box.width) >
-		    SEXT(PWC_PaneMinimum(PW_ManagedChildren(w)[j])))
-		{
-		    if (delta < (SEXT(boxes[j].box.width) -
-			      SEXT(PWC_PaneMinimum(PW_ManagedChildren(w)[j]))))
-		    {
-			boxes[j].box.width = SEXT(boxes[j].box.width) - delta;
-
-			/* move lower boxes up */
-			for (k = j + 1; k < PW_PaneCount(w); k++)
-			{
-			    boxes[k].box.x -= delta;
-			    sashes[k - 1].box.x -= delta;
-			    seps[k - 1].box.x -= delta;
-			}
-
-			break;	/* out of j-loop */
-		    }
-		    else
-		    {
-			int ddt;
-
-			ddt = SEXT(boxes[j].box.width) -
-			    SEXT(PWC_PaneMinimum(PW_ManagedChildren(w)[j]));
-
-			boxes[j].box.width = SEXT(boxes[j].box.width) - ddt;
-
-			delta -= ddt;
-
-			/* move lower boxes up */
-			for (k = j + 1; k < PW_PaneCount(w); k++)
-			{
-			    boxes[k].box.x -= ddt;
-			    sashes[k - 1].box.x -= ddt;
-			    seps[k - 1].box.x -= ddt;
-			}
-		    }
-		}
-	    }			/* end j loop */
-	}
-    }
+    /* rippling code no longer needed */
 
     /* now set childgeom */
     if (rchild && childgeom)
@@ -1934,10 +1878,13 @@ _XmPanedWVLayout(Widget w, Widget rchild,
 		XmKidGeometry *boxptr, XmKidGeometry *sashptr,
 		XmKidGeometry *sepptr)
 {
+#define SEXT(x) \
+       ((int)((short)(x)))
     int i, save_mode;
     Dimension height, nh, curw, curh;
     Widget child;
     XmKidGeometry boxes, sashes, seps;
+    int total_delta ;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "Pane Vertical Layout\n"));
 
@@ -1952,21 +1899,77 @@ _XmPanedWVLayout(Widget w, Widget rchild,
     seps = (XmKidGeometry)XtCalloc(PW_PaneCount(w) + 1,
 				   sizeof(XmKidGeometryRec));
 
-    /* compute the optimimum layout */
-    height = PW_MarginHeight(w);
-
+    /* ------- New ------------- */
+    /* Pass 1 - each pane can be its desired height but within its PaneMaximum and PaneMinimum */
+    height = 2 * PW_MarginHeight(w);   /* margins on either side */
     for (i = 0; i < PW_PaneCount(w); i++)
     {
 	child = PW_ManagedChildren(w)[i];
 
-	if (i == PW_PaneCount(w) - 1)
+       boxes[i].box.height = PWC_DHeight(child);
+       if (SEXT(boxes[i].box.height) > SEXT(PWC_PaneMaximum(child)))
+           boxes[i].box.height = PWC_PaneMaximum(child) ;
+       else if (SEXT(boxes[i].box.height) < SEXT(PWC_PaneMinimum(child)))
+           boxes[i].box.height = PWC_PaneMinimum(child) ;
+       height += boxes[i].box.height + PW_Spacing(w) ;
+    }
+    height -= PW_Spacing(w) ;  /* no need for last spacing */
+    total_delta = curh - height ;      /* adjustment needed to fill out to current height */
+
+    /* pass 2 - expand or contract each pane to make fit within curh */
+    for (i = PW_PaneCount(w) - 1; i >= 0; i--)
+    {
+       child = PW_ManagedChildren(w)[i];
+
+       if (PWC_SkipAdjust(child) )     /* we can't change the size of this one */
+           continue;
+       if (total_delta > 0)    /* need to make some pane(s) bigger */
 	{
-	    nh = curh - height - PW_MarginHeight(w);
+           if (SEXT(boxes[i].box.height) < SEXT(PWC_PaneMaximum(child)))       /* can grow */
+           {
+               if (SEXT(boxes[i].box.height) + total_delta <= SEXT(PWC_PaneMaximum(child)))
+               {
+                   boxes[i].box.height += total_delta; /* can absorb full delta, so done */
+                   break;
+               }
+               else
+               {
+                   /* make pane maximum, but this leaves some delta left so continue */
+                   total_delta -= SEXT(PWC_PaneMaximum(child)) - SEXT(boxes[i].box.height);
+                   boxes[i].box.height = PWC_PaneMaximum(child) ;
+               }
+           }
 	}
-	else
+       else if (total_delta < 0)       /* need to make some pane(s) smaller */
 	{
-	    nh = PWC_DHeight(child);
+           if (SEXT(boxes[i].box.height) > SEXT(PWC_PaneMinimum(child)))       /* allowed to shrink */
+           {
+               if (SEXT(boxes[i].box.height) + total_delta >= SEXT(PWC_PaneMinimum(child)))
+               {
+                   boxes[i].box.height += total_delta; /* can contract by full delta, so done */
+                   break;
+               }
+               else
+               {
+                   /* make pane minimum, but this leaves some delta left so continue */
+                   total_delta -= SEXT(PWC_PaneMinimum(child)) - SEXT(boxes[i].box.height);
+                   boxes[i].box.height = PWC_PaneMinimum(child) ;
+               }
+           }
 	}
+    }
+    /* ------- End New ------------- */
+
+    /* now we have the sizes of the panes determined and we can position
+       them along with their sashes and Separators */
+    height = PW_MarginHeight(w);
+
+    for (i = 0; i < PW_PaneCount(w); i++)
+    {
+       child = PW_ManagedChildren(w)[i];
+
+       nh = boxes[i].box.height ;
+
 	boxes[i].kid = child;
 	boxes[i].box.x = PW_MarginWidth(w);
 	boxes[i].box.width = curw - 2 * PW_MarginWidth(w);
@@ -1975,7 +1978,7 @@ _XmPanedWVLayout(Widget w, Widget rchild,
 	boxes[i].box.border_width = XtBorderWidth(child);
 
 	PWC_DY(child) = height;
-	height += PWC_DHeight(child);
+       height += nh;
 
 	if (i != (PW_PaneCount(w) - 1))
 	{
@@ -2016,119 +2019,7 @@ _XmPanedWVLayout(Widget w, Widget rchild,
 	}
     }
 
-#define SEXT(x) \
-	((int)((short)(x)))
-
-    /* now make it fit, by rippling the geom differences */
-    for (i = PW_PaneCount(w) - 1; i >= 0; i--)
-    {
-	int delta;
-	int j;
-
-	child = PW_ManagedChildren(w)[i];
-
-	if (SEXT(boxes[i].box.height) > SEXT(PWC_PaneMaximum(child)))
-	{
-	    delta = SEXT(boxes[i].box.height) - SEXT(PWC_PaneMaximum(child));
-
-	    boxes[i].box.height = PWC_PaneMaximum(child);
-
-	    for (j = i - 1; j >= 0; j--)
-	    {
-		int k;
-
-		if (SEXT(boxes[j].box.height) <
-		    SEXT(PWC_PaneMaximum(PW_ManagedChildren(w)[j])))
-		{
-		    if (delta <
-			(SEXT(PWC_PaneMaximum(PW_ManagedChildren(w)[j])) -
-			 SEXT(boxes[j].box.height)))
-		    {
-			boxes[j].box.height = SEXT(boxes[j].box.height) + delta;
-
-			/* move lower boxes down */
-			for (k = j + 1; k < PW_PaneCount(w); k++)
-			{
-			    boxes[k].box.y += delta;
-			    sashes[k - 1].box.y += delta;
-			    seps[k - 1].box.y += delta;
-			}
-
-			break;	/* out of j-loop */
-		    }
-		    else
-		    {
-			int ddt;
-
-			ddt = SEXT(PWC_PaneMaximum(PW_ManagedChildren(w)[j])) -
-			    SEXT(boxes[j].box.height);
-
-			boxes[j].box.height = SEXT(boxes[j].box.height) + ddt;
-
-			delta -= ddt;
-
-			/* move lower boxes down */
-			for (k = j + 1; k < PW_PaneCount(w); k++)
-			{
-			    boxes[k].box.y += ddt;
-			    sashes[k - 1].box.y += ddt;
-			    seps[k - 1].box.y += ddt;
-			}
-		    }
-		}
-	    }			/* end j loop */
-	}
-	else if (SEXT(boxes[i].box.height) < SEXT(PWC_PaneMinimum(child)))
-	{
-	    delta = SEXT(PWC_PaneMinimum(child)) - SEXT(boxes[i].box.height);
-
-	    boxes[i].box.height = PWC_PaneMinimum(child);
-
-	    for (j = i - 1; j >= 0; j--)
-	    {
-		int k;
-
-		if (SEXT(boxes[j].box.height) >
-		    SEXT(PWC_PaneMinimum(PW_ManagedChildren(w)[j])))
-		{
-		    if (delta < (SEXT(boxes[j].box.height) -
-			      SEXT(PWC_PaneMinimum(PW_ManagedChildren(w)[j]))))
-		    {
-			boxes[j].box.height = SEXT(boxes[j].box.height) - delta;
-
-			/* move lower boxes up */
-			for (k = j + 1; k < PW_PaneCount(w); k++)
-			{
-			    boxes[k].box.y -= delta;
-			    sashes[k - 1].box.y -= delta;
-			    seps[k - 1].box.y -= delta;
-			}
-
-			break;	/* out of j-loop */
-		    }
-		    else
-		    {
-			int ddt;
-
-			ddt = SEXT(boxes[j].box.height) -
-			    SEXT(PWC_PaneMinimum(PW_ManagedChildren(w)[j]));
-
-			boxes[j].box.height = SEXT(boxes[j].box.height) - ddt;
-
-			delta -= ddt;
-
-			/* move lower boxes up */
-			for (k = j + 1; k < PW_PaneCount(w); k++)
-			{
-			    boxes[k].box.y -= ddt;
-			    sashes[k - 1].box.y -= ddt;
-			    seps[k - 1].box.y -= ddt;
-			}
-		    }
-		}
-	    }			/* end j loop */
-	}
-    }
+    /* rippling code no longer needed */
 
     /* now set childgeom */
     if (rchild && childgeom)
