@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/SpinBox.c,v 1.20 1998/01/09 15:18:21 miers Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/SpinBox.c,v 1.22 1998/09/24 15:37:40 pgw Exp $
  *
  * Copyright (C) 1997-1998 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/SpinBox.c,v 1.20 1998/01/09 15:18:21 miers Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/SpinBox.c,v 1.22 1998/09/24 15:37:40 pgw Exp $";
 
 #include <stdio.h>
 #include <LTconfig.h>
@@ -461,7 +461,7 @@ initialize(Widget request, Widget nw,
 #endif
 
 	SPB_TextW(nw) = NULL;
-	SPB_TimerId(nw) = NULL;
+	SPB_TimerId(nw) = (XtIntervalId)0;
 
 	if (XtWidth(request) == 0 && XtHeight(request) == 0) {
 		XtWidth(nw) = 200;
@@ -633,6 +633,7 @@ PreferredSize(	Widget w, Widget instig, XtWidgetGeometry *ig,
 	Dimension	hh = 0;
 
 	xx = SPB_MarginWidth(w);
+	yy = SPB_MarginHeight(w);
 
 	for (i=0; i<MGR_NumChildren(w); i++)
 		if (XtIsManaged(MGR_Children(w)[i])) {
@@ -1072,7 +1073,7 @@ SpinBDisarm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
 	if (SPB_TimerId(w))
 		XtRemoveTimeOut(SPB_TimerId(w));
-	SPB_TimerId(w) = NULL;
+	SPB_TimerId(w) = (XtIntervalId)0;
 
 	SPB_UpArrowPressed(w) = False;
 	SPB_DownArrowPressed(w) = False;
@@ -1128,7 +1129,6 @@ static void
 InsertChild(Widget w)
 {
     Widget	spb = XtParent(w);
-	Arg		a;
 
     DEBUGOUT(XdbDebug2(__FILE__, (Widget)spb, w, "SpinBox InsertChild\n"));
 
@@ -1138,6 +1138,8 @@ InsertChild(Widget w)
 
 	if (XmeTraitGet(XtClass(w), XmQTaccessTextual)) {
 #if 0
+		Arg		a;
+
 		DEBUGOUT(XdbDebug2(__FILE__, spb, w,
 			"Adding accelerators to child\n"));
 
