@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ArrowB.c,v 1.18 1998/03/24 23:00:45 rwscott Exp $
+ * $Id: ArrowB.c,v 1.24 1998/10/10 07:40:44 gritton Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: ArrowB.c,v 1.18 1998/03/24 23:00:45 rwscott Exp $";
+static const char rcsid[] = "$Id: ArrowB.c,v 1.24 1998/10/10 07:40:44 gritton Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -39,6 +39,7 @@ static char rcsid[] = "$Id: ArrowB.c,v 1.18 1998/03/24 23:00:45 rwscott Exp $";
  * Try to add trait stuff by #ifdeffing it.
  */
 #if XmVERSION > 1
+#include <Xm/TraitP.h>
 #include <Xm/ActivatableT.h>
 
 void _XmArrowB_TraitAddCallback(Widget, XtCallbackProc, XtPointer, Boolean);
@@ -50,7 +51,9 @@ static XmActivatableTraitRec _XmArrowBTraitRec = {
 #endif
 
 /* Forward Declarations */
+#if XmVERSION > 1
 static void class_initialize();
+#endif
 static void class_part_initialize(WidgetClass w_class);
 
 static void initialize(Widget request, Widget new_w,
@@ -60,9 +63,11 @@ static void destroy(Widget w);
 
 static void expose(Widget w, XEvent *event, Region region);
 
+#if 0
 static XtGeometryResult query_geometry(Widget w,
 				       XtWidgetGeometry *proposed,
 				       XtWidgetGeometry *answer);
+#endif
 
 static Boolean set_values(Widget current, Widget request, Widget new_w,
 			  ArgList args, Cardinal *num_args);
@@ -198,17 +203,17 @@ XmArrowButtonClassRec xmArrowButtonClassRec = {
 
 WidgetClass xmArrowButtonWidgetClass = (WidgetClass)&xmArrowButtonClassRec;
 
+#if XmVERSION > 1
 static void
 class_initialize()
 {
-#if XmVERSION > 1
     if (! XmeTraitSet(xmArrowButtonWidgetClass, XmQTactivatable,
 		&_XmArrowBTraitRec)) {
 	_XmWarning(NULL,
 	    "XmArrowButtonWidget ClassInitialize: XmeTraitSet failed\n");
     }
-#endif
 }
+#endif
 
 static void
 class_part_initialize(WidgetClass widget_class)
@@ -269,11 +274,11 @@ initialize(Widget request, Widget new_w,
 {
     if (XtWidth(request) == (Dimension)0)
     {
-	XtWidth(new_w) = 20;
+	XtWidth(new_w) += 15;
     }
     if (XtHeight(request) == (Dimension)0)
     {
-	XtHeight(new_w) = 20;
+	XtHeight(new_w) += 15;
     }
 
     AB_Armed(new_w) = False;
@@ -345,6 +350,15 @@ expose(Widget w, XEvent *event, Region region)
 	myGC = AB_InsensitiveGC(w);
     }
 
+    if (Prim_Highlighted(w))
+    {
+	    (*PrimC_BorderHighlight(XtClass(w))) (w);
+    }
+    else
+    {
+	(*PrimC_BorderUnhighlight(XtClass(w))) (w);
+    }
+
     _XmDrawShadows(XtDisplay(w),
 		   XtWindow(w),
 		   Prim_TopShadowGC(w),
@@ -370,7 +384,7 @@ expose(Widget w, XEvent *event, Region region)
 	   this
 	   Prim_ShadowThickness(w),
 	 */
-		     1,
+		     2,
 		     AB_Direction(w));
     }
     else
@@ -386,11 +400,12 @@ expose(Widget w, XEvent *event, Region region)
 	   this
 	   Prim_ShadowThickness(w),
 	 */
-		     1,
+		     2,
 		     AB_Direction(w));
     }
 }
 
+#if 0
 static XtGeometryResult
 query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 {
@@ -415,6 +430,7 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 	return XtGeometryAlmost;
     }
 }
+#endif
 
 
 static void
@@ -425,27 +441,30 @@ Arm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
     DEBUGOUT(XdbDebug(__FILE__, w, "Arm\n"));
 
-    XmProcessTraversal(w, XmTRAVERSE_CURRENT);
-
-    AB_Armed(w) = True;
-    if (event)
+    if (!AB_Armed(w))
     {
-	AB_ArmTimeStamp(w) = event->xbutton.time;
-    }
+	XmProcessTraversal(w, XmTRAVERSE_CURRENT);
 
-    (*exp) (w, event, (Region)NULL);
+	AB_Armed(w) = True;
+	if (event)
+	{
+	    AB_ArmTimeStamp(w) = event->xbutton.time;
+	}
 
-    if (AB_ArmCallback(w))
-    {
-	cbs.reason = XmCR_ARM;
-	cbs.event = event;
-	cbs.click_count = AB_ClickCount(w);
+	(*exp) (w, event, (Region)NULL);
 
-	XFlush(XtDisplay(w));
+	if (AB_ArmCallback(w))
+	{
+	    cbs.reason = XmCR_ARM;
+	    cbs.event = event;
+	    cbs.click_count = AB_ClickCount(w);
 
-	XtCallCallbackList(w,
-			   AB_ArmCallback(w),
-			   (XtPointer)&cbs);
+	    XFlush(XtDisplay(w));
+
+	    XtCallCallbackList(w,
+			       AB_ArmCallback(w),
+			       (XtPointer)&cbs);
+	}
     }
 }
 
@@ -514,65 +533,50 @@ Disarm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 ArmTimeout(XtPointer data, XtIntervalId *id)
 {
-    XmArrowButtonCallbackStruct cbs;
-    XmArmTimeoutRec *p = (XmArmTimeoutRec *)data;
-    Widget w = p->w;
+    Widget w = (Widget)data;
     XtExposeProc exp = XtClass(w)->core_class.expose;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "ArmTimeout\n"));
  
-    if (AB_Armed(w) == False)
+    AB_Timer(w) = 0;
+
+    if (XtIsRealized(w))
     {
-        return;
+	(*exp) (w, NULL, NULL);
+	XFlush(XtDisplay(w));
     }
- 
-    AB_Armed(w) = False;
-
-    (*exp) (w, NULL, NULL);
- 
-    if (AB_ActivateCallback(w))
-    {
-        cbs.reason = XmCR_ACTIVATE;
- 
-        /* Modified by JAC to allow for null event */
-        if (p->has_event)
-        {
-            cbs.event = &p->ev;
-        }
-        else
-        {
-            cbs.event = NULL;
-        }
-        cbs.click_count = AB_ClickCount(w);
- 
-        XFlush(XtDisplay(w));
- 
-        XtCallCallbackList(w,
-                           AB_ActivateCallback(w),
-                           (XtPointer)&cbs);
-    }
-
-    Disarm(w, NULL, NULL, 0);
-
-    XtFree(data);
 }
 
 static void
 ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    XmArmTimeoutRec *p = (XmArmTimeoutRec *)XtCalloc(1,
-						     sizeof(XmArmTimeoutRec));
+    XmArrowButtonCallbackStruct cbs;
  
     DEBUGOUT(XdbDebug(__FILE__, w, "ArmAndActivate\n"));
 
+    /* Arm, Activate, and Disarm now, but draw the disarmed state later */
+
     Arm(w, event, params, num_params);
-
-    p->w = w;
-
-    if (event)
+    AB_Armed(w) = False;
+    if (AB_ActivateCallback(w))
     {
-        p->ev = *event;
-        p->has_event=True;
+	XFlush(XtDisplay(w));
+	cbs.reason = XmCR_ACTIVATE;
+	cbs.event = event;
+	cbs.click_count = 1;
+	XtCallCallbackList(w,
+			   AB_ActivateCallback(w),
+			   (XtPointer)&cbs);
+    }
+    if (AB_DisarmCallback(w))
+    {
+	XFlush(XtDisplay(w));
+	cbs.reason = XmCR_DISARM;
+	cbs.event = event;
+	cbs.click_count = 1;
+	XtCallCallbackList(w,
+			   AB_DisarmCallback(w),
+			   (XtPointer)&cbs);
     }
 
     if (AB_Timer(w) != 0)
@@ -581,7 +585,7 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     }
 
     AB_Timer(w) = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
-				  ACTIVATE_DELAY, ArmTimeout, (XtPointer)p);
+				  ACTIVATE_DELAY, ArmTimeout, (XtPointer)w);
 }
 
 static void
@@ -608,7 +612,7 @@ EnterWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	   this
 	   Prim_ShadowThickness(w),
 	 */
-		     1,
+		     2,
 		     AB_Direction(w));
     }
 }
@@ -637,7 +641,7 @@ LeaveWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	   this
 	   Prim_ShadowThickness(w),
 	 */
-		     1,
+		     2,
 		     AB_Direction(w));
     }
 }
