@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ArrowBG.c,v 1.13 1998/03/24 23:00:47 rwscott Exp $
+ * $Id: ArrowBG.c,v 1.21 1998/10/10 07:40:44 gritton Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: ArrowBG.c,v 1.13 1998/03/24 23:00:47 rwscott Exp $";
+static const char rcsid[] = "$Id: ArrowBG.c,v 1.21 1998/10/10 07:40:44 gritton Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -38,6 +38,7 @@ static char rcsid[] = "$Id: ArrowBG.c,v 1.13 1998/03/24 23:00:47 rwscott Exp $";
  * Try to add trait stuff by #ifdeffing it.
  */
 #if XmVERSION > 1
+#include <Xm/TraitP.h>
 #include <Xm/ActivatableT.h>
 
 void _XmArrowBG_TraitAddCallback(Widget, XtCallbackProc, XtPointer, Boolean);
@@ -51,7 +52,9 @@ static XmActivatableTraitRec _XmArrowBGTraitRec = {
 /* Forward Declarations */
 
 
+#if XmVERSION > 1
 static void class_initialize();
+#endif
 
 static void class_part_initialize(WidgetClass w_class);
 
@@ -62,9 +65,11 @@ static void destroy(Widget w);
 
 static void expose(Widget w, XEvent *event, Region region);
 
+#if 0
 static XtGeometryResult query_geometry(Widget w,
 				       XtWidgetGeometry *proposed,
 				       XtWidgetGeometry *answer);
+#endif
 
 static Boolean set_values(Widget current, Widget request, Widget new_w,
 			  ArgList args, Cardinal *num_args);
@@ -200,7 +205,7 @@ XmArrowButtonGadgetClassRec xmArrowButtonGadgetClassRec = {
 	/* version               */ XtVersion,
 	/* callback offsets      */ NULL,
 	/* tm_table              */ NULL,
-	/* query_geometry        */ query_geometry,
+	/* query_geometry        */ NULL /* query_geometry */,
 	/* display_accelerator   */ NULL,
 	/* extension             */ (XtPointer)NULL /*&_XmArrowBGRectClassExtRec*/
     },
@@ -227,9 +232,7 @@ XmArrowButtonGadgetClassRec xmArrowButtonGadgetClassRec = {
 WidgetClass xmArrowButtonGadgetClass =
 			(WidgetClass)&xmArrowButtonGadgetClassRec;
 
-#define ABG_DEFAULT_HEIGHT	20
-#define ABG_DEFAULT_WIDTH	20
-
+#if XmVERSION > 1
 static void
 class_initialize()
 {
@@ -237,15 +240,13 @@ class_initialize()
     _XmArrowBGRectClassExtRec.record_type = XmQmotif;
 #endif
 
-#if XmVERSION > 1
     if (! XmeTraitSet(xmArrowButtonGadgetClass, XmQTactivatable,
 		&_XmArrowBGTraitRec)) {
 	_XmWarning(NULL,
 	    "XmArrowButtonGadget ClassInitialize: XmeTraitSet failed\n");
     }
-#endif
-
 }
+#endif
 
 static void
 class_part_initialize(WidgetClass widget_class)
@@ -309,14 +310,14 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 	_XmError(new_w, "parent should be manager.");
     }
 
-    if (XtWidth(new_w) == 0)
+    if (XtWidth(request) == 0)
     {
-	XtWidth(new_w) = ABG_DEFAULT_WIDTH;
+	XtWidth(new_w) += 15;
     }
 
-    if (XtHeight(new_w) == 0)
+    if (XtHeight(request) == 0)
     {
-	XtHeight(new_w) = ABG_DEFAULT_HEIGHT;
+	XtHeight(new_w) += 15;
     }
 
     /* Gadget override */
@@ -399,6 +400,16 @@ expose(Widget w, XEvent *event, Region region)
 	myGC = ABG_InsensitiveGC(w);
     }
 
+    if (G_Highlighted(w))
+    {
+	    (*GC_BorderHighlight(XtClass(w))) (w);
+    }
+    else
+    {
+	(*GC_BorderUnhighlight(XtClass(w))) (w);
+    }
+
+
     _XmDrawShadows(XtDisplayOfObject(w),
 		   XtWindowOfObject(w),
 		   XmParentTopShadowGC(w),
@@ -425,7 +436,7 @@ expose(Widget w, XEvent *event, Region region)
 	   this
 	   G_ShadowThickness(w),
 	 */
-		     1,
+		     2,
 		     ABG_Direction(w));
     }
     else
@@ -441,11 +452,12 @@ expose(Widget w, XEvent *event, Region region)
 	   this
 	   G_ShadowThickness(w),
 	 */
-		     1,
+		     2,
 		     ABG_Direction(w));
     }
 }
 
+#if 0
 static XtGeometryResult
 query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 {
@@ -472,6 +484,7 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 	return XtGeometryAlmost;
     }
 }
+#endif
 
 static void
 Arm(Widget w, XEvent *event, String *params, Cardinal *num_params)
@@ -479,21 +492,24 @@ Arm(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XmArrowButtonCallbackStruct cbs;
     XtExposeProc exp = XtClass(w)->core_class.expose;
 
-    ABG_Armed(w) = True;
-
-    (*exp)(w, NULL, (Region)NULL);
-
-    if (ABG_ArmCallback(w))
+    if (!ABG_Armed(w))
     {
-	cbs.reason = XmCR_ARM;
-	cbs.event = event;
-	cbs.click_count = ABG_ClickCount(w);
+	ABG_Armed(w) = True;
 
-	XFlush(XtDisplay(w));
+	(*exp)(w, NULL, (Region)NULL);
 
-	XtCallCallbackList(w,
-			   ABG_ArmCallback(w),
-			   (XtPointer)&cbs);
+	if (ABG_ArmCallback(w))
+	{
+	    cbs.reason = XmCR_ARM;
+	    cbs.event = event;
+	    cbs.click_count = ABG_ClickCount(w);
+
+	    XFlush(XtDisplay(w));
+
+	    XtCallCallbackList(w,
+			       ABG_ArmCallback(w),
+			       (XtPointer)&cbs);
+	}
     }
 }
 
@@ -552,64 +568,50 @@ Activate(Widget w,
 static void
 ArmTimeout(XtPointer data, XtIntervalId *id)
 {
-    XmArrowButtonCallbackStruct cbs;
-    XmArmTimeoutRec *p = (XmArmTimeoutRec *)data;
-    Widget w = p->w;
+    Widget w = (Widget)data;
     XtExposeProc exp = XtClass(w)->core_class.expose;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "ArmTimeout\n"));
  
-    if (ABG_Armed(w) == False)
+    ABG_Timer(w) = 0;
+
+    if (XtIsRealized(w))
     {
-        return;
+	(*exp) (w, NULL, NULL);
+	XFlush(XtDisplayOfObject(w));
     }
- 
-    ABG_Armed(w) = False;
-
-    (*exp) (w, NULL, NULL);
- 
-    if (ABG_ActivateCallback(w))
-    {
-        cbs.reason = XmCR_ACTIVATE;
- 
-        if (p->has_event)
-        {
-            cbs.event = &p->ev;
-        }
-        else
-        {
-            cbs.event = NULL;
-        }
-        cbs.click_count = ABG_ClickCount(w);
- 
-        XFlush(XtDisplay(w));
- 
-        XtCallCallbackList(w,
-                           ABG_ActivateCallback(w),
-                           (XtPointer)&cbs);
-    }
-
-    Disarm(w, NULL, NULL, 0);
-
-    XtFree(data);
 }
 
 static void
 ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    XmArmTimeoutRec *p = (XmArmTimeoutRec *)XtCalloc(1,
-						     sizeof(XmArmTimeoutRec));
+    XmArrowButtonCallbackStruct cbs;
  
     DEBUGOUT(XdbDebug(__FILE__, w, "ArmAndActivate\n"));
 
+    /* Arm, Activate, and Disarm now, but draw the disarmed state later */
+
     Arm(w, event, params, num_params);
-
-    p->w = w;
-
-    if (event)
+    ABG_Armed(w) = False;
+    if (ABG_ActivateCallback(w))
     {
-        p->ev = *event;
-        p->has_event=True;
+	XFlush(XtDisplayOfObject(w));
+	cbs.reason = XmCR_ACTIVATE;
+	cbs.event = event;
+	cbs.click_count = 1;
+	XtCallCallbackList(w,
+			   ABG_ActivateCallback(w),
+			   (XtPointer)&cbs);
+    }
+    if (ABG_DisarmCallback(w))
+    {
+	XFlush(XtDisplayOfObject(w));
+	cbs.reason = XmCR_DISARM;
+	cbs.event = event;
+	cbs.click_count = 1;
+	XtCallCallbackList(w,
+			   ABG_DisarmCallback(w),
+			   (XtPointer)&cbs);
     }
 
     if (ABG_Timer(w) != 0)
@@ -618,7 +620,7 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     }
 
     ABG_Timer(w) = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
-				   ACTIVATE_DELAY, ArmTimeout, (XtPointer)p);
+				   ACTIVATE_DELAY, ArmTimeout, (XtPointer)w);
 }
 
 static void
@@ -641,7 +643,7 @@ EnterWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	   this
 	   Prim_ShadowThickness(w),
 	 */
-		     1,
+		     2,
 		     ABG_Direction(w));
     }
 }
@@ -666,7 +668,7 @@ LeaveWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	   this
 	   Prim_ShadowThickness(w),
 	 */
-		     1,
+		     2,
 		     ABG_Direction(w));
     }
 }
