@@ -1,6 +1,6 @@
 /**
  *
- * $Id: BulletinBoard.c,v 1.24 1998/04/14 22:54:44 rwscott Exp $
+ * $Id: BulletinBoard.c,v 1.38 1998/10/23 17:27:39 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: BulletinBoard.c,v 1.24 1998/04/14 22:54:44 rwscott Exp $";
+static const char rcsid[] = "$Id: BulletinBoard.c,v 1.38 1998/10/23 17:27:39 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -229,6 +229,7 @@ static XtActionsRec actions[] =
 };
 
 static XtTranslations mapTrans;
+static XtTranslations defTrans;
 /* *INDENT-OFF* */
 #if 0
 static XmBaseClassExtRec _XmBulletinBCoreClassExtRec = {
@@ -364,6 +365,7 @@ class_initialize()
 #endif
 
     mapTrans = XtParseTranslationTable(_XmBulletinB_mapTranslations);
+    defTrans = XtParseTranslationTable(_XmBulletinB_defaultTranslations);
 }
 
 static void
@@ -460,8 +462,14 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
     }
 
     XtAugmentTranslations(new_w, mapTrans);
+    XtAugmentTranslations(new_w, defTrans);
+    /* rws 30 Aug 1998
+       We seem to loose a few hundred bytes every time we parse a translation
+       table, so like mapTrans lets parse it in class_initialize and use the
+       parsed version here.
     XtAugmentTranslations(new_w,
 		  XtParseTranslationTable(_XmBulletinB_defaultTranslations));
+		  */
 
     if (XmIsDialogShell(XtParent(new_w)))
     {
@@ -494,7 +502,7 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 	 * shell -- first here. Otherwise we have *real trouble* with
 	 * the grab mechanism. This replaces the call to XtRealizeWidget()
 	 * within the initialize_posthook() method of the BulletinBoard
-	 * class. And it does get us rid off that %"&!$ double installed
+	 * class. And it does get us rid off that %&!$ double installed
 	 * grabs when using a dialog shell.
 	 */
 	XtRealizeWidget(XtParent(new_w));
@@ -559,6 +567,17 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 static void
 destroy(Widget w)
 {
+  Widget ve;
+
+    XmFontListFree(BB_LabelFontList(w));
+    XmFontListFree(BB_ButtonFontList(w));
+    XmFontListFree(BB_TextFontList(w));
+    if ((ve = _LtFindVendorExt(w)) != NULL)
+      {
+	XtRemoveAllCallbacks(ve,XmNfocusMovedCallback);
+      }
+    
+
 }
 
 static Boolean
@@ -702,7 +721,9 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
     XmBulletinBoardWidgetClass bbc = (XmBulletinBoardWidgetClass)XtClass(w);
     XtGeometryResult res;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "QueryGeometry\n"));
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:query_geometry(%d) - proposed %s\n",
+    		__FILE__, __LINE__,
+    		XdbWidgetGeometry2String(proposed)));
 
     if (bbc->bulletin_board_class.geo_matrix_create)
     {
@@ -784,11 +805,21 @@ resize(Widget w)
     BB_OldShadowThickness(w) = 0;
 #endif
 
+#if 0
+    /* rws 21 Jul 1998
+       _XmGMDoLayout will request a new size from the parent if it wants to.
+       This is bad in the resize method, we have to live with the size we
+       are given!  Since BulletinBoard, without a GeoMatrix proc, doesn't
+       layout much of anything, let's just skip this and see what happens.
+       This should not affect any of the standard dialogs since they all
+       use the GeoMatirx. ( test/Xm/pushbg/test8 )
+     */
     if (XtIsRealized(w) || XtWidth(w) == 0 || XtHeight(w) == 0)
     {
 	_XmGMDoLayout(w, BB_MarginWidth(w), BB_MarginHeight(w),
 		      BB_ResizePolicy(w), True);
     }
+#endif
 
 #if 0
     if ((XtWidth(w) < BB_OldWidth(w) || XtHeight(w) < BB_OldHeight(w)) &&
@@ -818,7 +849,8 @@ handle_geometry_manager(Widget w,
     XtGeometryResult res;
 
     DEBUGOUT(XdbDebug2(__FILE__, bb, w,
-		       "handle_geometry_manager\n"));
+		       "handle_geometry_manager\n\tdesired %s\n",
+		       	XdbWidgetGeometry2String(desired)));
 
     if (!(desired->request_mode & (CWWidth | CWHeight)))
     {
@@ -842,10 +874,10 @@ handle_geometry_manager(Widget w,
     {
 	if (XtIsRealized(bb))
 	{
-	    _XmClearBorder(XtDisplay(w), XtWindow(w),
+	    _XmClearBorder(XtDisplay(bb), XtWindow(bb),
 		       0,0,
-			BB_OldWidth(w), BB_OldHeight(w),
-			BB_OldShadowThickness(w));
+			BB_OldWidth(bb), BB_OldHeight(bb),
+			BB_OldShadowThickness(bb));
 	    _XmDrawShadows(XtDisplay(bb), XtWindow(bb),
 			   MGR_TopShadowGC(bb), MGR_BottomShadowGC(bb),
 			   0, 0, XtWidth(bb), XtHeight(bb),
@@ -933,6 +965,12 @@ handle_change_managed(Widget w, XmGeoCreateProc mat_make)
 
     if (wd == XtWidth(w) && ht == XtHeight(w))
     {
+	/* rws 16 Aug 1998
+	   Ok, so the BB hasn't changed size so we do not need to make the
+	   geo request but, the kids may not have been positioned yet, see
+	   messagebox/test14.  So, let's always do the set.
+	 */
+	_XmGeoMatrixSet(geo);
 	_XmGeoMatrixFree(geo);
 	_XmNavigChangeManaged(w);
 
@@ -945,7 +983,24 @@ handle_change_managed(Widget w, XmGeoCreateProc mat_make)
     request.request_mode |= CWBorderWidth;
     request.border_width = XtBorderWidth(w);
 
-    _XmMakeGeometryRequest(w, &request);
+    {
+    XtGeometryResult res;
+
+	res = _XmMakeGeometryRequest(w, &request);
+	DEBUGOUT(XdbDebug(__FILE__, w, "handle_change_managed request %s %s\n",
+		XdbWidgetGeometry2String(&request),
+		XdbGeometryResult2String(res)));
+    	if (res == XtGeometryNo)
+    	{
+	    /* rws 16 Aug 1998
+	       If the parent says NO we must re-layout the kids, since they
+	       are laid out to their preferred places at this point.
+	       (messagebox/test13)
+	     */
+	    request.width = XtWidth(w);
+	    request.height = XtHeight(w);
+    	}
+    }
 
     if (request.width != wd || request.height != ht)
     {
@@ -996,7 +1051,8 @@ change_managed(Widget w)
 
     BB_OldShadowThickness(w) = 0;
 
-    if (XtIsRealized(w) || XtWidth(w) == 0 || XtHeight(w) == 0)
+    /* mainw/test13 */
+    if (/* XtIsRealized(w) || */ XtWidth(w) == 0 || XtHeight(w) == 0)
     {
 	_XmGMDoLayout(w, BB_MarginWidth(w), BB_MarginHeight(w),
 		      BB_ResizePolicy(w), False);
@@ -1177,7 +1233,15 @@ realize(Widget w,
     /* Motif inherits this method */
     XmBulletinBoardClassRec *bb = (XmBulletinBoardClassRec *)XtClass(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Realize\n"));
+#if 1
+    /* bulletinboard/test11 */
+    if (XtWidth(w) == 0) XtWidth(w) = 1;
+    if (XtHeight(w) == 0) XtHeight(w) = 1;
+#endif
+
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:realize(%d) - %dx%d\n",
+    	__FILE__, __LINE__,
+    	XtWidth(w), XtHeight(w)));
 
 #define superclass (&xmManagerClassRec)
     (*superclass->core_class.realize) (w, value_mask, attributes);
@@ -1190,6 +1254,11 @@ realize(Widget w,
 	return;
     }
 
+#if 0
+    /* rws 21 Oct 1998
+       This should not be necessary. All this should get done in change_managed.
+       (bulletinboard/test14)
+     */
     _XmGMEnforceMargin(w,
 		       BB_MarginWidth(w), BB_MarginHeight(w),
 		       False);
@@ -1213,6 +1282,7 @@ realize(Widget w,
     BB_OldWidth(w) = XtWidth(w);
     BB_OldHeight(w) = XtHeight(w);
     BB_OldShadowThickness(w) = MGR_ShadowThickness(w);
+#endif
 }
 
 static void
@@ -1267,8 +1337,13 @@ _XmBbButton(Widget w, XtPointer client, XtPointer call)
 	    XtUnmanageChild((Widget)bb);
 	    DEBUGOUT(XdbDebug2(__FILE__, (Widget)bb, w, "AutoUnmanage\n"));
 
+#if 0
+	    /* rws 19 Sep 1998
+	       This is a job for Shell, not BB
+	     */
 	    /* XtNpopdownCallback */
-	    XtCallCallbackList(s, Shell_PopdownCallback(s), NULL);
+	    XtCallCallbackList(s, Shell_PopdownCallback(s), call);
+#endif
 	}
     }
 }
@@ -1460,61 +1535,91 @@ void
 _XmBulletinBoardFocusMoved(Widget wid, XtPointer client_data, XtPointer data)
 {
     XmFocusMovedCallback cbs = (XmFocusMovedCallback)data;
-    Widget bb = (Widget)client_data, par;
-    Boolean call = False;
+    Widget bb = (Widget)client_data;
     XmAnyCallbackStruct cb;
+    Boolean to_bb = False, from_bb = False;
+    Widget default_button = NULL;
+    Widget par;
 
     DEBUGOUT(XdbDebug(__FILE__, wid, "_XmBulletinBoardFocusMoved\n"));
 
-    if (cbs->new_focus == wid)
+    if (!cbs->cont)
     {
-	call = True;
+	return;
     }
-    else if (cbs->new_focus)
+    for (par = cbs->new_focus; par != NULL && !XtIsShell(par);
+	 par = XtParent(par))
     {
-	par = XtParent(cbs->new_focus);
 
-	for (;;)
+	if (bb == par)
 	{
-	    if (par == bb)
+	    to_bb = True;
+	    break;
+	}
+	if (XmIsBulletinBoard(par))
+	{
+	    if (default_button == NULL)
 	    {
-		call = True;
-		if (BB_DefaultButton(bb) && XtParent(cbs->new_focus) == bb &&
-		    cbs->new_focus &&
-		    (XmIsPushButton(cbs->new_focus) ||
-		     XmIsPushButtonGadget(cbs->new_focus) ||
-		     XmIsDrawnButton(cbs->new_focus)))
-		{
-		    _XmBulletinBoardSetDynDefaultButton(bb, cbs->new_focus);
-		}
-
-		break;
-
+		default_button = BB_DefaultButton(par);
 	    }
-
-	    if (XtIsShell(par))
-	    {
-		break;
-	    }
-	    if (!par)
-	    {
-		break;
-	    }
-
-	    par = XtParent(par);
 	}
     }
-
-    if (call && BB_FocusCallback(bb))
+    for (par = cbs->old_focus; par != NULL && !XtIsShell(par);
+	 par = XtParent(par))
     {
-	cb.reason = XmCR_FOCUS;
-	cb.event = cbs->event;
-	XtCallCallbackList(bb, BB_FocusCallback(bb), data);
+	if (bb == par)
+	{
+	    from_bb = True;
+	    break;
+	}
+    }
+    if (to_bb)
+    {
+	if (default_button == NULL)
+	{
+	    default_button = BB_DefaultButton(bb);
+	}
+	if (default_button == NULL)
+	{
+	    BB_DynamicDefaultButton(bb) = NULL;
+	}
+	else
+	{
+	    par = cbs->new_focus;
+	    if (XmIsPushButton(cbs->new_focus) ||
+		XmIsPushButtonGadget(cbs->new_focus))
+	    {
+		_XmBulletinBoardSetDynDefaultButton(bb, cbs->new_focus);
+	    }
+	    else if (cbs->focus_policy == XmEXPLICIT ||
+		     !XmIsManager(cbs->new_focus) ||
+		     cbs->old_focus == NULL ||
+		     (!XmIsPushButtonGadget(cbs->old_focus) &&
+		      !XmIsPushButton(cbs->old_focus)))
+	    {
+		if (XtIsManaged(default_button))
+		{
+		    _XmBulletinBoardSetDynDefaultButton(bb, default_button);
+		}
+	    }
+	}
+	if (!from_bb)
+	{
+	    cb.reason = XmCR_FOCUS;
+	    cb.event = cbs->event;
+	    XtCallCallbackList(bb, BB_FocusCallback(bb), data);
+	}
+    }
+    else
+    {
+	if (from_bb && cbs->new_focus != NULL)
+	{
+	    _XmBulletinBoardSetDynDefaultButton(bb, NULL);
+	}
     }
-
-    cbs->cont = True;
 }
 
+
 void
 _XmBulletinBoardMap(Widget w, XEvent *event,
 		    String *params, Cardinal *numParams)
@@ -1654,6 +1759,10 @@ _XmBulletinBoardSetDynDefaultButton(Widget wid, Widget newDefaultButton)
 {
     Arg argl[2];
 
+    if (newDefaultButton && wid != XtParent(newDefaultButton))
+    {
+    	return;
+    }
     if (BB_DynamicDefaultButton(wid) != NULL)
     {
 	if (!BB_DynamicDefaultButton(wid)->core.being_destroyed)
