@@ -1,6 +1,6 @@
 /**
  *
- * $Id: CascadeB.c,v 1.82 1998/05/08 19:41:49 rwscott Exp $
+ * $Id: CascadeB.c,v 1.96 1998/10/17 23:09:57 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: CascadeB.c,v 1.82 1998/05/08 19:41:49 rwscott Exp $";
+static const char rcsid[] = "$Id: CascadeB.c,v 1.96 1998/10/17 23:09:57 rwscott Exp $";
 
 #define	DO_PIXMAP
 #define	DO_SANITY
@@ -342,6 +342,8 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
     DEBUGOUT(XdbDebug(__FILE__, new_w, "%s:initialize()\n", __FILE__));
 
     CB_SetArmed(new_w, False);
+    CB_ArmedPixmap(new_w) = (Pixmap)NULL;
+    CB_CascadePixmap(new_w) = (Pixmap)NULL;
 
     if (!XtIsSubclass(XtParent(new_w), xmRowColumnWidgetClass))
     {
@@ -396,8 +398,6 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 	(RC_Type(XtParent(new_w)) == XmMENU_OPTION))	/* rws 23 Mar 1997 */
     {
 	_XmCreateArrowPixmaps(new_w);
-
-	_XmCBCalcDimensions(new_w);
     }
 
     if (CB_Submenu(new_w))
@@ -406,6 +406,15 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 	RC_MenuSubmenu(new_w);
     }
 
+    _XmCBCalcDimensions(new_w);
+    if (XtWidth(request) != 0)
+    {
+    	XtWidth(new_w) = XtWidth(request);
+    }
+    if (XtHeight(request) != 0)
+    {
+    	XtHeight(new_w) = XtHeight(request);
+    }
     CB_Timer(new_w) = 0;
     LabClass_MenuProcs(XtClass(new_w)) = MenuProcEntry;
 }
@@ -523,7 +532,9 @@ _XmCBCalcDimensions(Widget w)
 static void
 resize(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:resize()\n", __FILE__));
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:resize(%d) - %dx%d\n",
+    			__FILE__, __LINE__,
+    			XtWidth(w), XtHeight(w)));
 
 #define superclass (&xmLabelClassRec)
     (*superclass->core_class.resize) (w);
@@ -542,11 +553,17 @@ destroy(Widget w)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "%s:destroy()\n", __FILE__));
 
-    _XmFreeScratchPixmap((XmScreen)XmGetXmScreen(XtScreen(w)),
+    if (CB_ArmedPixmap(w))
+    {
+	_XmFreeScratchPixmap((XmScreen)XmGetXmScreen(XtScreen(w)),
 			 CB_ArmedPixmap(w));
+    }
 
-    _XmFreeScratchPixmap((XmScreen)XmGetXmScreen(XtScreen(w)),
+    if (CB_CascadePixmap(w))
+    {
+	_XmFreeScratchPixmap((XmScreen)XmGetXmScreen(XtScreen(w)),
 			 CB_CascadePixmap(w));
+    }
 }
 
 static Boolean
@@ -578,14 +595,20 @@ set_values(Widget old, Widget request, Widget new_w,
 #endif
 	_XmCreateArrowPixmaps(new_w);
 
-	_XmCBCalcDimensions(new_w);
+	if (Lab_RecomputeSize(new_w))
+	{
+	    _XmCBCalcDimensions(new_w);
+	}
 
 	refresh_needed = True;
     }
 
     if (Lab_Label(new_w) != Lab_Label(old))
     {
-	_XmCBCalcDimensions(new_w);
+	if (Lab_RecomputeSize(new_w))
+	{
+	    _XmCBCalcDimensions(new_w);
+	}
 
 	refresh_needed = True;
     }
@@ -603,7 +626,10 @@ set_values(Widget old, Widget request, Widget new_w,
     {
 	/* Make sure the RC also knows how to locate us */
 	RC_MenuSubmenu(new_w);
-	_XmCBCalcDimensions(new_w);
+	if (Lab_RecomputeSize(new_w))
+	{
+	    _XmCBCalcDimensions(new_w);
+	}
 	refresh_needed = True;
     }
 
@@ -787,8 +813,13 @@ _XmCBMenuBarDoSelect(Widget w, XEvent *event,
     else
     {				/* We have no menu */
 	XtCallbackList cbl;
-	Widget p;
 
+	/* rws 8 Jul 1998
+	   We have no menu but, we may have been armed, filesb/test5. So
+	   let's clean up anything that needs to be first.
+	 */
+	RCClass_MenuProcs(XtClass(XtParent(w)))(XmMENU_BAR_CLEANUP,
+                                                    XtParent(w), NULL);
 	/* Actually all this no-menu thing is totally wrong.
 	 *
 	 * This code does call the callback, which makes apps work,
@@ -826,27 +857,9 @@ _XmCBMenuBarDoSelect(Widget w, XEvent *event,
 		XtCallCallbackList(w, cbl, &cbs);
 	    }
 	}
-
-	/* Activate the cascade button */
-	/* ??? FIX ME */
-	/* Unpost menus */
-	/* Only call CleanupMenuBar if we're in a menu pane */
-	for (p = w; p && !XmIsMenuShell(p); p = XtParent(p))
-	{
-	}
-
-	if (p)
-	{
-            RCClass_MenuProcs(XtClass(XtParent(w)))(XmMENU_BAR_CLEANUP,
-                                                    XtParent(w), NULL);
-	}
     }
 
     CB_SetArmed(w, False);
-
-    /*
-    RC_SetArmed(XtParent(w), 0);
-    */
 }
 
 void
@@ -875,7 +888,13 @@ _XmCBMenuBarSelect(Widget w, XEvent *event,
     if (!menu)
     {
 	DEBUGOUT(XdbDebug(__FILE__, w, "_XmCBMenuBarSelect: no menu!\n"));
+	/* rws 24 Jun 1998
+	   Oh well, Motif still highlights the button and gives up the menu
+	   cursor. So lets go for it anyway!
+	 */
+	/*
 	return;
+	*/
     }
 
     RC_MenuShellPopdown(w, event, &poppedUp);
@@ -1147,7 +1166,9 @@ static void
 MenuProcEntry(int proc, Widget w,...)
 {
     va_list arg_list;
+    /*
     XEvent *event;
+    */
     extern void _XmTearOffArm();
  
     va_start(arg_list, w);
@@ -1158,7 +1179,9 @@ MenuProcEntry(proc, w, va_alist)
      va_dcl
 {
     va_list arglist;
+    /*
     XEvent *event;
+    */
     extern void _XmTearOffArm();
  
     va_start(arglist);
@@ -1168,7 +1191,7 @@ MenuProcEntry(proc, w, va_alist)
     {
     case XmMENU_ARM:
     	{
-	XtExposeProc exp = XtClass(w)->core_class.expose;
+	  /* XtExposeProc exp = XtClass(w)->core_class.expose; */
 
 	    CB_SetArmed(w, True);
 	    XmCascadeButtonHighlight(w, True);
@@ -1179,7 +1202,7 @@ MenuProcEntry(proc, w, va_alist)
     	break;
     case XmMENU_DISARM:
     	{
-	XtExposeProc exp = XtClass(w)->core_class.expose;
+	  /* XtExposeProc exp = XtClass(w)->core_class.expose; */
 
 	    CB_SetArmed(w, False);
 	    XmCascadeButtonHighlight(w, False);
