@@ -1,6 +1,6 @@
 /**
  *
- * $Id: CascadeBG.c,v 1.75 1998/03/16 21:52:09 rwscott Exp $
+ * $Id: CascadeBG.c,v 1.85 1998/10/02 14:57:45 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: CascadeBG.c,v 1.75 1998/03/16 21:52:09 rwscott Exp $";
+static const char rcsid[] = "$Id: CascadeBG.c,v 1.85 1998/10/02 14:57:45 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -488,8 +488,12 @@ _XmCreateArrowPixmaps(Widget w)
 	hh = 1;
     }
 
-    pm = XCreatePixmap(dpy, win, hh, hh, depth);
-    apm = XCreatePixmap(dpy, win, hh, hh, depth);
+    pm = _XmAllocScratchPixmap((XmScreen)XmGetXmScreen(XtScreen(w)),
+    				depth,
+    				hh, hh);
+    apm = _XmAllocScratchPixmap((XmScreen)XmGetXmScreen(XtScreen(w)),
+    				depth,
+    				hh, hh);
 
     XFillRectangle(dpy, pm, fgc, 0, 0, hh, hh);
     XFillRectangle(dpy, apm, fgc, 0, 0, hh, hh);
@@ -633,6 +637,8 @@ initialize(Widget request, Widget new_w,
     CBG_Cascade_height(new_w) = 0;
 
     CBG_SetArmed(new_w, False);
+    CBG_ArmedPixmap(new_w) = (Pixmap)NULL;
+    CBG_CascadePixmap(new_w) = (Pixmap)NULL;
 
     if (LabG_MenuType(new_w) == XmMENU_BAR ||
 	LabG_MenuType(new_w) == XmMENU_POPUP ||
@@ -671,12 +677,8 @@ initialize(Widget request, Widget new_w,
 	LabG_MenuType(new_w) == XmMENU_OPTION)	/* rws 23 Mar 1997 */
     {
 	_XmCreateArrowPixmaps(new_w);
-
-	_XmCBGCalcDimensions(new_w);
     }
 
-    CBG_Timer(new_w) = 0;
-
     if (CBG_Submenu(new_w))
     {
 	/* Make sure the RC also knows how to locate us */
@@ -689,6 +691,8 @@ initialize(Widget request, Widget new_w,
 	XmHELP_EVENT | XmBDRAG_EVENT;
 
     G_TraversalOn(new_w) = True;   /* Danny 18/5/1996 - FIX ME (not sure) */
+    _XmCBGCalcDimensions(new_w);
+    CBG_Timer(new_w) = 0;
     LabGClass_MenuProcs(XtClass(new_w)) = MenuProcEntry;
 }
 
@@ -824,6 +828,17 @@ static void
 destroy(Widget w)
 {
     _XmCacheDelete((XtPointer)CBG_Cache(w));
+    if (CBG_ArmedPixmap(w))
+    {
+	_XmFreeScratchPixmap((XmScreen)XmGetXmScreen(XtScreen(w)),
+			 CBG_ArmedPixmap(w));
+    }
+
+    if (CBG_CascadePixmap(w))
+    {
+	_XmFreeScratchPixmap((XmScreen)XmGetXmScreen(XtScreen(w)),
+			 CBG_CascadePixmap(w));
+    }
 }
 
 static Boolean
@@ -1386,8 +1401,6 @@ DelayedArm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	CBG_SetArmed(w, True);
 
 	XmCascadeButtonHighlight(w, True);
-
-	MGR_SelectedGadget(XtParent(w)) = (XmGadget)w;
     }
 }
 
@@ -1588,7 +1601,7 @@ static void
 MenuProcEntry(int proc, Widget w,...)
 {
     va_list arg_list;
-    XEvent *event;
+    XEvent *event = NULL;
     extern void _XmTearOffArm();
  
     va_start(arg_list, w);
@@ -1599,7 +1612,7 @@ MenuProcEntry(proc, w, va_alist)
      va_dcl
 {
     va_list arglist;
-    XEvent *event;
+    XEvent *event = NULL;
     extern void _XmTearOffArm();
  
     va_start(arglist);
