@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DebugUtil.c,v 1.26 1998/04/08 21:07:28 rwscott Exp $
+ * $Id: DebugUtil.c,v 1.39 1998/10/16 23:41:37 rwscott Exp $
  * 
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: DebugUtil.c,v 1.26 1998/04/08 21:07:28 rwscott Exp $";
+static const char rcsid[] = "$Id: DebugUtil.c,v 1.39 1998/10/16 23:41:37 rwscott Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -37,6 +37,7 @@ static char rcsid[] = "$Id: DebugUtil.c,v 1.26 1998/04/08 21:07:28 rwscott Exp $
 #include <X11/IntrinsicP.h>
 #include <X11/Xresource.h>
 #include <Xm/VendorSEP.h>
+#include <Xm/DrawingAP.h>
 
 /*
  * Functionality in this file is influenced at run time by these
@@ -195,6 +196,28 @@ XdbPrintCompleteTree(Widget w)
 }
 #endif
 
+#if XmVERSION > 1
+char *
+XdbComboBoxType2String(unsigned char type)
+{
+    switch(type)
+    {
+    case XmDROP_DOWN_LIST:
+    	return("XmDROP_DOWN_LIST");
+	break;
+    case XmDROP_DOWN_COMBO_BOX:
+    	return("XmDROP_DOWN_COMBO_BOX");
+	break;
+    case XmCOMBO_BOX:
+    	return("XmCOMBO_BOX");
+	break;
+    default:
+	return("UNKNOWN");
+	break;
+    }
+}
+#endif
+
 char *
 XdbGeoAction2String(int action)
 {
@@ -240,6 +263,22 @@ XdbGeometryResult2String(XtGeometryResult r)
     }
 }
 
+char *
+XdbScrollingPolicy2String(unsigned char r)
+{
+    switch (r)
+    {
+    case XmAUTOMATIC:
+	return "XmAUTOMATIC";
+
+    case XmCONSTANT:
+	return "XmCONSTANT";
+
+    default:
+	return "UNKNOWN";
+    }
+}
+
 char *
 XdbDialogStyle2String(int a)
 {
@@ -364,12 +403,15 @@ char *
 _XdbFindDebugSources(Widget w)
 {
     static char *env = NULL;
+    static int checked=False;
 
-    if (env == NULL)
-    {
+    if (!checked) {
+      checked=True;
+      if (env == NULL) 	
+      {
 	env = getenv("DEBUGSOURCES");
+      }
     }
-
     return env;
 }
 
@@ -377,12 +419,15 @@ static char *
 _XdbPrintWidgetID()
 {
     static char *env = NULL;
+    static int checked=False;
 
-    if (env == NULL)
-    {
+    if (!checked) {
+      checked=True;
+      if (env == NULL)  
+      {
 	env = getenv("DEBUG_PRINT_WIDGETID");
+      }
     }
-
     return env;
 }
 
@@ -510,6 +555,26 @@ XdbInDebug(char *fn, Widget w)
 #endif
 }
 
+char *
+XdbMenuFocusOp2String(int f)
+{
+    switch (f)
+    {
+    case XmMENU_FOCUS_SAVE:
+    	return ("XmMENU_FOCUS_SAVE");
+    	break;
+    case XmMENU_FOCUS_RESTORE:
+    	return ("XmMENU_FOCUS_RESTORE");
+    	break;
+    case XmMENU_FOCUS_SET:
+    	return ("XmMENU_FOCUS_SET");
+    	break;
+    default:
+    	return("Unknown focus op");
+    	break;
+    }
+}
+
 char *
 XdbMenuEnum2String(int f)
 {
@@ -740,6 +805,8 @@ XdbWidgetGeometry2String(XtWidgetGeometry *g)
 #define XdbLIST_SIZE_POLICY	15
 #define XdbSB_PLACEMENT		16
 #define XdbRESIZE_POLICY	17
+#define XdbCOMBOBOX_TYPE	18
+#define XdbSCROLLING_POLICY	19
 
 static struct
 {
@@ -749,6 +816,30 @@ static struct
 }
 XdbTypes[] =
 {
+    {
+	XmNscrollingPolicy, XdbSCROLLING_POLICY, NULL
+    }
+    ,
+    {
+	XmNmaximum, XdbINT, NULL
+    }
+    ,
+    {
+	XmNminimum, XdbINT, NULL
+    }
+    ,
+    {
+	XmNsliderSize, XdbINT, NULL
+    }
+    ,
+    {
+	XmNpageIncrement, XdbINT, NULL
+    }
+    ,
+    {
+	XmNincrement, XdbINT, NULL
+    }
+    ,
     {
 	XmNx, XdbINT, NULL
     }
@@ -769,6 +860,10 @@ XdbTypes[] =
 	XmNlabelString, XdbXMSTRING, NULL
     }
     ,
+    {
+	XmNmessageString, XdbXMSTRING, NULL
+    }
+    ,
     {
 	XmNrowColumnType, XdbNONE, NULL
     }
@@ -833,6 +928,10 @@ XdbTypes[] =
 	XmNshadowThickness, XdbINT, NULL
     }
     ,
+    {
+	XmNhighlightThickness, XdbINT, NULL
+    }
+    ,
     {
 	XmNtopAttachment, XdbATTACHMENT, NULL
     }
@@ -881,6 +980,14 @@ XdbTypes[] =
 	XmNrightPosition, XdbINT, NULL
     }
     ,
+    {
+	XmNdefaultButton, XdbWIDGET, NULL
+    }
+    ,
+    {
+	XmNmessageWindow, XdbWIDGET, NULL
+    }
+    ,
     {
 	XmNtopWidget, XdbWIDGET, NULL
     }
@@ -901,6 +1008,10 @@ XdbTypes[] =
 	XmNresizable, XdbBOOLEAN, NULL
     }
     ,
+    {
+	XmNmustMatch, XdbBOOLEAN, NULL
+    }
+    ,
     {
 	XmNresizeHeight, XdbBOOLEAN, NULL
     }
@@ -933,6 +1044,18 @@ XdbTypes[] =
 	XmNfileListItemCount, XdbINT, NULL
     }
     ,
+    {
+	XmNtextString, XdbXMSTRING, NULL
+    }
+    ,
+    {
+	XmNdirSpec, XdbXMSTRING, NULL
+    }
+    ,
+    {
+	XmNdirMask, XdbXMSTRING, NULL
+    }
+    ,
     {
 	XmNitems, XdbXMSTRING_LIST, XmNitemCount
     }
@@ -1001,6 +1124,14 @@ XdbTypes[] =
 	XmNworkWindow, XdbWIDGET, NULL
     }
     ,
+    {
+	XmNmenuBar, XdbWIDGET, NULL
+    }
+    ,
+    {
+	XmNcommandWindow, XdbWIDGET, NULL
+    }
+    ,
     {
 	XmNvisibleItemCount, XdbSHORT, NULL
     }
@@ -1057,6 +1188,12 @@ XdbTypes[] =
 	XmNnumColumns, XdbSHORT, NULL
     }
     ,
+#if XmVERSION > 1
+    {
+	XmNcomboBoxType, XdbCOMBOBOX_TYPE, NULL
+    }
+    ,
+#endif
     {
 	NULL, 0, NULL
     }				/* the end */
@@ -1140,6 +1277,12 @@ XdbPrintArgList(char *fn, Widget w, ArgList al, int n, Boolean Get)
 			: XdbSBDisplayPolicy2String((unsigned char)al[i].value));
 		break;
 
+	    case XdbSCROLLING_POLICY:
+		fprintf(XdbFile, "Arg[%d] : %s %s\n", i, al[i].name,
+		    (Get) ? XdbScrollingPolicy2String(*(unsigned char *)al[i].value)
+			: XdbScrollingPolicy2String((unsigned char)al[i].value));
+		break;
+
 	    case XdbSB_PLACEMENT:
 		fprintf(XdbFile, "Arg[%d] : %s %s\n", i, al[i].name,
 		    (Get) ? XdbSBPlacement2String(*(unsigned char *)al[i].value)
@@ -1210,6 +1353,13 @@ XdbPrintArgList(char *fn, Widget w, ArgList al, int n, Boolean Get)
 		}
 
 		break;
+#if XmVERSION > 1
+	    case XdbCOMBOBOX_TYPE:
+		fprintf(XdbFile, "Arg[%d] : %s %s\n", i, al[i].name,
+			XdbComboBoxType2String(al[i].value));
+		break;
+#endif
+
 	    }
 	}
     }
@@ -1268,6 +1418,9 @@ XdbResizePolicy2String(int n)
     case XmRESIZE_ANY:
 	return "XmRESIZE_ANY";
 
+    case XmRESIZE_SWINDOW:
+	return "XmRESIZE_SWINDOW";
+
     default:
 	return "XmNscrollBarDisplayPolicy - illegal";
     }
