@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DragBS.c,v 1.7 1997/08/25 23:34:29 miers Exp $
+ * $Id: DragBS.c,v 1.10 1998/09/22 22:03:42 jon Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: DragBS.c,v 1.7 1997/08/25 23:34:29 miers Exp $";
+static const char rcsid[] = "$Id: DragBS.c,v 1.10 1998/09/22 22:03:42 jon Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -1295,6 +1295,14 @@ _XmTargetsToIndex(Widget shell, Atom *tlist, Cardinal numTargets)
     int i;
     Atom *atoms;
 
+#ifdef NS45PR1
+    /* For Netscape 4.5 PR1 binary compatibility testing.
+       NS seems to be calling this routine with an invalid (huge) number
+       of numTargets.  I just bail out of the whole routine for testing.
+       -jac Sept 22, 1998 */
+    return 0;
+#endif
+
     if ((targets = get_targets_table(dpy)) == NULL)
     {
 	_XmInitTargetsTable(dpy);
@@ -1663,3 +1671,15 @@ _XmByteOrder(void)
 
     return byte_order;
 }
+
+#ifdef NS45PR1
+/* netscape 4.5PR1 wants these; dunno what they're supposed to do -jac */
+int _XmDSIAddChild(){return NULL;}
+int _XmDSIRemoveChild(){return NULL;}
+int _XmDSIDestroy(){return NULL;}
+int _XmDSIGetBorderWidth(){return NULL;}
+int _XmDSIReplaceChild(){return NULL;}
+int _XmDSIGetChildPosition(){return NULL;}
+int _XmDSISwapChildren(){return NULL;}
+#endif
+
