@@ -1,6 +1,6 @@
-/**
+/** -*- Mode: C; tab-width: 8; indent-tabs-mode: nil -*-
  *
- * $Id: DragOverS.c,v 1.8 1998/02/22 01:00:30 rwscott Exp $
+ * $Id: DragOverS.c,v 1.13 1998/10/19 23:56:28 jon Exp $
  *
  * derived from Xt Vendor class.c 
  *
@@ -25,7 +25,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: DragOverS.c,v 1.8 1998/02/22 01:00:30 rwscott Exp $";
+static const char rcsid[] = "$Id: DragOverS.c,v 1.13 1998/10/19 23:56:28 jon Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -40,7 +40,7 @@ static char rcsid[] = "$Id: DragOverS.c,v 1.8 1998/02/22 01:00:30 rwscott Exp $"
 
 #include <XmI/DebugUtil.h>
 
-static void class_initialize();
+/* static void class_initialize(); */
 
 static void class_part_initialize(WidgetClass wclass);
 
@@ -623,48 +623,54 @@ blend_icons(Widget w, XmDragOverBlend blend,
 	    XSetFunction(disp, blend->gc, GXor);
 	}
 
-	rect.x = DI_XOffset(state);
-	rect.y = DI_YOffset(state);
-	rect.width = DI_Width(state);
-	rect.height = DI_Height(state);
-	XSetClipRectangles(disp, blend->gc, 0, 0, &rect, 1, Unsorted);
-	if (DI_Mask(state) != XmUNSPECIFIED_PIXMAP &&
-	    bm != XmBLEND_JUST_SOURCE)
-	{
-	    XCopyPlane(disp, DI_Mask(state), DI_Mask(blend->mixedIcon),
-		       blend->gc, 0, 0, DI_Width(state), DI_Height(state),
-		       DI_XOffset(state),
-		       DI_YOffset(state), 1);
-	}
-	else if (bm != XmBLEND_JUST_SOURCE)
-	{
-	    XSetFunction(disp, blend->gc, GXset);
-	    XFillRectangles(disp, DI_Mask(blend->mixedIcon),
-			    blend->gc, &rect, 1);
-	    XSetFunction(disp, blend->gc, GXor);
-	}
+	if ( state != NULL )
+	  {
+		rect.x = DI_XOffset(state);
+		rect.y = DI_YOffset(state);
+		rect.width = DI_Width(state);
+		rect.height = DI_Height(state);
+		XSetClipRectangles(disp, blend->gc, 0, 0, &rect, 1, Unsorted);
+		if (DI_Mask(state) != XmUNSPECIFIED_PIXMAP &&
+			bm != XmBLEND_JUST_SOURCE)
+		  {
+			XCopyPlane(disp, DI_Mask(state), DI_Mask(blend->mixedIcon),
+					   blend->gc, 0, 0, DI_Width(state), DI_Height(state),
+					   DI_XOffset(state),
+					   DI_YOffset(state), 1);
+		  }
+		else if (bm != XmBLEND_JUST_SOURCE)
+		  {
+			XSetFunction(disp, blend->gc, GXset);
+			XFillRectangles(disp, DI_Mask(blend->mixedIcon),
+							blend->gc, &rect, 1);
+			XSetFunction(disp, blend->gc, GXor);
+		  }
+	  }
 
-	rect.x = DI_XOffset(op);
-	rect.y = DI_YOffset(op);
-	rect.width = DI_Width(op);
-	rect.height = DI_Height(op);
-	XSetClipRectangles(disp, blend->gc, 0, 0, &rect, 1, Unsorted);
-	if (DI_Mask(op) != XmUNSPECIFIED_PIXMAP &&
-	    bm != XmBLEND_JUST_SOURCE && bm != XmBLEND_STATE_SOURCE)
-	{
-	    XCopyPlane(disp, DI_Mask(op), DI_Mask(blend->mixedIcon),
-		       blend->gc, 0, 0, DI_Width(op), DI_Height(op),
-		       DI_XOffset(op),
-		       DI_YOffset(op), 1);
+	if ( op != NULL )
+	  {
+		rect.x = DI_XOffset(op);
+		rect.y = DI_YOffset(op);
+		rect.width = DI_Width(op);
+		rect.height = DI_Height(op);
+		XSetClipRectangles(disp, blend->gc, 0, 0, &rect, 1, Unsorted);
+		if (DI_Mask(op) != XmUNSPECIFIED_PIXMAP &&
+			bm != XmBLEND_JUST_SOURCE && bm != XmBLEND_STATE_SOURCE)
+		  {
+			XCopyPlane(disp, DI_Mask(op), DI_Mask(blend->mixedIcon),
+					   blend->gc, 0, 0, DI_Width(op), DI_Height(op),
+					   DI_XOffset(op),
+					   DI_YOffset(op), 1);
+		  }
+		else if (bm != XmBLEND_JUST_SOURCE && bm != XmBLEND_STATE_SOURCE)
+		  {
+			XSetFunction(disp, blend->gc, GXset);
+			XFillRectangles(disp, DI_Mask(blend->mixedIcon),
+							blend->gc, &rect, 1);
+			XSetFunction(disp, blend->gc, GXor);
+		  }
+	  }
 	}
-	else if (bm != XmBLEND_JUST_SOURCE && bm != XmBLEND_STATE_SOURCE)
-	{
-	    XSetFunction(disp, blend->gc, GXset);
-	    XFillRectangles(disp, DI_Mask(blend->mixedIcon),
-			    blend->gc, &rect, 1);
-	    XSetFunction(disp, blend->gc, GXor);
-	}
-    }
 
     values.function = GXclear;
     XChangeGC(disp, blend->gc,
@@ -1020,10 +1026,16 @@ make_soup(Widget w,
 							  args, argc);
     DI_XOffset(src) = srx;
     DI_YOffset(src) = sry;
-    DI_XOffset(op) = sox;
-    DI_YOffset(op) = soy;
-    DI_XOffset(state) = stx;
-    DI_YOffset(state) = sty;
+	if ( op != NULL )
+	  {
+		DI_XOffset(op) = sox;
+		DI_YOffset(op) = soy;
+	  }
+	if ( state != NULL )
+	  {
+		DI_XOffset(state) = stx;
+		DI_YOffset(state) = sty;
+	  }
 
     blend_icons(w, blend, src, state, op);
 }
@@ -1151,6 +1163,7 @@ get_colors(Widget w)
     XColor backfore[2];
     XGCValues values;
     Boolean new_gc = False;
+    static int warned = False;
 
     backfore[0].pixel = DC_CursorBackground(par);
 
@@ -1174,12 +1187,15 @@ get_colors(Widget w)
 	break;
     }
 
-    XQueryColors(disp, CoreColormap(w), backfore, 2);
-
-    if (!XAllocColor(disp, DefaultColormapOfScreen(scr), &backfore[0]) ||
-	!XAllocColor(disp, DefaultColormapOfScreen(scr), &backfore[1]) ||
-	backfore[0].pixel == backfore[1].pixel)
+    if ((CoreColormap(w) != DefaultColormapOfScreen(scr) &&
+         (XQueryColors(disp, CoreColormap(w), backfore, 2),
+          !XAllocColor(disp, DefaultColormapOfScreen(scr), &backfore[0]) ||
+          !XAllocColor(disp, DefaultColormapOfScreen(scr), &backfore[1]))) ||
+        backfore[0].pixel == backfore[1].pixel)
     {
+        warned = True;
+        _XmWarning(w,
+            "Cannot allocate colormap entry, some colors may be incorrect");
 	backfore[0].pixel = BlackPixelOfScreen(scr);
 	backfore[1].pixel = WhitePixelOfScreen(scr);
     }
@@ -1357,15 +1373,124 @@ change_window(Widget w)
 static void
 melt_cursor(Widget w)
 {
+  DEBUGOUT(XdbDebug( __FILE__, w, "Melting not implemented.\n" ));
 }
 
 /*
  * return the icon back to the drag site
  */
+#define ZAP_EFFECT2 5
+#define ZAP_SIZE2 6
+
 static void
 zap_cursor(Widget w)
 {
+    Screen  *screen = XtScreen(w);
+    GC gc;
+    XGCValues	gc_values;
+    XSegment segments[4];
+    int zap_count, idx;
+    int adjust_w, adjust_h;
+
+    DEBUGOUT(XdbDebug(__FILE__, w, "zap_cursor is implemented.\n"));
+    _XmWarning(NULL,  "Testing zap_cursor");
+    zap_count = 0;
+    gc = DO_RootBlend(w).gc;
+
+    for ( idx = 0; idx < 4; idx++ )
+    {
+    	segments[idx].x1 = DO_InitialX(w);
+    	segments[idx].y1 = DO_InitialY(w);
+    }
+    segments[0].x2 = XtX(w);
+    segments[0].y2 = XtY(w);
+
+    segments[1].x2 = XtX(w);
+    segments[1].y2 = XtY(w) + XtHeight(w);
+
+    segments[2].x2 = XtX(w) + XtWidth(w);
+    segments[2].y2 = XtY(w) + XtHeight(w);
+
+    segments[3].x2 = XtX(w) + XtWidth(w);
+    segments[3].y2 = XtX(w);
+
+    adjust_h = (DO_InitialY(w) - (XtY(w) + (XtHeight(w)>>1))) / ZAP_EFFECT2;
+    adjust_w = (DO_InitialX(w) - (XtX(w) + (XtWidth(w)>>1))) / ZAP_EFFECT2;
+
+    gc_values.foreground = 1;
+    gc_values.function = GXxor;
+    gc_values.clip_mask = (Pixmap)NULL;
+    XChangeGC( DisplayOfScreen(screen), gc,
+    	       GCFunction |GCForeground | GCClipMask,
+    	       &gc_values );
+
+    XDrawSegments( DisplayOfScreen(screen), RootWindowOfScreen(screen),
+    	    	   gc, segments, 4);
+
+    XFlush( DisplayOfScreen(screen) );
+
+    if ( abs(adjust_h) <= ZAP_SIZE2
+    && abs(adjust_w) <= ZAP_SIZE2 )
+    {
+    	zap_count = ZAP_EFFECT2;
+    }
+
+    for ( ; ; zap_count++)
+    {
+	_XmMicroSleep( 50000 );
+
+	XDrawSegments( DisplayOfScreen(screen), RootWindowOfScreen(screen),
+    	    	       gc, segments, 4);
+
+	gc_values.foreground = DO_CursorForeground(w);
+	gc_values.function = GXcopy;
+	XChangeGC( DisplayOfScreen(screen), gc,
+	    	   GCFunction | GCForeground,
+    		   &gc_values );
+
+	XCopyArea( DisplayOfScreen(screen), DO_Backing(w).pixmap,
+    		   RootWindowOfScreen(screen), gc,
+		   0, 0, XtWidth(w), XtHeight(w),
+		   segments[0].x2, segments[0].y2 );
+
+	if ( zap_count == 5 )
+    	    break;
+
+    	segments[0].x2 += adjust_w;
+    	segments[0].y2 += adjust_h;
+    	segments[1].x2 += adjust_w;
+    	segments[1].y2 += adjust_h;
+    	segments[2].x2 += adjust_w;
+    	segments[2].y2 += adjust_h;
+    	segments[3].x2 += adjust_w;
+    	segments[3].y2 += adjust_h;
+
+    	XCopyArea( DisplayOfScreen(screen), RootWindowOfScreen(screen),
+    	       DO_Backing(w).pixmap, gc,
+	       segments[0].x2, segments[0].y2,
+	       XtWidth(w), XtHeight(w), 0 ,0 );
+
+    	copy_to_window( w,
+	    DO_RootBlend(w).mixedIcon ? DO_RootBlend(w).mixedIcon
+	    : DO_CursorBlend(w).mixedIcon,
+    	    RootWindowOfScreen(screen),
+	    segments[0].x2, segments[0].y2 );
+
+    	gc_values.foreground = 1;
+    	gc_values.function = GXxor;
+    	XChangeGC( DisplayOfScreen(screen), gc,
+	    	GCFunction | GCForeground,
+    	    	&gc_values );
+
+    	XDrawSegments( DisplayOfScreen(screen), RootWindowOfScreen(screen),
+	    	       gc, segments, 4);
+    	XFlush( DisplayOfScreen(screen) );
+    }
+
+    XFlush( DisplayOfScreen(screen) );
 }
+#undef ZAP_EFFECT2
+#undef ZAP_SIZE2
 
 void
 _XmDragOverHide(Widget w, Position clipOriginX, Position clipOriginY,
