@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DrawingArea.c,v 1.21 1998/05/04 22:04:36 rwscott Exp $
+ * $Id: DrawingArea.c,v 1.30 1998/10/20 19:41:36 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: DrawingArea.c,v 1.21 1998/05/04 22:04:36 rwscott Exp $";
+static const char rcsid[] = "$Id: DrawingArea.c,v 1.30 1998/10/20 19:41:36 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -31,6 +31,7 @@ static char rcsid[] = "$Id: DrawingArea.c,v 1.21 1998/05/04 22:04:36 rwscott Exp
 #include <Xm/DrawingAP.h>
 #include <Xm/TransltnsP.h>
 #include <Xm/ScrolledWP.h>
+#include <Xm/RepType.h>
 #include <XmI/MacrosI.h>
 
 #include <XmI/DebugUtil.h>
@@ -102,6 +103,21 @@ static XtResource resources[] =
 	sizeof(unsigned char), Offset(resize_policy),
 	XmRImmediate, (XtPointer)XmRESIZE_ANY
     }
+#if LESSTIF_BACK_COMPAT
+#else
+#if XmVERSION > 1
+    ,{
+	XmNconvertCallback, XmCCallback, XmRCallback,
+	sizeof(XtCallbackList), Offset(convert_callback),
+	XmRImmediate, (XtPointer)NULL
+    },
+    {
+	XmNdestinationCallback, XmCCallback, XmRCallback,
+	sizeof(XtCallbackList), Offset(destination_callback),
+	XmRImmediate, (XtPointer)NULL
+    }
+#endif
+#endif
 };
 
 static XmSyntheticResource syn_resources[] =
@@ -266,15 +282,37 @@ static void
 initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "%s:initialize: %i args\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__,
+		      *num_args,
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+
+    if (DA_ResizePolicy(new_w) != XmRESIZE_SWINDOW &&
+	!XmRepTypeValidValue(XmRepTypeGetId(XmRResizePolicy),
+			     DA_ResizePolicy(new_w),
+			     new_w))
+    {
+	DA_ResizePolicy(new_w) = XmRESIZE_ANY;
+    }
+
+#if 0
     if (XtWidth(request) == 0)
     {
-	XtWidth(new_w) = 2 * DA_MarginWidth(new_w);
+	XtWidth(new_w) = /* 2 * */ DA_MarginWidth(new_w);
     }
 
     if (XtHeight(request) == 0)
     {
-	XtHeight(new_w) = 2 * DA_MarginHeight(new_w);
+	XtHeight(new_w) = /* 2 * */ DA_MarginHeight(new_w);
     }
+#endif
 
     DEBUGOUT(XdbDebug(__FILE__, new_w,
 		      "DrawingArea initialize: w/h %d %d\n",
@@ -287,13 +325,39 @@ set_values(Widget old, Widget request, Widget new_w,
 {
     Boolean need_refresh = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "setvalues()\n"));
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "%s:set_values: %i args\n"
+		      "\t    old X %5i Y %5i W %5i H %5i\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__,
+		      *num_args,
+		      XtX(old), XtY(old),
+		      XtWidth(old), XtHeight(old),
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
 
 #define NE(x)	(x(old) != x(new_w))
 
+    if (NE(DA_ResizePolicy))
+    {
+	if (!XmRepTypeValidValue(XmRepTypeGetId(XmRResizePolicy),
+			     DA_ResizePolicy(new_w),
+			     new_w))
+	{
+	    DA_ResizePolicy(new_w) = DA_ResizePolicy(old);
+	}
+	else
+	{
+	    need_refresh = True;
+	}
+    }
+    
     if (NE(DA_MarginHeight) ||
-	NE(DA_MarginWidth) ||
-	NE(DA_ResizePolicy))
+	NE(DA_MarginWidth))
     {
 	need_refresh = True;
     }
@@ -326,12 +390,14 @@ resize(Widget w)
 {
     XmDrawingAreaCallbackStruct cb;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:resize() %dx%d\n",
+    	__FILE__,
+    	XtWidth(w), XtHeight(w)));
+
     cb.reason = XmCR_RESIZE;
     cb.event = NULL;
     cb.window = XtWindow(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "DA_ResizeCallback\n"));
-
     XtCallCallbackList(w, DA_ResizeCallback(w), (XtPointer)&cb);
 }
 
@@ -345,11 +411,15 @@ PreferedSize(Widget w, Widget instigator, XtWidgetGeometry *ir,
     *hei = 0;
 
     DEBUGOUT(XdbDebug(__FILE__, w,
-		      "DrawingArea PreferredSize: w/h %d %d\n",
-		      XtWidth(w), XtHeight(w)));
+		      "%s:PreferredSize: %dx%d inst %s %s\n",
+		      __FILE__,
+		      XtWidth(w), XtHeight(w),
+		      instigator ? XtName(instigator) : "NULL",
+		      XdbWidgetGeometry2String(ir)
+		      ));
 
-    if (DA_ResizePolicy(w) == XmRESIZE_NONE ||
-	_XmGeoCount_kids((CompositeRec *)w) == 0)
+    if (DA_ResizePolicy(w) == XmRESIZE_NONE /* ||
+	_XmGeoCount_kids((CompositeRec *)w) == 0 */)
     {
 	*wid = XtWidth(w);
 	*hei = XtHeight(w);
@@ -406,10 +476,27 @@ PreferedSize(Widget w, Widget instigator, XtWidgetGeometry *ir,
      * Let's see what happens if we return our current size instead in those
      * cases.
      */
+     /* rws 7 Oct 1998
+        drawingarea/test7
+      */
     if (cnt == 0)
     {
-	*wid = XtWidth(w);
-	*hei = XtHeight(w);
+	if (XtWidth(w) == 0)
+	{
+	    *wid = DA_MarginWidth(w);
+	}
+	else
+	{
+	    *wid = XtWidth(w);
+	}
+	if (XtHeight(w) == 0)
+	{
+	    *hei = DA_MarginHeight(w);
+	}
+	else
+	{
+	    *hei = XtHeight(w);
+	}
 
 	return;
     }
@@ -420,10 +507,14 @@ PreferedSize(Widget w, Widget instigator, XtWidgetGeometry *ir,
      * increment the total width and height of the drawing area to
      * include the margins on the other side.
      */
-    *wid += DA_MarginWidth(w);
-    *hei += DA_MarginHeight(w);
+    if (DA_ResizePolicy(w) != XmRESIZE_SWINDOW)
+    {
+	*wid += DA_MarginWidth(w);
+	*hei += DA_MarginHeight(w);
+    }
 
-    if (DA_ResizePolicy(w) == XmRESIZE_ANY)
+    if (DA_ResizePolicy(w) == XmRESIZE_ANY ||
+	DA_ResizePolicy(w) == XmRESIZE_SWINDOW)
     {
 	return;
     }
@@ -446,10 +537,13 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 
 #define	Wants(x)	((geo.request_mode & x) == x)
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "query_geometry\n"));
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:query_geometry\n",
+    	__FILE__,
+    	XdbWidgetGeometry2String(proposed) ));
 
     geo = *proposed;
 
+#if 0
     answer->request_mode = 0;
 
     PreferedSize(w, NULL, NULL, &wid, &hei);
@@ -468,6 +562,14 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
     }
 
     return XtGeometryYes;
+#else
+    PreferedSize(w, NULL, NULL, &wid, &hei);
+
+    answer->width = wid;
+    answer->height = hei;
+
+    return _XmGMReplyToQueryGeometry(w, &geo, answer);
+#endif
 }
 
 static XtGeometryResult
@@ -483,61 +585,19 @@ geometry_manager(Widget w,
     DEBUGOUT(XdbDebug2(__FILE__, dw, w, "geometry_manager(%s)\n",
 		     (DA_ResizePolicy(dw) == XmRESIZE_NONE) ? "XmRESIZE_NONE" :
 		     (DA_ResizePolicy(dw) == XmRESIZE_GROW) ? "XmRESIZE_GROW" :
-	      (DA_ResizePolicy(dw) == XmRESIZE_ANY) ? "XmRESIZE_ANY" : "???"));
+		     (DA_ResizePolicy(dw) == XmRESIZE_ANY) ? "XmRESIZE_ANY" :
+      (DA_ResizePolicy(dw) == XmRESIZE_SWINDOW) ? "XmRESIZE_SWINDOW" : "???"));
 
     if (XtIsSubclass(sw, xmScrolledWindowWidgetClass) &&
 	SW_ScrollPolicy(sw) == XmAUTOMATIC &&
 	(Widget)SW_ClipWindow(sw) == dw)
     {
+	
 	DEBUGOUT(XdbDebug2(__FILE__, dw, w,
 			   "geometry_manager: in ScrolledWindow (%s) => YES\n",
 			   XdbWidgetGeometry2String(request)));
 
-	*reply = *request;
-
-	if (!(reply->request_mode & CWX))
-	{
-	    reply->x = XtX(w);
-	}
-	if (!(reply->request_mode & CWY))
-	{
-	    reply->y = XtY(w);
-	}
-	if (!(reply->request_mode & CWWidth))
-	{
-	    reply->width = XtWidth(w);
-	}
-	if (!(reply->request_mode & CWHeight))
-	{
-	    reply->height = XtHeight(w);
-	}
-	if (!(reply->request_mode & CWBorderWidth))
-	{
-	    reply->border_width = XtBorderWidth(w);
-	}
-
-	/*
-	 * Resize the widget as requested
-	 * FIX ME : ScrolledWindow should do this for us
-	 * MLM: Nah.  w is a child of dw, not of dw's ScrolledWindow
-	 * parent.  *We're* supposed to do this
-	 */
-	XtX(w) = reply->x;
-	XtY(w) = reply->y;
-	XtWidth(w) = reply->width;
-	XtHeight(w) = reply->height;
-	XtBorderWidth(w) = reply->border_width;
-
-	/* Trigger the ScrolledWindow's geometry algorithms so it can
-	 * handle scrollbars and such more.
-	 *
-	 * Not using _XmMakeGeometryRequest here since this is a special case.
-	 */
-	DEBUGOUT(XdbDebug(__FILE__, dw, "XtMakeResizeRequest (phony) %d %d\n",
-			  reply->width, reply->height));
-
-	XtMakeResizeRequest(dw, reply->width, reply->height, NULL, NULL);
-
+	XtMakeGeometryRequest(dw, request, reply);
 	return XtGeometryYes;
     }
 
@@ -637,6 +697,7 @@ geometry_manager(Widget w,
 	break;
 
     case XmRESIZE_ANY:
+    case XmRESIZE_SWINDOW:
 	if ((wid != XtWidth(dw)) || (hei != XtHeight(dw)))
 	{
 
@@ -695,6 +756,18 @@ changed_managed(Widget w)
     XtGeometryResult result;
     XtWidgetGeometry req;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:change_managed()\n",
+    	__FILE__));
+    if (DA_ResizePolicy(w) == XmRESIZE_SWINDOW)
+    {
+	XtWidgetProc manager;
+
+	manager = ((CompositeWidgetClass) (XtClass(XtParent(w))))
+	    ->composite_class.change_managed;
+	(*manager)(XtParent(w));
+	_XmNavigChangeManaged(w) ;
+	return;
+    }
     /*
     _XmGMEnforceMargin(w, DA_MarginWidth(w), DA_MarginHeight(w), False);
     */
@@ -703,19 +776,28 @@ changed_managed(Widget w)
     {
 	_XmGMEnforceMargin(w, DA_MarginWidth(w), DA_MarginHeight(w), False);
 	PreferedSize(w, NULL, NULL, &wid, &hei);
+	wid = XtIsRealized(w) ? wid : XtWidth(w) == 0 ? wid : XtWidth(w);
+	hei = XtIsRealized(w) ? hei : XtHeight(w) == 0 ? hei : XtHeight(w);
 	if (wid > XtWidth(w) || hei > XtHeight(w))
 	{
 
 	    req.request_mode = (CWWidth | CWHeight);
 	    req.width = wid;
 	    req.height = hei;
+	    DEBUGOUT(XdbDebug(__FILE__, w, "%s:change_managed() - request %s\n",
+		__FILE__,
+		XdbWidgetGeometry2String(&req)));
 	    result = _XmMakeGeometryRequest(w, &req);
+	    DEBUGOUT(XdbDebug(__FILE__, w, "%s:change_managed() - got %s\n",
+		__FILE__,
+		XdbGeometryResult2String(result)));
 	    reply_wid = req.width;
 	    reply_hei = req.height;
 
 	}
 	/* else we're not getting any bigger... */
-	else if (DA_ResizePolicy(w) == XmRESIZE_ANY)
+	else if (DA_ResizePolicy(w) == XmRESIZE_ANY ||
+		 DA_ResizePolicy(w) == XmRESIZE_SWINDOW)
 	{
 	    req.request_mode = (CWWidth | CWHeight);
 	    req.width = wid;
@@ -778,3 +860,4 @@ widget_navigable(Widget w)
 
     return XmNOT_NAVIGABLE;
 }
+
