@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DrawnB.c,v 1.23 1998/02/22 01:00:31 rwscott Exp $
+ * $Id: DrawnB.c,v 1.29 1998/10/24 22:28:36 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: DrawnB.c,v 1.23 1998/02/22 01:00:31 rwscott Exp $";
+static const char rcsid[] = "$Id: DrawnB.c,v 1.29 1998/10/24 22:28:36 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -39,6 +39,7 @@ static char rcsid[] = "$Id: DrawnB.c,v 1.23 1998/02/22 01:00:31 rwscott Exp $";
  * Try to add trait stuff by #ifdeffing it.
  */
 #if XmVERSION > 1
+#include <Xm/TraitP.h>
 #include <Xm/ActivatableT.h>
 
 void _XmDrawnB_TraitAddCallback(Widget, XtCallbackProc, XtPointer, Boolean);
@@ -50,7 +51,9 @@ static XmActivatableTraitRec _XmDrawnBTraitRec = {
 #endif
 
 /* Forward Declarations */
+#if XmVERSION > 1
 static void class_initialize();
+#endif
 static void class_part_initialize(WidgetClass w_class);
 
 static void initialize(Widget request, Widget new_w,
@@ -251,17 +254,17 @@ XmDrawnButtonClassRec xmDrawnButtonClassRec = {
 
 WidgetClass xmDrawnButtonWidgetClass = (WidgetClass)&xmDrawnButtonClassRec;
 
+#if XmVERSION > 1
 static void
 class_initialize()
 {
-#if XmVERSION > 1
     if (! XmeTraitSet(xmDrawnButtonWidgetClass, XmQTactivatable,
                 &_XmDrawnBTraitRec)) {
         _XmWarning(NULL,
             "XmDrawnButton ClassInitialize: XmeTraitSet failed\n");
     }
-#endif
 }
+#endif
 
 static void
 class_part_initialize(WidgetClass widget_class)
@@ -292,8 +295,12 @@ initialize(Widget request, Widget new_w,
 
     if (Lab_Label(new_w))
     {
+    XmString tmp;
+
 	_XmStringFree(Lab_Label(new_w));
-	Lab_Label(new_w) = _XmStringCreate(XmStringCreateSimple("\0"));
+	tmp = XmStringCreateSimple("\0");
+	Lab_Label(new_w) = _XmStringCreate(tmp);
+	XmStringFree(tmp);
     }
 
     DB_Armed(new_w) = False;
@@ -372,9 +379,43 @@ expose(Widget w, XEvent *event, Region region)
     DEBUGOUT(XdbDebug(__FILE__, w, "Expose (%dx%d%+d%+d)\n",
 		      XtWidth(w), XtHeight(w),XtX(w),XtY(w)));
 
+/*
 #define superclass (&xmLabelClassRec)
     (*superclass->core_class.expose) (w, event, region);
 #undef superclass
+*/
+#if 1
+    /* rws 24 Oct 1998
+       mfm clipboard pixmap
+     */
+    if (XtSensitive(w) &&
+	     Lab_Pixmap(w) != XmUNSPECIFIED_PIXMAP)
+    {
+	XCopyArea(XtDisplay(w),
+		  Lab_Pixmap(w),
+		  XtWindow(w),
+		  Lab_NormalGC(w),
+		  0,
+		  0,
+		  Lab_TextRect_width(w),
+		  Lab_TextRect_height(w),
+		  (XtWidth(w) - Lab_TextRect_width(w)) / 2,
+		  (XtHeight(w) - Lab_TextRect_height(w)) / 2);
+    }
+    else if (!XtSensitive(w) &&
+	     Lab_PixmapInsensitive(w) != XmUNSPECIFIED_PIXMAP)
+    {
+	XCopyArea(XtDisplay(w),
+		  Lab_PixmapInsensitive(w),
+		  XtWindow(w),
+		  Lab_InsensitiveGC(w),
+		  0,
+		  0,
+		  Lab_TextRect_width(w),
+		  Lab_TextRect_height(w),
+		  Lab_TextRect_x(w), Lab_TextRect_y(w));
+    }
+#endif
 
     if (DB_PushButtonEnabled(w))
     {
@@ -427,6 +468,7 @@ resize(Widget w)
 	return;
     }
 
+/*
     if (DB_PushButtonEnabled(w))
     {
 	_XmDrawShadows(XtDisplay(w), XtWindow(w),
@@ -449,6 +491,7 @@ resize(Widget w)
 		       Prim_ShadowThickness(w),
 		       DB_ShadowType(w));
     }
+*/
 
     if (DB_ResizeCallback(w))
     {
@@ -471,6 +514,8 @@ Arm(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XmDrawnButtonCallbackStruct cbs;
     XtExposeProc exp;
 
+    XmProcessTraversal(w, XmTRAVERSE_CURRENT);
+
     DB_Armed(w) = True;
 
     exp = XtClass(w)->core_class.expose;
@@ -555,64 +600,52 @@ Disarm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 ArmTimeout(XtPointer data, XtIntervalId *id)
 {
-    XmDrawnButtonCallbackStruct cbs;
-    XmArmTimeoutRec *p = (XmArmTimeoutRec *)data;
-    Widget w = p->w;
+    Widget w = (Widget)data;
     XtExposeProc exp = XtClass(w)->core_class.expose;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "ArmTimeout\n"));
  
-    if (DB_Armed(w) == False)
+    DB_Timer(w) = 0;
+
+    if (XtIsRealized(w))
     {
-        return;
+	(*exp) (w, NULL, NULL);
+	XFlush(XtDisplay(w));
     }
- 
-    DB_Armed(w) = False;
-
-    (*exp) (w, NULL, NULL);
- 
-    if (!Lab_SkipCallback(w) && DB_ActivateCallback(w))
-    {
-        cbs.reason = XmCR_ACTIVATE;
- 
-        if (p->has_event)
-        {
-            cbs.event = &p->ev;
-        }
-        else
-        {
-            cbs.event = NULL;
-        }
-        cbs.click_count = DB_ClickCount(w);
- 
-        XFlush(XtDisplay(w));
- 
-        XtCallCallbackList(w,
-                           DB_ActivateCallback(w),
-                           (XtPointer)&cbs);
-    }
-
-    Disarm(w, NULL, NULL, 0);
-
-    XtFree(data);
 }
 
 static void
 ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    XmArmTimeoutRec *p = (XmArmTimeoutRec *)XtCalloc(1,
-						     sizeof(XmArmTimeoutRec));
+    XmDrawnButtonCallbackStruct cbs;
  
     DEBUGOUT(XdbDebug(__FILE__, w, "ArmAndActivate\n"));
 
+    /* Arm, Activate, and Disarm now, but draw the disarmed state later */
+
     Arm(w, event, params, num_params);
-
-    p->w = w;
-
-    if (event)
+    DB_Armed(w) = False;
+    if (!Lab_SkipCallback(w) && DB_ActivateCallback(w))
     {
-        p->ev = *event;
-        p->has_event=True;
+	XFlush(XtDisplay(w));
+	cbs.reason = XmCR_ACTIVATE;
+	cbs.event = event;
+	cbs.window = XtWindow(w);
+	cbs.click_count = 1;
+	XtCallCallbackList(w,
+			   DB_ActivateCallback(w),
+			   (XtPointer)&cbs);
+    }
+    if (DB_DisarmCallback(w))
+    {
+	XFlush(XtDisplay(w));
+	cbs.reason = XmCR_DISARM;
+	cbs.event = event;
+	cbs.window = XtWindow(w);
+	cbs.click_count = 1;
+	XtCallCallbackList(w,
+			   DB_DisarmCallback(w),
+			   (XtPointer)&cbs);
     }
 
     if (DB_Timer(w) != 0)
@@ -621,7 +654,7 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     }
 
     DB_Timer(w) = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
-				  ACTIVATE_DELAY, ArmTimeout, (XtPointer)p);
+				  ACTIVATE_DELAY, ArmTimeout, (XtPointer)w);
 }
 
 static void
