@@ -1,6 +1,6 @@
-/**
+/** -*- Mode: C; tab-width: 8; indent-tabs-mode: nil -*-
  *
- * $Id: DropSMgr.c,v 1.15 1998/02/22 01:00:32 rwscott Exp $
+ * $Id: DropSMgr.c,v 1.17 1998/09/12 15:05:25 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: DropSMgr.c,v 1.15 1998/02/22 01:00:32 rwscott Exp $";
+static const char rcsid[] = "$Id: DropSMgr.c,v 1.17 1998/09/12 15:05:25 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -1174,6 +1174,10 @@ loc_to_info(XmDropSiteManagerObject dsm, XmDropSiteInfo dsi,
     return NULL;
 }
 
+/**
+ * Waider 14/06/98 I've just noticed that this function /has/ no
+ * function other than, at the moment, to generate core dumps.
+ */
 static void
 animate(XmDropSiteManagerObject dsm,
 	XmDragMotionClientDataStruct * cd,
@@ -1182,7 +1186,7 @@ animate(XmDropSiteManagerObject dsm,
     XmDropSiteInfo dsi = (XmDropSiteInfo) DS_CurInfo(dsm);
     Widget dc;
 
-    if (dsi->animationStyle == XmDRAG_UNDER_NONE)
+    if (dsi && dsi->animationStyle == XmDRAG_UNDER_NONE)
     {
 	return;
     }
@@ -1450,7 +1454,8 @@ __XmDSMProcessMotion(XmDropSiteManagerObject dsm,
 {
     XmDragMotionCallbackStruct *cbs = (XmDragMotionCallbackStruct *)callData;
     XmDragMotionClientDataStruct *cd = (XmDragMotionClientDataStruct *) clientData;
-    XmDropSiteInfo dsi = (XmDropSiteInfo) DS_CurInfo(dsm), ndsi;
+    XmDropSiteInfo dsi = (XmDropSiteInfo) DS_CurInfo(dsm);
+    XmDropSiteInfo ndsi;
     unsigned char ps;
 
     DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "DSM ProcessMotion\n"));
@@ -1482,6 +1487,8 @@ __XmDSMProcessMotion(XmDropSiteManagerObject dsm,
 		drop_site_leave(dsm, cd, cbs, dsi, ps);
 	    }
 
+            DEBUGOUT(XdbDebug(__FILE__, 
+                              (Widget)dsm, "Setting dsi to %p\n", ndsi ));
 	    DS_CurInfo(dsm) = (XtPointer)ndsi;
 
 	    if (ndsi)
@@ -1491,6 +1498,8 @@ __XmDSMProcessMotion(XmDropSiteManagerObject dsm,
 	}
     }
 
+    DEBUGOUT(XdbDebug(__FILE__, 
+                      (Widget)dsm, "Dragging on %p\n", dsi ));
     drag_motion(dsm, cd, cbs, dsi, ps);
 }
 
