@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Form.c,v 1.109 1998/05/09 19:57:40 rwscott Exp $
+ * $Id: Form.c,v 1.123 1998/10/24 22:28:37 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Form.c,v 1.109 1998/05/09 19:57:40 rwscott Exp $";
+static const char rcsid[] = "$Id: Form.c,v 1.123 1998/10/24 22:28:37 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -295,44 +295,6 @@ static XmSyntheticResource constraint_syn_resources[] =
 	*/
     }
 };
-#if 0
-/* *INDENT-OFF* */
-static XmBaseClassExtRec _XmFormCoreClassExtRec =
-{
-    /* next_extension            */ NULL,
-    /* record_type               */ NULLQUARK,
-    /* version                   */ XmBaseClassExtVersion,
-    /* size                      */ sizeof(XmBaseClassExtRec),
-    /* initialize_prehook        */ NULL,
-    /* set_values_prehook        */ NULL,
-    /* initialize_posthook       */ NULL,
-    /* set_values_posthook       */ NULL,
-    /* secondary_object_class    */ NULL,
-    /* secondary_object_create   */ NULL,
-    /* get_secondary_resources   */ NULL,
-    /* fast_subclass             */ {0},
-    /* get_values_prehook        */ NULL,
-    /* get_values_posthook       */ NULL,
-    /* class_part_init_prehook   */ NULL,
-    /* class_part_init_posthook  */ NULL,
-    /* ext_resources             */ NULL,
-    /* compiled_ext_resources    */ NULL,
-    /* num_ext_resources         */ 0,
-    /* use_sub_resources         */ False,
-    /* widget_navigable          */ XmInheritWidgetNavigable,
-    /* focus_change              */ XmInheritFocusChange,
-    /* wrapper_data              */ NULL
-};
-
-static XmManagerClassExtRec _XmFormMClassExtRec =
-{
-    /* next_extension            */ NULL,
-    /* record_type               */ NULLQUARK,
-    /* version                   */ XmManagerClassExtVersion,
-    /* record_size               */ sizeof(XmManagerClassExtRec),
-    /* traversal_children        */ NULL
-};
-#endif
 
 XmFormClassRec xmFormClassRec =
 {
@@ -369,7 +331,7 @@ XmFormClassRec xmFormClassRec =
 	/* tm_table              */ XtInheritTranslations,
 	/* query_geometry        */ query_geometry,
 	/* display_accelerator   */ NULL,
-	/* extension             */ NULL /*(XtPointer)&_XmFormCoreClassExtRec*/
+	/* extension             */ NULL
     },
     /* Composite class part */
     {
@@ -397,18 +359,18 @@ XmFormClassRec xmFormClassRec =
 	/* syn_constraint_resources     */ constraint_syn_resources,
 	/* num_syn_constraint_resources */ XtNumber(constraint_syn_resources),
 	/* parent_process               */ XmInheritParentProcess,
-	/* extension                    */ NULL /*(XtPointer)&_XmFormMClassExtRec*/
+	/* extension                    */ NULL
     },
     /* XmBulletinBoard part */
     {
 	/* always_install_accelerators  */ False,
 	/* geo_matrix_create            */ NULL,
 	/* focus_moved_proc             */ XmInheritFocusMovedProc,
-	/* extension */ NULL
+	/* extension                    */ NULL
     },
     /* XmForm part */
     {
-	/* extension */ NULL
+	/* extension                    */ NULL
     }
 };
 /* *INDENT-ON* */
@@ -427,9 +389,10 @@ initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
     DEBUGOUT(XdbDebug(__FILE__, new_w,
-		      "initialize: %i args\n"
+		      "%s:initialize(%d) - %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__, __LINE__,
 		      *num_args,
 		      XtX(request), XtY(request),
 		      XtWidth(request), XtHeight(request),
@@ -464,7 +427,9 @@ realize(Widget w,
    of -2, because of the topOffset, and BulletinBoard was then enforcing
    a margin of 0.
 */
-    DEBUGOUT(XdbDebug(__FILE__, w, "Realize\n"));
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:realize(%d) - %dx%d\n",
+    	__FILE__, __LINE__,
+    	XtWidth(w), XtHeight(w)));
 
 #define superclass (&xmManagerClassRec)
     (*superclass->core_class.realize) (w, value_mask, attributes);
@@ -478,10 +443,11 @@ set_values(Widget old, Widget request, Widget new_w,
     Boolean relayout = False, refresh = False;
 
     DEBUGOUT(XdbDebug(__FILE__, new_w,
-		      "set_values: %i args\n"
+		      "%s:set_values(%d) - %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__, __LINE__,
 		      *num_args,
 		      XtX(old), XtY(old),
 		      XtWidth(old), XtHeight(old),
@@ -533,7 +499,8 @@ resize(Widget w)
 {
     Dimension wd, ht;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Resize (width %d height %d)\n",
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:resize(%d) - %dx%d\n",
+		      __FILE__, __LINE__,
 		      XtWidth(w), XtHeight(w)));
 
     wd = XtWidth(w);
@@ -560,10 +527,12 @@ resize(Widget w)
 static XtGeometryResult
 query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *desired)
 {
+    /* form/test24 */
     Dimension wd, ht;
     XtWidgetGeometry pp;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "query_geometry %s\n",
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:query_geometry(%d) - proposed %s\n",
+		      __FILE__, __LINE__,
 		      XdbWidgetGeometry2String(proposed)));
 
     pp = *proposed;
@@ -575,9 +544,12 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *desired)
     }
     else
     {
-	wd = 0;
-	ht = 0;
+	wd = XtIsRealized(w) ? 0 : XtWidth(w);
+	ht = XtIsRealized(w) ? 0 : XtHeight(w);
+
 	_XmFormLayout(w, NULL, NULL, &wd, &ht);
+	wd = pp.request_mode & CWWidth ? pp.width < wd ? wd : pp.width : wd;
+	ht = pp.request_mode & CWHeight ? pp.height < ht ? ht : pp.height : ht;
 
 	if (BB_ResizePolicy(w) == XmRESIZE_GROW)
 	{
@@ -592,55 +564,14 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *desired)
 	}
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "query returned geometry %s\n",
-		      XdbWidgetGeometry2String(proposed)));
-
     desired->request_mode = CWWidth | CWHeight;
-    desired->width = wd;
-    desired->height = ht;
+    desired->width = wd == 0 ? 1 : wd;
+    desired->height = ht == 0 ? 1 : ht;
 
-    if ((pp.request_mode & (CWWidth | CWHeight)) == 0)
-    {
-	if (XtIsRealized(w))
-	{
-	    return XtGeometryNo;
-	}
-	else
-	{
-	    return XtGeometryAlmost;
-	}
-    }
-    else if ((pp.request_mode & (CWWidth | CWHeight)) == (CWWidth | CWHeight) &&
-	     pp.width == desired->width && pp.height == desired->height)
-    {
-	return XtGeometryYes;
-    }
-    else if (((pp.request_mode & CWWidth) == CWWidth) &&
-	     pp.width == desired->width)
-    {
-	return XtGeometryAlmost;
-    }
-    else if (((pp.request_mode & CWHeight) == CWHeight) &&
-	     pp.height == desired->height)
-    {
-	return XtGeometryAlmost;
-    }
-    else if (desired->width == XtWidth(w) &&
-	     desired->height == XtHeight(w))
-    {
-	if (XtIsRealized(w))
-	{
-	    return XtGeometryNo;
-	}
-	else
-	{
-	    return XtGeometryAlmost;
-	}
-    }
-    else
-    {
-	return XtGeometryAlmost;
-    }
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:query_geometry(%d) - desired %s\n",
+		      __FILE__, __LINE__,
+		      XdbWidgetGeometry2String(desired)));
+    return _XmGMReplyToQueryGeometry(w, &pp, desired);
 }
 
 /*
@@ -678,8 +609,10 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
     }
     if (XdbInDebug(__FILE__, f))
     {
-	DEBUGOUT(XdbDebug2(__FILE__, f, w,
-			   "GeometryManager : request "));
+	DEBUGOUT(XdbDebug2(__FILE__, f, w, "%s:geometry_manager(%d) - request %s\n",
+			   __FILE__, __LINE__,
+			   XdbWidgetGeometry2String(request)));
+	/*
 	if (Wants(CWX))
 	    DEBUGOUT(XdbDebug0(__FILE__, f, "X %d ", request->x));
 	if (Wants(CWY))
@@ -689,26 +622,27 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 	if (Wants(CWHeight))
 	    DEBUGOUT(XdbDebug0(__FILE__, f, "H %d ", request->height));
 	DEBUGOUT(XdbDebug0(__FILE__, f, "\n"));
+	*/
     }
 
     mr = *request;
-       /*
-       o      Every child has a preferred width, based on  geome-
-              try  requests it makes (whether they are granted or
-              not).
-	*/
-	if (mr.request_mode & CWWidth)
-	{
+   /*
+   o      Every child has a preferred width, based on  geome-
+	  try  requests it makes (whether they are granted or
+	  not).
+    */
+    {
         XmFormConstraints con = (XmFormConstraints)CoreConstraints(w);
 
+	if (mr.request_mode & CWWidth)
+	{
 	    FCP_PrefW(con) = mr.width;
 	}
 	if (mr.request_mode & CWHeight)
 	{
-        XmFormConstraints con = (XmFormConstraints)CoreConstraints(w);
-
 	    FCP_PrefH(con) = mr.height;
 	}
+    }
 
     wd = XtIsRealized(f) ? 0 : XtWidth(f);
     ht = XtIsRealized(f) ? 0 : XtHeight(f);
@@ -736,19 +670,10 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
     }
     else
     {
-#if 0
-	/* rws 24 Mar 1998
-	   This was taken out to fix up mgv's magstep form. Essentially if
-	   the parent says NO to a size change then relayout ourself with
-	   the requested geo and the current size.
-	 */
-	return XtGeometryNo;
-#else
 	mr = *request;
 	wd = XtWidth(f);
 	ht = XtHeight(f);
 	_XmFormLayout(f, w, &mr, &wd, &ht);
-#endif
     }
 
     /*
@@ -909,10 +834,11 @@ constraint_initialize(Widget request, Widget new_w,
     int i;
 
     DEBUGOUT(XdbDebug2(__FILE__, f, new_w,
-		       "constraint_initialize: %i args\n"
+		       "%s:constraint_initialize(%d) - %i args\n"
 		       "\trequest X %5i Y %5i W %5i H %5i\n"
 		       "\t  new_w X %5i Y %5i W %5i H %5i\n"
 		       "\t   Form W %5i H %5i\n",
+		       __FILE__, __LINE__,
 		       *num_args,
 		       XtX(request), XtY(request),
 		       XtWidth(request), XtHeight(request),
@@ -944,8 +870,16 @@ constraint_initialize(Widget request, Widget new_w,
 	}
     }
 
+    /* rws 2 Jul 1998
+       Mozilla NavCenter window stop the left hand selector pane from coming
+       up 12 pixels wide
+     */
+    /*
     FCP_PrefW(con) = XtWidth(new_w);
     FCP_PrefH(con) = XtHeight(new_w);
+    */
+    FCP_PrefW(con) = XtWidth(request);
+    FCP_PrefH(con) = XtHeight(request);
 }
 
 static Boolean
@@ -959,11 +893,12 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
     int i;
 
     DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, new_w,
-		       "constraint_set_values: %i args\n"
+		       "%s:constraint_set_values(%d) - %i args\n"
 		       "\tcurrent X %5i Y %5i W %5i H %5i\n"
 		       "\trequest X %5i Y %5i W %5i H %5i\n"
 		       "\t  new_w X %5i Y %5i W %5i H %5i\n"
 		       "\t   Form W %5i H %5i\n",
+		       __FILE__, __LINE__,
 		       *num_args,
 		       XtX(current), XtY(current),
 		       XtWidth(current), XtHeight(current),
@@ -1041,7 +976,12 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
 			   "constraint_set_values - refused resize\n"));
     }
 
-    if (changed && XtIsManaged(f))
+#if 1
+    /* rws 24 Oct 1998 */
+    /* scrolledwindow/test13 */
+    /* form/test15 */
+    /* Xinvest when centering in the frame */
+    if (changed && XtIsRealized(f) && XtIsManaged(f))
     {
 	Dimension wd, ht;
 
@@ -1074,6 +1014,7 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
 	    _XmFormConfigureChildren(f, NULL, NULL);
 	}
     }
+#endif
 
     return changed;
 }
@@ -1346,6 +1287,8 @@ XmFormFindPreferred(Widget f, Widget cw, XtWidgetGeometry *cg)
     Boolean changed;		/* is needed because of SETX macros */
 
     DEBUGOUT(XdbDebug(__FILE__, (Widget)f, "XmFormFindPreferred() - start\n"));
+    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, cw, "XmFormFindPreferred() - request %s\n",
+    			XdbWidgetGeometry2String(cg)));
     for (i = 0; i < MGR_NumChildren(f); i++)
     {
 	Widget child = MGR_Children(f)[i];
@@ -1356,7 +1299,11 @@ XmFormFindPreferred(Widget f, Widget cw, XtWidgetGeometry *cg)
 	    XtWidgetGeometry p;
 
 	    if (cw && cw == child && cg)
-		p = *cg;
+	    {
+		p.width = (cg->request_mode & CWWidth) ? cg->width : XtWidth(child);
+		p.height = (cg->request_mode & CWHeight) ? cg->height : XtHeight(child);
+		p.border_width = (cg->request_mode & CWBorderWidth) ? cg->border_width : XtBorderWidth(child);
+	    }
 	    else
 	    {
 		p.width = XtWidth(child);
@@ -1470,52 +1417,69 @@ XmFormPrintAttachmentReport(Widget f)
  *
  * Return the size of the Form in that direction.
  */
-static Dimension
+static int
 XmFormPath(Widget f, Widget c, int dir, XmFormAttachment atta)
 {
     XmFormConstraints con;
     int x;
 
-    if (c == NULL)
+    if (c == NULL || !XtIsManaged(c))
     {
-	return 0;
+	x = 0;
+    }
+    else
+    {
+	con = (XmFormConstraints)CoreConstraints(c);
+
+	atta[dir].type = FCP_Atta(con, dir).type;
+	atta[dir].value = FCP_Atta(con, dir).value;
+	atta[dir].offset = FCP_Atta(con, dir).offset;
+	atta[dir].w = c;
+	x = 0;
+	if (dir == TOP || dir == BOTTOM)
+	{
+	    x = FCP_H(con);
+	}
+	else if (dir == LEFT || dir == RIGHT)
+	{
+	    x = FCP_W(con);
+	}
+
+	switch (FCP_Atta(con, dir).type)
+	{
+	case XmATTACH_FORM:
+	    if (dir == TOP || dir == BOTTOM)
+	    {
+		x += BB_MarginHeight(f);
+	    }
+	    else if (dir == LEFT || dir == RIGHT)
+	    {
+		x += BB_MarginWidth(f);
+	    }
+	    /* rws 15 Oct 1998
+	       Fall through on purpose for now.  What happens if a margin _and_
+	       an offset is specified!
+	     */
+	case XmATTACH_WIDGET:
+	    x += FCP_Atta(con, dir).offset;
+	    break;
+	case XmATTACH_OPPOSITE_WIDGET:
+	    {
+	    XmFormConstraints con1;
+
+		con1 = (XmFormConstraints)CoreConstraints(FCP_Atta(con, dir).w);
+
+		x += FCP_Atta(con1, dir).offset;
+	    }
+	    break;
+	}
+
+	if (FCP_Atta(con, dir).type == XmATTACH_WIDGET)
+	{
+	    x += XmFormPath(f, FCP_Atta(con, dir).w, dir, atta);
+	}
     }
 
-    con = (XmFormConstraints)CoreConstraints(c);
-
-    if (!XtIsManaged(c))
-    {
-	return (0);
-    }
-
-    atta[dir].type = FCP_Atta(con, dir).type;
-    atta[dir].value = FCP_Atta(con, dir).value;
-    atta[dir].offset = FCP_Atta(con, dir).offset;
-    atta[dir].w = c;
-    x = 0;
-    if (dir == TOP || dir == BOTTOM)
-    {
-	x = FCP_H(con);
-    }
-    if (dir == LEFT || dir == RIGHT)
-    {
-	x = FCP_W(con);
-    }
-
-    switch (FCP_Atta(con, dir).type)
-    {
-    case XmATTACH_FORM:
-    case XmATTACH_WIDGET:
-	x += FCP_Atta(con, dir).offset;
-	break;
-    }
-
-    if (FCP_Atta(con, dir).type == XmATTACH_WIDGET)
-    {
-	x += XmFormPath(f, FCP_Atta(con, dir).w, dir, atta);
-    }
-
-    if (x < 0) x = 0;
     return (x);
 }
 
@@ -1544,19 +1508,21 @@ XmFormAllPaths(Widget f, Dimension *ww, Dimension *hh)
 	        /* test 2 test30 nedit */
 	    	y = atta[TOP].offset + atta[BOTTOM].offset + (y * Form_FractionBase(f) / (atta[BOTTOM].value -
 			atta[TOP].value));
-	        y -= 2 * BB_MarginHeight(f);
+//	        y -= 2 * BB_MarginHeight(f);
 	    }
 	    else if (atta[BOTTOM].type == XmATTACH_POSITION)
 	    {
 	    /* test18 test19 test27 */
-		y = (int)((y + atta[BOTTOM].offset) * Form_FractionBase(f) /
-			atta[BOTTOM].value);
+		y = (int)(((10 * (y + atta[BOTTOM].offset) * Form_FractionBase(f) /
+			atta[BOTTOM].value) + 5) / 10);
+//	        y -= 1 * BB_MarginHeight(f);
 	    }
 	    else if (atta[TOP].type == XmATTACH_POSITION)
 	    {
 	    /* test28 test29 */
-		y = (y  + atta[TOP].offset ) * Form_FractionBase(f) /
-			(Form_FractionBase(f) - atta[TOP].value);
+		y = (int)(((10 * (y  + atta[TOP].offset ) * Form_FractionBase(f) /
+			(Form_FractionBase(f) - atta[TOP].value)) + 5) / 10);
+//	        y -= 1 * BB_MarginHeight(f);
 	    }
 	    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
 			       "XmFormAllPaths H %5i\n", y));
@@ -1566,7 +1532,7 @@ XmFormAllPaths(Widget f, Dimension *ww, Dimension *hh)
 	    }
 	}
     }
-    *hh += 2 * BB_MarginHeight(f);
+//    *hh += 2 * BB_MarginHeight(f);
 
     *ww = 0;
     for (i = 0; i < MGR_NumChildren(f); i++)
@@ -1585,18 +1551,20 @@ XmFormAllPaths(Widget f, Dimension *ww, Dimension *hh)
 	    {
 	    	y = atta[LEFT].offset + atta[RIGHT].offset + (y * Form_FractionBase(f) / (atta[RIGHT].value -
 			atta[LEFT].value));
-	        y -= 2 * BB_MarginHeight(f);
+//	        y -= 2 * BB_MarginWidth(f);
 	    }
 	    else if (atta[RIGHT].type == XmATTACH_POSITION)
 	    {
-		y = (y + atta[RIGHT].offset) * Form_FractionBase(f) /
-			atta[RIGHT].value;
+		y = ((10 * (y + atta[RIGHT].offset) * Form_FractionBase(f) /
+			atta[RIGHT].value) + 5) / 10;
+//	        y -= 1 * BB_MarginWidth(f);
 	    }
 	    else if (atta[LEFT].type == XmATTACH_POSITION)
 	    {
 	    /* test13 test31 mgdiff */
-		y = (y + atta[LEFT].offset) * Form_FractionBase(f) /
-			(Form_FractionBase(f) - atta[LEFT].value);
+		y = ((10 * (y + atta[LEFT].offset) * Form_FractionBase(f) /
+			(Form_FractionBase(f) - atta[LEFT].value)) + 5) / 10;
+//	        y -= 1 * BB_MarginWidth(f);
 	    }
 	    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
 			       "XmFormAllPaths W %5i\n", y));
@@ -1607,7 +1575,7 @@ XmFormAllPaths(Widget f, Dimension *ww, Dimension *hh)
 	}
     }
 
-    *ww += 2 * BB_MarginWidth(f);
+//    *ww += 2 * BB_MarginWidth(f);
 
     DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
 		      "XmFormAllPaths returning %d %d\n", *ww, *hh));
@@ -1672,8 +1640,11 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 #define	Wants(x)	(cg->request_mode & x)
 
     DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
-		      "XmFormLayout with width %d height %d\n",
+		      "_XmFormLayout with width %d height %d\n",
 		      XtWidth(f), XtHeight(f)));
+    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, cw,
+		      "_XmFormLayout request %s\n",
+		      XdbWidgetGeometry2String(cg)));
 
     /*
      * Special case ... should this be a special case (maybe our code below
@@ -1683,7 +1654,7 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
     if (_XmGeoCount_kids((CompositeRec *)f) == 0)
     {
 	DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
-			  "_XmFormLayout: Form has no kids!\n"));
+			  "_XmFormLayout: Form has no managed kids!\n"));
 
 	*width = XtWidth(f);
 	*height = XtHeight(f);
@@ -1962,29 +1933,15 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 
 	    /* TOP */
 	    case XmATTACH_WIDGET:
-		/* 11 12 13 14 17 19 23 25 4 6 */
+		/* 11 12 13 14 17 19 23 25 36 37 4 6 */
 		if (!(FCP_Atta(con, TOP).w))
 		{
-		    /* rws 4 Jan 1998
-		       Now that the geometry stuff is working much better
-		       this does not seem to be necessary and in fact breaks
-		       test32.
-		    if (XtIsRealized(f))
-		    */
-		    {
-			/* if it is not realized just ignore it and hope the
-			 * attachment will be specified before it is realized
-			 * (mgdiff does alot of this)
-			 * rws 26 Feb 1997
-			 */
 #ifdef	LESSTIF_VERBOSE
-			_XmWarning(child, "top: XmATTACH_WIDGET without "
-				   "widget changed to XmATTACH_FORM");
+		    _XmWarning(child, "top: XmATTACH_WIDGET without "
+			       "widget changed to XmATTACH_FORM");
 #endif
-			FCP_Atta(con, TOP).type = XmATTACH_FORM;
-			changed = True;
-		    }
-
+		    FCP_Atta(con, TOP).type = XmATTACH_FORM;
+		    changed = True;
 		    break;
 		}
 
@@ -1996,26 +1953,13 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 			     AttachedHt(FCP_Atta(con, TOP).w) +
 			     FCP_Atta(con, TOP).offset;
 
-		    if (yy < 0)
+		    if (FCP_Atta(con, BOTTOM).type != XmATTACH_NONE)
 		    {
-		        /* */
-		        _XmWarning(child,"top ATTACH_WIDGET %s %d"
-					 " no test coverage",__FILE__,__LINE__);
-
-			SETWY(FCP_Atta(con, TOP),
-			      FCP_Y(con) - AttachedHt(FCP_Atta(con, TOP).w) -
-				FCP_Atta(con, TOP).offset);
+			/* 14 17 19 23 25 37 6 */
+			SETH(FCP_Y(con) + FCP_H(con) - (yy));
 		    }
-		    else
-		    {
-			if (FCP_Atta(con, BOTTOM).type != XmATTACH_NONE)
-			{
-			    /* 14 17 19 23 25 6 */
-			    SETH(FCP_Y(con) + FCP_H(con) - (yy));
-			}
 
-			SETY(yy);
-		    }
+		    SETY(yy);
 		}
 		break;
 
@@ -2024,26 +1968,12 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		/* 12 13 20 4 */
 		if (!(FCP_Atta(con, TOP).w))
 		{
-		    /* rws 4 Jan 1998
-		       Now that the geometry stuff is working much better
-		       this does not seem to be necessary and in fact breaks
-		       test32.
-		    if (XtIsRealized(f))
-		    */
-		    {
-			/* if it is not realized just ignore it and hope the
-			 * attachment will be specified before it is realized
-			 * (mgdiff does alot of this)
-			 * rws 26 Feb 1997
-			 */
 #ifdef	LESSTIF_VERBOSE
-			_XmWarning(child, "top: XmATTACH_OPPOSITE_WIDGET "
-				   "without widget changed to XmATTACH_FORM");
+		    _XmWarning(child, "top: XmATTACH_OPPOSITE_WIDGET "
+			       "without widget changed to XmATTACH_FORM");
 #endif
-			FCP_Atta(con, TOP).type = XmATTACH_FORM;
-			changed = True;
-		    }
-
+		    FCP_Atta(con, TOP).type = XmATTACH_FORM;
+		    changed = True;
 		    break;
 		}
 
@@ -2053,21 +1983,12 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		    int yy = AttachedY(FCP_Atta(con, TOP).w) +
 			     FCP_Atta(con, TOP).offset;
 
-		    if (yy < 0)
+		    if (FCP_Atta(con, BOTTOM).type != XmATTACH_NONE)
 		    {
-			/* 4 */
-			SETWY(FCP_Atta(con, TOP),
-			      FCP_Y(con) - FCP_Atta(con, TOP).offset);
-		    }
-		    else
-		    {
-			if (FCP_Atta(con, BOTTOM).type != XmATTACH_NONE)
-			{
-				/* 12 13 */
-				SETH(FCP_Y(con) - (yy));
-			}
-			SETY(yy);
+			    /* 12 13 */
+			    SETH(FCP_Y(con) - (yy));
 		    }
+		    SETY(yy);
 		}
 		break;
 
@@ -2177,27 +2098,13 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		/* 14 15 25 30 8 */
 		if (!(FCP_Atta(con, BOTTOM).w))
 		{
-		    /* rws 4 Jan 1998
-		       Now that the geometry stuff is working much better
-		       this does not seem to be necessary and in fact breaks
-		       test32.
-		    if (XtIsRealized(f))
-		    */
-		    {
-			/* if it is not realized just ignore it and hope the
-			 * attachment will be specified before it is realized
-			 * (mgdiff does alot of this)
-			 * rws 26 Feb 1997
-			 */
 #ifdef	LESSTIF_VERBOSE
-			_XmWarning(child, "bottom: XmATTACH_WIDGET without "
-				   "widget changed to XmATTACH_FORM");
+		    _XmWarning(child, "bottom: XmATTACH_WIDGET without "
+			       "widget changed to XmATTACH_FORM");
 #endif
 
-			FCP_Atta(con, BOTTOM).type = XmATTACH_FORM;
-			changed = True;
-		    }
-
+		    FCP_Atta(con, BOTTOM).type = XmATTACH_FORM;
+		    changed = True;
 		    break;
 		}
 
@@ -2246,27 +2153,13 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		/* 12 13 21 8 */
 		if (!(FCP_Atta(con, BOTTOM).w))
 		{
-		    /* rws 4 Jan 1998
-		       Now that the geometry stuff is working much better
-		       this does not seem to be necessary and in fact breaks
-		       test32.
-		    if (XtIsRealized(f))
-		    */
-		    {
-			/* if it is not realized just ignore it and hope the
-			 * attachment will be specified before it is realized
-			 * (mgdiff does alot of this)
-			 * rws 26 Feb 1997
-			 */
 #ifdef	LESSTIF_VERBOSE
-			_XmWarning(child, "bottom: XmATTACH_OPPOSITE_WIDGET "
-				   "without widget changed to XmATTACH_FORM");
+		    _XmWarning(child, "bottom: XmATTACH_OPPOSITE_WIDGET "
+			       "without widget changed to XmATTACH_FORM");
 #endif
 
-			FCP_Atta(con, BOTTOM).type = XmATTACH_FORM;
-			changed = True;
-		    }
-
+		    FCP_Atta(con, BOTTOM).type = XmATTACH_FORM;
+		    changed = True;
 		    break;
 		}
 
@@ -2359,26 +2252,13 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		/* 1 15 20 21 22 4 6 8 */
 		if (!(FCP_Atta(con, LEFT).w))
 		{
-		    /* rws 4 Jan 1998
-		       Now that the geometry stuff is working much better
-		       this does not seem to be necessary and in fact breaks
-		       test32.
-		    if (XtIsRealized(f))
-		    */
-		    {
-			/* if it is not realized just ignore it and hope the
-			 * attachment will be specified before it is realized
-			 * (mgdiff does alot of this)
-			 * rws 26 Feb 1997
-			 */
 #ifdef	LESSTIF_VERBOSE
-			_XmWarning(child, "left: XmATTACH_WIDGET without "
-				   "widget changed to XmATTACH_FORM");
+		    _XmWarning(child, "left: XmATTACH_WIDGET without "
+			       "widget changed to XmATTACH_FORM");
 #endif
 
-			FCP_Atta(con, LEFT).type = XmATTACH_FORM;
-			changed = True;
-		    }
+		    FCP_Atta(con, LEFT).type = XmATTACH_FORM;
+		    changed = True;
 		    break;
 		}
 		if (FCP_X(con) != (AttachedX(FCP_Atta(con, LEFT).w) +
@@ -2389,25 +2269,13 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 			     AttachedWd(FCP_Atta(con, LEFT).w) +
 			     FCP_Atta(con, LEFT).offset;
 
-		    if (xx < 0)
+		    if (FCP_Atta(con, RIGHT).type != XmATTACH_NONE)
 		    {
-			/* */
-			_XmWarning(child,"left ATTACH_WIDGET %s %d"
-					 " no test coverage",__FILE__,__LINE__);
-			SETWX(FCP_Atta(con, LEFT),
-			      FCP_X(con) - AttachedWd(FCP_Atta(con, LEFT).w) -
-				FCP_Atta(con, LEFT).offset);
+			/* 1 15 22 4 6 8 */
+			SETW(FCP_X(con) + FCP_W(con) - (xx));
 		    }
-		    else
-		    {
-			if (FCP_Atta(con, RIGHT).type != XmATTACH_NONE)
-			{
-			    /* 1 15 22 4 6 8 */
-			    SETW(FCP_X(con) + FCP_W(con) - (xx));
-			}
 
-			SETX(xx);
-		    }
+		    SETX(xx);
 		}
 		break;
 
@@ -2416,26 +2284,13 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		/* 4 */
 		if (!(FCP_Atta(con, LEFT).w))
 		{
-		    /* rws 4 Jan 1998
-		       Now that the geometry stuff is working much better
-		       this does not seem to be necessary and in fact breaks
-		       test32.
-		    if (XtIsRealized(f))
-		    */
-		    {
-			/* if it is not realized just ignore it and hope the
-			 * attachment will be specified before it is realized
-			 * (mgdiff does alot of this)
-			 * rws 26 Feb 1997
-			 */
 #ifdef	LESSTIF_VERBOSE
-			_XmWarning(child, "left: XmATTACH_OPPOSITE_WIDGET "
-				   "without widget changed to XmATTACH_FORM");
+		    _XmWarning(child, "left: XmATTACH_OPPOSITE_WIDGET "
+			       "without widget changed to XmATTACH_FORM");
 #endif
 
-			FCP_Atta(con, LEFT).type = XmATTACH_FORM;
-			changed = True;
-		    }
+		    FCP_Atta(con, LEFT).type = XmATTACH_FORM;
+		    changed = True;
 		    break;
 		}
 		if (FCP_X(con) != AttachedX(FCP_Atta(con, LEFT).w) +
@@ -2444,24 +2299,13 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		    int xx = AttachedX(FCP_Atta(con, LEFT).w) +
 			     FCP_Atta(con, LEFT).offset;
 
-		    if (xx < 0)
+		    if (FCP_Atta(con, RIGHT).type != XmATTACH_NONE)
 		    {
-			/* */
-			_XmWarning(child,"left ATTACH_OPPOSITE_WIDGET %s %d"
-					 " no test coverage",__FILE__,__LINE__);
-			SETWX(FCP_Atta(con, LEFT),
-			      FCP_X(con) - FCP_Atta(con, LEFT).offset);
+			/* 4 */
+			SETW(FCP_X(con) + FCP_W(con) - (xx));
 		    }
-		    else
-		    {
-			if (FCP_Atta(con, RIGHT).type != XmATTACH_NONE)
-			{
-			    /* 4 */
-			    SETW(FCP_X(con) + FCP_W(con) - (xx));
-			}
 
-			SETX(xx);
-		    }
+		    SETX(xx);
 		}
 		break;
 
@@ -2533,26 +2377,13 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	    case XmATTACH_WIDGET:
 		if (!(FCP_Atta(con, RIGHT).w))
 		{
-		    /* rws 4 Jan 1998
-		       Now that the geometry stuff is working much better
-		       this does not seem to be necessary and in fact breaks
-		       test32.
-		    if (XtIsRealized(f))
-		    */
-		    {
-			/* if it is not realized just ignore it and hope the
-			 * attachment will be specified before it is realized
-			 * (mgdiff does alot of this)
-			 * rws 26 Feb 1997
-			 */
 #ifdef	LESSTIF_VERBOSE
-			_XmWarning(child, "right: XmATTACH_WIDGET without "
-				   "widget changed to XmATTACH_FORM");
+		    _XmWarning(child, "right: XmATTACH_WIDGET without "
+			       "widget changed to XmATTACH_FORM");
 #endif
 
-			FCP_Atta(con, RIGHT).type = XmATTACH_FORM;
-			changed = True;
-		    }
+		    FCP_Atta(con, RIGHT).type = XmATTACH_FORM;
+		    changed = True;
 		    break;
 		}
 		if (FCP_X(con) + FCP_W(con) !=
@@ -2575,27 +2406,13 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		/* 4 */
 		if (!(FCP_Atta(con, RIGHT).w))
 		{
-		    /* rws 4 Jan 1998
-		       Now that the geometry stuff is working much better
-		       this does not seem to be necessary and in fact breaks
-		       test32.
-		    if (XtIsRealized(f))
-		    */
-		    {
-			/* if it is not realized just ignore it and hope the
-			 * attachment will be specified before it is realized
-			 * (mgdiff does alot of this)
-			 * rws 26 Feb 1997
-			 */
 #ifdef	LESSTIF_VERBOSE
-			_XmWarning(child, "right : XmATTACH_OPPOSITE_WIDGET "
-				   "without widget changed to XmATTACH_FORM");
+		    _XmWarning(child, "right : XmATTACH_OPPOSITE_WIDGET "
+			       "without widget changed to XmATTACH_FORM");
 #endif
 
-			FCP_Atta(con, RIGHT).type = XmATTACH_FORM;
-			changed = True;
-		    }
-
+		    FCP_Atta(con, RIGHT).type = XmATTACH_FORM;
+		    changed = True;
 		    break;
 		}
 
