@@ -1,6 +1,6 @@
 /** 
  *
- * $Id: Frame.c,v 1.35 1998/04/03 14:22:46 rwscott Exp $
+ * $Id: Frame.c,v 1.44 1998/10/13 16:09:58 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,12 @@
  *
  **/
 
-static char rcsid[] = "$Id: Frame.c,v 1.35 1998/04/03 14:22:46 rwscott Exp $";
+static const char rcsid[] = "$Id: Frame.c,v 1.44 1998/10/13 16:09:58 rwscott Exp $";
+
+/*
+ * Warning : this widget has a small amount of Motif 2.0 support.
+ * It is surrounded by #if XmVERSION > 1.
+ */
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -48,7 +53,7 @@ static char rcsid[] = "$Id: Frame.c,v 1.35 1998/04/03 14:22:46 rwscott Exp $";
  * Forward Declarations
  */
 /* core */
-static void class_initialize(void);
+/* static void class_initialize(void); */
 
 static void class_part_initialize(WidgetClass widget_class);
 
@@ -152,11 +157,31 @@ static XmSyntheticResource syn_resources[] =
 #define Offset(field) XtOffsetOf(XmFrameConstraintRec, frame.field)
 static XtResource frameConstraintResources[] =
 {
+#if 0
+    /*
+     * This is a weird one. XmNchildType is obsoleted, and replaced by
+     * XmNframeChildType. Unfortunately the former is also still supposed
+     * to work.
+     */
+    {
+	XmNframeChildType, XmCChildType, XmRChildType,
+	sizeof(unsigned char), Offset(child_type),
+	XtRImmediate, (XtPointer)XmFRAME_WORKAREA_CHILD
+    },
+#else
+    /*
+     * This is the Motif 1.2 version
+     * We can't have both of these in here (if an application sets
+     * only one of the resources, the other's default value would
+     * still apply).
+     * Therefore we'll have to simulate one in code.
+     */
     {
 	XmNchildType, XmCChildType, XmRChildType,
 	sizeof(unsigned char), Offset(child_type),
 	XtRImmediate, (XtPointer)XmFRAME_WORKAREA_CHILD
     },
+#endif
     {
 	XmNchildHorizontalAlignment, XmCChildHorizontalAlignment,
 	 XmRChildHorizontalAlignment,
@@ -498,6 +523,7 @@ expose(Widget w, XEvent *event, Region region)
 static XtGeometryResult
 query_geometry(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 {
+#if 0
     Dimension wd, ht;
     XtGeometryResult r;
     XtWidgetGeometry rr = *request;
@@ -536,6 +562,20 @@ query_geometry(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 		      XdbWidgetGeometry2String(reply)));
 
     return r;
+#else
+    Dimension wd, ht;
+
+    DEBUGOUT(XdbDebug(__FILE__, w,
+		      "Frame query_geometry %p W %i H %i\n",
+		       w, XtWidth(w), XtHeight(w)));
+
+    _XmFrameComputeSize(w, NULL, NULL, &wd, &ht);
+
+    reply->width = wd;
+    reply->height = ht;
+
+    return _XmGMReplyToQueryGeometry(w, request, reply);
+#endif
 }
 
 /*
@@ -765,13 +805,36 @@ static void
 constraint_initialize(Widget request, Widget new_w,
 		      ArgList args, Cardinal *num_args)
 {
+#if XmVERSION > 1
+    int	i;
+#endif
+
+    DEBUGOUT(XdbDebug2(__FILE__, XtParent(new_w), new_w, "%s:constraint_initialize()\n", __FILE__));
+
+#if XmVERSION > 1
+    if (num_args)
+	for (i=0; i<*num_args; i++) {
+	    if (strcmp(args[i].name, XmNframeChildType) == 0) {
+		FrameC_ChildType(new_w) = args[i].value;
+	    }
+	}
+#endif
+
     if (FrameC_ChildHSpacing(new_w) == XmINVALID_DIMENSION)
 	FrameC_ChildHSpacing(new_w) = Frame_MarginWidth(XtParent(new_w)) + 10;
+    if (FrameC_ChildType(new_w) == XmFRAME_WORKAREA_CHILD)
+    {
+	    Frame_WorkArea(XtParent(new_w))  = new_w;
+    }
+    if (FrameC_ChildType(new_w) == XmFRAME_TITLE_CHILD)
+    {
+	    Frame_TitleArea(XtParent(new_w))  = new_w;
+    }
 }
 
 static void constraint_destroy(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:constraint_destroy()", __FILE__));
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:constraint_destroy()\n", __FILE__));
 }
 
 static Boolean
@@ -780,6 +843,17 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
 {
     Boolean redisplay = False;
 
+#if XmVERSION > 1
+    int	i;
+
+    if (num_args)
+	for (i=0; i<*num_args; i++) {
+	    if (strcmp(args[i].name, XmNframeChildType) == 0) {
+		FrameC_ChildType(new_w) = args[i].value;
+	    }
+	}
+#endif
+
     if (FrameC_ChildType(current) != FrameC_ChildType(new_w) ||
 	FrameC_ChildHAlignment(current) != FrameC_ChildHAlignment(new_w) ||
 	FrameC_ChildHSpacing(current) != FrameC_ChildHSpacing(new_w) ||
@@ -864,13 +938,14 @@ _XmFrameComputeSize(Widget w, Widget instig, XtWidgetGeometry *desired,
 
 	    XtQueryGeometry(Frame_TitleArea(w), NULL, &geo);
 
-	    curh += geo.height + 2 * geo.border_width;
+	    curh += geo.height + 2 * geo.border_width - MGR_ShadowThickness(w);
 	    curw = geo.width + 2 * geo.border_width +
 		2 * FrameC_ChildHSpacing(Frame_TitleArea(w));
 	}
     }
 
-    if (Frame_WorkArea(w) && XtIsManaged(Frame_WorkArea(w)))
+    /* This seems a little wacky but look at frame/test6 */
+    if (Frame_WorkArea(w) /*&& XtIsManaged(Frame_WorkArea(w))*/)
     {
 	if (instig == Frame_WorkArea(w))
 	{
@@ -949,8 +1024,8 @@ _XmFrameConfigureChildren(Widget w, Widget instig, XtWidgetGeometry *desired,
 	{
 	case XmALIGNMENT_CENTER:
 	    title_y = 0;
-	    workarea_y = XtHeight(Frame_TitleArea(w)) + Frame_MarginHeight(w) +
-				MGR_ShadowThickness(w);
+	    workarea_y = XtHeight(Frame_TitleArea(w)) + Frame_MarginHeight(w) /* +
+				MGR_ShadowThickness(w) */;
 	    break;
 
 	case XmALIGNMENT_BASELINE_BOTTOM:
@@ -1091,7 +1166,7 @@ XmCreateFrame(Widget parent,
 	      Arg *arglist,
 	      Cardinal argCount)
 {
-    return XtCreateManagedWidget(name,
+    return XtCreateWidget(name,
 				 xmFrameWidgetClass,
 				 parent,
 				 arglist,
