@@ -1,6 +1,6 @@
 /**
  *
- * $Id: GeoUtils.c,v 1.22 1998/04/10 12:35:37 u27113 Exp $
+ * $Id: GeoUtils.c,v 1.35 1998/10/16 23:41:37 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: GeoUtils.c,v 1.22 1998/04/10 12:35:37 u27113 Exp $";
+static const char rcsid[] = "$Id: GeoUtils.c,v 1.35 1998/10/16 23:41:37 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -132,7 +132,7 @@ dump_matrix(XmGeoMatrix geo)
 }
 #endif
 
-void
+static void
 FitBoxesProportional(XmKidGeometry boxes, Dimension box_count,
 		     Dimension box_wd, short fill_wd)
 {
@@ -192,7 +192,7 @@ FitBoxesProportional(XmKidGeometry boxes, Dimension box_count,
     }
 }
 
-int
+static int
 sort_by_width(XmConst void *a, XmConst void *b)
 {
     XmKidGeometry geoA, geoB;
@@ -208,7 +208,7 @@ sort_by_width(XmConst void *a, XmConst void *b)
     return 1;
 }
 
-void
+static void
 FitBoxesAveraging(XmKidGeometry boxes, Dimension box_count,
 		  Dimension box_wd, Dimension fill_wd)
 {
@@ -264,18 +264,34 @@ FitBoxesAveraging(XmKidGeometry boxes, Dimension box_count,
     XtFree((char *)tbox);
 }
 
-void
-_XmGeoCalcFill(Dimension width, Dimension margin, Dimension box_count,
+
+
+static void
+_XmGeoCalcFill(Dimension width, Dimension margin, int box_count,
 	       Dimension space_end, Dimension space_between,
 	       Dimension *start, Dimension *tween)
 {
+    /*
     Dimension tmp, tmp2;
+    */
 
     DEBUGOUT(XdbDebug(__FILE__, NULL,
 		      "_XmGeoCalcFill: %d %d %d %d %d %d %d\n",
 		      width, margin, box_count, space_end, space_between,
 		      *start, *tween));
 
+#if 1
+	if (box_count - 1 > 0)
+	{
+	    *start = margin;
+	    *tween = (width - (2 * margin)) / (box_count - 1);
+	}
+	else
+	{
+		*start = width / 2;
+		*tween = 0;
+	}
+#else
     if (space_end == 0)
     {
 	if (margin != 1)
@@ -314,9 +330,12 @@ _XmGeoCalcFill(Dimension width, Dimension margin, Dimension box_count,
     }
 
     *tween = (tmp * width) / space_end;
+#endif
 }
 
-Dimension
+
+
+static Position
 _XmGeoLayoutSimple(XmKidGeometry boxes, XmGeoRowLayout layout,
 		   Position x, Position y,
 		   Dimension end, Dimension start, Dimension space_between)
@@ -365,164 +384,164 @@ _XmGeoLayoutSimple(XmKidGeometry boxes, XmGeoRowLayout layout,
     return y + layout->max_box_height;
 }
 
-void
-SegmentFill(XmKidGeometry boxes, int boxcount, XmGeoRowLayout layout,
+static void
+SegmentFill(XmKidGeometry boxes, int box_count, XmGeoRowLayout layout,
 	    Position x, Dimension width, Dimension margin,
-	    Position curx, Position end, Dimension start,
+	    Position curx, Dimension max_x, Dimension end_space,
 	    Dimension space_between)
 {
-    XmKidGeometry kend, kbox;
-    Widget tkid;
-    Dimension box_wd;
-    short fill_wd, adjust;
+    XmKidGeometry end_box = boxes + box_count;
+    Widget end_kid;
+    int fill_wd;
+    int space_width, box_width;
 
     DEBUGOUT(XdbDebug(__FILE__, NULL,
-		      "SEGMENT FILL: %d %d %d %d %d %d %d %d\n",
-		      boxcount, x, width, margin, curx,
-		      end, start, space_between));
-
-    kend = boxes + boxcount;
-    tkid = kend->kid;
-    kend->kid = NULL;
-
-    for (kbox = boxes, box_wd = 0; kbox->kid; kbox++)
+ 		      "SEGMENT FILL: %d %d %d %d %d %d %d %d\n",
+ 		      box_count, x, width, margin, curx,
+ 		      max_x, end_space, space_between));
+    
+    space_width = 2 * end_space + space_between * (box_count - 1);
+    box_width = curx - space_width + end_space;
+    if (box_width <= 0)
     {
-	box_wd += kbox->box.border_width * 2 + kbox->box.width;
+	box_width = 1;
     }
 
-    fill_wd = (end - curx) + (curx - box_wd);
-    adjust = fill_wd - width;
-
+    end_kid = end_box->kid;
+    end_box->kid = NULL;
     if (layout->fill_mode == XmGEO_CENTER)
     {
-	_XmGeoCalcFill(fill_wd, margin, boxcount,
-		       layout->space_end, space_between,
-		       &start, &space_between);
+	if (width - (space_width + box_width) > 0)
+	{
+	    fill_wd = width - box_width;
+	}
+	else
+	{
+	    fill_wd = margin * 2;
+	}
+	
+	_XmGeoCalcFill(fill_wd, margin, box_count, end_space,
+		       space_between, &end_space, &space_between);
     }
-    else
+    else if (layout->fill_mode == XmGEO_EXPAND)
     {
-	FitBoxesProportional(boxes, boxcount, box_wd, adjust);
+	FitBoxesProportional(boxes, box_count, box_width, curx - end_space);
     }
-
-    DEBUGOUT(XdbDebug(__FILE__, NULL,
-		      "SEGMENT FILL: %d %d %d %d\n",
-		      x, end, start, space_between));
-
-    _XmGeoLayoutSimple(boxes, layout,
-		       x, boxes->box.y, end, start, space_between);
-
-    kend->kid = tkid;
+    fill_wd = x + end_space;
+    while (boxes->kid != NULL)
+    {
+	boxes->box.x = fill_wd;
+	fill_wd += 
+	    boxes->box.width + 2 * boxes->box.border_width + space_between;
+	boxes++;
+    }
+    end_box->kid = end_kid;
 }
 
-Dimension
+
+/*
+ *  FUNCTION: _XmGeoLayoutWrap
+ *  RETURNS: The position below the last used row
+ *  DESCRIPTION:
+ *  This function arranges one (locical) row to fit inside width and margin,
+ *  and possibly breaking the row into 2 or more rows.
+ *  CALLED FROM _XmGeoArrangeList
+*/
+
+static Position
 _XmGeoLayoutWrap(XmKidGeometry boxes, XmGeoRowLayout layout,
 		 Position x, Position y,
 		 Dimension start, Dimension space_between, Dimension end,
 		 Dimension width, Dimension margin)
 {
-    Dimension curx, hincr, boxcount;
+    Position left_edge, right_edge;
+    int boxcount;
+    Dimension kid_width, kid_height;
     XmKidGeometry curbox;
-    Position e;			/* Replacement for "end" which is unsigned */
 
     DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmGeoLayoutWrap\n"));
 
-    e = end + margin;
-
-    curx = x + start;
-
-    curbox = boxes;
-
+    left_edge = x + start;
     boxcount = 0;
-
-    while (curbox->kid)
+    curbox = boxes;
+    while(curbox->kid != NULL)
     {
-	hincr = curbox->box.width + curbox->box.border_width * 2;
-
-	if (curx + hincr > (e - margin) && boxcount)
+	kid_width = curbox->box.width + 2 * curbox->box.border_width;
+	kid_height = curbox->box.height + 2 * curbox->box.border_width;
+	right_edge = left_edge + kid_width;
+	if (right_edge > end)
 	{
-	    SegmentFill(boxes, boxcount, layout,
-			x, width, margin, curx, e, start, space_between);
-
-	    boxes = curbox;
-
-	    y += layout->max_box_height;
-
-	    hincr = curbox->box.width + curbox->box.border_width * 2;
-
-	    curx = x + start;
-
-	    boxcount = 0;
-	}
-
-	if (curx + hincr > e - margin)
-	{
-	    if (curx + hincr - e < curbox->box.width &&
-		curx + hincr - e != 0)
+	    if (boxcount != 0)
 	    {
-		curbox->box.width -= curx + hincr - e;
+		/* move the kid one row down and finish this one */
+		
+		SegmentFill(boxes, boxcount, layout, x, width, margin,
+			    left_edge - space_between, end, start,
+			    space_between);
+		
+		boxcount = 0;
+		left_edge = x + start;
+		y += layout->max_box_height;
+		boxes = curbox;
+		right_edge = left_edge + kid_width;
+	    }
+	}
+	if (right_edge > end)
+	{
+	    /* If the kid is alone on the line can't i move it to the 
+	     * next row, but i most squeeze it to fit
+	     */
+	    int missing_space = right_edge + start - end - margin; 
+	    
+	    if (missing_space < boxes->box.width)
+	    {
+		if (missing_space > 0)
+		{
+		    curbox->box.width -= missing_space;
+		}
 	    }
 	    else
 	    {
-		curbox->box.width = boxcount;
+		curbox->box.width = 1;
 	    }
-
-	    y += layout->max_box_height;
-
-	    curbox++;
-
-	    continue;
+	    kid_width = curbox->box.width + 2 * curbox->box.border_width;
+	    right_edge = left_edge + kid_width;
 	}
-
-	curx += hincr;
-
-	curbox->box.x = curx;
-	curbox->box.height += 2 * curbox->box.border_width;
-
-	if (curbox->box.height == layout->max_box_height)
+	curbox->box.x = left_edge;
+	curbox->box.y = y;
+	if (layout->max_box_height != kid_height)
 	{
-	    boxes->box.y = y;
+	    curbox->box.y += layout->max_box_height - kid_height / 2;
 	}
-	else
-	{
-	    boxes->box.y += (layout->max_box_height - curbox->box.height) / 2;
-	}
-
-	curx += space_between;
-
-	curbox++;
-
+	left_edge = space_between + right_edge;
 	boxcount++;
+	curbox++;
     }
-
-    if (boxcount)
-    {
-	SegmentFill(boxes, boxcount, layout,
-		    x, width, margin, curx, e, start, space_between);
-
-	y += layout->max_box_height;
-    }
-
+    SegmentFill(boxes, boxcount, layout, x, width, start,
+		left_edge - space_between, end, start,
+		space_between);
     if (layout->sticky_end)
     {
 	curbox--;
-
-	curx = boxes->box.border_width * 2 + boxes->box.width;
-
-	e -= curx;
-
-	if (e < curbox->box.x)
+	kid_width = curbox->box.width + 2 * curbox->box.border_width;
+	if (curbox->box.x + kid_width < end)
 	{
-	    curbox->box.x = e;
+	    curbox->box.x  = end - kid_width;
 	}
     }
-
-    return y;
+    return y + layout->max_box_height;
 }
 
+
 /*
- * called from arrange boxes
- */
-Position
+ *  FUNCTION: _XmGeoArrangeList
+ *  RETURNS: The position below the last used row
+ *  DESCRIPTION:
+ *  This function arranges one (locical) row to fit inside width and margin,
+ *  and possibly breaking the row into 2 or more rows.
+ *  CALLED FROM _XmGeoArrangeBoxes
+*/
+static Position
 _XmGeoArrangeList(XmKidGeometry boxes, XmGeoRowLayout layout,
 		  Position x, Position y,
 		  Dimension width, Dimension margin)
@@ -538,9 +557,9 @@ _XmGeoArrangeList(XmKidGeometry boxes, XmGeoRowLayout layout,
     fill_wd = layout->boxes_width + layout->fill_width + 2 * margin;
     adjust = fill_wd - width;
 
-    if (layout->space_above > margin)
+    if (layout->space_end > margin)
     {
-	start = layout->space_above;
+	start = layout->space_end;
     }
     else
     {
@@ -596,7 +615,7 @@ _XmGeoArrangeList(XmKidGeometry boxes, XmGeoRowLayout layout,
 /*
  * also called from arrange boxes
  */
-Dimension
+static Dimension
 _XmGeoStretchVertical(XmGeoMatrix geoSpec, Dimension height, Dimension maxh)
 {
     register XmGeoRowLayout layout;
@@ -872,7 +891,7 @@ _XmGeoArrangeBoxes(XmGeoMatrix geoSpec,
 
     y -= oy;
 
-    if (*pH != 0 && *pH != y)
+    if (*pH != 0 && *pH > y)
     {
 	if (geoSpec->stretch_boxes)
 	{
@@ -926,6 +945,7 @@ _XmGeoBoxesSameWidth(XmKidGeometry rowPtr,
 	    boxes++;
 	}
     }
+    wd += width;
 
     DEBUGOUT(XdbDebug0(__FILE__, NULL, " %i", wd));
     boxes = rowPtr;
@@ -970,6 +990,7 @@ _XmGeoBoxesSameHeight(XmKidGeometry rowPtr,
 	    boxes++;
 	}
     }
+    ht += height;
 
     DEBUGOUT(XdbDebug0(__FILE__, NULL, " %i", ht));
     boxes = rowPtr;
@@ -1588,6 +1609,7 @@ _XmHandleQueryGeometry(Widget wid,
 		       unsigned char policy,
 		       XmGeoCreateProc createMatrix)
 {
+#if 0
     Dimension wd, ht;
     XmGeoMatrix cache;
 
@@ -1647,6 +1669,51 @@ _XmHandleQueryGeometry(Widget wid,
     }
 
     return XtGeometryYes;
+#else
+    Dimension wd, ht;
+    XmGeoMatrix cache;
+
+    DEBUGOUT(XdbDebug(__FILE__, wid, "_XmHandleQueryGeometry - intended %s\n",
+    		XdbWidgetGeometry2String(intended)));
+
+    if (policy == XmRESIZE_NONE)
+    {
+	desired->width = XtWidth(wid);
+	desired->height = XtHeight(wid);
+    }
+    else
+    {
+	cache = createMatrix(wid, NULL, NULL);
+
+	_XmGeoMatrixGet(cache, XmGET_PREFERRED_SIZE);
+
+	wd = XtIsRealized(wid) ? 0 : XtWidth(wid);
+	ht = XtIsRealized(wid) ? 0 : XtHeight(wid);
+
+	_XmGeoArrangeBoxes(cache, 0, 0, &wd, &ht);
+	_XmGeoMatrixFree(cache);
+
+	if (policy == XmRESIZE_GROW)
+	{
+	    if (wd < XtWidth(wid))
+	    {
+		wd = XtWidth(wid);
+	    }
+	    if (ht < XtHeight(wid))
+	    {
+		ht = XtHeight(wid);
+	    }
+	}
+
+	desired->request_mode = CWWidth | CWHeight;
+	desired->width  = wd == 0 ? 1 : wd;
+	desired->height = ht == 0 ? 1 : ht;
+    }
+
+    DEBUGOUT(XdbDebug(__FILE__, wid, "_XmHandleQueryGeometry - desired %s\n",
+    		XdbWidgetGeometry2String(desired)));
+    return _XmGMReplyToQueryGeometry(wid, intended, desired);
+#endif
 }
 
 static XtGeometryResult
@@ -2249,6 +2316,7 @@ _XmGMReplyToQueryGeometry(Widget w,
 			  XtWidgetGeometry *request,
 			  XtWidgetGeometry *reply)
 {
+#if 0
 
     if (XdbInDebug(__FILE__, w))
     {
@@ -2313,7 +2381,7 @@ _XmGMReplyToQueryGeometry(Widget w,
 	return XtGeometryYes;
     }
 
-    if (Wants(CWHeight) && request->height != reply->height && !Wants(CWWidth))
+    if (Wants(CWHeight) && request->height == reply->height && !Wants(CWWidth))
     {
 	DEBUGOUT(XdbDebug(__FILE__, w,
 			  "_XmGMReplyToQueryGeometry (%d %d) => Yes\n",
@@ -2341,6 +2409,88 @@ _XmGMReplyToQueryGeometry(Widget w,
 		      XtWidth(w), XtHeight(w)));
 
     return XtGeometryAlmost;
+#else
+    XtGeometryResult result;
+
+    if (XmIsLabel(w))
+    {
+	/* rather not change */
+	reply->request_mode = CWWidth | CWHeight;
+    }
+    else if (XmIsList(w))
+    {
+	/* rather not change */
+	reply->request_mode = CWWidth | CWHeight;
+    }
+    else if (XmIsText(w))
+    {
+	/* don't care if I change */
+	reply->request_mode = 0;
+    }
+    else if (XmIsTextField(w))
+    {
+	/* don't care if I change */
+	reply->request_mode = 0;
+    }
+    else if (XmIsBulletinBoard(w))
+    {
+	/* rather not change */
+	reply->request_mode = CWWidth | CWHeight;
+    }
+    else if (XmIsForm(w))
+    {
+	/* don't care if I change */
+	reply->request_mode = 0;
+    }
+    else if (XmIsDrawingArea(w))
+    {
+	/* don't care if I change */
+	reply->request_mode = 0;
+    }
+    else if (XmIsFrame(w))
+    {
+	/* rather not change */
+	reply->request_mode = CWWidth | CWHeight;
+    }
+    else if (XmIsScrolledWindow(w))
+    {
+	/* don't care if I change */
+	reply->request_mode = 0;
+    }
+    else if (XmIsRowColumn(w))
+    {
+	/* rather not change */
+	reply->request_mode = CWWidth | CWHeight;
+    }
+    else
+    {
+	/* don't care if I change */
+	reply->request_mode = 0;
+    }
+    if (((request->request_mode & (CWWidth | CWHeight)) == (CWWidth | CWHeight)) && 
+		request->width == reply->width && 
+		request->height == reply->height)
+    {
+	result = XtGeometryYes;
+    }
+    else if ((reply->width == XtWidth(w) && reply->height == XtHeight(w)) &&
+             ((reply->request_mode & (CWWidth | CWHeight)) == (CWWidth | CWHeight)))
+    {
+	result = XtGeometryNo;
+    }
+    else
+    {
+	result = XtGeometryAlmost;
+    }
+    reply->request_mode = CWWidth | CWHeight;
+
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:_XmGMReplyToQueryGeometry(%d)\n\trequest (%s) reply (%s) result %s\n",
+		      __FILE__, __LINE__,
+		      XdbWidgetGeometry2String(request),
+		      XdbWidgetGeometry2String(reply),
+		      XdbGeometryResult2String(result)));
+    return result;
+#endif
 }
 #undef	Wants
 
@@ -2350,6 +2500,7 @@ _XmGMHandleQueryGeometry(Widget w,
 			 Dimension margin_width, Dimension margin_height,
 			 unsigned char resize_policy)
 {
+#if 0
     Dimension wantw, wanth;
 
     DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmGMHandleQueryGeometry\n"));
@@ -2369,7 +2520,7 @@ _XmGMHandleQueryGeometry(Widget w,
 	{
 	    if (resize_policy == XmRESIZE_GROW && XtWidth(w) > wantw)
 	    {
-		wanth = XtWidth(w);
+		wantw = XtWidth(w);
 	    }
 	    if (resize_policy == XmRESIZE_GROW && XtHeight(w) > wanth)
 	    {
@@ -2394,6 +2545,34 @@ _XmGMHandleQueryGeometry(Widget w,
     answer->height = wanth;
 
     return XtGeometryAlmost;
+#else
+    Dimension wantw, wanth;
+
+    DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmGMHandleQueryGeometry\n"));
+
+    if (resize_policy == XmRESIZE_NONE)
+    {
+	wantw = XtWidth(w);
+	wanth = XtHeight(w);
+    }
+    else
+    {
+	_XmGMCalcSize(w, margin_width, margin_height, &wantw, &wanth);
+
+	if (resize_policy == XmRESIZE_GROW && XtWidth(w) > wantw)
+	{
+	    wantw = XtWidth(w);
+	}
+	if (resize_policy == XmRESIZE_GROW && XtHeight(w) > wanth)
+	{
+	    wanth = XtHeight(w);
+	}
+    }
+    answer->width = wantw;
+    answer->height = wanth;
+
+    return _XmGMReplyToQueryGeometry(w, proposed, answer);
+#endif
 }
 
 void
