@@ -47,9 +47,6 @@
 #include <Xm/XmP.h>
 
 #include <stdio.h>
-#ifdef __EMX__
-typedef char *caddr_t;
-#endif
 
 #define _LesstifEditResPutBool _LesstifEditResPut8
 #define _LesstifEditResPutResourceType _LesstifEditResPut8
@@ -1112,10 +1109,12 @@ DumpChildren(w, stream, count)
     _LesstifEditResPutString8(stream, class);	/* Insert class */
 
     if (XtIsWidget(w))
+      {
 	if (XtIsRealized(w))
 	    window = XtWindow(w);
 	else
 	    window = EDITRES_IS_UNREALIZED;
+      }
     else
 	window = EDITRES_IS_OBJECT;
 
