@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/Label.c,v 1.63 1998/05/08 19:41:50 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/Label.c,v 1.80 1998/09/28 01:35:16 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/Label.c,v 1.63 1998/05/08 19:41:50 rwscott Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/Label.c,v 1.80 1998/09/28 01:35:16 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -593,6 +593,7 @@ initialize(Widget request, Widget new_w,
 					   XtName(new_w));
 
 	Lab_Label(new_w) = _XmStringCreate(xmstring);
+	XmStringFree(xmstring);
     }
 
     if (_XmStringIsXmString((XmString)Lab_Label(new_w)))
@@ -607,8 +608,11 @@ initialize(Widget request, Widget new_w,
     }
     else
     {
-	Lab_AcceleratorText(new_w) =
-	    _XmStringCreate(XmStringCreateLocalized(""));
+    XmString tmp;
+
+	tmp = XmStringCreateLocalized("");
+	Lab_AcceleratorText(new_w) = _XmStringCreate(tmp);
+	XmStringFree(tmp);
     }
 
     /* if the parent is a row column, set the menu_type to
@@ -719,10 +723,15 @@ MLM: WHY TRAVERSAL?
 static void
 destroy(Widget w)
 {
-    XtReleaseGC(w, Lab_NormalGC(w));
-    XtReleaseGC(w, Lab_InsensitiveGC(w));
-    _XmManagerUninstallAccelerator(XtParent(w), w);
     _XmManagerUninstallMnemonic(XtParent(w), w);
+    _XmManagerUninstallAccelerator(XtParent(w), w);
+    XtFree(Lab_Accelerator(w));
+    XtFree(Lab_MnemonicCharset(w));
+    XtReleaseGC(w, Lab_InsensitiveGC(w));
+    XtReleaseGC(w, Lab_NormalGC(w));
+    _XmStringFree(Lab_AcceleratorText(w));
+    _XmStringFree(Lab_Label(w));
+    XmFontListFree(Lab_Font(w));
 }
 
 
@@ -764,7 +773,19 @@ set_values(Widget old, Widget request, Widget new_w,
 {
     Boolean refresh_needed = False, relayout_needed = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "set_values()\n"));
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "%s(%d):set_values: %i args\n"
+		      "\t    old X %5i Y %5i W %5i H %5i\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__, __LINE__, *num_args,
+		      XtX(old), XtY(old),
+		      XtWidth(old), XtHeight(old),
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     if (XtSensitive(old) != XtSensitive(new_w))
     {
@@ -820,8 +841,16 @@ set_values(Widget old, Widget request, Widget new_w,
 
     if (Lab_Label(new_w) == NULL)
     {
-	Lab_Label(new_w) =
-	    _XmStringCreate(XmStringCreateSimple(XtName(new_w)));
+    XmString tmp;
+
+	tmp = XmStringCreateSimple(XtName(new_w));
+	Lab_Label(new_w) = _XmStringCreate(tmp);
+	XmStringFree(tmp);
+
+	if (Lab_Label(old))
+	{
+	    _XmStringFree(Lab_Label(old));
+	}
 
 	if (Lab_RecomputeSize(new_w))
 	{
@@ -1053,19 +1082,43 @@ expose(Widget w, XEvent *event, Region region)
     /* Set a clip rectangle for the GC - ensure we don't overwrite shadows */
     /* the rectangle used to include MarginWidth and MarginHeight.  Leaving
      * those two out makes Xinvest and Xmcd look better */
-    cliprect.x = Prim_HighlightThickness(w) + Prim_ShadowThickness(w);
-    cliprect.y = Prim_HighlightThickness(w) + Prim_ShadowThickness(w);
-    cliprect.width = XtWidth(w) -
-	2 * (Prim_ShadowThickness(w) +
-	     Prim_HighlightThickness(w));
-    cliprect.height = XtHeight(w) -
-	2 * (Prim_ShadowThickness(w) +
-	     Prim_HighlightThickness(w));
+    /*
+     * However, not accounting for the margin screws up the default button
+     * rendering; PushButton will fiddle with the margins to keep the label
+     * from drawing over the extra shadow.  Basically we want to make the
+     * label draw itself as though it were smaller than it is but we want
+     * to keep geometry management doing the "right thing" so we can't
+     * just fiddle with XmNwidth and XmNheight.  The problem will only
+     * show up if a PushButton is forced to be smaller than it wants to
+     * be _and_ there is a defaultButton.  Inspection has revealed that
+     * the (Top|Bottom|Left|Right)Margin's are the ones we want (yes,
+     * I did count the pixels).
+     *
+     * See similar things in LabelG.c.
+     */
+    cliprect.x = Prim_HighlightThickness(w)
+               + Prim_ShadowThickness(w)
+               + Lab_MarginLeft(w);
+    cliprect.y = Prim_HighlightThickness(w)
+               + Prim_ShadowThickness(w)
+               + Lab_MarginTop(w);
+    cliprect.width  = XtWidth(w)
+                    - 2 * (Prim_ShadowThickness(w) + Prim_HighlightThickness(w))
+                    - Lab_MarginLeft(w) - Lab_MarginRight(w)
+                    + Lab_AccTextRect(w).width + Lab_AccTextRect(w).x;
+    cliprect.height = XtHeight(w)
+                    - 2 * (Prim_ShadowThickness(w) + Prim_HighlightThickness(w))
+                    - Lab_MarginTop(w) - Lab_MarginBottom(w)
+                    + Lab_AccTextRect(w).height + Lab_AccTextRect(w).y;
 
     DEBUGOUT(XdbDebug(__FILE__, w,
 		      "Expose: cliprect %d %d %dx%d\n",
 		      cliprect.x, cliprect.y,
 		      cliprect.width, cliprect.height));
+    DEBUGOUT(XdbDebug(__FILE__, w,
+                     "Expose: acctextrect %d %d %dx%d\n",
+                     Lab_AccTextRect(w).x, Lab_AccTextRect(w).y,
+                     Lab_AccTextRect(w).width, Lab_AccTextRect(w).height));
 
     XSetClipRectangles(XtDisplay(w), myGC, 0, 0, &cliprect, 1, Unsorted);
 
@@ -1260,6 +1313,7 @@ resize(Widget w)
 static XtGeometryResult
 query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 {
+#if 0
     XtWidgetGeometry a;
     Dimension wd, ht;
 
@@ -1353,6 +1407,51 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
     {
 	return XtGeometryAlmost;
     }
+#else
+    Dimension wd = XtWidth(w);
+    Dimension ht = XtHeight(w);
+    XtWidgetGeometry pc;
+
+	pc = *proposed;
+	/* rws 27 Sep 1998
+	   label/test8 should prove whether this IsRealized should be here
+	   or not, and it should not.
+	   label/test9 should prove whether Lab_RecomputeSize should be here
+	   or not,
+	 */
+	if (Lab_RecomputeSize(w))
+	{
+	    _XmCalcLabelDimensions(w);
+	    answer->width = XtWidth(w);
+	    answer->height = XtHeight(w);
+	    XtWidth(w) = wd;
+	    XtHeight(w) = ht;
+	}
+	else
+	{
+	    answer->width = XtWidth(w);
+	    answer->height = XtHeight(w);
+	}
+
+#if 0
+	if (((pc.request_mode & (CWWidth | CWHeight)) == (CWWidth | CWHeight)) && 
+		pc.width == answer->width && 
+		pc.height == answer->height)
+	{
+	    return XtGeometryYes;
+	}
+	else if (answer->width == XtWidth(w) && answer->height == XtHeight(w))
+	{
+	    return XtGeometryNo;
+	}
+	else
+	{
+	    return XtGeometryAlmost;
+	}
+#else
+	return _XmGMReplyToQueryGeometry(w, &pc, answer);
+#endif
+#endif
 }
 
 static Boolean
@@ -1415,7 +1514,10 @@ ProcessDrag(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
     dc = XmDragStart(w, event, args, n);
 
-    XtAddCallback(dc, XmNdragDropFinishCallback, drag_drop_finish, NULL);
+    if (dc)
+    {
+	XtAddCallback(dc, XmNdragDropFinishCallback, drag_drop_finish, NULL);
+    }
 }
 
 static void
@@ -1465,7 +1567,7 @@ _XmLabelGetPixmapSize(Widget w, Pixmap Pix,
 
     if (Pix == XmUNSPECIFIED_PIXMAP || Pix == None)
     {
-	*width = *height = 10;
+	*width = *height = 0;
 	return;
     }
 
@@ -1508,7 +1610,7 @@ _XmLabelShowsMnemonic(Widget w)
      *
      * Xmcd-2.2 bug #1. Danny 29/8/1997
      */
-#ifdef	notdef
+#if 0
     if (!XtIsSubclass(XtParent(w), xmRowColumnWidgetClass))
     {
 	return False;
