@@ -1,6 +1,6 @@
 /**
  *
- * $Id: LabelG.c,v 1.54 1998/04/16 23:32:54 rwscott Exp $
+ * $Id: LabelG.c,v 1.67 1998/09/28 01:35:18 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: LabelG.c,v 1.54 1998/04/16 23:32:54 rwscott Exp $";
+static const char rcsid[] = "$Id: LabelG.c,v 1.67 1998/09/28 01:35:18 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -909,7 +909,11 @@ _XmProcessDrag(Widget w, XEvent *event,
 	     _XmGetTextualDragIcon(XtParent(w))); n++;
 
     dc = XmDragStart(w, event, args, n);
-    XtAddCallback(dc, XmNdragDropFinishCallback, drag_drop_finish, NULL);
+
+    if (dc)
+    {
+	XtAddCallback(dc, XmNdragDropFinishCallback, drag_drop_finish, NULL);
+    }
 }
 
 /******************************* GADGET PART *********************************/
@@ -1108,8 +1112,7 @@ initialize(Widget request, Widget new_w,
     else
     {
 	xmstring = XmStringCreateLocalized("");
-	LabG_AcceleratorText(new_w) =
-	    _XmStringCreate(xmstring);
+	LabG_AcceleratorText(new_w) = _XmStringCreate(xmstring);
 	XmStringFree(xmstring);
     }
 
@@ -1219,10 +1222,12 @@ MLM: WHY TRAVERSAL?
 static void
 destroy(Widget w)
 {
-    XtReleaseGC(w, LabG_NormalGC(w));
-    XtReleaseGC(w, LabG_InsensitiveGC(w));
+    XmFontListFree(LabG_Font(w));
     _XmStringFree(LabG_Label(w));
     _XmStringFree(LabG_AcceleratorText(w));
+    XtReleaseGC(w, LabG_NormalGC(w));
+    XtReleaseGC(w, LabG_InsensitiveGC(w));
+    XtFree(LabG_MnemonicCharset(w));
     _XmManagerUninstallAccelerator(XtParent(w), w);
     _XmManagerUninstallMnemonic(XtParent(w), w);
 
@@ -1780,12 +1785,34 @@ expose(Widget w,
     /* Set a clip rectangle for the GC - ensure we don't overwrite shadows */
     /* the rectangle used to include MarginWidth and MarginHeight.  Leaving
      * those two out makes Xinvest and Xmcd look better */
-    cliprect.x = XtX(w) + G_HighlightThickness(w) + G_ShadowThickness(w);
-    cliprect.y = XtY(w) + G_HighlightThickness(w) + G_ShadowThickness(w);
-    cliprect.width = XtWidth(w) -
-	2 * (G_ShadowThickness(w) + G_HighlightThickness(w));
-    cliprect.height = XtHeight(w) -
-	2 * (G_ShadowThickness(w) + G_HighlightThickness(w));
+    /*
+     * See Label.c:expose to learn about the margins.
+     */
+    cliprect.x = XtX(w)
+               + G_HighlightThickness(w)
+               + G_ShadowThickness(w)
+               + LabG_MarginLeft(w);
+    cliprect.y = XtY(w)
+               + G_HighlightThickness(w)
+               + G_ShadowThickness(w)
+               + LabG_MarginTop(w);
+    cliprect.width  = XtWidth(w) 
+                   - 2 * (G_ShadowThickness(w) + G_HighlightThickness(w))
+                   - LabG_MarginLeft(w) - LabG_MarginRight(w)
+                   + LabG_AccTextRect(w).x + LabG_AccTextRect(w).width;
+    cliprect.height = XtHeight(w)
+                    - 2 * (G_ShadowThickness(w) + G_HighlightThickness(w))
+                    - LabG_MarginTop(w) - LabG_MarginBottom(w)
+                   + LabG_AccTextRect(w).y + LabG_AccTextRect(w).height;
+
+    DEBUGOUT(XdbDebug(__FILE__, w,
+             "Expose: cliprect %d %d %dx%d\n",
+            cliprect.x, cliprect.y,
+             cliprect.width, cliprect.height));
+    DEBUGOUT(XdbDebug(__FILE__, w,
+             "Expose: acctextrect %d %d %dx%d\n",
+            LabG_AccTextRect(w).x, LabG_AccTextRect(w).y,
+             LabG_AccTextRect(w).width, LabG_AccTextRect(w).height));
 
     XSetClipRectangles(XtDisplay(w), myGC, 0, 0, &cliprect, 1, Unsorted);
 
@@ -1893,6 +1920,7 @@ expose(Widget w,
 static XtGeometryResult
 query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 {
+#if 0
     XtWidgetGeometry a;		/* Standin for answer if NULL parameter */
     Dimension wd, ht;
 
@@ -1984,6 +2012,51 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
     {
 	return XtGeometryAlmost;
     }
+#else
+    Dimension wd = XtWidth(w);
+    Dimension ht = XtHeight(w);
+    XtWidgetGeometry pc;
+
+	pc = *proposed;
+	/* rws 27 Sep 1998
+	   label/test8 should prove whether this IsRealized should be here
+	   or not, and it should not.
+	   label/test9 should prove whether Lab_RecomputeSize should be here
+	   or not,
+	 */
+	if (LabG_RecomputeSize(w))
+	{
+	    _XmCalcLabelGDimensions(w);
+	    answer->width = XtWidth(w);
+	    answer->height = XtHeight(w);
+	    XtWidth(w) = wd;
+	    XtHeight(w) = ht;
+	}
+	else
+	{
+	    answer->width = XtWidth(w);
+	    answer->height = XtHeight(w);
+	}
+
+#if 0
+	if (((pc.request_mode & (CWWidth | CWHeight)) == (CWWidth | CWHeight)) && 
+		pc.width == answer->width && 
+		pc.height == answer->height)
+	{
+	    return XtGeometryYes;
+	}
+	else if (answer->width == XtWidth(w) && answer->height == XtHeight(w))
+	{
+	    return XtGeometryNo;
+	}
+	else
+	{
+	    return XtGeometryAlmost;
+	}
+#else
+	return _XmGMReplyToQueryGeometry(w, &pc, answer);
+#endif
+#endif
 }
 
 static Cardinal
