@@ -1,6 +1,6 @@
 /**
  *
- * $Id: MainW.c,v 1.28 1998/03/24 23:01:04 rwscott Exp $
+ * $Id: MainW.c,v 1.43 1998/10/23 20:57:12 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: MainW.c,v 1.28 1998/03/24 23:01:04 rwscott Exp $";
+static const char rcsid[] = "$Id: MainW.c,v 1.43 1998/10/23 20:57:12 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -40,6 +40,8 @@ static char rcsid[] = "$Id: MainW.c,v 1.28 1998/03/24 23:01:04 rwscott Exp $";
 
 #include <XmI/DebugUtil.h>
 
+#define	VALID(w)	(w != NULL && XtIsManaged(w))
+
 /* Forward Declarations */
 
 #if 0
@@ -90,6 +92,8 @@ void _XmRepositionScrolledWindow(Widget w,
 
 void _XmFixupScrollBars(Widget w, Dimension ww, Dimension wh);
 
+static void ReparentChild(Widget w, Widget child);
+
 /*
  * Resources for the MainWindow class
  */
@@ -308,9 +312,18 @@ initialize(Widget request, Widget new_w,
 								*num_args);
     if (MW_ShowSep(new_w))
     {
-	XtManageChild((Widget)MW_Sep1(new_w));
-	XtManageChild((Widget)MW_Sep2(new_w));
-	XtManageChild((Widget)MW_Sep3(new_w));
+	if (VALID(MW_MenuBar(new_w)))
+	{
+	    XtManageChild((Widget)MW_Sep1(new_w));
+	}
+	if (VALID(MW_CommandWindow(new_w)))
+	{
+	    XtManageChild((Widget)MW_Sep2(new_w));
+	}
+	if(VALID(MW_MessageWindow(new_w)))
+	{
+	    XtManageChild((Widget)MW_Sep3(new_w));
+	}
     }
 }
 
@@ -321,6 +334,21 @@ set_values(Widget old, Widget request, Widget new_w,
     Boolean refresh = False;
 #define NE(x)	(x(old) != x(new_w))
 
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "%s:set_values(%d) - %i args\n"
+		      "\t    old X %5i Y %5i W %5i H %5i\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__, __LINE__,
+		      *num_args,
+		      XtX(old), XtY(old),
+		      XtWidth(old), XtHeight(old),
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+
     if (NE(MW_CommandLoc))
     {
 	refresh = True;
@@ -329,7 +357,7 @@ set_values(Widget old, Widget request, Widget new_w,
     if (NE(SW_WorkWindow))
     {
         DEBUGOUT(XdbDebug(__FILE__, new_w,
-			  "SetValues: Changed the work window!\n"));
+			  "SetValues: Changed the work window !\n"));
 
 	if (MW_CommandWindow(new_w) == SW_WorkWindow(new_w))
 	{
@@ -350,7 +378,13 @@ set_values(Widget old, Widget request, Widget new_w,
     if (NE(MW_CommandWindow))
     {
         DEBUGOUT(XdbDebug(__FILE__, new_w,
-			  "SetValues: Changed the work window!\n"));
+			  "SetValues: Changed the command window !\n"));
+
+	if (SW_VisualPolicy(new_w) == XmCONSTANT &&
+	    XtParent(MW_CommandWindow(new_w)) == (Widget)SW_ClipWindow(new_w))
+	{
+	    ReparentChild(new_w, MW_CommandWindow(new_w));
+	}
 
 	if (SW_WorkWindow(new_w) == MW_CommandWindow(new_w))
 	{
@@ -371,7 +405,13 @@ set_values(Widget old, Widget request, Widget new_w,
     if (NE(MW_MessageWindow))
     {
         DEBUGOUT(XdbDebug(__FILE__, new_w,
-			  "SetValues: Changed the work window!\n"));
+			  "SetValues: Changed the message window !\n"));
+
+	if (SW_VisualPolicy(new_w) == XmCONSTANT &&
+	    XtParent(MW_MessageWindow(new_w)) == (Widget)SW_ClipWindow(new_w))
+	{
+	    ReparentChild(new_w, MW_MessageWindow(new_w));
+	}
 
 	if (MW_CommandWindow(new_w) == MW_MessageWindow(new_w))
 	{
@@ -392,7 +432,13 @@ set_values(Widget old, Widget request, Widget new_w,
     if (NE(MW_MenuBar))
     {
         DEBUGOUT(XdbDebug(__FILE__, new_w,
-			  "SetValues: Changed the work window!\n"));
+			  "SetValues: Changed the menu bar !\n"));
+
+	if (SW_VisualPolicy(new_w) == XmCONSTANT &&
+	    XtParent(MW_MenuBar(new_w)) == (Widget)SW_ClipWindow(new_w))
+	{
+	    ReparentChild(new_w, MW_MenuBar(new_w));
+	}
 
 	if (MW_CommandWindow(new_w) == MW_MenuBar(new_w))
 	{
@@ -410,21 +456,38 @@ set_values(Widget old, Widget request, Widget new_w,
 	refresh = True;
     }
 
-    if (NE(MW_ShowSep))
+    if (MW_ShowSep(new_w))
     {
-	if (MW_ShowSep(new_w))
+	if (VALID(MW_MenuBar(new_w)))
 	{
 	    XtManageChild((Widget)MW_Sep1(new_w));
-	    XtManageChild((Widget)MW_Sep2(new_w));
-	    XtManageChild((Widget)MW_Sep3(new_w));
 	}
 	else
 	{
 	    XtUnmanageChild((Widget)MW_Sep1(new_w));
+	}
+	if (VALID(MW_CommandWindow(new_w)))
+	{
+	    XtManageChild((Widget)MW_Sep2(new_w));
+	}
+	else
+	{
 	    XtUnmanageChild((Widget)MW_Sep2(new_w));
+	}
+	if(VALID(MW_MessageWindow(new_w)))
+	{
+	    XtManageChild((Widget)MW_Sep3(new_w));
+	}
+	else
+	{
 	    XtUnmanageChild((Widget)MW_Sep3(new_w));
 	}
-
+    }
+    else if (NE(MW_ShowSep))
+    {
+	XtUnmanageChild((Widget)MW_Sep1(new_w));
+	XtUnmanageChild((Widget)MW_Sep2(new_w));
+	XtUnmanageChild((Widget)MW_Sep3(new_w));
 	refresh = True;
     }
 
@@ -440,7 +503,7 @@ set_values(Widget old, Widget request, Widget new_w,
 	refresh = True;
     }
 
-    if (refresh)
+    if (XtIsRealized(new_w) && refresh)
     {
 	XmMWValues vals;
 
@@ -457,7 +520,8 @@ resize(Widget w)
 {
     XmMWValues vals;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Resize (w %d h %d)\n",
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:resize(%d) (%dx%d)\n",
+		      __FILE__, __LINE__,
 		      XtWidth(w), XtHeight(w)));
 
     SW_FromResize(w) = True;
@@ -478,8 +542,11 @@ static void
 realize(Widget w, Mask *value_mask, XSetWindowAttributes *attributes)
 {
 /* Motif inherits this method */
+/* and we could also, if ScrolledW gets rid of the geo stuff in realize */
 
+    /*
     XmMWValues vals;
+    */
 
     DEBUGOUT(XdbDebug(__FILE__, w, "Realize ...\n"));
 
@@ -487,6 +554,8 @@ realize(Widget w, Mask *value_mask, XSetWindowAttributes *attributes)
     (*superclass->core_class.realize) (w, value_mask, attributes);
 #undef superclass
 
+#if 0
+    /* mainw/test13 */
     if (XtWidth(w) != 0 && XtWidth(w) != 1)
     {
 	SW_GivenWidth(w) = XtWidth(w);
@@ -509,6 +578,7 @@ realize(Widget w, Mask *value_mask, XSetWindowAttributes *attributes)
 
     DEBUGOUT(XdbDebug(__FILE__, w, "Realize => size %d %d\n",
 		      XtWidth(w), XtHeight(w)));
+#endif
 }
 
 static XtGeometryResult
@@ -516,6 +586,7 @@ query_geometry(Widget w,
 	       XtWidgetGeometry *intended,
 	       XtWidgetGeometry *preferred)
 {
+#if 0
     XtWidgetGeometry wants;
     XmMWValues vals;
 
@@ -549,15 +620,141 @@ query_geometry(Widget w,
     }
 
     return XtGeometryYes;
+#else
+    XtWidgetGeometry wants;
+    XmMWValues vals;
+
+    wants = *intended;
+
+    _XmMainWindowPreferredSize(w, w, NULL, &vals);
+
+    if (preferred)
+    {
+	preferred->width = vals.MwW;
+	preferred->height = vals.MwH;
+    }
+
+    return _XmGMReplyToQueryGeometry(w, &wants, preferred);
+#endif
 }
 
+static void
+CreateManagedList(Widget w, Widget **managed, int *num_managed,
+		  int *num_allocated)
+{
+    if (XtIsComposite(w) && MGR_NumChildren(w) > 0)
+    {
+	int i;
+	for (i = 0; i < MGR_NumChildren(w); i++)
+	{
+	    CreateManagedList(MGR_Children(w)[i], managed, num_managed,
+			      num_allocated);
+	}
+    }
+    if (XtIsManaged(w))
+    {
+	if (*num_managed == *num_allocated)
+	{
+	/* Allocate more space */
+	*num_allocated += (*num_allocated / 2) + 2;
+	*managed =  
+	    (WidgetList) XtRealloc((XtPointer) *managed,
+				   (unsigned) (*num_allocated *
+					       sizeof(Widget)));
+	}
+	*managed[*num_managed++] = w;
+    }
+}
+
+static void
+ReparentChild(Widget w, Widget child)
+{
+
+    Widget old_parent = XtParent(child), *managed;
+    int i, j, num_managed, num_allocated;
+    Boolean is_realized;
+
+    if (old_parent == w)
+    {
+	return;
+    }
+    /* first make a list of children who needs to be managed again */
+    num_managed = 0;
+    num_allocated = 2;
+    managed = (Widget *) XtMalloc(num_allocated * sizeof(Widget));
+    CreateManagedList(child, &managed, & num_managed, &num_allocated);
+    for (i = 0; i < MGR_NumChildren(old_parent); i++)
+    {
+	if (MGR_Children(old_parent)[i] == child)
+	{
+	    break;
+	}
+    }
+    if (MGR_NumChildren(w) == MGR_NumSlots(w)) {
+	/* Allocate more space */
+	MGR_NumSlots(w) += (MGR_NumSlots(w) / 2) + 2;
+	MGR_Children(w) =  
+	    (WidgetList) XtRealloc((XtPointer) MGR_Children(w),
+				   (unsigned) (MGR_NumSlots(w) *
+					       sizeof(Widget)));
+    }
+    if ((is_realized = XtIsRealized(child)))
+    {
+	XtUnrealizeWidget(child);
+    } 
+    MGR_Children(w)[MGR_NumChildren(w)++] = child;
+    XtParent(child) = w;
+    if (is_realized)
+    {
+	XtRealizeWidget(child);
+    }
+    for (j = 0; j < num_managed; j++)
+    {
+	XtManageChild(managed[j]);
+    }
+    XtFree((char *) managed);
+ 
+    for (j = i + 1; j < MGR_NumChildren(old_parent); j++)
+    {
+	MGR_Children(old_parent)[j - 1] = MGR_Children(old_parent)[j];
+    }
+    MGR_NumChildren(old_parent)--;
+}
+
+
+
+
 static void
 change_managed(Widget w)
 {
     XmMWValues vals;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "ChangeManaged\n"));
+    /*
+     * It is possible that some of the MainWindows children is made children
+     * of ClipWindow by the ScrolledWindow. Steal them back.
+     */
+    if (SW_VisualPolicy(w) == XmCONSTANT &&
+	MGR_NumChildren(SW_ClipWindow(w)) > 1)
+    {
+	XmDrawingAreaWidget cw = SW_ClipWindow(w);
+	Widget mw_children[3];
+	int i, num_mw_children = 0;
 
+	for (i = 0 ; i < MGR_NumChildren(cw); i++)
+	{
+	    if (MGR_Children(cw)[i] == MW_CommandWindow(w) ||
+		MGR_Children(cw)[i] == MW_MenuBar(w) ||
+		MGR_Children(cw)[i] == MW_MessageWindow(w))
+	    {
+		mw_children[num_mw_children++] = MGR_Children(cw)[i];
+	    }
+	}
+	for (i = 0; i < num_mw_children; i++)
+	{
+	    ReparentChild(w, mw_children[i]);
+	}
+    }
     if (SW_HSB(w) && XtIsManaged(SW_HSB(w)))
     {
 	SW_HasHSB(w) = True;
@@ -576,8 +773,47 @@ change_managed(Widget w)
 	SW_HasVSB(w) = False;
     }
 
+    if (MW_ShowSep(w))
+    {
+	if (VALID(MW_MenuBar(w)))
+	{
+	    XtManageChild((Widget)MW_Sep1(w));
+	}
+	else
+	{
+	    XtUnmanageChild((Widget)MW_Sep1(w));
+	}
+	if (VALID(MW_CommandWindow(w)))
+	{
+	    XtManageChild((Widget)MW_Sep2(w));
+	}
+	else
+	{
+	    XtUnmanageChild((Widget)MW_Sep2(w));
+	}
+	if(VALID(MW_MessageWindow(w)))
+	{
+	    XtManageChild((Widget)MW_Sep3(w));
+	}
+	else
+	{
+	    XtUnmanageChild((Widget)MW_Sep3(w));
+	}
+    }
     _XmMainWindowPreferredSize(w, NULL, NULL, &vals);
 
+    if (!XtIsRealized(w))
+    {
+	/* mainw/test11 */
+	if (SW_GivenWidth(w) != 0)
+	{
+	    vals.MwW = SW_GivenWidth(w);
+	}
+	if (SW_GivenHeight(w) != 0)
+	{
+	    vals.MwH = SW_GivenHeight(w);
+	}
+    }
     _XmMainWindowGeomRequest(w, &vals);
 
     /* ordinarily, we'd look at the return code and reset w/h based on
@@ -588,7 +824,6 @@ change_managed(Widget w)
     _XmMainWindowConfigureChildren(w, NULL, NULL, &vals);
 }
 
-#define	VALID(w)	(w != NULL && XtIsManaged(w))
 
 /*
  * This is the new implementation of MainWindow Layout.
@@ -639,13 +874,8 @@ _XmMainWindowPreferredSize(Widget w, Widget child, XtWidgetGeometry *cg,
      */
 
     /* Talk */
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmMainWindowPreferredSize"));
-
-    if (child)
-    {
-	DEBUGOUT(XdbDebug0(__FILE__, w, " Child %s, geo %s\n",
-			   XtName(child), XdbWidgetGeometry2String(cg)));
-    }
+    DEBUGOUT(XdbDebug2(__FILE__, w, child, "_XmMainWindowPreferredSize request %s\n",
+    		XdbWidgetGeometry2String(cg)));
 
     /*
      * Find out how big we need to be.
@@ -659,9 +889,32 @@ _XmMainWindowPreferredSize(Widget w, Widget child, XtWidgetGeometry *cg,
     vals->mbw = vals->mbh = vals->mbx = vals->mby = 0;
     if (VALID(MW_MenuBar(w)))
     {
+    XtWidgetGeometry rcg, rcg_r;
+
 	vals->mbw = XtWidth(MW_MenuBar(w));
 	vals->mbh = XtHeight(MW_MenuBar(w));
 
+	if (SW_FromResize(w))
+	{
+	    rcg.request_mode = CWWidth;
+	    rcg.width = XtWidth(w);
+	}
+	else
+	{
+	    if (cg && (cg->request_mode & CWWidth))
+	    {
+		rcg.request_mode = CWWidth;
+		rcg.width = cg->width;
+	    }
+	    else
+	    {
+		rcg.request_mode = 0;
+	    }
+	}
+	XtQueryGeometry(MW_MenuBar(w), &rcg, &rcg_r);
+	vals->mbw = rcg_r.width;
+	vals->mbh = rcg_r.height;
+
 	if (MW_MenuBar(w) == child)
 	{
 	    if (Wants(CWWidth))
@@ -675,7 +928,7 @@ _XmMainWindowPreferredSize(Widget w, Widget child, XtWidgetGeometry *cg,
 	}
 
 	DEBUGOUT(XdbDebug2(__FILE__, w, MW_MenuBar(w),
-			   "_XmMainWindoPreferredSize: menu bar w %d h %d\n",
+			   "menu bar w %d h %d\n",
 			   vals->mbw, vals->mbh));
 
 	curh += vals->mbh;
@@ -727,9 +980,32 @@ _XmMainWindowPreferredSize(Widget w, Widget child, XtWidgetGeometry *cg,
     vals->mww = vals->mwh = vals->mwx = vals->mwy = 0;
     if (VALID(MW_MessageWindow(w)))
     {
+    XtWidgetGeometry rcg, rcg_r;
+
 	vals->mww = XtWidth(MW_MessageWindow(w));
 	vals->mwh = XtHeight(MW_MessageWindow(w));
 
+	if (SW_FromResize(w))
+	{
+	    rcg.request_mode = CWWidth;
+	    rcg.width = XtWidth(w);
+	}
+	else
+	{
+	    if (cg && (cg->request_mode & CWWidth))
+	    {
+		rcg.request_mode = CWWidth;
+		rcg.width = cg->width;
+	    }
+	    else
+	    {
+		rcg.request_mode = 0;
+	    }
+	}
+	XtQueryGeometry(MW_MessageWindow(w), &rcg, &rcg_r);
+	vals->mww = rcg_r.width;
+	vals->mwh = rcg_r.height;
+
 	if (MW_MessageWindow(w) == child)
 	{
 	    if (Wants(CWWidth))
@@ -743,9 +1019,8 @@ _XmMainWindowPreferredSize(Widget w, Widget child, XtWidgetGeometry *cg,
 	}
 
 	DEBUGOUT(XdbDebug2(__FILE__, w, MW_MessageWindow(w),
-			   "_XmMainWindowPreferredSize:"
-			   " message window %p : w %d h %d\n",
-			   MW_MessageWindow(w), vals->mww, vals->mwh));
+			   "message window w %d h %d\n",
+			   vals->mww, vals->mwh));
 
 	curh += vals->mwh;
 	if (curw < vals->mww)
@@ -1110,7 +1385,10 @@ _XmMainWindowGeomRequest(Widget w, XmMWValues * vals)
     /*
      * MLM: This breaks xmgr.  Don't do this!
      * Well, ok, don't use 100x100
+     * also check out mainw/test12
      */
+#if 0
+#if 0
     if (SW_GivenWidth(w) != 0)
     {
 	vals->MwW = SW_GivenWidth(w);
@@ -1120,6 +1398,20 @@ _XmMainWindowGeomRequest(Widget w, XmMWValues * vals)
     {
 	vals->MwH = SW_GivenHeight(w);
     }
+#else
+    if (!XtIsRealized(w))
+    {
+	if (SW_GivenWidth(w) != 0)
+	{
+	    vals->MwW = SW_GivenWidth(w);
+	}
+	if (SW_GivenHeight(w) != 0)
+	{
+	    vals->MwH = SW_GivenHeight(w);
+	}
+    }
+#endif
+#endif
 
     geo.width = vals->MwW;
     geo.height = vals->MwH;
@@ -1858,9 +2150,7 @@ XmMainWindowSetAreas(Widget widget,
 		     Widget vertical_scrollbar,
 		     Widget work_region)
 {
-    /* color computation */
-    XColor _widgetBackground;
-    XColor troughColor;
+    Pixel trough;
  
     DEBUGOUT(XdbDebug(__FILE__, widget, "XmMainWindowSetAreas ["));
 
@@ -1921,22 +2211,8 @@ XmMainWindowSetAreas(Widget widget,
 
     if (horizontal_scrollbar || vertical_scrollbar)
     {
-	_widgetBackground.pixel = XtBackground(widget);
- 
-	XQueryColor(XtDisplay(widget),
-		    CoreColormap(widget),
-		    &_widgetBackground);
- 
-	troughColor.blue = _widgetBackground.blue * .80;
-	troughColor.green = _widgetBackground.green * .80;
-	troughColor.red = _widgetBackground.red * .80;
- 
-	if (!XAllocColor(XtDisplay(widget),
-			 CoreColormap(widget),
-			 &troughColor))
-	{
-	    troughColor.pixel = _XmWhitePixelOfObject(widget);
-	}
+	XmGetColors(XtScreen(widget), CoreColormap(widget),
+		    XtBackground(widget), NULL, NULL, NULL, &trough);
     }
 
     if (horizontal_scrollbar)
@@ -1944,7 +2220,7 @@ XmMainWindowSetAreas(Widget widget,
 	XtVaSetValues(horizontal_scrollbar,
 		      XmNforeground, MGR_Foreground(widget),
 		      XmNbackground, XtBackground(widget),
-		      XmNtroughColor, troughColor.pixel,
+		      XmNtroughColor, trough,
 		      XmNtopShadowColor, MGR_TopShadowColor(widget),
 		      XmNtopShadowPixmap, MGR_TopShadowPixmap(widget),
 		      XmNbottomShadowColor, MGR_BottomShadowColor(widget),
@@ -1956,7 +2232,7 @@ XmMainWindowSetAreas(Widget widget,
 	XtVaSetValues(vertical_scrollbar,
 		      XmNforeground, MGR_Foreground(widget),
 		      XmNbackground, XtBackground(widget),
-		      XmNtroughColor, troughColor.pixel,
+		      XmNtroughColor, trough,
 		      XmNtopShadowColor, MGR_TopShadowColor(widget),
 		      XmNtopShadowPixmap, MGR_TopShadowPixmap(widget),
 		      XmNbottomShadowColor, MGR_BottomShadowColor(widget),
