@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Manager.c,v 1.49 1998/04/16 21:46:44 u27113 Exp $
+ * $Id: Manager.c,v 1.57 1998/10/18 22:01:00 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: Manager.c,v 1.49 1998/04/16 21:46:44 u27113 Exp $";
+static const char rcsid[] = "$Id: Manager.c,v 1.57 1998/10/18 22:01:00 rwscott Exp $";
 
 #include <LTconfig.h>
 
@@ -306,6 +306,17 @@ static XtResource resources[] =
 	sizeof(Widget), Offset(initial_focus),
 	XmRImmediate, (XtPointer)NULL
     }
+
+#if LESSTIF_BACK_COMPAT
+#else
+#if XmVERSION > 1
+    ,{
+	XmNpopupHandlerCallback, XmCCallback, XmRCallback,
+	sizeof(XtCallbackList), Offset(popup_handler_callback),
+	XtRPointer, (XtPointer)NULL
+    }
+#endif
+#endif
 };
 
 #define MOffset(field)	XtOffset(XmManagerWidget, manager.field)
@@ -794,6 +805,11 @@ static void
 realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
 {
     Boolean wz = False, hz = False;
+
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:realize(%d) - %dx%d\n",
+    	__FILE__, __LINE__,
+    	XtWidth(w), XtHeight(w)));
+
     /*
      * MLM 960524 - if you get an XError: widget has zero width or height.
      */
@@ -817,6 +833,8 @@ realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
     XtCreateWindow(w, (unsigned int)InputOutput,
 		   (Visual *)CopyFromParent, *value_mask, attributes);
 
+    /* rws 7 Oct 1998
+       drawingarea/test7 shows that this does not happen
     if (wz)
     {
 	XtWidth(w) = 0;
@@ -825,6 +843,7 @@ realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
     {
 	XtHeight(w) = 0;
     }
+    */
 }
 
 static Boolean
@@ -984,9 +1003,7 @@ constraint_initialize(Widget request,
 
 	    if (G_EventMask(new_w) & XmMOTION_EVENT
 		|| G_EventMask(new_w) & XmENTER_EVENT
-		|| G_EventMask(new_w) & XmLEAVE_EVENT
-		|| G_EventMask(new_w) & XmFOCUS_IN
-		|| G_EventMask(new_w) & XmFOCUS_OUT)
+		|| G_EventMask(new_w) & XmLEAVE_EVENT)
 	    {
 		DEBUGOUT(XdbDebug(__FILE__, manager, "adding event handler\n"));
 
@@ -1013,17 +1030,23 @@ _XmGadgetArm(Widget w,
 
     DEBUGOUT(XdbDebug(__FILE__, w, "Inside _XmGadgetArm()\n"));
 
-    if (_XmIsNavigable(g))
+    if (g)
     {
-	if (g)
-	{
-	    _XmDispatchGadgetInput(g,
-				   event,
-				   XmARM_EVENT);
-	}
-
+	XmProcessTraversal(g, XmTRAVERSE_CURRENT);
+	_XmDispatchGadgetInput(g,
+			       event,
+			       XmARM_EVENT);
+	
 	MGR_SelectedGadget(w) = (XmGadget)g;
     }
+    else
+    {
+	if (_XmIsNavigable(w)) 
+	{
+	    XmProcessTraversal(w, XmTRAVERSE_CURRENT);
+	}
+    }
+    MGR_EligibleForMultiButtonEvent(w) = NULL;
 }
 
 void
@@ -1041,6 +1064,7 @@ _XmGadgetActivate(Widget w,
 	_XmDispatchGadgetInput((Widget)g,
 			       event,
 			       XmACTIVATE_EVENT);
+	MGR_EligibleForMultiButtonEvent(w) = (XmGadget)g;
     }
     MGR_SelectedGadget(w) = NULL;
 }
@@ -1058,12 +1082,34 @@ _XmGadgetMultiArm(Widget w,
 
     if (g)
     {
-	_XmDispatchGadgetInput(g,
-			       event,
-			       XmMULTI_ARM_EVENT);
+	/* This only counts as a multiArm if we're arming the gadget we just
+	 * activated.  Otherwise it's just a regular arm.
+	 */
 
+	if (g == (Widget)MGR_EligibleForMultiButtonEvent(w))
+	{
+	    _XmDispatchGadgetInput(g,
+				   event,
+				   XmMULTI_ARM_EVENT);
+	}
+	else
+	{
+	    XmProcessTraversal(g, XmTRAVERSE_CURRENT);
+	    _XmDispatchGadgetInput(g,
+				   event,
+				   XmARM_EVENT);
+	    MGR_EligibleForMultiButtonEvent(w) = NULL;
+	}
 	MGR_SelectedGadget(w) = (XmGadget)g;
     }
+    else
+    {
+	if (_XmIsNavigable(w)) 
+	{
+	    XmProcessTraversal(w, XmTRAVERSE_CURRENT);
+	}
+	MGR_EligibleForMultiButtonEvent(w) = NULL;
+    }
 }
 
 void
@@ -1078,9 +1124,23 @@ _XmGadgetMultiActivate(Widget w,
 
     if (g)
     {
-	_XmDispatchGadgetInput((Widget)g,
-			       event,
-			       XmMULTI_ACTIVATE_EVENT);
+	/* This only counts as a multiActivate if we're activating a
+	 * multiArmed gadget.  Otherwise it's just a regular activate.
+	 */
+
+	if (g == (Widget)MGR_EligibleForMultiButtonEvent(w))
+	{
+	    _XmDispatchGadgetInput((Widget)g,
+				   event,
+				   XmMULTI_ACTIVATE_EVENT);
+	}
+	else
+	{
+	    _XmDispatchGadgetInput((Widget)g,
+				   event,
+				   XmACTIVATE_EVENT);
+	    MGR_EligibleForMultiButtonEvent(w) = (XmGadget)g;
+	}
     }
 
     MGR_SelectedGadget(w) = NULL;
@@ -1406,8 +1466,6 @@ _XmManagerEventHandler(Widget w,
 {
     XmGadget g;
 
-    unsigned char foc;
-
     if (CoreBeingDestroyed(w))
     {
 	return;
@@ -1443,9 +1501,12 @@ _XmManagerEventHandler(Widget w,
      * MLM
      *
      * 961114 -- Sometimes I am slow.  POINTER or InDragMode, stupid.  MLM
+     *
+     * Not so.  Enter and Leave events are quite independent of focus,
+     * and despite its name, MGR_HighlightedWidget is only used for tracking
+     * Enter/Leave.  So regardless of focus policy, set MGR_HighlightedWidget
+     * to track the gadget under the pointer.  JHG
      */
-    foc = _XmGetFocusPolicy(w);
-
     if (g == NULL)
     {
 	/* 
@@ -1463,8 +1524,7 @@ _XmManagerEventHandler(Widget w,
 				   event,
 				   XmLEAVE_EVENT);
 
-	    if (foc == XmPOINTER || (XmIsRowColumn(w) && _XmGetInDragMode(w)))
-		MGR_HighlightedWidget(w) = NULL;
+	    MGR_HighlightedWidget(w) = NULL;
 	}
     }
     else
@@ -1487,8 +1547,7 @@ _XmManagerEventHandler(Widget w,
 	    _XmDispatchGadgetInput((Widget)g,
 				   event,
 				   XmENTER_EVENT);
-	    if (foc == XmPOINTER || (XmIsRowColumn(w) && _XmGetInDragMode(w)))
-		MGR_HighlightedWidget(w) = (Widget)g;
+	    MGR_HighlightedWidget(w) = (Widget)g;
 	}
 	else
 	{
@@ -1510,11 +1569,7 @@ _XmManagerEventHandler(Widget w,
 				       event,
 				       XmENTER_EVENT);
 
-		if (foc == XmPOINTER ||
-		    (XmIsRowColumn(w) && _XmGetInDragMode(w)))
-		{
-		    MGR_HighlightedWidget(w) = (Widget)g;
-		}
+		MGR_HighlightedWidget(w) = (Widget)g;
 	    }
 	}
     }
@@ -1531,6 +1586,24 @@ _XmClearShadowType(Widget w,
 		   Dimension old_shadow_thickness,
 		   Dimension old_highlight_thickness)
 {
+    if (old_shadow_thickness > 0 && XtIsRealized(w))
+    {
+	if (XtWidth(w) >= old_width)
+	{
+	    XClearArea(XtDisplay(w), XtWindow(w),
+		       old_width - old_shadow_thickness - old_highlight_thickness,
+		       0, old_shadow_thickness,
+		       old_height - old_highlight_thickness, False);
+	}
+	if (XtHeight(w) >= old_height)
+	{
+	    XClearArea(XtDisplay(w), XtWindow(w),
+		       0,
+		       old_height - old_shadow_thickness - old_highlight_thickness,
+		       old_width - old_highlight_thickness,
+		       old_shadow_thickness, False);
+	}
+    }
 }
 
 /*
@@ -1567,6 +1640,7 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
 {
     int i;
     XtActionProc act = NULL;
+    Widget comp = (Widget)client_data;
 
     DEBUGOUT(XdbDebug(__FILE__, w,
 		      "Got Accelerator key: type %d code %04x "
@@ -1587,6 +1661,24 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
 		      event->xkey.keycode, event->xkey.state, client_data,
 		      XtName(client_data)));
 
+    /*
+     * Insensitive widgets shouldn't be activated by accelerators or
+     * mnemonics.  We could keep inserting and removing things from
+     * the MGR_KeyboardList as widgets change sensitivity but that's
+     * just sick.  Hence, we just filter out insensitive widgets.
+     * This test could go inside the following search loop but there's
+     * no point in searching when we know that we can't do anything with
+     * what we find.
+     *         --mu@trends.net, 1998.09.06
+     */
+    if(!XtIsSensitive(comp))
+    {
+       DEBUGOUT(XdbDebug(__FILE__, w,
+                         "_XmAcceleratorHandler: %s is insensitive, ignoring",
+                         XtName(comp)));
+       return;
+    }
+
     for (i = 0; i < MGR_NumKeyboardEntries(w); i++)
     {
 	if (MGR_KeyboardList(w)[i].eventType == event->type &&
@@ -1594,7 +1686,6 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
 	    MGR_KeyboardList(w)[i].modifiers == event->xkey.state &&
 	    MGR_KeyboardList(w)[i].component == client_data)
 	{
-	    Widget comp = (Widget)client_data;
 	    Cardinal num_params = 0;
 
 	    DEBUGOUT(XdbDebug(__FILE__, w,
@@ -1648,7 +1739,7 @@ AddKeyboardEntry(Widget m, Widget w, XmKeyboardData *item)
     int i;
 
     /* Register with the manager widget */
-    if (MGR_NumKeyboardEntries(m) == 0 ||
+    if (MGR_SizeKeyboardList(m) == 0 ||
 	(MGR_NumKeyboardEntries(m) == MGR_SizeKeyboardList(m)))
     {
 	i = MGR_SizeKeyboardList(m);
