@@ -1,6 +1,6 @@
 /**
  * 
- * $Id: MenuShell.c,v 1.67 1998/04/22 22:00:58 u27113 Exp $
+ * $Id: MenuShell.c,v 1.70 1998/10/16 14:43:24 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: MenuShell.c,v 1.67 1998/04/22 22:00:58 u27113 Exp $";
+static const char rcsid[] = "$Id: MenuShell.c,v 1.70 1998/10/16 14:43:24 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -227,6 +227,28 @@ static void
 initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "%s:initialize(%d) - %i args\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__, __LINE__,
+		      *num_args,
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+
+    if (MS_DefaultFontList(new_w) != NULL)
+    {
+	MS_DefaultFontList(new_w) =
+	    XmFontListCopy(MS_DefaultFontList(new_w));
+    }
+    else
+    {
+	MS_DefaultFontList(new_w) = _XmGetDefaultFontList(new_w,
+							 XmBUTTON_FONTLIST);
+    }
     if (MS_ButtonFontList(new_w) == NULL)
     {
 	if (MS_DefaultFontList(new_w) != NULL)
@@ -653,9 +675,11 @@ MenuShellPopdownDone(Widget w, XEvent *event,
     if (_XmMenuGetInPMMode(w) && event && event->type == ButtonRelease &&
 	event->xbutton.time <= state->RC_ButtonEventStatus.time)
     {
+      DEBUGOUT(XdbDebug(__FILE__, w, "MenuShellPopdownDone(): exiting b/c already processed\n"));
 	return;
     }
 
+#if 0
     /*
      * Don't use an event that's within the multi click time.
      * Hopefully this is an event which we can discard :-)
@@ -670,15 +694,18 @@ MenuShellPopdownDone(Widget w, XEvent *event,
 	event->xbutton.time <= state->RC_ButtonEventStatus.time
 				+ XtGetMultiClickTime(XtDisplay(w)))
     {
+      DEBUGOUT(XdbDebug(__FILE__, w, "MenuShellPopdownDone(): exiting b/c user too fast\n"));
 	return;
     }
+#endif
 
     /* must be a menu shell here. */
     assert(XmIsMenuShell(w));
 
-    if (MGR_NumChildren(w) == 0)
+    if (MGR_NumChildren(w) == 0) {
+      DEBUGOUT(XdbDebug(__FILE__, w, "MenuShellPopdownDone(): exiting b/c no children\n"));
 	return;
-
+    }
     /* Find a child that is managed */
     rc = NULL;
     for (i = 0; i < MGR_NumChildren(w); i++)
