@@ -1,6 +1,6 @@
 /**
  *
- * $Id: MenuUtil.c,v 1.29 1998/05/01 15:07:02 rwscott Exp $
+ * $Id: MenuUtil.c,v 1.32 1998/10/17 06:25:04 gritton Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: MenuUtil.c,v 1.29 1998/05/01 15:07:02 rwscott Exp $";
+static const char rcsid[] = "$Id: MenuUtil.c,v 1.32 1998/10/17 06:25:04 gritton Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -131,7 +131,7 @@ _XmGrabPointer(Widget w,
 
 	if (retries)
 	{
-	    _XmMicroSleep(1);
+	    _XmSleep(1);
 	}
     }
 
@@ -180,7 +180,7 @@ _XmGrabKeyboard(Widget widget,
 
 	if (retries)
 	{
-	    _XmMicroSleep(1);
+	    _XmSleep(1);
 	}
     }
 
@@ -200,7 +200,7 @@ _XmUngrabKeyboard(Widget w, Time t)
 void
 _XmMenuEscape(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
-    Widget cb = NULL;
+    /* Widget cb = NULL; */
     XmRowColumnWidget rc;
 
     if (!_XmIsEventUnique(ev))
@@ -765,8 +765,16 @@ _XmRestoreCoreClassTranslations(Widget widget)
 void
 XmSetMenuCursor(Display *display, Cursor cursorId)
 {
-    XmScreen scr = (XmScreen)XmGetXmScreen(DefaultScreenOfDisplay(display));
-    Screen_MenuCursor(scr) = cursorId;
+    int	i;
+
+    /* Set the menuCursor in the screen object for each screen in the display.
+     * But don't bother with that pesky resource interface.
+     */
+
+    if (display && cursorId)
+	for (i = ScreenCount(display) - 1; i >= 0; i--)
+	    Screen_MenuCursor(XmGetXmScreen(ScreenOfDisplay(display, i))) =
+		cursorId;
 }
 
 Cursor
