@@ -1,6 +1,6 @@
 /**
  *
- * $Id: MessageB.c,v 1.20 1998/02/22 01:00:45 rwscott Exp $
+ * $Id: MessageB.c,v 1.28 1998/09/30 03:41:28 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: MessageB.c,v 1.20 1998/02/22 01:00:45 rwscott Exp $";
+static const char rcsid[] = "$Id: MessageB.c,v 1.28 1998/09/30 03:41:28 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -39,6 +39,7 @@ static char rcsid[] = "$Id: MessageB.c,v 1.20 1998/02/22 01:00:45 rwscott Exp $"
 #include <Xm/RowColumnP.h>
 #include <Xm/DialogS.h>
 #include <Xm/LabelGP.h>
+#include <Xm/RepType.h>
 
 #include <XmI/DebugUtil.h>
 
@@ -322,10 +323,84 @@ initialize(Widget request, Widget new_w,
     Widget default_button;
     Arg al[3];
 
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "initialize: %i args\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      *num_args,
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+
     /* core dump avoidance */
     MB_Message(new_w) = NULL;
     MB_Symbol(new_w) = NULL;
 
+    if (!XmRepTypeValidValue(XmRepTypeGetId(XmRDefaultButtonType),
+			     MB_DefaultType(new_w), new_w))
+    {
+	MB_DefaultType(new_w) = XmDIALOG_OK_BUTTON;
+    }
+
+    if (!XmRepTypeValidValue(XmRepTypeGetId(XmRDialogType),
+			     MB_DialogType(new_w), new_w))
+    {
+	MB_DialogType(new_w) = XmDIALOG_MESSAGE;
+    }
+
+    if (!XmRepTypeValidValue(XmRepTypeGetId(XmRAlignment),
+			     MB_MessageAlignment(new_w), new_w))
+    {
+	MB_MessageAlignment(new_w) = XmALIGNMENT_BEGINNING;
+    }
+
+    if (MB_DialogType(new_w) != XmDIALOG_TEMPLATE ||
+	(MB_SymbolPixmap(new_w) != None &&
+	 MB_SymbolPixmap(new_w) != XmUNSPECIFIED_PIXMAP))
+    {
+	MB_Symbol(new_w) = _XmBB_CreateLabelG(new_w, NULL, "Symbol");
+	if (MB_SymbolPixmap(new_w) == XmUNSPECIFIED_PIXMAP)
+	{
+	    MB_SymbolPixmap(new_w) = _XmMessageBoxPixmap(new_w,
+							 MB_DialogType(new_w));
+	}
+
+	XtSetArg(al[0], XmNlabelPixmap, MB_SymbolPixmap(new_w));
+	XtSetArg(al[1], XmNlabelType, XmPIXMAP);
+	XtSetValues(MB_Symbol(new_w), al, 2);
+    }
+    else
+    {
+	MB_Symbol(new_w) = NULL;
+    }
+
+    if (MB_DialogType(new_w) != XmDIALOG_TEMPLATE ||
+	MB_MessageString(new_w) != NULL)
+    {
+	if (MB_MessageString(new_w) == NULL)
+	{
+	    MB_MessageString(new_w) =
+		XmStringCreateLtoR("", XmFONTLIST_DEFAULT_TAG);
+	}
+	MB_Message(new_w) = _XmBB_CreateLabelG(new_w, MB_MessageString(new_w),
+					       "Message");
+
+	XtSetArg(al[0], XmNalignment, MB_MessageAlignment(new_w));
+	XtSetValues(MB_Message(new_w), al, 1);
+    }
+    else
+    {
+	MB_Message(new_w) = NULL;
+    }
+
+#ifdef USE_WIDGETS
+    MB_Separator(new_w) = XmCreateSeparator(new_w, "Separator", NULL, 0);
+#else
+    MB_Separator(new_w) = XmCreateSeparatorGadget(new_w, "Separator", NULL, 0);
+#endif
+
     if (MB_DialogType(new_w) != XmDIALOG_TEMPLATE ||
 	MB_OKLabelString(new_w) != NULL ||
 	MB_OKCall(new_w) != NULL)
@@ -366,46 +441,6 @@ initialize(Widget request, Widget new_w,
 	MB_HelpButton(new_w) = NULL;
     }
 
-#ifdef USE_WIDGETS
-    MB_Separator(new_w) = XmCreateSeparator(new_w, "Separator", NULL, 0);
-#else
-    MB_Separator(new_w) = XmCreateSeparatorGadget(new_w, "Separator", NULL, 0);
-#endif
-
-    if (MB_DialogType(new_w) != XmDIALOG_TEMPLATE ||
-	(MB_SymbolPixmap(new_w) != None &&
-	 MB_SymbolPixmap(new_w) != XmUNSPECIFIED_PIXMAP))
-    {
-	MB_Symbol(new_w) = _XmBB_CreateLabelG(new_w, NULL, "Symbol");
-	if (MB_SymbolPixmap(new_w) == XmUNSPECIFIED_PIXMAP)
-	{
-	    MB_SymbolPixmap(new_w) = _XmMessageBoxPixmap(new_w,
-							 MB_DialogType(new_w));
-	}
-
-	XtSetArg(al[0], XmNlabelPixmap, MB_SymbolPixmap(new_w));
-	XtSetArg(al[1], XmNlabelType, XmPIXMAP);
-	XtSetValues(MB_Symbol(new_w), al, 2);
-    }
-    else
-    {
-	MB_Symbol(new_w) = NULL;
-    }
-
-    if (MB_DialogType(new_w) != XmDIALOG_TEMPLATE ||
-	MB_MessageString(new_w) != NULL)
-    {
-	MB_Message(new_w) = _XmBB_CreateLabelG(new_w, MB_MessageString(new_w),
-					       "Message");
-
-	XtSetArg(al[0], XmNalignment, MB_MessageAlignment(new_w));
-	XtSetValues(MB_Message(new_w), al, 1);
-    }
-    else
-    {
-	MB_Message(new_w) = NULL;
-    }
-
     /* Remove auto_unmanage */
     if (MB_HelpButton(new_w))
     {
@@ -446,12 +481,12 @@ initialize(Widget request, Widget new_w,
 
     XtManageChild(MB_Separator(new_w));
 
-    if (MB_DialogType(new_w) != XmDIALOG_MESSAGE && MB_Symbol(new_w))
+    if (MB_Symbol(new_w))
     {
 	XtManageChild(MB_Symbol(new_w));
     }
 
-    if (MB_MessageString(new_w) && MB_Message(new_w))
+    if (MB_Message(new_w))
     {
 	XtManageChild(MB_Message(new_w));
     }
@@ -556,6 +591,18 @@ static unsigned char question_bits[] =
     0x00, 0x0f, 0x00, 0x00, 0x0c, 0x06, 0x00, 0x0c, 0x0f, 0x00, 0x0c, 0x0f,
     0xf0, 0x03, 0x06, 0xc0, 0x03, 0x00, 0xc0, 0x03, 0x00};
 #else
+#define question_width 22
+#define question_height 22
+static unsigned char question_bits[] =
+{
+   0xf0, 0x3f, 0x00, 0x58, 0x55, 0x00, 0xac, 0xaa, 0x00, 0xd6, 0x5f, 0x01,
+   0xea, 0xbf, 0x02, 0xf6, 0x7f, 0x01, 0xea, 0xba, 0x02, 0xf6, 0x7d, 0x05,
+   0xea, 0xba, 0x0a, 0x56, 0x7d, 0x15, 0xaa, 0xbe, 0x1e, 0x56, 0x5f, 0x01,
+   0xac, 0xaf, 0x02, 0x58, 0x57, 0x01, 0xb0, 0xaf, 0x00, 0x60, 0x55, 0x01,
+   0xa0, 0xaa, 0x00, 0x60, 0x17, 0x00, 0xa0, 0x2f, 0x00, 0x60, 0x57, 0x00,
+   0xb0, 0xaa, 0x00, 0x50, 0x55, 0x01};
+#endif
+#if 0
 #define question_width 32
 #define question_height 32
 static unsigned char question_bits[] =
@@ -743,11 +790,8 @@ XmMbSetSubresources(Widget new_w, Widget old, ArgList args, Cardinal *nargs)
     {
 	Arg al[2];
 
-	if (MB_SymbolPixmap(new_w) == XmUNSPECIFIED_PIXMAP)
-	{
-	    MB_SymbolPixmap(new_w) = _XmMessageBoxPixmap(new_w,
+	MB_SymbolPixmap(new_w) = _XmMessageBoxPixmap(new_w,
 							 MB_DialogType(new_w));
-	}
 
 	XtSetArg(al[0], XmNlabelPixmap, MB_SymbolPixmap(new_w));
 	XtSetArg(al[1], XmNlabelType, XmPIXMAP);
@@ -845,6 +889,20 @@ set_values(Widget old, Widget request, Widget new_w,
 {
     Boolean refresh_needed;
 
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "set_values: %i args\n"
+		      "\t    old X %5i Y %5i W %5i H %5i\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      *num_args,
+		      XtX(old), XtY(old),
+		      XtWidth(old), XtHeight(old),
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+
     BB_InSetValues(new_w) = True;
 
     /* Must do this call first */
@@ -1016,10 +1074,12 @@ _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     newRow = False;
     tookChild = False;
     if (MB_Symbol(_w) && XtIsManaged(MB_Symbol(_w)) &&
-	_XmGeoSetupKid(boxPtr, MB_Symbol(_w)))
+	_XmGeoSetupKid(boxPtr, MB_Symbol(_w)) &&
+	 MB_DialogType(_w) != XmDIALOG_MESSAGE &&
+	 MB_DialogType(_w) != XmDIALOG_TEMPLATE)
     {
 	layoutPtr->even_width = 0;
-	layoutPtr->even_height = 1;
+	layoutPtr->even_height = 0;
 	layoutPtr->space_above = BB_MarginHeight(_w);
 	layoutPtr->space_between = BB_MarginWidth(_w);
 	newRow = True;
@@ -1029,8 +1089,9 @@ _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     if (MB_Message(_w) && XtIsManaged(MB_Message(_w)) &&
 	_XmGeoSetupKid(boxPtr, MB_Message(_w)))
     {
+	layoutPtr->space_end = 0;
 	layoutPtr->even_width = 0;
-	layoutPtr->even_height = 1;
+	layoutPtr->even_height = 0;
 	layoutPtr->space_above = BB_MarginHeight(_w);
 	layoutPtr->space_between = BB_MarginWidth(_w);
 	newRow = True;
@@ -1104,8 +1165,8 @@ _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     {
 	layoutPtr->fill_mode = XmGEO_CENTER;
 	layoutPtr->fit_mode = XmGEO_WRAP;
-	layoutPtr->even_width = 1;
-	layoutPtr->even_height = 1;
+	layoutPtr->even_width = 4;
+	layoutPtr->even_height = 4;
 	layoutPtr->space_above = BB_MarginHeight(_w);
 	newRow = True;
     }
@@ -1118,8 +1179,8 @@ _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
 	    _XmBulletinBoardSetDefaultShadow(extras[i]);
 	    layoutPtr->fill_mode = XmGEO_CENTER;
 	    layoutPtr->fit_mode = XmGEO_WRAP;
-	    layoutPtr->even_width = 1;
-	    layoutPtr->even_height = 1;
+	    layoutPtr->even_width = 4;
+	    layoutPtr->even_height = 4;
 	    layoutPtr->space_above = BB_MarginHeight(_w);
 	    newRow = True;
 	}
@@ -1130,8 +1191,8 @@ _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     {
 	layoutPtr->fill_mode = XmGEO_CENTER;
 	layoutPtr->fit_mode = XmGEO_WRAP;
-	layoutPtr->even_width = 1;
-	layoutPtr->even_height = 1;
+	layoutPtr->even_width = 4;
+	layoutPtr->even_height = 4;
 	layoutPtr->space_above = BB_MarginHeight(_w);
 	newRow = True;
     }
@@ -1140,8 +1201,8 @@ _XmMessageBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     {
 	layoutPtr->fill_mode = XmGEO_CENTER;
 	layoutPtr->fit_mode = XmGEO_WRAP;
-	layoutPtr->even_width = 1;
-	layoutPtr->even_height = 1;
+	layoutPtr->even_width = 4;
+	layoutPtr->even_height = 4;
 	layoutPtr->space_above = BB_MarginHeight(_w);
 	newRow = True;
     }
