@@ -1,6 +1,6 @@
 /**
  *
- * $Id: PanedW.c,v 1.36 1998/06/03 01:49:36 jon Exp $
+ * $Id: PanedW.c,v 1.42 1998/10/18 22:01:01 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: PanedW.c,v 1.36 1998/06/03 01:49:36 jon Exp $";
+static const char rcsid[] = "$Id: PanedW.c,v 1.42 1998/10/18 22:01:01 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -262,7 +262,9 @@ static void _XmPanedWLayout(Widget w, Widget instig, XtWidgetGeometry *cg,
 			    XmKidGeometry *sashes, XmKidGeometry *seps);
 
 /* manager */
+#if 0
 static Boolean traversal_children(Widget mw, Widget **children, Cardinal *num_children);
+#endif
 
 /*
  * Resources for the PanedWindow class
@@ -663,6 +665,18 @@ static void
 initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "%s:initialize(%d) - %i args\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__, __LINE__,
+		      *num_args,
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+
     PW_StartY(new_w) = 0;
     PW_PaneCount(new_w) = 0;
     PW_NumSlots(new_w) = DEFAULT_NUM_SLOTS;
@@ -682,7 +696,19 @@ static void
 constraint_initialize(Widget request, Widget new_w,
 		      ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "PanedWindow constraint initialize\n"));
+    DEBUGOUT(XdbDebug2(__FILE__, XtParent(new_w), new_w,
+		       "%s:constraint_initialize(%d) - %i args\n"
+		       "\trequest X %5i Y %5i W %5i H %5i\n"
+		       "\t  new_w X %5i Y %5i W %5i H %5i\n"
+		       "\t   Form W %5i H %5i\n",
+		       __FILE__, __LINE__,
+		       *num_args,
+		       XtX(request), XtY(request),
+		       XtWidth(request), XtHeight(request),
+		       XtX(new_w), XtY(new_w),
+		       XtWidth(new_w), XtHeight(new_w),
+		       XtWidth(XtParent(new_w)), XtHeight(XtParent(new_w))));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     ValidatePaneMin(new_w);
     ValidatePaneMax(new_w);
@@ -701,6 +727,10 @@ constraint_initialize(Widget request, Widget new_w,
 static void
 destroy(Widget w)
 {
+    if (PW_ManagedChildren(w))
+    {
+	XtFree((String)PW_ManagedChildren(w));
+    }
     XtReleaseGC(w, PW_FlipGC(w));
 }
 
@@ -917,14 +947,22 @@ realize(Widget w,
 	XtValueMask *value_mask,
 	XSetWindowAttributes *attributes)
 {
+    /*
     XtWidgetGeometry geo;
+    */
+
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:realize(%d) - %dx%d\n",
+    	__FILE__, __LINE__,
+    	XtWidth(w), XtHeight(w)));
 
 #define superclass (&xmManagerClassRec)
     (*superclass->core_class.realize) (w, value_mask, attributes);
 #undef superclass
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "PanedWindow Realize\n"));
-
+#if 0
+    /* rws 17 Oct 1998
+       This should not be necessary. All this should get done in change_managed.
+     */
     _XmPanedWPreferredSize(w, NULL, NULL, &geo);
 
     if (_XmMakeGeometryRequest(w, &geo) == XtGeometryYes)
@@ -937,6 +975,7 @@ realize(Widget w,
 	geo.height = XtHeight(w);
 	_XmPanedWLayout(w, NULL, NULL, &geo, NULL, NULL, NULL);
     }
+#endif
 }
 
 
@@ -970,6 +1009,9 @@ insert_child(Widget w)
     XmPanedWindowWidget pw = (XmPanedWindowWidget)XtParent(w);
     int i;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:insert_child(%d)\n",
+		      __FILE__, __LINE__));
+
     if (PW_RecursivelyCalled(pw))
     {
 
@@ -1055,6 +1097,7 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *desired)
 
     _XmPanedWPreferredSize(w, NULL, NULL, desired);
 
+#if 0
     /* Doesn't seem to be set otherwise */
     desired->request_mode |= CWWidth | CWHeight;
 
@@ -1077,6 +1120,9 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *desired)
     }
 
     return XtGeometryAlmost;
+#else
+    return _XmGMReplyToQueryGeometry(w, proposed, desired);
+#endif
 }
 
 static XtGeometryResult
@@ -1382,6 +1428,7 @@ change_managed(Widget w)
     _XmNavigChangeManaged(w);
 }
 
+#if 0
 static Boolean
 traversal_children(Widget mw, Widget **children, Cardinal *num_children)
 {
@@ -1416,6 +1463,7 @@ traversal_children(Widget mw, Widget **children, Cardinal *num_children)
 
     return True;
 }
+#endif
 
 static void
 _XmPanedWPreferredSize(Widget w, Widget rchild,
@@ -1489,10 +1537,13 @@ _XmPanedWLayout(Widget w, Widget rchild,
 		XmKidGeometry *boxptr, XmKidGeometry *sashptr,
 		XmKidGeometry *sepptr)
 {
+#define SEXT(x) \
+       ((int)((short)(x)))
     int i, save_mode;
     Dimension height, nh, curw, curh;
     Widget child;
     XmKidGeometry boxes, sashes, seps;
+    int total_delta ;
 
     curw = pwg->width;
     curh = pwg->height;
@@ -1505,21 +1556,77 @@ _XmPanedWLayout(Widget w, Widget rchild,
     seps = (XmKidGeometry)XtCalloc(PW_PaneCount(w) + 1,
 				   sizeof(XmKidGeometryRec));
 
-    /* compute the optimimum layout */
-    height = PW_MarginHeight(w);
-
+    /* ------- New ------------- */
+    /* Pass 1 - each pane can be its desired height but within its PaneMaximum and PaneMinimum */
+    height = 2 * PW_MarginHeight(w);   /* margins on either side */
     for (i = 0; i < PW_PaneCount(w); i++)
     {
 	child = PW_ManagedChildren(w)[i];
 
-	if (i == PW_PaneCount(w) - 1)
+       boxes[i].box.height = PWC_DHeight(child);
+       if (SEXT(boxes[i].box.height) > SEXT(PWC_PaneMaximum(child)))
+           boxes[i].box.height = PWC_PaneMaximum(child) ;
+       else if (SEXT(boxes[i].box.height) < SEXT(PWC_PaneMinimum(child)))
+           boxes[i].box.height = PWC_PaneMinimum(child) ;
+       height += boxes[i].box.height + PW_Spacing(w) ;
+    }
+    height -= PW_Spacing(w) ;  /* no need for last spacing */
+    total_delta = curh - height ;      /* adjustment needed to fill out to current height */
+
+    /* pass 2 - expand or contract each pane to make fit within curh */
+    for (i = PW_PaneCount(w) - 1; i >= 0; i--)
+    {
+       child = PW_ManagedChildren(w)[i];
+
+       if (PWC_SkipAdjust(child) )     /* we can't change the size of this one */
+           continue;
+       if (total_delta > 0)    /* need to make some pane(s) bigger */
 	{
-	    nh = curh - height - PW_MarginHeight(w);
+           if (SEXT(boxes[i].box.height) < SEXT(PWC_PaneMaximum(child)))       /* can grow */
+           {
+               if (SEXT(boxes[i].box.height) + total_delta <= SEXT(PWC_PaneMaximum(child)))
+               {
+                   boxes[i].box.height += total_delta; /* can absorb full delta, so done */
+                   break;
+               }
+               else
+               {
+                   /* make pane maximum, but this leaves some delta left so continue */
+                   total_delta -= SEXT(PWC_PaneMaximum(child)) - SEXT(boxes[i].box.height);
+                   boxes[i].box.height = PWC_PaneMaximum(child) ;
+               }
+           }
 	}
-	else
+       else if (total_delta < 0)       /* need to make some pane(s) smaller */
 	{
-	    nh = PWC_DHeight(child);
+           if (SEXT(boxes[i].box.height) > SEXT(PWC_PaneMinimum(child)))       /* allowed to shrink */
+           {
+               if (SEXT(boxes[i].box.height) + total_delta >= SEXT(PWC_PaneMinimum(child)))
+               {
+                   boxes[i].box.height += total_delta; /* can contract by full delta, so done */
+                   break;
+               }
+               else
+               {
+                   /* make pane minimum, but this leaves some delta left so continue */
+                   total_delta -= SEXT(PWC_PaneMinimum(child)) - SEXT(boxes[i].box.height);
+                   boxes[i].box.height = PWC_PaneMinimum(child) ;
+               }
+           }
 	}
+    }
+    /* ------- End New ------------- */
+
+    /* now we have the sizes of the panes determined and we can position
+       them along with their sashes and Separators */
+    height = PW_MarginHeight(w);
+
+    for (i = 0; i < PW_PaneCount(w); i++)
+    {
+       child = PW_ManagedChildren(w)[i];
+
+       nh = boxes[i].box.height ;
+
 	boxes[i].kid = child;
 	boxes[i].box.x = PW_MarginWidth(w);
 	boxes[i].box.width = curw - 2 * PW_MarginWidth(w);
@@ -1528,7 +1635,7 @@ _XmPanedWLayout(Widget w, Widget rchild,
 	boxes[i].box.border_width = XtBorderWidth(child);
 
 	PWC_DY(child) = height;
-	height += PWC_DHeight(child);
+       height += nh;
 
 	if (i != (PW_PaneCount(w) - 1))
 	{
@@ -1558,120 +1665,7 @@ _XmPanedWLayout(Widget w, Widget rchild,
 	}
     }
 
-#define SEXT(x) \
-	((int)((short)(x)))
-
-    /* now make it fit, by rippling the geom differences */
-    for (i = PW_PaneCount(w) - 1; i >= 0; i--)
-    {
-	int delta;
-	int j;
-
-	child = PW_ManagedChildren(w)[i];
-
-	if (SEXT(boxes[i].box.height) > SEXT(PWC_PaneMaximum(child)))
-	{
-	    delta = SEXT(boxes[i].box.height) - SEXT(PWC_PaneMaximum(child));
-
-	    boxes[i].box.height = PWC_PaneMaximum(child);
-
-	    for (j = i - 1; j >= 0; j--)
-	    {
-		int k;
-
-		if (SEXT(boxes[j].box.height) <
-		    SEXT(PWC_PaneMaximum(PW_ManagedChildren(w)[j])))
-		{
-		    if (delta <
-			(SEXT(PWC_PaneMaximum(PW_ManagedChildren(w)[j])) -
-			 SEXT(boxes[j].box.height)))
-		    {
-			boxes[j].box.height = SEXT(boxes[j].box.height) + delta;
-
-			/* move lower boxes down */
-			for (k = j + 1; k < PW_PaneCount(w); k++)
-			{
-			    boxes[k].box.y += delta;
-			    sashes[k - 1].box.y += delta;
-			    seps[k - 1].box.y += delta;
-			}
-
-			break;	/* out of j-loop */
-		    }
-		    else
-		    {
-			int ddt;
-
-			ddt = SEXT(PWC_PaneMaximum(PW_ManagedChildren(w)[j])) -
-			    SEXT(boxes[j].box.height);
-
-			boxes[j].box.height = SEXT(boxes[j].box.height) + ddt;
-
-			delta -= ddt;
-
-			/* move lower boxes down */
-			for (k = j + 1; k < PW_PaneCount(w); k++)
-			{
-			    boxes[k].box.y += ddt;
-			    sashes[k - 1].box.y += ddt;
-			    seps[k - 1].box.y += ddt;
-			}
-		    }
-		}
-	    }			/* end j loop */
-	}
-	else if (SEXT(boxes[i].box.height) < SEXT(PWC_PaneMinimum(child)))
-	{
-	    delta = SEXT(PWC_PaneMinimum(child)) - SEXT(boxes[i].box.height);
-
-	    boxes[i].box.height = PWC_PaneMinimum(child);
-
-	    for (j = i - 1; j >= 0; j--)
-	    {
-		int k;
-
-		if (SEXT(boxes[j].box.height) >
-		    SEXT(PWC_PaneMinimum(PW_ManagedChildren(w)[j])))
-		{
-		    if (delta < (SEXT(boxes[j].box.height) -
-			      SEXT(PWC_PaneMinimum(PW_ManagedChildren(w)[j]))))
-		    {
-			boxes[j].box.height = SEXT(boxes[j].box.height) - delta;
-
-			/* move lower boxes up */
-			for (k = j + 1; k < PW_PaneCount(w); k++)
-			{
-			    boxes[k].box.y -= delta;
-			    sashes[k - 1].box.y -= delta;
-			    seps[k - 1].box.y -= delta;
-			}
-
-			break;	/* out of j-loop */
-		    }
-		    else
-		    {
-			int ddt;
-
-			ddt = SEXT(boxes[j].box.height) -
-			    SEXT(PWC_PaneMinimum(PW_ManagedChildren(w)[j]));
-
-			boxes[j].box.height = SEXT(boxes[j].box.height) - ddt;
-
-			delta -= ddt;
-
-			/* move lower boxes up */
-			for (k = j + 1; k < PW_PaneCount(w); k++)
-			{
-			    boxes[k].box.y -= ddt;
-			    sashes[k - 1].box.y -= ddt;
-			    seps[k - 1].box.y -= ddt;
-			}
-		    }
-		}
-	    }			/* end j loop */
-	}
-    }
-
+    /* rippling code no longer needed */
     /* now set childgeom */
     if (rchild && childgeom)
     {
