@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Primitive.c,v 1.28 1998/04/16 21:46:42 u27113 Exp $
+ * $Id: Primitive.c,v 1.34 1998/10/10 07:38:59 gritton Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: Primitive.c,v 1.28 1998/04/16 21:46:42 u27113 Exp $";
+static const char rcsid[] = "$Id: Primitive.c,v 1.34 1998/10/10 07:38:59 gritton Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -37,6 +37,11 @@ static char rcsid[] = "$Id: Primitive.c,v 1.28 1998/04/16 21:46:42 u27113 Exp $"
 
 #include <XmI/DebugUtil.h>
 
+#if XmVERSION > 1
+/* Includes for 2.0 features */
+#include <XmI/DirectionI.h>
+#endif
+
 /* Forward Declarations */
 
 static void class_initialize();
@@ -182,6 +187,28 @@ static XtResource resources[] =
 	sizeof(XtPointer), Offset(user_data),
 	XmRImmediate, (XtPointer)NULL
     }
+
+#if LESSTIF_BACK_COMPAT
+#else
+#if XmVERSION > 1
+    ,{
+        XmNconvertCallback, XmCCallback, XmRCallback,
+        sizeof(XtCallbackList), Offset(convert_callback),
+        XtRPointer, (XtPointer)NULL
+    },
+    {
+        XmNpopupHandlerCallback, XmCCallback, XmRCallback,
+        sizeof(XtCallbackList), Offset(popup_handler_callback),
+        XtRPointer, (XtPointer)NULL
+    },
+    {
+        XmNlayoutDirection, XmCLayoutDirection, XmRDirection,
+        sizeof(XmDirection), Offset(layout_direction),
+	XmRCallProc, (XtPointer)_XmDirectionDefault
+    }
+#endif
+#endif
+
 };
 
 #define POffset(field)	XtOffset(XmPrimitiveWidget, primitive.field)
@@ -250,8 +277,8 @@ static XtActionsRec actions[] =
     {"PrimitiveParentCancel", _XmPrimitiveParentCancel},
     {"unmap", _XmPrimitiveUnmap},
     {"Help", _XmPrimitiveHelp},
-    {"enter", _XmPrimitiveLeave},
-    {"leave", _XmPrimitiveEnter},
+    {"enter", _XmPrimitiveEnter},
+    {"leave", _XmPrimitiveLeave},
     {"PrevTabGroup", _XmTraverseNextTabGroup},
     {"NextTabGroup", _XmTraversePrevTabGroup},
 };
@@ -854,6 +881,11 @@ primitive_border_unhighlight(Widget w)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "PrimitiveBorderUnhighlight\n"));
 
+    if (!XtIsRealized(w))
+    {
+        return;
+    }
+
     /* with zero width, we don't need this... */
     if (Prim_HighlightThickness(w) == 0)
     {
@@ -884,6 +916,11 @@ primitive_border_highlight(Widget w)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "PrimitiveBorderHighlight\n"));
 
+    if (!XtIsRealized(w))
+    {
+        return;
+    }
+
     /* with zero width, we don't need this... */
     if (Prim_HighlightThickness(w) == 0)
     {
@@ -1041,12 +1078,15 @@ _XmDifferentBackground(Widget w, Widget parent)
     {
 	return True;
     }
-    if (!XmIsPrimitive(w))
+    if (XmIsPrimitive(w) && XmIsManager(parent) &&
+	(XtBackground(w) != XtBackground(parent)))
     {
 	return True;
     }
-
-    return (XtBackground(w) != XtBackground(parent));
+    else
+    {
+	return False;
+    }
 }
 
 void
@@ -1086,7 +1126,7 @@ _XmPrimitiveTopShadowPixmapDefault(Widget w, int offset, XrmValue *value)
     {
 	pixmap = XmGetPixmapByDepth(XtScreen(w), "50_foreground",
 				    Prim_TopShadowColor(w),
-				    XtBackground(w),
+				    Prim_Foreground(w),
 				    CoreDepth(w));
     }
     else if (DefaultDepthOfScreen(XtScreen(w)) == 1)
