@@ -1,6 +1,6 @@
 /**
  *
- * $Id: PushB.c,v 1.35 1998/03/16 21:52:13 rwscott Exp $
+ * $Id: PushB.c,v 1.48 1998/09/25 03:22:26 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: PushB.c,v 1.35 1998/03/16 21:52:13 rwscott Exp $";
+static const char rcsid[] = "$Id: PushB.c,v 1.48 1998/09/25 03:22:26 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -50,6 +50,7 @@ static const char rcsid[] = "$Id: PushB.c,v 1.35 1998/03/16 21:52:13 rwscott Exp
  * Try to add trait stuff by #ifdeffing it.
  */
 #if XmVERSION > 1
+#include <Xm/TraitP.h>
 #include <Xm/ActivatableT.h>
 
 void _XmPushB_TraitAddCallback(Widget, XtCallbackProc, XtPointer, Boolean);
@@ -503,6 +504,12 @@ initialize(Widget request, Widget new_w,
 
     PB_UnarmPixmap(new_w) = Lab_Pixmap(new_w);
 
+    if (PB_MultiClick(new_w) == (unsigned char)XmUNSPECIFIED)
+    {
+	PB_MultiClick(new_w) =
+	    IN_MENU(new_w) ? XmMULTICLICK_DISCARD : XmMULTICLICK_KEEP;
+    }
+
     if (IN_MENU(new_w))
     {
 	Lab_Highlight(new_w) = 0;
@@ -638,9 +645,13 @@ set_values(Widget old, Widget request, Widget new_w,
 
 	if (PB_DefaultButtonShadow(new_w) != PB_DefaultButtonShadow(old))
 	{
+	/* pushbutton/test16 */
 	    margin = 2 * PB_DefaultButtonShadow(new_w) + Lab_Shadow(new_w);
-	    margin_extra = Xm3D_ENHANCE_PIXEL;
+	    margin_extra = PB_DefaultButtonShadow(new_w) != 0
+			 ? Xm3D_ENHANCE_PIXEL
+			 : 0;
 
+#if 0
 	    Lab_MarginLeft(new_w) = margin + margin_extra;
 	    Lab_MarginRight(new_w) = margin + margin_extra;
 	    Lab_MarginTop(new_w) = margin + margin_extra;
@@ -660,6 +671,30 @@ set_values(Widget old, Widget request, Widget new_w,
 
 	    (*xmLabelClassRec.core_class.resize) (new_w);
 
+#else
+	    Lab_MarginLeft(new_w) = PB_DefaultButtonShadow(new_w) ? margin + margin_extra : Lab_MarginWidth(new_w);
+	    Lab_MarginRight(new_w) = PB_DefaultButtonShadow(new_w) ? margin + margin_extra : Lab_MarginWidth(new_w);
+	    Lab_MarginTop(new_w) = PB_DefaultButtonShadow(new_w) ? margin + margin_extra : Lab_MarginHeight(new_w);
+	    Lab_MarginBottom(new_w) = PB_DefaultButtonShadow(new_w) ? margin + margin_extra : Lab_MarginHeight(new_w);
+
+		if (Lab_RecomputeSize(new_w) && XtWidth(new_w) == XtWidth(old))
+		{
+		    XtWidth(new_w) = 0;
+		}
+		if (Lab_RecomputeSize(new_w) && XtHeight(new_w) == XtHeight(old))
+		{
+		    XtHeight(new_w) = 0;
+		}
+
+		_XmCalcLabelDimensions(new_w);
+
+		(*xmLabelClassRec.core_class.resize) (new_w);
+
+	    Lab_MarginLeft(new_w) = PB_DefaultButtonShadow(new_w) ? margin + margin_extra : 0;
+	    Lab_MarginRight(new_w) = PB_DefaultButtonShadow(new_w) ? margin + margin_extra : 0;
+	    Lab_MarginTop(new_w) = PB_DefaultButtonShadow(new_w) ? margin + margin_extra : 0;
+	    Lab_MarginBottom(new_w) = PB_DefaultButtonShadow(new_w) ? margin + margin_extra : 0;
+#endif
 	    refresh_needed = True;
 	}
     }
@@ -743,28 +778,34 @@ set_values(Widget old, Widget request, Widget new_w,
 
 	if (PB_ShowAsDefault(new_w) && !PB_ShowAsDefault(old))
 	{
-	    _XmDrawShadows(XtDisplay(new_w), XtWindow(new_w),
-			   Prim_TopShadowGC(new_w),
-			   Prim_BottomShadowGC(new_w),
-			   normal_shadow_x - (2 * shad + Lab_Shadow(new_w)),
-			   normal_shadow_y - (2 * shad + Lab_Shadow(new_w)),
-			   normal_shadow_width + 2 *
-			   (2 * shad + Lab_Shadow(new_w)),
-			   normal_shadow_height + 2 *
-			   (2 * shad + Lab_Shadow(new_w)),
-			   shad,
-			   XmSHADOW_IN);
+	    if (XtIsManaged(new_w))
+	    {
+		_XmDrawShadows(XtDisplay(new_w), XtWindow(new_w),
+			       Prim_TopShadowGC(new_w),
+			       Prim_BottomShadowGC(new_w),
+			       normal_shadow_x - (2*shad + Lab_Shadow(new_w)),
+			       normal_shadow_y - (2*shad + Lab_Shadow(new_w)),
+			       normal_shadow_width + 2 *
+			       (2 * shad + Lab_Shadow(new_w)),
+			       normal_shadow_height + 2 *
+			       (2 * shad + Lab_Shadow(new_w)),
+			       shad,
+			       XmSHADOW_IN);
+	    }
 	}
 	else
 	{
-	    _XmClearBorder(XtDisplay(new_w), XtWindow(new_w),
-			   normal_shadow_x - (2 * shad + Lab_Shadow(new_w)),
-			   normal_shadow_y - (2 * shad + Lab_Shadow(new_w)),
-			   normal_shadow_width + 2 *
-			   (2 * shad + Lab_Shadow(new_w)),
-			   normal_shadow_height + 2 *
-			   (2 * shad + Lab_Shadow(new_w)),
-			   shad);
+	    if (XtIsManaged(new_w))
+	    {
+		_XmClearBorder(XtDisplay(new_w), XtWindow(new_w),
+			       normal_shadow_x - (2*shad + Lab_Shadow(new_w)),
+			       normal_shadow_y - (2*shad + Lab_Shadow(new_w)),
+			       normal_shadow_width + 2 *
+			       (2 * shad + Lab_Shadow(new_w)),
+			       normal_shadow_height + 2 *
+			       (2 * shad + Lab_Shadow(new_w)),
+			       shad);
+	    }
 	}
     }
 
@@ -935,32 +976,39 @@ Arm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
     XmProcessTraversal(w, XmTRAVERSE_CURRENT);
 
-    PB_Armed(w) = True;
-    /* JAC added check for existence of event */
-    if (event)
+    if (!PB_Armed(w))
     {
-	PB_ArmTimeStamp(w) = event->xbutton.time;
+	PB_Armed(w) = True;
+	/* JAC added check for existence of event */
+	if (event)
+	{
+	    PB_ArmTimeStamp(w) = event->xbutton.time;
+	}
+	else
+	{
+	    PB_ArmTimeStamp(w) = CurrentTime;
+	}
+
+	(*exp) (w, event, NULL);
+
+	if (PB_ArmCallback(w))
+	{
+	    cbs.reason = XmCR_ARM;
+	    cbs.event = event;
+	    cbs.click_count = PB_ClickCount(w);
+
+	    XFlush(XtDisplay(w));
+
+	    XtCallCallbackList(w,
+			       PB_ArmCallback(w),
+			       (XtPointer)&cbs);
+	}
+	/* rws 11 Sep 1998
+	   This should not be here. ActiveChild is used by Manager to track
+	   gadgets.
+	 */
+	MGR_ActiveChild(XtParent(w)) = w;
     }
-    else
-    {
-	PB_ArmTimeStamp(w) = CurrentTime;
-    }
-
-    (*exp) (w, event, NULL);
-
-    if (PB_ArmCallback(w))
-    {
-	cbs.reason = XmCR_ARM;
-	cbs.event = event;
-	cbs.click_count = PB_ClickCount(w);
-
-	XFlush(XtDisplay(w));
-
-	XtCallCallbackList(w,
-			   PB_ArmCallback(w),
-			   (XtPointer)&cbs);
-    }
-    MGR_ActiveChild(XtParent(w)) = w;
 }
 
 
@@ -1053,69 +1101,55 @@ Disarm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 ArmTimeout(XtPointer data, XtIntervalId *id)
 {
-    XmPushButtonCallbackStruct cbs;
-    XmArmTimeoutRec *p = (XmArmTimeoutRec *)data;
-    Widget w = p->w;
+    Widget w = (Widget)data;
     XtExposeProc exp = XtClass(w)->core_class.expose;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "ArmTimeout\n"));
  
-    if (PB_Armed(w))
+    PB_Timer(w) = 0;
+
+    if (XtIsRealized(w))
     {
- 
-	PB_Armed(w) = False;
-
 	(*exp) (w, NULL, NULL);
-     
-	if (!Lab_SkipCallback(w) && PB_ActivateCallback(w))
-	{
-	    cbs.reason = XmCR_ACTIVATE;
-     
-	    if (p->has_event)
-	    {
-		cbs.event = &p->ev;
-	    }
-	    else
-	    {
-		cbs.event = NULL;
-	    }
-	    cbs.click_count = PB_ClickCount(w);
-     
-	    XFlush(XtDisplay(w));
-     
-	    XtCallCallbackList(w,
-			       PB_ActivateCallback(w),
-			       (XtPointer)&cbs);
-	}
-
-	Disarm(w, NULL, NULL, 0);
-
+	XFlush(XtDisplay(w));
     }
-    XtFree(data);
 }
 
 static void
 ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    XmArmTimeoutRec *p = (XmArmTimeoutRec *)XtCalloc(1,
-						     sizeof(XmArmTimeoutRec));
+    XmPushButtonCallbackStruct cbs;
     Boolean poppedUp;
  
     DEBUGOUT(XdbDebug(__FILE__, w, "ArmAndActivate\n"));
 
+    /* Arm, Activate, and Disarm now, but draw the disarmed state later */
+
     Arm(w, event, params, num_params);
-
-    p->w = w;
-
-    if (event)
+    PB_Armed(w) = False;
+    if (!Lab_SkipCallback(w) && PB_ActivateCallback(w))
     {
-        p->ev = *event;
-        p->has_event=True;
+	XFlush(XtDisplay(w));
+	cbs.reason = XmCR_ACTIVATE;
+	cbs.event = event;
+	cbs.click_count = 1;
+	XtCallCallbackList(w,
+			   PB_ActivateCallback(w),
+			   (XtPointer)&cbs);
+    }
+    if (PB_DisarmCallback(w))
+    {
+	XFlush(XtDisplay(w));
+	cbs.reason = XmCR_DISARM;
+	cbs.event = event;
+	cbs.click_count = 1;
+	XtCallCallbackList(w,
+			   PB_DisarmCallback(w),
+			   (XtPointer)&cbs);
     }
 
     if (IN_MENU(w))
     {
-	ArmTimeout((XtPointer)p, 0);
 	RC_MenuButtonPopdown(w, event, &poppedUp);
     }
     else
@@ -1126,7 +1160,7 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	}
 
 	PB_Timer(w) = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
-				  ACTIVATE_DELAY, ArmTimeout, (XtPointer)p);
+				  ACTIVATE_DELAY, ArmTimeout, (XtPointer)w);
     }
 }
 
@@ -1199,7 +1233,9 @@ LeaveWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	_XmPrimitiveLeave(w, event, NULL, NULL);
 	if (PB_Armed(w))
 	{
+	    PB_Armed(w) = False;
 	    (*exp) (w, event, NULL);
+	    PB_Armed(w) = True;
 	}
     }
     else
@@ -1444,7 +1480,6 @@ static void
 MenuProcEntry(int proc, Widget w,...)
 {
     va_list arg_list;
-    XEvent *event;
     extern void _XmTearOffArm();
  
     va_start(arg_list, w);
@@ -1455,7 +1490,6 @@ MenuProcEntry(proc, w, va_alist)
      va_dcl
 {
     va_list arglist;
-    XEvent *event;
     extern void _XmTearOffArm();
  
     va_start(arglist);
