@@ -1,6 +1,6 @@
 /**
  *
- * $Id: PushBG.c,v 1.37 1998/05/03 17:03:36 rwscott Exp $
+ * $Id: PushBG.c,v 1.52 1998/09/25 03:22:26 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: PushBG.c,v 1.37 1998/05/03 17:03:36 rwscott Exp $";
+static const char rcsid[] = "$Id: PushBG.c,v 1.52 1998/09/25 03:22:26 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -53,6 +53,7 @@ static const char rcsid[] = "$Id: PushBG.c,v 1.37 1998/05/03 17:03:36 rwscott Ex
  * Try to add trait stuff by #ifdeffing it.
  */
 #if XmVERSION > 1
+#include <Xm/TraitP.h>
 #include <Xm/ActivatableT.h>
 
 void _XmPushBG_TraitAddCallback(Widget, XtCallbackProc, XtPointer, Boolean);
@@ -672,6 +673,12 @@ initialize(Widget request, Widget new_w,
 
     PBG_UnarmPixmap(new_w) = LabG_Pixmap(new_w);
 
+    if (PBG_MultiClick(new_w) == (unsigned char)XmUNSPECIFIED)
+    {
+	PBG_MultiClick(new_w) =
+	    IN_MENU(new_w) ? XmMULTICLICK_DISCARD : XmMULTICLICK_KEEP;
+    }
+
     if (IN_MENU(new_w))
     {
 	LabG_Highlight(new_w) = 0;
@@ -912,18 +919,44 @@ set_values(Widget old, Widget request, Widget new_w,
 
 	if (PBG_DefaultButtonShadow(new_w) != PBG_DefaultButtonShadow(old))
 	{
+	    /*
+	     * If PBG_DefaultButtonShadow is zero then we're not the default
+	     * button (right?) so we want to undo the margin hack to get the
+	     * label's clipping area correct again.  Blah! default buttons
+	     * are a pain.  See pushbg/test8 (note:  this test shows two bugs,
+	     * setting margin_extra to zero will fix one of them, the resize
+	     * problem may or may not be fixed when you read this).
+	     *
+	     *	--mu@trends.net, 1998.08.06
+	     */
 	    margin = 2 * PBG_DefaultButtonShadow(new_w) + LabG_Shadow(new_w);
-	    margin_extra = Xm3D_ENHANCE_PIXEL;
+	    margin_extra = PBG_DefaultButtonShadow(new_w) != 0
+	                 ? Xm3D_ENHANCE_PIXEL
+	                 : 0;
 
-	    LabG_MarginLeft(new_w) = margin + margin_extra;
-	    LabG_MarginRight(new_w) = margin + margin_extra;
-	    LabG_MarginTop(new_w) = margin + margin_extra;
-	    LabG_MarginBottom(new_w) = margin + margin_extra;
+	    LabG_MarginLeft(new_w) = PBG_DefaultButtonShadow(new_w) ? margin + margin_extra : LabG_MarginWidth(new_w);
+	    LabG_MarginRight(new_w) = PBG_DefaultButtonShadow(new_w) ? margin + margin_extra : LabG_MarginWidth(new_w);
+	    LabG_MarginTop(new_w) = PBG_DefaultButtonShadow(new_w) ? margin + margin_extra : LabG_MarginHeight(new_w);
+	    LabG_MarginBottom(new_w) = PBG_DefaultButtonShadow(new_w) ? margin + margin_extra : LabG_MarginHeight(new_w);
 
-	    XtWidth(new_w) += 2 * (margin + margin_extra);
-	    XtHeight(new_w) += 2 * (margin + margin_extra);
+		if (LabG_RecomputeSize(new_w) && XtWidth(new_w) == XtWidth(old))
+		{
+		    XtWidth(new_w) = 0;
+		}
+		if (LabG_RecomputeSize(new_w) && XtHeight(new_w) == XtHeight(old))
+		{
+		    XtHeight(new_w) = 0;
+		}
+
+		_XmCalcLabelGDimensions(new_w);
+
+		(*xmLabelGadgetClassRec.rect_class.resize) (new_w);
+
+	    LabG_MarginLeft(new_w) = PBG_DefaultButtonShadow(new_w) ? margin + margin_extra : 0;
+	    LabG_MarginRight(new_w) = PBG_DefaultButtonShadow(new_w) ? margin + margin_extra : 0;
+	    LabG_MarginTop(new_w) = PBG_DefaultButtonShadow(new_w) ? margin + margin_extra : 0;
+	    LabG_MarginBottom(new_w) = PBG_DefaultButtonShadow(new_w) ? margin + margin_extra : 0;
 
-	    (*xmLabelGadgetClassRec.rect_class.resize) (new_w);
 
 	    refresh_needed = True;
 	}
@@ -1005,32 +1038,38 @@ set_values(Widget old, Widget request, Widget new_w,
 
 	if (PBG_ShowAsDefault(new_w) && !PBG_ShowAsDefault(old))
 	{
-	    _XmDrawShadows(XtDisplay(new_w), XtWindow(new_w),
-			   XmParentTopShadowGC(new_w),
-			   XmParentBottomShadowGC(new_w),
-			   XtX(new_w) + normal_shadow_x -
-			   (2 * shad + LabG_Shadow(new_w)),
-			   XtY(new_w) + normal_shadow_y -
-			   (2 * shad + LabG_Shadow(new_w)),
-			   normal_shadow_width + 2 *
-			   (2 * shad + LabG_Shadow(new_w)),
-			   normal_shadow_height + 2 *
-			   (2 * shad + LabG_Shadow(new_w)),
-			   shad,
-			   XmSHADOW_IN);
+	    if (XtIsManaged(new_w))
+	    {
+		_XmDrawShadows(XtDisplay(new_w), XtWindow(new_w),
+			       XmParentTopShadowGC(new_w),
+			       XmParentBottomShadowGC(new_w),
+			       XtX(new_w) + normal_shadow_x -
+			       (2 * shad + LabG_Shadow(new_w)),
+			       XtY(new_w) + normal_shadow_y -
+			       (2 * shad + LabG_Shadow(new_w)),
+			       normal_shadow_width + 2 *
+			       (2 * shad + LabG_Shadow(new_w)),
+			       normal_shadow_height + 2 *
+			       (2 * shad + LabG_Shadow(new_w)),
+			       shad,
+			       XmSHADOW_IN);
+	    }
 	}
 	else if (!PBG_ShowAsDefault(new_w) && PBG_ShowAsDefault(old))
 	{
-	    _XmClearBorder(XtDisplay(new_w), XtWindow(new_w),
-			   XtX(new_w) + normal_shadow_x -
-			   (2 * shad + LabG_Shadow(new_w)),
-			   XtY(new_w) + normal_shadow_y -
-			   (2 * shad + LabG_Shadow(new_w)),
-			   normal_shadow_width + 2 *
-			   (2 * shad + LabG_Shadow(new_w)),
-			   normal_shadow_height + 2 *
-			   (2 * shad + LabG_Shadow(new_w)),
-			   shad);
+	    if (XtIsManaged(new_w))
+	    {
+		_XmClearBorder(XtDisplay(new_w), XtWindow(new_w),
+			       XtX(new_w) + normal_shadow_x -
+			       (2 * shad + LabG_Shadow(new_w)),
+			       XtY(new_w) + normal_shadow_y -
+			       (2 * shad + LabG_Shadow(new_w)),
+			       normal_shadow_width + 2 *
+			       (2 * shad + LabG_Shadow(new_w)),
+			       normal_shadow_height + 2 *
+			       (2 * shad + LabG_Shadow(new_w)),
+			       shad);
+	    }
 	}
     }
 
@@ -1271,25 +1310,28 @@ Arm(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XmProcessTraversal(w, XmTRAVERSE_CURRENT);
     */
 
-    PBG_Armed(w) = True;
-
-    (*exp) (w, event, NULL);
-
-    if (PBG_ArmCallback(w))
+    if (!PBG_Armed(w))
     {
-	cbs.reason = XmCR_ARM;
-	cbs.event = event;
-	cbs.click_count = PBG_ClickCount(w);
+	PBG_Armed(w) = True;
 
-	XFlush(XtDisplay(w));
+	(*exp) (w, event, NULL);
 
-	XtCallCallbackList(w,
-			   PBG_ArmCallback(w),
-			   (XtPointer)&cbs);
+	if (PBG_ArmCallback(w))
+	{
+	    cbs.reason = XmCR_ARM;
+	    cbs.event = event;
+	    cbs.click_count = PBG_ClickCount(w);
+
+	    XFlush(XtDisplay(w));
+
+	    XtCallCallbackList(w,
+			       PBG_ArmCallback(w),
+			       (XtPointer)&cbs);
+	}
+	/*
+	MGR_ActiveChild(XtParent(w)) = w;
+	*/
     }
-    /*
-    MGR_ActiveChild(XtParent(w)) = w;
-    */
 }
 
 static void
@@ -1367,66 +1409,55 @@ Activate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 ArmTimeout(XtPointer data, XtIntervalId *id)
 {
-    XmPushButtonCallbackStruct cbs;
-    XmArmTimeoutRec *p = (XmArmTimeoutRec *)data;
-    Widget w = p->w;
+    Widget w = (Widget)data;
+    XtExposeProc exp = XtClass(w)->core_class.expose;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "ArmTimeout\n"));
  
-    if (PBG_Armed(w))
-    {
-	/* rws 18 Mar 1998
-	   This must be done before the ACTIVATE callback
-	   see comments near the end of ArmAndActivate
-	 */
-	Disarm(w, NULL, NULL, 0);
-	if (!LabG_SkipCallback(w) && PBG_ActivateCallback(w))
-	{
-	    cbs.reason = XmCR_ACTIVATE;
-     
-	    if (p->has_event)
-	    {
-		cbs.event = &p->ev;
-	    }
-	    else
-	    {
-		cbs.event = NULL;
-	    }
-	    cbs.click_count = PBG_ClickCount(w);
-     
-	    XFlush(XtDisplay(w));
-     
-	    XtCallCallbackList(w,
-			       PBG_ActivateCallback(w),
-			       (XtPointer)&cbs);
-	}
-    }
+    PBG_Timer(w) = 0;
 
-    XtFree(data);
+    if (XtIsRealized(w))
+    {
+	(*exp) (w, NULL, NULL);
+	XFlush(XtDisplayOfObject(w));
+    }
 }
 
 static void
 ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    XmArmTimeoutRec *p = (XmArmTimeoutRec *)XtCalloc(1,
-						     sizeof(XmArmTimeoutRec));
+    XmPushButtonCallbackStruct cbs;
     Boolean poppedUp;
  
     DEBUGOUT(XdbDebug(__FILE__, w, "ArmAndActivate\n"));
 
+    /* Arm, Activate, and Disarm now, but draw the disarmed state later */
+
     Arm(w, event, params, num_params);
-
-    p->w = w;
-
-    if (event)
+    PBG_Armed(w) = False;
+    if (!LabG_SkipCallback(w) && PBG_ActivateCallback(w))
     {
-        p->ev = *event;
-        p->has_event=True;
+	XFlush(XtDisplayOfObject(w));
+	cbs.reason = XmCR_ACTIVATE;
+	cbs.event = event;
+	cbs.click_count = 1;
+	XtCallCallbackList(w,
+			   PBG_ActivateCallback(w),
+			   (XtPointer)&cbs);
+    }
+    if (PBG_DisarmCallback(w))
+    {
+	XFlush(XtDisplayOfObject(w));
+	cbs.reason = XmCR_DISARM;
+	cbs.event = event;
+	cbs.click_count = 1;
+	XtCallCallbackList(w,
+			   PBG_DisarmCallback(w),
+			   (XtPointer)&cbs);
     }
 
     if (IN_MENU(w))
     {
-	ArmTimeout((XtPointer)p, 0);
 	RC_MenuButtonPopdown(w, event, &poppedUp);
     }
     else
@@ -1436,27 +1467,8 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	    XtRemoveTimeOut(PBG_Timer(w));
 	}
 
-#if 1
-	/* rws 8 Mar 1998
-	   This is wacky. In xdir select Connect->Connect to Anonymous.
-	   Fill in User Name: with anonymous, Password: with you email address
-	   then Host Name: with ftp.lesstif.org. Now with the focus in the
-	   Host Name: text field press return.  This takes you on a wild trip
-	   through textfield activate ParentActivate and you eventually end
-	   up here.  If you call ArmTimeout immediatly, this case, then all
-	   works well. You connect to ftp.lesstif.org. If you go through the
-	   timeout method, you connect the window for the remote host
-	   comes up then segfault. In xdir's activate callback the dialog
-	   is XtDestroy'ed. That should not affect this because the activate
-	   callback is called in ArmTimeout. What difference does it make
-	   whether we call ArmTimeout directly or wait a while??
-	   What gives??????
-	 */
-	ArmTimeout((XtPointer)p, 0);
-#else
 	PBG_Timer(w) = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
-				       ACTIVATE_DELAY, ArmTimeout, (XtPointer)p);
-#endif
+				       ACTIVATE_DELAY, ArmTimeout, (XtPointer)w);
     }
 }
 
@@ -1492,8 +1504,6 @@ EnterWindow(Widget w,
 
 	    (*exp) (w, event, NULL);
 
-	    MGR_SelectedGadget(XtParent(w)) = (XmGadget)w;
-
 	    if (PBG_ArmCallback(w))
 	    {
 		cbs.reason = XmCR_ARM;
@@ -1521,7 +1531,9 @@ LeaveWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	_XmLeaveGadget(w, event, NULL, NULL);
 	if (PBG_Armed(w))
 	{
+	    PBG_Armed(w) = False;
 	    (*exp) (w, event, NULL);
+	    PBG_Armed(w) = True;
 	}
     }
     else
@@ -1533,8 +1545,6 @@ LeaveWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
 	    (*exp) (w, event, NULL);
 
-	    MGR_SelectedGadget(XtParent(w)) = NULL;
-
 	    if (PBG_DisarmCallback(w))
 	    {
 		cbs.reason = XmCR_DISARM;
@@ -1645,8 +1655,6 @@ ButtonDown(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
     _XmSetInDragMode(w, True);
 
-    MGR_SelectedGadget(XtParent(w)) = (XmGadget)w;
-
     RC_MenuShellPopdown(w, event, &poppedUp);
 
     _XmDrawShadows(XtDisplay(w),
@@ -1814,7 +1822,7 @@ static void
 MenuProcEntry(int proc, Widget w,...)
 {
     va_list arg_list;
-    XEvent *event;
+    XEvent *event = NULL;
     extern void _XmTearOffArm();
  
     va_start(arg_list, w);
@@ -1825,7 +1833,7 @@ MenuProcEntry(proc, w, va_alist)
      va_dcl
 {
     va_list arglist;
-    XEvent *event;
+    XEvent *event = NULL;
     extern void _XmTearOffArm();
  
     va_start(arglist);
