@@ -1,6 +1,6 @@
 /**
  *
- * $Id: RCUtils.c,v 1.74 1998/05/01 22:51:03 rwscott Exp $
+ * $Id: RCUtils.c,v 1.90 1998/10/26 21:38:37 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: RCUtils.c,v 1.74 1998/05/01 22:51:03 rwscott Exp $";
+static const char rcsid[] = "$Id: RCUtils.c,v 1.90 1998/10/26 21:38:37 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -180,6 +180,8 @@ PreferredSizeHT(Widget rc, XtWidgetGeometry *rcg,
 	DEBUGOUT(XdbDebug0(__FILE__, rc, "%s",
 			   RC_FromResize(rc) ? "" : "ParentResize "));
 	DEBUGOUT(XdbDebug0(__FILE__, rc, ")\n"));
+	DEBUGOUT(XdbDebug0(__FILE__, rc, "requesting %s\n",
+		XdbWidgetGeometry2String(rcg)));
     }
 
     *max_height = 0, *max_width = 0;
@@ -205,10 +207,10 @@ PreferredSizeHT(Widget rc, XtWidgetGeometry *rcg,
 	/*
 	 * If we don't have enough space in this row, start another one.
 	 */
-	if (RC_FromResize(rc) &&
-	    current_x + kid_geometry->box.width > XtWidth(rc))
+	if ((RC_FromResize(rc) || (rcg->request_mode & CWWidth)) &&
+	    current_x + kid_geometry->box.width > (rcg->request_mode & CWWidth ? rcg->width : XtWidth(rc)))
 	{
-	    current_y += *max_height + RC_Spacing(rc);
+	    current_y += *max_height + RC_Spacing(rc) + RC_MarginW(rc) + MGR_ShadowThickness(rc);
 	    current_x = RC_MarginW(rc) + MGR_ShadowThickness(rc);
 	}
 
@@ -315,7 +317,9 @@ DoLayoutHT(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 		RC_Boxes(rc)[j].box.height = lmh;
 	    }
 
-	    current_y += lmh;
+	    printf("lmh %d spacing %d %d %d\n", lmh, RC_Spacing(rc),
+	    	RC_MarginW(rc), MGR_ShadowThickness(rc));
+	    current_y += lmh + RC_MarginW(rc) + MGR_ShadowThickness(rc); /* mainw/test14 */
 	    current_x = RC_MarginW(rc) + MGR_ShadowThickness(rc);
 	    first_in_row = i;
 	}
@@ -361,12 +365,13 @@ DoLayoutHT(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 		RC_Boxes(rc)[j].box.height = lmh;
 	    }
 
-	    current_y += lmh;
+	    current_y += lmh + RC_MarginW(rc) + MGR_ShadowThickness(rc);
 	    current_x = RC_MarginW(rc) + MGR_ShadowThickness(rc);
 	    first_in_row = i;
 	}
 
 	kid_geometry->box.x = (XtWidth(rc)
+				    - RC_MarginW(rc)
 				    - MGR_ShadowThickness(rc)
 				    - help_kid_geometry->box.width);
 	kid_geometry->box.y = current_y;
@@ -526,10 +531,11 @@ static void
 DoLayoutHC(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 	   int max_width, int max_height)
 {
-    int i;
+    int i, cnt;
     int number_per_row;
     Dimension current_x = RC_MarginW(rc) + MGR_ShadowThickness(rc);
     Dimension current_y = RC_MarginH(rc) + MGR_ShadowThickness(rc);
+    Dimension box_width;
 
     if (RC_NCol(rc) == 0)
     {
@@ -540,9 +546,17 @@ DoLayoutHC(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 
     number_per_row = ((_XmGeoCount_kids((CompositeWidget)rc) - 1) /
 		      RC_NCol(rc)) + 1;
+    if (XtIsRealized(rc) && max_width < (XtWidth(rc) - 2 * (RC_MarginW(rc) + MGR_ShadowThickness(rc)) - (RC_Spacing(rc) * (number_per_row - 1))) / number_per_row)
+    {
+	box_width = (XtWidth(rc) - 2 * (RC_MarginW(rc) + MGR_ShadowThickness(rc)) - (RC_Spacing(rc) * (number_per_row - 1))) / number_per_row;
+    }
+    else
+    {
+    	box_width = max_width;
+    }
 
     /* now we lay out the children */
-    for (i = 0; i < MGR_NumChildren(rc); i++)
+    for (i = 0, cnt = 0; i < MGR_NumChildren(rc); i++)
     {
 	XmRCKidGeometry kid_geometry;
 	Widget kid;
@@ -555,7 +569,7 @@ DoLayoutHC(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 	    continue;
 	}
 
-	if (i != 0 && (i % number_per_row) == 0)
+	if (cnt != 0 && (cnt % number_per_row) == 0)
 	{
 	    current_x = RC_MarginW(rc) + MGR_ShadowThickness(rc);
 	    current_y += max_height + RC_Spacing(rc);
@@ -572,7 +586,8 @@ DoLayoutHC(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 	    *instig_request = kid_geometry->box;
 	}
 
-	current_x += RC_Spacing(rc) + max_width;
+	current_x += RC_Spacing(rc) + box_width;
+	cnt++;
     }
 
     /* now we fill in the last row, so it takes up the remaining height, 
@@ -599,6 +614,7 @@ DoLayoutHC(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 					   + RC_MarginH(rc)
 					   + MGR_ShadowThickness(rc)));
 
+	    kid_geometry->box.width = box_width;
 	    if (kid == instig && instig_request != NULL)
 	    {
 		*instig_request = kid_geometry->box;
@@ -623,12 +639,13 @@ static void
 PreferredSizeVT(Widget rc, XtWidgetGeometry *rcg,
 		int *max_width, int *max_height)
 {
-    int i;
+    int i, cols;
     Dimension current_x = RC_MarginW(rc) + MGR_ShadowThickness(rc);
     Dimension current_y = RC_MarginH(rc) + MGR_ShadowThickness(rc);
     Dimension curw = RC_MarginW(rc) + MGR_ShadowThickness(rc);
 
     *max_width = 0, *max_height = 0;
+    cols = 1;
 
     /* Calculate the max_width */
     for (i = 0; i < MGR_NumChildren(rc); i++)
@@ -648,6 +665,7 @@ PreferredSizeVT(Widget rc, XtWidgetGeometry *rcg,
 	if (RC_FromResize(rc) &&
 	    current_y + kid_geometry->box.height > XtHeight(rc))
 	{
+	    cols++;
 	    current_x += *max_width + RC_Spacing(rc);
 	    current_y = RC_MarginH(rc) + MGR_ShadowThickness(rc);
 	}
@@ -680,6 +698,24 @@ PreferredSizeVT(Widget rc, XtWidgetGeometry *rcg,
 
     rcg->request_mode = CWWidth | CWHeight | CWBorderWidth;
     rcg->width = curw + RC_MarginW(rc) + MGR_ShadowThickness(rc);
+
+    if (RC_FromResize(rc) && rcg->width > XtWidth(rc))
+    {
+    	/* rws 25 Oct 1998 (rowcolumn/test42, xephem, xmgrace)
+    	   If we are being resized then we can't be any wider than the RC.
+    	   In this case split the available width up evenly between all of
+    	   the columns.
+    	   This, and the next if, should maybe be done in the Layout portion,
+    	   since this is a constraint affecting the "preferred" size.  Then
+    	   again, it is the preferred size given the constraints????
+    	 */
+    	rcg->width = XtWidth(rc);
+    	*max_width = (XtWidth(rc) 
+    	              - 2 * (RC_MarginW(rc) + MGR_ShadowThickness(rc)) 
+    	              - (RC_Spacing(rc) * (cols - 1))) 
+    	             / cols;
+    }
+
     if (*max_height == 0)
     {
 	rcg->height = 2 * (RC_MarginH(rc) + MGR_ShadowThickness(rc));
@@ -916,6 +952,8 @@ DoLayoutVC(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
     number_per_column = ((_XmGeoCount_kids((CompositeWidget)rc) - 1) /
 			 RC_NCol(rc)) + 1;
 
+    {
+    int cnt = 0;
     /* now we lay out the children */
     for (i = 0; i < MGR_NumChildren(rc); i++)
     {
@@ -930,7 +968,7 @@ DoLayoutVC(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 	    continue;
 	}
 
-	if (i != 0 && (i % number_per_column) == 0)
+	if (cnt != 0 && (cnt % number_per_column) == 0)
 	{
 	    current_x += max_width + RC_Spacing(rc);
 	    current_y = RC_MarginH(rc) + MGR_ShadowThickness(rc);
@@ -951,6 +989,8 @@ DoLayoutVC(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 	}
 
 	current_y += RC_Spacing(rc) + max_height;
+	cnt++;
+    }
     }
 
     /*
@@ -997,6 +1037,46 @@ DoLayoutVC(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
     }
 }
 
+static void
+find_largest_option_selection(Widget w, Dimension *width, Dimension *height)
+{
+    int i;
+    Widget child;
+    XtWidgetGeometry preferred;
+    
+    if (w == NULL)
+	return;
+    for (i = 0; i < MGR_NumChildren(w); i++)
+    {
+	child = MGR_Children(w)[i];
+	if (XtIsManaged(child))
+	{
+	    if (XmIsCascadeButton(child))
+	    {
+		find_largest_option_selection(CB_Submenu(child),
+					      width, height);
+	    }
+	    else if (XmIsCascadeButtonGadget(child))
+	    {
+		find_largest_option_selection(CBG_Submenu(child)
+					      , width, height);
+	    }
+	    else
+	    {
+		XtQueryGeometry(child, NULL, &preferred);
+		if (*width < preferred.width)
+		{
+		    *width = preferred.width;
+		}
+		if (*height < preferred.height)
+		{
+		    *height = preferred.height;
+		}
+	    }
+	}
+    }
+}
+
 /*
  * This function was a major inefficiency provider of the RC widget -
  * it gets called by setvalues, insert_child, and change_managed;
@@ -1070,12 +1150,19 @@ _XmRCGetKidGeo(Widget w, Widget instig,
 	 * correct (according to the Motif comments), and it helps
 	 * the new form algorithm work.
 	 */
+
+	/*
+	 * But it breaks the layout of xmgr File->Describe menu - the text
+	 * widget becomes mutch to small.
+	 */
+#if 0
 	if (RC_Type(w) == XmWORK_AREA)
 	{
 	    _XmGeoLoadValues(kid_geometry->kid, geoType,
 			     NULL, NULL, &kid_geometry->box);
 	}
 	else
+#endif
 	{
 	    _XmGeoLoadValues(kid_geometry->kid, geoType,
 			     instig, instig_request, &kid_geometry->box);
@@ -1175,6 +1262,37 @@ _XmRCGetKidGeo(Widget w, Widget instig,
 	    }
 	}
     }
+    if (RC_Type(w) == XmMENU_OPTION)
+    {
+	Widget obg = XmOptionButtonGadget(w);
+	Dimension width = 0, height = 0;
+
+	for (i = 0; i < MGR_NumChildren(w); i++)
+	{
+	    kid_geometry = &boxes[i];
+	    if (obg == kid_geometry->kid)
+	    {
+		break;
+	    }
+	}
+	if (obg == kid_geometry->kid)
+	{
+	    find_largest_option_selection(CBG_Submenu(obg), &width,
+					  &height);
+	    width += LabG_MarginRight(obg) + 
+		2 * G_HighlightThickness(obg) + G_ShadowThickness(obg) +
+		Xm3D_ENHANCE_PIXEL;
+	    height += 2 * G_HighlightThickness(obg);
+	    if (width > kid_geometry->box.width)
+	    {
+		kid_geometry->box.width = width;
+	    }
+	    if (height > kid_geometry->box.height)
+	    {
+		kid_geometry->box.height = height;
+	    }
+	}
+    }
 
     return boxes;
 }
@@ -1337,7 +1455,7 @@ _XmRCAdjustSize(Widget rc, Widget instig, XtWidgetGeometry *instig_request)
 
     DEBUGOUT(XdbDebug(__FILE__, rc, "XmRCAdjustSize (%s) %s\n",
 		      XdbRcType2String(RC_Type(rc)),
-		      !RC_FromResize(rc) ? "" : "ParentResize"));
+		      RC_FromResize(rc) ? "FromResize" : ""));
 
     if (RC_Boxes(rc))
 	XtFree((char *)RC_Boxes(rc));
@@ -1351,6 +1469,7 @@ _XmRCAdjustSize(Widget rc, Widget instig, XtWidgetGeometry *instig_request)
      * As you can see the rcg parameter of the PreferredSize*() functions is
      * never NULL
      */
+    rcg.request_mode = 0;
     switch (RC_Orientation(rc))
     {
     case XmHORIZONTAL:
@@ -1397,22 +1516,40 @@ _XmRCAdjustSize(Widget rc, Widget instig, XtWidgetGeometry *instig_request)
 	max_height = 1;
     }
 
-    if (XtIsRealized(rc) || !RC_FromResize(rc))
+    if (!RC_FromResize(rc))
     {
+    /* messagebox/test8, test12 budget */
 	DEBUGOUT(XdbDebug(__FILE__, rc,
 			  "_XmRCAdjustSize: requesting %s from parent %s\n",
 			XdbWidgetGeometry2String(&rcg), XtName(XtParent(rc))));
 
-	if (!RC_ResizeWidth(rc) && XtIsRealized(rc))
+	if (!RC_ResizeWidth(rc) /* && XtIsRealized(rc) */)
 	{
 	    rcg.request_mode &= ~CWWidth;
 	    rcg.width = XtWidth(rc);
 	}
-	if (!RC_ResizeHeight(rc) && XtIsRealized(rc))
+	if (!RC_ResizeHeight(rc) /* && XtIsRealized(rc) */)
 	{
 	    rcg.request_mode &= ~CWHeight;
 	    rcg.height = XtHeight(rc);
 	}
+#if 0
+	if (!XtIsRealized(rc))
+	{
+		/* rowcolumn/test1 test25 */
+		/* mfm Xquote->Option->Details */
+		/*
+		if (XtWidth(rc) != 0 && XtWidth(rc) != 1)
+		{
+			rcg.width = XtWidth(rc);
+		}
+		if (XtHeight(rc) != 0 && XtHeight(rc) != 1)
+		{
+			rcg.height = XtHeight(rc);
+		}
+		*/
+	}
+#endif
 
 	if ((ret = _XmMakeGeometryRequest(rc, &rcg)) == XtGeometryYes)
 	{
@@ -1423,8 +1560,10 @@ _XmRCAdjustSize(Widget rc, Widget instig, XtWidgetGeometry *instig_request)
 	else
 	{
 	    DEBUGOUT(XdbDebug(__FILE__, rc,
-		       "_XmMakeGeometryRequest for RC no: size stays %d %d.\n",
-			      XtWidth(rc), XtHeight(rc)));
+		       "_XmMakeGeometryRequest for RC %s: size stays %d %d (%s).\n",
+			      XdbGeometryResult2String(ret),
+			      XtWidth(rc), XtHeight(rc),
+			      XdbWidgetGeometry2String(&rcg)));
 	}
     }
     else
@@ -1666,7 +1805,13 @@ XmCreateOptionMenu(Widget parent, char *name,
     ArgList combined;
 
     XtSetArg(myArgList[n], XmNrowColumnType, XmMENU_OPTION); n++;
+    /* rws 23 Jun 1998
+       We take care of setting this in initialize. Setting it here means
+       that we cannot pick this up from a resource. So... get rid of it.
+     */
+    /*
     XtSetArg(myArgList[n], XmNorientation, XmHORIZONTAL); n++;
+    */
     XtSetArg(myArgList[n], XmNentryAlignment, XmALIGNMENT_CENTER); n++;
 
     combined = XtMergeArgLists(myArgList, n, arglist, argcount);
@@ -1703,19 +1848,30 @@ XmCreatePopupMenu(Widget parent, char *name,
     XtSetArg(myArgList[n], XmNrowColumnType, XmMENU_POPUP);
     n++;
 
-    combined = XtMergeArgLists(myArgList, n, arglist, argcount);
-
     shell_ac = 0;
     XtSetArg(shell_args[shell_ac], XmNwidth, 10); shell_ac++;
     XtSetArg(shell_args[shell_ac], XmNheight, 10); shell_ac++;
     XtSetArg(shell_args[shell_ac], XmNallowShellResize, True); shell_ac++;
 
+    /* rws 10 Jun 1998
+       Mozilla is assuming that some of the args passed in here get set
+       on the created shell.  Therefore lets combine the arglists with those
+       we want on the shell with those passed in.
+       Thanks to Peter Haight <psh1@cornell.edu> for narrowing this one
+       down.
+     */
+    combined = XtMergeArgLists(shell_args, shell_ac, arglist, argcount);
+
     ms = XtCreatePopupShell(popup_name,
 			    xmMenuShellWidgetClass,
-			    parent, shell_args, shell_ac);
+			    parent, combined, shell_ac + argcount);
+
+    XtFree((char *)combined);
 
     XtFree(popup_name);
 
+    combined = XtMergeArgLists(myArgList, n, arglist, argcount);
+
     w = XtCreateWidget(name,
 		       xmRowColumnWidgetClass,
 		       ms,
