@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ResInd.c,v 1.4 1997/08/19 19:29:14 miers Exp $
+ * $Id: ResInd.c,v 1.9 1998/10/08 05:24:51 gritton Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -28,7 +28,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: ResInd.c,v 1.4 1997/08/19 19:29:14 miers Exp $";
+static const char rcsid[] = "$Id: ResInd.c,v 1.9 1998/10/08 05:24:51 gritton Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -43,13 +43,6 @@ static char rcsid[] = "$Id: ResInd.c,v 1.4 1997/08/19 19:29:14 miers Exp $";
 
 #include <XmI/DebugUtil.h>
 
-#define MMsPerInch		(25.4)
-#define InchesPerMM		(1.0 / MMsPerInch)
-#define MMsPerPoint		(MMsPerInch / 72.0)
-#define PointsPerMM		(72.0 / MMsPerInch)
-#define InchesPerPoint		(1.0 / 72.0)
-#define PointsPerInch		(72.0)
-
 #ifdef __cplusplus
 extern "C"
 {
@@ -1056,9 +1049,9 @@ _XmManagerImportArgs(Widget w,
     }
 }
 
-/*
- * WANTED: more accurate algorithms
- */
+#define	FU		0x1	/* (Pixels per) Font Unit		*/
+#define	CMMPP		0x2	/* 100ths of millimeters per pixel	*/
+
 int
 _XmConvertUnits(Screen *screen,
 		int orientation,
@@ -1066,125 +1059,79 @@ _XmConvertUnits(Screen *screen,
 		int from_val,
 		int to_type)
 {
-    double factor = 1.0, sfact = 1.0;
-
-    /*
-     * factor is the number of pixels of the screen over the number of
-     * millimeters of the screen, i.e., the number of pixels per millimeter.
-     * 1 point is 1/72nd of an inch.
-     * font units are specified by the XmScreen resources XmNhorizontalFontUnit
-     * and XmNverticalFontUnit.
-     */
-    switch (orientation)
+    int		fu, cmmpp;
+    struct conv
     {
-    case XmHORIZONTAL:
-	factor = (double)WidthOfScreen(screen) /
-	    (double)WidthMMOfScreen(screen);
-	sfact = (double)_XmGetFontUnit(screen, XmHORIZONTAL);
-	break;
-    case XmVERTICAL:
-	factor = (double)HeightOfScreen(screen) /
-	    (double)HeightMMOfScreen(screen);
-	sfact = (double)_XmGetFontUnit(screen, XmVERTICAL);
-	break;
-    }
+	int	num, denom;	/* Constant conversion factor		*/
+	int	numv, denomv;	/* Variable conversion factor		*/
+    }		*cp;
+    static struct conv conversions[5][5] =
+    {{				/* from PIXELS to ...			*/
+	{1, 1, 0, 0},		/* PIXELS				*/
+	{1, 1, CMMPP, 0},	/* 100TH_MILLIMETERS = CMMPP		*/
+	{50, 127, CMMPP, 0},	/* 1000TH_INCHES = CMMPP * 10/25.4	*/
+	{360, 127, CMMPP, 0},	/* 100TH_POINTS = CMMPP * 72/25.4	*/
+	{100, 1, 0, FU}		/* 100TH_FONT_UNITS = 100/FU		*/
+    },
+    {				/* from 100TH_MILLIMETERS to ...	*/
+	{1, 1, 0, CMMPP},	/* PIXELS = 1/CMMPP			*/
+	{1, 1, 0, 0},		/* 100TH_MILLIMETERS			*/
+	{50, 127, 0, 0},	/* 1000TH_INCHES = 10/25.4		*/
+	{360, 127, 0, 0},	/* 100TH_POINTS = 72/25.4		*/
+	{100, 1, 0, CMMPP | FU}	/* 100TH_FONT_UNITS = 1/CMMPP * 100/FU	*/
+    },
+    {				/* from 1000TH_INCHES to ...		*/
+	{127, 50, 0, CMMPP},	/* PIXELS = 1/CMMPP * 25.4/10		*/
+	{127, 50, 0, 0},	/* 100TH_MILLIMETERS = 25.4/10		*/
+	{1, 1, 0, 0},		/* 1000TH_INCHES			*/
+	{36, 5, 0, 0},		/* 100TH_POINTS = 72/10			*/
+	{254, 1, 0, CMMPP | FU}	/* 100TH_FONT_UNITS = 1/CMMPP * 25.4/10	*/
+    },				/*		      * 100/FU		*/
+    {				/* from 100TH_POINTS to ...		*/
+	{127, 360, 0, CMMPP},	/* PIXELS = 1/CMMPP * 25.4/72		*/
+	{127, 360, 0, 0},	/* 100TH_MILLIMETERS = 25.4/72		*/
+	{5, 36, 0, 0},		/* 1000TH_INCHES = 10/72		*/
+	{1, 1, 0, 0},		/* 100TH_POINTS				*/
+	{635, 18, 0, CMMPP | FU}/* 100TH_FONT_UNITS = 1/CMMPP * 25.4/72	*/
+    },				/*		      * 100/FU		*/
+    {				/* from 100TH_FONT_UNITS to ...		*/
+	{1, 100, FU, 0},	/* PIXELS = FU/100			*/
+	{1, 100, FU | CMMPP, 0},/* 100TH_MILLIMETERS = CMPP * FU/100	*/
+	{1, 254, FU | CMMPP, 0},/* 1000TH_INCHES = CMPP * 10/25.4	*/
+				/*		   * FU/100		*/
+	{18, 635, FU | CMMPP,0},/* 100TH_POINTS = CMPP * 72/25.4	*/
+				/*		  * FU/100		*/
+	{1, 1, 0, 0}		/* 100TH_FONT_UNITS			*/
+    }};
 
-    switch (from_type)
-    {
-    case XmPIXELS:
-	switch (to_type)
-	{
-	case XmPIXELS:
-	    return from_val;
-	    break;
-	case Xm100TH_MILLIMETERS:
-	    return (int)((from_val / factor) / 100.0 + 0.5);
-	    break;
-	case Xm1000TH_INCHES:
-	    return (int)((from_val / factor) * (InchesPerMM * 1000.0) + 0.5);
-	    break;
-	case Xm100TH_POINTS:
-	    return (int)((from_val / factor) * (PointsPerMM * 100.0) + 0.5);
-	    break;
-	case Xm100TH_FONT_UNITS:
-	    break;
-	}
-	break;
-    case Xm100TH_MILLIMETERS:
-	switch (to_type)
-	{
-	case XmPIXELS:
-	    return (int)((from_val / 100.0) * factor + 0.5);
-	    break;
-	case Xm100TH_MILLIMETERS:
-	    return from_val;
-	    break;
-	case Xm1000TH_INCHES:
-	    return (int)((from_val / 100.0) * (InchesPerMM * 1000.0) + 0.5);
-	    break;
-	case Xm100TH_POINTS:
-	    return (int)((from_val / 100.0) * (PointsPerMM * 100.0) + 0.5);
-	    break;
-	case Xm100TH_FONT_UNITS:
-	    break;
-	}
-	break;
-    case Xm1000TH_INCHES:
-	switch (to_type)
-	{
-	case XmPIXELS:
-	    return (int)((from_val / 1000.0) * MMsPerInch * factor + 0.5);
-	    break;
-	case Xm100TH_MILLIMETERS:
-	    return (int)((from_val / 1000.0) * (MMsPerInch * 100.0) + 0.5);
-	    break;
-	case Xm1000TH_INCHES:
-	    return from_val;
-	    break;
-	case Xm100TH_POINTS:
-	    return (int)((from_val / 1000.0) * (PointsPerInch * 100.0) + 0.5);
-	    break;
-	case Xm100TH_FONT_UNITS:
-	    break;
-	}
-	break;
-    case Xm100TH_POINTS:
-	switch (to_type)
-	{
-	case XmPIXELS:
-	    return (int)((from_val / 100.0) * MMsPerPoint * factor);
-	    break;
-	case Xm100TH_MILLIMETERS:
-	    return (int)((from_val / 100.0) * (MMsPerPoint * 100.0) + 0.5);
-	    break;
-	case Xm1000TH_INCHES:
-	    return (int)((from_val / 100.0) * (InchesPerPoint * 1000.0) + 0.5);
-	    break;
-	case Xm100TH_POINTS:
-	    return from_val;
-	    break;
-	case Xm100TH_FONT_UNITS:
-	    break;
-	}
-	break;
-    case Xm100TH_FONT_UNITS:
-	switch (to_type)
-	{
-	case XmPIXELS:
-	    break;
-	case Xm100TH_MILLIMETERS:
-	    break;
-	case Xm1000TH_INCHES:
-	    break;
-	case Xm100TH_POINTS:
-	    break;
-	case Xm100TH_FONT_UNITS:
-	    return from_val;
-	    break;
-	}
-	break;
-    }
-    return 0;
+    /* The trivial case - converting a value to itself */
+
+    if (from_type == to_type)
+	return from_val;
+
+    /* Fetch the font units and/or screen dimensions if we need them */
+
+    cp = &conversions[from_type][to_type];
+    if ((cp->numv | cp->denomv) & FU)
+	fu = _XmGetFontUnit(screen, orientation);
+    if ((cp->numv | cp->denomv) & CMMPP)
+	cmmpp = orientation == XmHORIZONTAL
+	    ? (100 * WidthMMOfScreen(screen)) / WidthOfScreen(screen)
+	    : (100 * HeightMMOfScreen(screen)) / HeightOfScreen(screen);
+
+    /* Do the actual conversion, first all multiplies, then all divides	*/
+
+    from_val *= cp->num;
+    if (cp->numv & FU)
+	from_val *= fu;
+    if (cp->numv & CMMPP)
+	from_val *= cmmpp;
+    from_val /= cp->denom;
+    if (cp->denomv & FU)
+	from_val /= fu;
+    if (cp->denomv & CMMPP)
+	from_val /= cmmpp;
+    return from_val;
 }
 
 /*
@@ -1427,9 +1374,27 @@ XmCvtFromVerticalPixels(Screen *screen, int from_val, int to_type)
 void
 XmSetFontUnits(Display *display, int h_value, int v_value)
 {
+    int		i;
+    XmScreen	sw;
+
+    /* Set the horizontalFontUnit and verticalFontUnit values in the
+     * screen object for each screen in the display.  But don't bother
+     * with that pesky resource interface.
+     */
+
+    if (display)
+	for (i = ScreenCount(display) - 1; i >= 0; i--)
+	{
+	    sw = (XmScreen)XmGetXmScreen(ScreenOfDisplay(display, i));
+	    if (h_value > 0)
+		Screen_HorizUnit(sw) = h_value;
+	    if (v_value > 0)
+		Screen_VertUnit(sw) = v_value;
+	}
 }
 
 void
 XmSetFontUnit(Display *display, int value)
 {
+    XmSetFontUnits( display, value, value);
 }
