@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Scale.c,v 1.25 1998/04/22 19:47:18 u27113 Exp $
+ * $Id: Scale.c,v 1.30 1998/10/24 00:38:14 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: Scale.c,v 1.25 1998/04/22 19:47:18 u27113 Exp $";
+static const char rcsid[] = "$Id: Scale.c,v 1.30 1998/10/24 00:38:14 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -132,11 +132,13 @@ static void computeValueSize(Widget w);
 
 static void showValue(Widget w, int scb_value, int scale_value);
 
+#if 0
 static void GetFocus(Widget w, XEvent *event,
 		     String *params, Cardinal *num_params);
 
 static void LoseFocus(Widget w, XEvent *event,
 		      String *params, Cardinal *num_params);
+#endif
 
 static void _XmScaleProcessingDirectionDefault(Widget w,
 					       int offset,
@@ -448,10 +450,22 @@ initialize(Widget request, Widget new_w,
     Widget sb;
     Arg argl[15];
     int argc;
+    /*
     XmKidGeometry boxes;
     Dimension wd, ht;
+    */
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "XmScale initialize()\n"));
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "%s:initialize(%d) - %i args\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__, __LINE__,
+		      *num_args,
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     if (Scale_Value(new_w) == INT_MAX)
     {
@@ -542,6 +556,7 @@ initialize(Widget request, Widget new_w,
 
     CreateForegroundGC(new_w);
 
+#if 0
     wd = ht = 0;
     boxes = _XmScalePreferredSize(new_w, NULL, NULL, &wd, &ht);
 
@@ -552,11 +567,13 @@ initialize(Widget request, Widget new_w,
     _XmScaleConfigureChildren(new_w, NULL, boxes);
 
     XtFree((char *)boxes);
+#endif
 }
 
 static void
 destroy(Widget w)
 {
+    XmFontListFree(Scale_FontList(w));
     XtDestroyGC(Scale_ForegroundGC(w));
 }
 
@@ -755,6 +772,7 @@ expose(Widget w,
 static XtGeometryResult
 query_geometry(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 {
+#if 0
     XtWidgetGeometry wants;
     XmKidGeometry boxes;
     Dimension wd, ht;
@@ -807,6 +825,21 @@ query_geometry(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
     return XtGeometryYes;
 
 #undef Wants
+#else
+    XtWidgetGeometry wants;
+    Dimension wd, ht;
+
+    DEBUGOUT(XdbDebug(__FILE__, w, "XmScale %s QueryGeometry\n", XtName(w)));
+
+    wants = *request;
+
+    _XmScalePreferredSize(w, NULL, NULL, &wd, &ht);
+
+    reply->width = wd;
+    reply->height = ht;
+
+    return _XmGMReplyToQueryGeometry(w, &wants, reply);
+#endif
 }
 
 /*
@@ -906,9 +939,11 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 static void
 realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
 {
+    /*
     Dimension wd, ht;
     Dimension pwd, pht;
     XmKidGeometry boxes;
+    */
 
     DEBUGOUT(XdbDebug(__FILE__, w, "XmScale realize()\n"));
 
@@ -916,6 +951,7 @@ realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
     (*superclass->core_class.realize) (w, value_mask, attributes);
 #undef superclass
 
+#if 0
     wd = ht = 0;
     boxes = _XmScalePreferredSize(w, NULL, NULL, &wd, &ht);
     pwd = wd;
@@ -928,6 +964,7 @@ realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
     _XmScaleConfigureChildren(w, NULL, boxes);
 
     XtFree((char *)boxes);
+#endif
 }
 
 /*
@@ -1400,6 +1437,7 @@ _XmScalePreferredSize(Widget scale, Widget child, XtWidgetGeometry *childgeom,
 	    curh += inch;
 
 	    curx += incw;
+	    curh += 2 * MGR_ShadowThickness(scale);
 	}
     }
     /* XmNorientation == XmVERTICAL */
@@ -1509,15 +1547,28 @@ _XmScalePreferredSize(Widget scale, Widget child, XtWidgetGeometry *childgeom,
 			     2 * (Prim_HighlightThickness(boxes[CHILD_SB].kid) +
 				  Prim_ShadowThickness(boxes[CHILD_SB].kid))));
 		}
+		printf("curh %i %i %i %i %i %i %i\n",
+			spacing,
+			curh,
+			HSLIDERSIZE,
+			maxh,
+			num_children,
+			Prim_HighlightThickness(boxes[CHILD_SB].kid),
+			Prim_ShadowThickness(boxes[CHILD_SB].kid));
 
 		for (i = 2; i < num_children; i++)
 		{
 		    boxes[i].box.x = (maxw - boxes[i].box.width) / 2;
-		    boxes[i].box.y = (HSLIDERSIZE - boxes[i].box.height) / 2 +
+		    boxes[i].box.y = 1 + (HSLIDERSIZE - boxes[i].box.height) / 2 +
 			    Prim_HighlightThickness(boxes[CHILD_SB].kid) +
 			    Prim_ShadowThickness(boxes[CHILD_SB].kid) +
 			    (i - 2) * spacing / (num_children - 3);
 		}
+#if 1
+		/* scale/test13 ???? */
+		curh++;
+		boxes[CHILD_SB].box.height++;
+#endif
 	    }
 	    else
 	    {
@@ -1546,6 +1597,7 @@ _XmScalePreferredSize(Widget scale, Widget child, XtWidgetGeometry *childgeom,
 
 	    curw += incw;
 	    cury += inch;
+	    curw += 2 * MGR_ShadowThickness(scale);
 /*
 printf("COMPUTED %d %d : %d %d\n", curw, cury, incw, inch);
 */
@@ -1770,8 +1822,8 @@ computeValueSize(Widget w)
         maxlen = maxlen < 3 ? 3 : maxlen;
     }
  
-    Scale_ShowValueWidth(w) = Scale_FontStruct(w)->max_bounds.width * maxlen;
-    Scale_ShowValueHeight(w) = Scale_FontStruct(w)->max_bounds.ascent +
+    Scale_ShowValueWidth(w) = 2 + Scale_FontStruct(w)->max_bounds.width * maxlen;
+    Scale_ShowValueHeight(w) = 3 + Scale_FontStruct(w)->max_bounds.ascent +
         Scale_FontStruct(w)->max_bounds.descent;
 }
 
@@ -1908,6 +1960,7 @@ _ScaleDrag(Widget sb, XtPointer cd, XtPointer data)
     }
 }
 
+#if 0
 static void
 GetFocus(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
@@ -1933,6 +1986,7 @@ LoseFocus(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XtCallActionProc((Widget)sb, "PrimitiveFocusOut", event,
 		     params, *num_params);
 }
+#endif
 
 static int
 _XmScaleConvertWidthToSliderSize(Widget w)
@@ -2013,17 +2067,19 @@ _XmScaleConvertSCBValueToScaleValue(Widget w, int value)
 	return Scale_Minimum(w);
     }
 
-    return (int)((float)(Scale_Maximum(w) - Scale_Minimum(w)) /
-		 (float)(SCB_MAX - SCB_MIN - Scale_SliderSize(w)) *
-		 (float)value + (float)Scale_Minimum(w) );
+    return (int)((float)(Scale_Maximum(w) - Scale_Minimum(w)) 
+		* (float)value 
+		/ (float)(SCB_MAX - SCB_MIN - Scale_SliderSize(w))
+		+ (float)Scale_Minimum(w) );
 }
 
 static int
 _XmScaleConvertScaleValueToSCBValue(Widget w)
 {
-    return (int)((float)(Scale_Value(w) - Scale_Minimum(w)) /
-		 (float)(Scale_Maximum(w) - Scale_Minimum(w)) *
-		 (float)(SCB_MAX - SCB_MIN - Scale_SliderSize(w)));
+    return (int)((float)(Scale_Value(w) - Scale_Minimum(w)) 
+		* (float)(SCB_MAX - SCB_MIN - Scale_SliderSize(w))
+		/ (float)(Scale_Maximum(w) - Scale_Minimum(w)) 
+		);
 }
 
 static void
