@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Screen.c,v 1.10 1998/02/22 01:00:51 rwscott Exp $
+ * $Id: Screen.c,v 1.15 1998/10/14 21:38:26 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: Screen.c,v 1.10 1998/02/22 01:00:51 rwscott Exp $";
+static const char rcsid[] = "$Id: Screen.c,v 1.15 1998/10/14 21:38:26 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -36,6 +36,7 @@ static char rcsid[] = "$Id: Screen.c,v 1.10 1998/02/22 01:00:51 rwscott Exp $";
 #include <Xm/DisplayP.h>
 #include <Xm/ScreenP.h>
 #include <Xm/DragIconP.h>
+#include <Xm/RepType.h>
 #include <XmI/AtomMgrI.h>
 
 #include <XmI/DebugUtil.h>
@@ -73,8 +74,8 @@ static Boolean set_values(Widget current, Widget request, Widget new_w,
 static void insert_child(Widget w);
 static void delete_child(Widget w);
 
-static void _XmComputeFontUnits(Widget w, XFontStruct *font,
-				int *h_unit, int *v_unit);
+static void default_horizontal_font_unit(Widget w, int offset, XrmValue *val);
+static void default_vertical_font_unit(Widget w, int offset, XrmValue *val);
 
 /*
  * Resources for the primitive class
@@ -150,12 +151,12 @@ static XtResource resources[] =
     {
 	XmNhorizontalFontUnit, XmCHorizontalFontUnit, XmRInt,
 	sizeof(int), Offset(h_unit),
-	XmRImmediate, (XtPointer)XmUNSPECIFIED
+	XmRCallProc, (XtPointer)default_horizontal_font_unit
     },
     {
 	XmNverticalFontUnit, XmCVerticalFontUnit, XmRInt,
 	sizeof(int), Offset(v_unit),
-	XmRImmediate, (XtPointer)XmUNSPECIFIED
+	XmRCallProc, (XtPointer)default_vertical_font_unit
     },
     {
 	XmNmoveOpaque, XmCMoveOpaque, XmRBoolean,
@@ -278,6 +279,9 @@ initialize(Widget request, Widget new_w,
 {
     XtPointer FirstScreenWidget;
 
+    DEBUGOUT(XdbDebug(__FILE__, new_w, "%s:initialize(%d)\n", 
+    			__FILE__, __LINE__));
+
     /*
      * This is due to the %&"% children management mechanism needed for the
      * shell "shadow hierarchy". See Desktop.c for more details on this.
@@ -315,9 +319,6 @@ initialize(Widget request, Widget new_w,
 
     Screen_MwmPresent(new_w) = XmIsMotifWMRunning(new_w);
 
-    _XmComputeFontUnits(new_w, Screen_FontStruct(new_w),
-			&Screen_HorizUnit(new_w), &Screen_VertUnit(new_w));
-
     Screen_ScratchPixmaps(new_w) = NULL;
     Screen_ScreenInfo(new_w) = NULL;
     Screen_CursorCache(new_w) = NULL;
@@ -340,7 +341,8 @@ initialize(Widget request, Widget new_w,
 static void
 destroy(Widget w)
 {
-    XmScratchPixmap pix, tmp;
+    XmScratchPixmap	pix, tmp;
+    XmScreen		sw;
 
     XtFree((char *)Screen_Children(w));
     for (pix = Screen_ScratchPixmaps(w); pix != NULL; pix = tmp)
@@ -349,6 +351,14 @@ destroy(Widget w)
 	XFreePixmap(XtDisplay(w), pix->pixmap);
 	XtFree((char *)pix);
     }
+
+    if (XFindContext(XtDisplay(w), RootWindowOfScreen(XtScreen(w)),
+		     PerScreenWidgetContext, (XPointer *)&sw) == XCSUCCESS
+	&& sw == (XmScreen)w)
+	XDeleteContext(XtDisplay(w), RootWindowOfScreen(XtScreen(w)),
+		       PerScreenWidgetContext);
+
+    _XmInvalidateColorCache(False);
 }
 
 static void
@@ -412,6 +422,9 @@ realize(Widget w,
 	XtValueMask *value_mask,
 	XSetWindowAttributes *attributes)
 {
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:realize(%d)\n", 
+    			__FILE__, __LINE__));
+
 }
 
 static Boolean
@@ -420,89 +433,87 @@ set_values(Widget old, Widget request, Widget new_w,
 {
     Boolean need_refresh = False;
 
+    if (Screen_UnpostBehavior(new_w) != Screen_UnpostBehavior(old)
+	&& !XmRepTypeValidValue( XmRepTypeGetId(XmRUnpostBehavior),
+				 Screen_UnpostBehavior(new_w),
+				 new_w))
+	Screen_UnpostBehavior(new_w) = Screen_UnpostBehavior(old);
+
+    if ((Screen_DarkThreshold(old) != Screen_DarkThreshold(new_w) ||
+	 Screen_LightThreshold(old) != Screen_LightThreshold(new_w) ||
+	 Screen_ForegroundThreshold(old) != Screen_ForegroundThreshold(new_w)))
+	_XmInvalidateColorCache(True);
+
+    if (Screen_FontStruct(new_w) != Screen_FontStruct(old))
+    {
+	XrmValue value;
+
+	if (Screen_HorizUnit(new_w) == Screen_HorizUnit(old))
+	{
+	    default_horizontal_font_unit(new_w, 0, &value);
+	    Screen_HorizUnit(new_w) = *(int *)value.addr;
+	}
+	if (Screen_VertUnit(new_w) == Screen_VertUnit(old))
+	{
+	    default_vertical_font_unit(new_w, 0, &value);
+	    Screen_VertUnit(new_w) = *(int *)value.addr;
+	}
+    }
+
     return need_refresh;
 }
 
-/*
- * helper function for font_unit stuff
- */
 static void
-_XmComputeFontUnits(Widget w, XFontStruct *font, int *h_unit, int *v_unit)
+default_horizontal_font_unit(Widget w, int offset, XrmValue *val)
 {
-    XFontProp *props;
-    int i, nprops;
-    Boolean got1;
+    unsigned long prop;
     Atom atom;
+    XFontStruct *font;
+    static int h_unit;
+    
 
-    if (font == NULL)
-    {
-	*h_unit = 10;
-	*v_unit = 10;
-	return;
-    }
+    val->addr = (XPointer)&h_unit;
+    val->size = sizeof h_unit;
+    h_unit =
+	/* No font specified: 10 */
+	!(font = Screen_FontStruct(w))
+	? 10
+	/* AVERAGE_WIDTH / 10 (if it exists) */
+	: ((atom = XmInternAtom(XtDisplay(w), _XA_AVERAGE_WIDTH, True))
+	   && XGetFontProperty(font, atom, &prop))
+	? (prop + 5) / 10
+	/* QUAD_WIDTH (if it exists) */
+	: XGetFontProperty(font, XA_QUAD_WIDTH, &prop)
+	? prop
+	/* (max + min) / 2.3 */
+	: ((font->min_bounds.width + font->max_bounds.width) * 10 + 11) / 23;
+}
 
-    props = font->properties;
-    nprops = font->n_properties;
+static void
+default_vertical_font_unit(Widget w, int offset, XrmValue *val)
+{
+    unsigned long prop, prop2;
+    Atom atom;
+    XFontStruct *font;
+    static int v_unit;
 
-    /* we calculate the value in reverse order as specified the programmer's
-     * reference; we let the higher precedence rules to override the lower
-     * ones. */
-    /* HUNITS */
-    *h_unit = (int)((font->min_bounds.width + font->max_bounds.width) / 2.3);
-    for (i = 0; i < nprops; i++)
-    {
-	if (props[i].name == XA_QUAD_WIDTH)
-	{
-	    *h_unit = props[i].card32;
-	    break;
-	}
-    }
-
-    atom = XInternAtom(XtDisplay(w), _XA_AVERAGE_WIDTH, False);
-    for (i = 0; i < nprops; i++)
-    {
-	if (props[i].name == atom)
-	{
-	    *h_unit = props[i].card32 / 10;
-	    break;
-	}
-    }
-
-    /* VUNITS */
-    *v_unit = (int)((font->ascent + font->descent) / 2.2);
-    got1 = False;
-    for (i = 0; i < nprops; i++)
-    {
-	if (props[i].name == XA_POINT_SIZE)
-	{
-	    got1 = True;
-	    *v_unit = props[i].card32;
-	    break;
-	}
-    }
-
-    if (got1)
-    {
-	atom = XInternAtom(XtDisplay(w), _XA_RESOLUTION_Y, False);
-	for (i = 0; i < nprops; i++)
-	{
-	    if (props[i].name == atom)
-	    {
-		*v_unit = (*v_unit * props[i].card32) / 1400;
-		break;
-	    }
-	}
-    }
-
-    atom = XInternAtom(XtDisplay(w), _XA_PIXEL_SIZE, False);
-    for (i = 0; i < nprops; i++)
-    {
-	if (props[i].name == atom)
-	{
-	    *v_unit = (int)(props[i].card32 / 1.8);
-	    break;
-	}
-    }
+    val->addr = (XPointer)&v_unit;
+    val->size = sizeof v_unit;
+    v_unit =
+	/* No font specified: 10 */
+	!(font = Screen_FontStruct(w))
+	? 10
+	/* PIXEL_SIZE / 1.8 (if it exists) */
+	: ((atom = XmInternAtom(XtDisplay(w), _XA_PIXEL_SIZE, True))
+	   && XGetFontProperty(font, atom, &prop))
+	? (prop * 10 + 9) / 18
+	/* POINT_SIZE * RESOLUTION_Y / 1400 (if they exist) */
+	: (XGetFontProperty(font, XA_POINT_SIZE, &prop)
+	   && (atom = XmInternAtom(XtDisplay(w), _XA_RESOLUTION_Y, True))
+	   && XGetFontProperty(font, atom, &prop2))
+	? (prop * prop2 + 700) / 1400
+	/* (ascent + descent) / 2.2 */
+	: ((font->ascent + font->descent) * 10 + 11) / 22;
 }
 
 XmDragIconObject
@@ -654,27 +665,38 @@ _XmAllocScratchPixmap(XmScreen xmScreen,
 {
     XmScratchPixmap pix = NULL;
 
+    DEBUGOUT(XdbDebug(__FILE__, (Widget)xmScreen, "%s:_XmAllocScratchPixmap(%d) - %dx%dx%d\n", 
+    			__FILE__, __LINE__,
+    			width, height, depth));
+
     for (pix = Screen_ScratchPixmaps(xmScreen); pix != NULL; pix = pix->next)
     {
 	if (!pix->inUse && pix->depth == depth &&
 	    pix->width == width && pix->height == height)
 	{
 	    pix->inUse = True;
-	    return pix->pixmap;
+	    DEBUGOUT(XdbDebug(__FILE__, (Widget)xmScreen, "\tfrom cache %p\n", 
+    			pix->pixmap));
+	    break;
 	}
     }
 
-    pix = (XmScratchPixmap)XtMalloc(sizeof(XmScratchPixmapRec));
-    pix->inUse = True;
-    pix->depth = depth;
-    pix->width = width;
-    pix->height = height;
-    pix->pixmap = XCreatePixmap(XtDisplay((Widget)xmScreen),
-				RootWindowOfScreen(XtScreen(xmScreen)),
-				width, height, depth);
-    pix->next = Screen_ScratchPixmaps(xmScreen);
-    Screen_ScratchPixmaps(xmScreen) = pix;
+    if (pix == NULL)
+    {
+	pix = (XmScratchPixmap)XtMalloc(sizeof(XmScratchPixmapRec));
+	pix->inUse = True;
+	pix->depth = depth;
+	pix->width = width;
+	pix->height = height;
+	pix->pixmap = XCreatePixmap(XtDisplay((Widget)xmScreen),
+				    RootWindowOfScreen(XtScreen(xmScreen)),
+				    width, height, depth);
+	pix->next = Screen_ScratchPixmaps(xmScreen);
+	Screen_ScratchPixmaps(xmScreen) = pix;
 
+	DEBUGOUT(XdbDebug(__FILE__, (Widget)xmScreen, "\tnew %p\n", 
+			    pix->pixmap));
+    }
     return pix->pixmap;
 }
 
@@ -683,10 +705,21 @@ _XmFreeScratchPixmap(XmScreen xmScreen, Pixmap pixmap)
 {
     XmScratchPixmap pix;
 
+    DEBUGOUT(XdbDebug(__FILE__, (Widget)xmScreen, "%s:_XmFreeScratchPixmap(%d) - %p\n", 
+    			__FILE__, __LINE__,
+    			pixmap));
     for (pix = Screen_ScratchPixmaps(xmScreen); pix != NULL; pix = pix->next)
     {
 	if (pix->pixmap == pixmap)
+	{
 	    pix->inUse = False;
+	    break;
+	}
+    }
+    if (pix == NULL)
+    {
+    	_XmWarning((Widget)xmScreen, "%s:_XmFreeScratchPixmap(%d) - Trying to free pixmap %p\n    that is not in the cache",
+    			__FILE__, __LINE__, pixmap);
     }
 }
 
@@ -869,6 +902,9 @@ XmGetXmScreen(Screen *scr)
     char name[128];
     int screen_no, max_screens;
 
+    DEBUGOUT(XdbDebug0(__FILE__, (Widget)NULL, "%s:XmGetXmScreen(%d)\n", 
+    			__FILE__, __LINE__));
+
     disp = XmGetXmDisplay(DisplayOfScreen(scr));
 
     if ((PerScreenWidgetContext == PSWC_None) ||
@@ -904,11 +940,3 @@ XmGetXmScreen(Screen *scr)
 
     return screen;
 }
-
-void
-_XmSetDefaultBackgroundColorSpec(Screen *screen, String string)
-{
-    DEBUGOUT(XdbDebug(__FILE__, NULL,
-		   "_XmSetDefaultBackgroundColorSpec(%s) is not implemented\n",
-		      string));
-}
