@@ -1,6 +1,6 @@
 /*
  *
- * $Id: ScrolledW.c,v 1.78 1998/05/27 18:52:23 u27113 Exp $
+ * $Id: ScrolledW.c,v 1.100 1998/10/24 03:18:41 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ScrolledW.c,v 1.78 1998/05/27 18:52:23 u27113 Exp $";
+static const char rcsid[] = "$Id: ScrolledW.c,v 1.100 1998/10/24 03:18:41 rwscott Exp $";
 
 /*
  * Current status of changes in layout (Danny 18/4/1997) :
@@ -55,7 +55,10 @@ static const char rcsid[] = "$Id: ScrolledW.c,v 1.78 1998/05/27 18:52:23 u27113
    problem can be seen if you start it and then select a directory from the
    middle list.  At this point the list on the right gets real tiny.
  */
-#undef MFM_BUGS
+/* rws 2 Jul 1998
+   With this undefined ddd 3.x gets into an infinite loop.
+ */
+#define MFM_BUGS
 
 /* Forward Declarations */
 
@@ -447,6 +450,17 @@ static void
 initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "%s:initialize: %i args\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__, *num_args,
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+
     if (SW_ScrollBarPolicy(new_w) == (unsigned char)XmUNSPECIFIED)
     {
 	if (SW_ScrollPolicy(new_w) == XmAUTOMATIC)
@@ -485,10 +499,26 @@ initialize(Widget request, Widget new_w,
 	}
     }
 
+    if (SW_VisualPolicy(new_w) == XmCONSTANT)
+    {
+	XtTranslations tr;
+
+	SW_ClipWindow(new_w) =
+	    (XmDrawingAreaWidget)XtCreateManagedWidget("clipWindow",
+				     xmDrawingAreaWidgetClass, new_w, NULL, 0);
+
+	tr = XtParseTranslationTable(_XmScrolledW_ClipWindowTranslationTable);
+	XtOverrideTranslations((Widget)SW_ClipWindow(new_w), tr);
+    }
+    else if (SW_VisualPolicy(new_w) != XmVARIABLE)
+    {
+	SW_VisualPolicy(new_w) = XmVARIABLE;
+    }
+
     if (SW_ScrollPolicy(new_w) == XmAUTOMATIC)
     {
 	SW_VSB(new_w) =
-	    (XmScrollBarWidget)XtVaCreateWidget("VertScrollBar",
+	    (XmScrollBarWidget)XtVaCreateManagedWidget("VertScrollBar",
 						xmScrollBarWidgetClass, new_w,
 						XmNorientation, XmVERTICAL,
 						XmNsliderSize, 100,
@@ -533,7 +563,7 @@ initialize(Widget request, Widget new_w,
 		      NULL);
 
 	SW_HSB(new_w) =
-	    (XmScrollBarWidget)XtVaCreateWidget("HorScrollBar",
+	    (XmScrollBarWidget)XtVaCreateManagedWidget("HorScrollBar",
 						xmScrollBarWidgetClass, new_w,
 						XmNorientation, XmHORIZONTAL,
 						XmNsliderSize, 100,
@@ -657,22 +687,6 @@ initialize(Widget request, Widget new_w,
     SW_CWX(new_w) = 0;
     SW_CWY(new_w) = 0;
 
-    if (SW_VisualPolicy(new_w) == XmCONSTANT)
-    {
-	XtTranslations tr;
-
-	SW_ClipWindow(new_w) =
-	    (XmDrawingAreaWidget)XtCreateManagedWidget("clipWindow",
-				     xmDrawingAreaWidgetClass, new_w, NULL, 0);
-
-	tr = XtParseTranslationTable(_XmScrolledW_ClipWindowTranslationTable);
-	XtOverrideTranslations((Widget)SW_ClipWindow(new_w), tr);
-    }
-    else if (SW_VisualPolicy(new_w) != XmVARIABLE)
-    {
-	SW_VisualPolicy(new_w) = XmVARIABLE;
-    }
-
     if (SW_VisualPolicy(new_w) == XmVARIABLE &&
 	SW_ScrollBarPolicy(new_w) != XmSTATIC)
     {
@@ -830,6 +844,11 @@ set_values(Widget old, Widget request, Widget new_w,
 			   : (SW_ScrollPolicy(new_w) == XmAUTOMATIC)
 			   ? "XmAUTOMATIC"
 			   : "???"));
+	/* rws 6 Jun 1998
+	   If we change the work area we definitly need to refresh the
+	   display.  This showed as 1x1 <textarea>'s in Mozilla
+	 */
+	r = True;
     }
 
     if (XtWidth(new_w) != XtWidth(old))
@@ -842,7 +861,12 @@ set_values(Widget old, Widget request, Widget new_w,
     }
     
     /* init chained from super to sub order */
-    if (r)
+    /* rws 6 Jun 1998
+       If we are not realized do not do all of this geo stuff.  It will be
+       done when we get realized.  This ties in with the Mozilla and ml stuff
+       above.
+     */
+    if (r && XtIsRealized(new_w))
     {
 	XmSWValues vals;
 
@@ -853,8 +877,12 @@ set_values(Widget old, Widget request, Widget new_w,
 	   put back to the original before the geom request. This shows up
 	   in ml when trying to read a message.
 	 */
+	/* rws 11 Sep 1998
+	   Ok, let's get rid of this!  It is causing an X proto error
+	   in xmgr File->Describe.
 	vals.SwW = XtWidth(new_w);
 	vals.SwH = XtHeight(new_w);
+	*/
 
 	_XmScrolledWGeomRequest(new_w, &vals);
 
@@ -910,10 +938,11 @@ realize(Widget w, Mask *value_mask, XSetWindowAttributes *attributes)
 
     DEBUGOUT(XdbDebug(__FILE__, w, "Realize\n"));
 
-#define superclass (&xmManagerClassRec)
-    (*superclass->core_class.realize) (w, value_mask, attributes);
-#undef superclass
-
+#if 1
+    /* rws 23 Oct 1998
+       This stuff should not be here.  It is handled in change_managed, which
+       is call just before realize
+     */
     /* not chained */
     if (XtWidth(w) != 0 && XtWidth(w) != 1)
     {
@@ -931,6 +960,11 @@ realize(Widget w, Mask *value_mask, XSetWindowAttributes *attributes)
     _XmScrolledWLayout(w, NULL, NULL, &vals);
 
     _XmScrolledWConfigureChildren(w, NULL, NULL, &vals);
+#endif
+
+#define superclass (&xmManagerClassRec)
+    (*superclass->core_class.realize) (w, value_mask, attributes);
+#undef superclass
 }
 
 /*
@@ -944,6 +978,7 @@ query_geometry(Widget w,
     XmSWValues vals;
     XtWidgetGeometry wants;
 
+#if 0
     DEBUGOUT(XdbDebug(__FILE__, w, "query_geometry(%s)\n",
 		      XdbWidgetGeometry2String(intended)));
 
@@ -994,6 +1029,28 @@ query_geometry(Widget w,
     DEBUGOUT(XdbDebug(__FILE__, w, "query_geometry => YES\n"));
 
     return XtGeometryYes;	/* I'm happy */
+#else
+    DEBUGOUT(XdbDebug(__FILE__, w, "query_geometry(%s)\n",
+		      XdbWidgetGeometry2String(intended)));
+
+    wants = *intended;
+
+    /* not chained */
+    _XmScrolledWPreferredSize(w, NULL, NULL, &vals);
+
+    /*
+    if (vals.SwW < 100) vals.SwW = 100;
+    if (vals.SwH < 100) vals.SwH = 100;
+    */
+
+    preferred->width = vals.SwW;
+    preferred->height = vals.SwH;
+
+    preferred->width = wants.request_mode & CWWidth ? wants.width : vals.SwW;
+    preferred->height = wants.request_mode & CWHeight ? wants.height : vals.SwH;
+
+    return _XmGMReplyToQueryGeometry(w, &wants, preferred);
+#endif
 }
 
 static XtGeometryResult
@@ -1050,12 +1107,14 @@ geometry_manager(Widget w, XtWidgetGeometry *desired, XtWidgetGeometry *allowed)
 	 */
 	_XmScrolledWPreferredSize(sw, SW_WorkWindow(sw), &wants, &vals);
 
-	_XmScrolledWLayout(sw, w, &wants, &vals);
+	_XmScrolledWLayout(sw, SW_WorkWindow(sw), &wants, &vals);
 
 	_XmConfigureScrollBars(sw, NULL, NULL, &vals);
 
 	_XmFixupScrollBars(sw, vals.WorkW, vals.WorkH);
 
+	_XmScrolledWConfigureChildren(sw, SW_WorkWindow(sw), &wants, &vals);
+
 	/* rws 23 Aug 1997
 	 * Taking this out gets the initial display of mfm correct and
 	 * does not seem to bother anything else
@@ -1159,7 +1218,9 @@ change_managed(Widget w)
 {
     XmSWValues vals;
 
-    if (XtIsManaged(w))
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:change_managed(%d)\n",
+			  __FILE__, __LINE__));
+    if (True || XtIsManaged(w))
     {
 	_XmScrolledWPreferredSize(w, NULL, NULL, &vals);
 
@@ -1274,6 +1335,11 @@ insert_child(Widget w)
     else if (SW_VisualPolicy(sw) == XmCONSTANT && SW_ClipWindow(sw) == NULL)
     {
 	(*superclass->composite_class.insert_child) (w);
+	/*
+	 * Stefan Birgersson <birger@musculus.mednet.gu.se> suggests to
+	 * assign the clip window here to aid in MainWindow.
+	 */
+	SW_ClipWindow(sw) = (XmDrawingAreaWidget)w;
     }
     /*
      * THIS IS SOO UGLY
@@ -1283,7 +1349,7 @@ insert_child(Widget w)
      */
     else if (SW_VisualPolicy(sw) == XmCONSTANT)
     {
-	DEBUGOUT(XdbDebug2(__FILE__, w, (Widget)sw, "Reparented to %s\n",
+	DEBUGOUT(XdbDebug2(__FILE__, (Widget)sw, (Widget)w, "Reparented to %s\n",
 			   XtName(SW_ClipWindow(sw))));
 
 	XtParent(w) = (Widget)SW_ClipWindow(sw);
@@ -1691,7 +1757,7 @@ _XmScrolledWPreferredSize(Widget w, Widget child, XtWidgetGeometry *childgeom,
     vals->ClipX = vals->ClipY = vals->WorkX = vals->WorkY = 0;
     vals->ClipW = vals->ClipH = vals->WorkW = vals->WorkH = 0;
 
-    if (SW_WorkWindow(w))
+    if (SW_WorkWindow(w) && XtIsManaged(SW_WorkWindow(w)))
     {
 	vals->WorkX = 0;
 	vals->WorkY = 0;
@@ -1701,13 +1767,13 @@ _XmScrolledWPreferredSize(Widget w, Widget child, XtWidgetGeometry *childgeom,
 	    XtWidgetGeometry workgeo;
 
 	    XtQueryGeometry(SW_WorkWindow(w), NULL, &workgeo);
-	    vals->WorkW = workgeo.width;
-	    vals->WorkH = workgeo.height;
+	    vals->WorkW = workgeo.width + 2 * workgeo.border_width;
+	    vals->WorkH = workgeo.height + 2 * workgeo.border_width;
 	}
 	else
 	{
-	    vals->WorkW = XtWidth(SW_WorkWindow(w));
-	    vals->WorkH = XtHeight(SW_WorkWindow(w));
+	    vals->WorkW = XtWidth(SW_WorkWindow(w)) + 2 * XtBorderWidth(SW_WorkWindow(w));
+	    vals->WorkH = XtHeight(SW_WorkWindow(w)) + 2 * XtBorderWidth(SW_WorkWindow(w));
 	}
 
 	DEBUGOUT(XdbDebug2(__FILE__, w, (Widget)SW_WorkWindow(w),
@@ -1718,8 +1784,8 @@ _XmScrolledWPreferredSize(Widget w, Widget child, XtWidgetGeometry *childgeom,
     {
 	vals->WorkX = 0;
 	vals->WorkY = 0;
-	vals->WorkW = 0;
-	vals->WorkH = 0;
+	vals->WorkW = 100;
+	vals->WorkH = 100;
     }
 
     /*
@@ -1727,8 +1793,16 @@ _XmScrolledWPreferredSize(Widget w, Widget child, XtWidgetGeometry *childgeom,
      */
     if (SW_VisualPolicy(w) == XmCONSTANT)
     {
+#if 0
+	/* rws 22 Oct 1998
+	   scrolledwindow/test6 test10 messagebox/test13
+	 */
 	vals->SwW = XtWidth(w);
 	vals->SwH = XtHeight(w);
+#else
+	vals->SwW = XtWidth(w) == 0 ? 100 : XtWidth(w);
+	vals->SwH = XtHeight(w) == 0 ? 100 : XtHeight(w);
+#endif
     }
     else
     {
@@ -1944,7 +2018,14 @@ _XmScrolledWGeomRequest(Widget w, XmSWValues *vals)
     request.width = vals->SwW;
     request.height = vals->SwH;
 
-#if 1
+#if 0
+/* rws 22 Oct 1998
+   list/test15 shows that this crap should not be here.  This also makes sense
+   since if we call _XmScrolledWGeomRequest with a request we should not be
+   altering it!  test15 shows that we are getting in here with the correct
+   values, but this is changing them.  If they need to be changed in some
+   situations _this is not the place to do it_.
+ */
 #ifdef MFM_BUGS
     if (SW_GivenWidth(w) != 0)
 #else
@@ -2087,9 +2168,9 @@ _XmScrolledWLayout(Widget w, Widget child, XtWidgetGeometry *childgeom,
     {
 	if (vals->ShowVSB && vals->HasVSB)
 	{
-	    vals->ClipW = vals->SwW - bw - 2 * MGR_ShadowThickness(w)
+	    vals->ClipW = vals->SwW - 2 * bw - 2 * MGR_ShadowThickness(w)
 		    - SW_Spacing(w) - vals->VsbW - 2 * SW_MarginWidth(w);
-	    vals->ClipH = vals->SwH - bh - 2 * MGR_ShadowThickness(w)
+	    vals->ClipH = vals->SwH - 2 * bh - 2 * MGR_ShadowThickness(w)
 		    - SW_Spacing(w) - vals->HsbH - 2 * SW_MarginHeight(w);
 
 	    vals->VsbH = vals->ClipH + 2 * bh + 2 * MGR_ShadowThickness(w);
@@ -2097,9 +2178,10 @@ _XmScrolledWLayout(Widget w, Widget child, XtWidgetGeometry *childgeom,
 	}
 	else
 	{
+	    vals->ClipY -= bh;
 	    vals->ClipW = vals->SwW - 2 * bw - 2 * MGR_ShadowThickness(w);
-	    vals->ClipH = vals->SwH - 2 * bh - 2 * MGR_ShadowThickness(w)
-		    - SW_Spacing(w) - vals->HsbH;
+	    vals->ClipH = vals->SwH - 1 * bh - 2 * MGR_ShadowThickness(w)
+		    - SW_Spacing(w) - vals->HsbH - 2 * SW_MarginHeight(w);
 
 	    vals->VsbH = vals->ClipH + 2 * MGR_ShadowThickness(w) + 2 * bh;
 	    vals->HsbW = vals->ClipW + 2 * MGR_ShadowThickness(w) + 2 * bw;
@@ -2108,8 +2190,9 @@ _XmScrolledWLayout(Widget w, Widget child, XtWidgetGeometry *childgeom,
     else if (vals->ShowVSB && vals->HasVSB)
     {
 	vals->ClipW = vals->SwW - 2 * bw - 2 * MGR_ShadowThickness(w)
-		- SW_Spacing(w) - vals->VsbW;
-	vals->ClipH = vals->SwH - 2 * bh - 2 * MGR_ShadowThickness(w);
+		- SW_Spacing(w) - vals->VsbW - 2 * SW_MarginWidth(w);
+	vals->ClipH = vals->SwH - 2 * bh - 2 * MGR_ShadowThickness(w)
+	    - 2 * SW_MarginHeight(w);
 
 	vals->VsbH = vals->ClipH + 2 * MGR_ShadowThickness(w) + 2 * bh;
 	vals->HsbW = vals->ClipW + 2 * MGR_ShadowThickness(w) + 2 * bw;
@@ -2253,8 +2336,12 @@ void
 _XmConfigureScrollBars(Widget w, Widget child, XtWidgetGeometry *childgeom,
 		    XmSWValues *vals)
 {
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:_XmConfigureScrollBars(%d)\n",
+    	__FILE__, __LINE__));
     /* Don't display scrollbars unless the child is managed */
-    if (SW_WorkWindow(w) && ! XtIsManaged(SW_WorkWindow(w))) {
+    if (SW_WorkWindow(w) && ! XtIsManaged(SW_WorkWindow(w)))
+    {
+#if 0
 	if (SW_VSB(w) != NULL)
 	{
 	    XtUnmanageChild((Widget)SW_VSB(w));
@@ -2263,6 +2350,7 @@ _XmConfigureScrollBars(Widget w, Widget child, XtWidgetGeometry *childgeom,
 	{
 	    XtUnmanageChild((Widget)SW_HSB(w));
 	}
+#endif
 	SW_HasVSB(w) = False;
 	SW_HasHSB(w) = False;
 	return;
@@ -2284,8 +2372,14 @@ _XmConfigureScrollBars(Widget w, Widget child, XtWidgetGeometry *childgeom,
 		DEBUGOUT(XdbDebug2(__FILE__, w, (Widget)SW_VSB(w),
 				   "Unmanaging VSB\n"));
 
+		vals->VsbX = vals->SwW;
+		vals->VsbH = vals->SwH;
+		/* rws 12 Oct 1998
+		   scrolledwindow/test10 shows that these do not get UnManaged
+		   just pushed off to the side
 		XtUnmanageChild((Widget)SW_VSB(w));
 		SW_HasVSB(w) = False;
+		*/
 	    }
 	}
 
@@ -2317,7 +2411,7 @@ _XmConfigureScrollBars(Widget w, Widget child, XtWidgetGeometry *childgeom,
 		DEBUGOUT(XdbDebug2(__FILE__, w, (Widget)SW_VSB(w),
 				   "Managing VSB\n"));
 
-		if (XtIsRealized(w))
+		if (True || XtIsRealized(w))
 		{
 		    XtManageChild((Widget)SW_VSB(w));
 		    SW_HasVSB(w) = True;
@@ -2339,8 +2433,14 @@ _XmConfigureScrollBars(Widget w, Widget child, XtWidgetGeometry *childgeom,
 		DEBUGOUT(XdbDebug2(__FILE__, w, (Widget)SW_HSB(w),
 				   "Unmanaging HSB\n"));
 
+		vals->HsbY = vals->SwH;
+		vals->HsbW = vals->SwW;
+		/* rws 12 Oct 1998
+		   scrolledwindow/test10 shows that these do not get UnManaged
+		   just pushed off to the side
 		XtUnmanageChild((Widget)SW_HSB(w));
 		SW_HasHSB(w) = False;
+		*/
 	    }
 	}
 
@@ -2372,7 +2472,7 @@ _XmConfigureScrollBars(Widget w, Widget child, XtWidgetGeometry *childgeom,
 		DEBUGOUT(XdbDebug2(__FILE__, w, (Widget)SW_HSB(w),
 				   "Managing HSB\n"));
 
-		if (XtIsRealized(w))
+		if (True || XtIsRealized(w))
 		{
 		    XtManageChild((Widget)SW_HSB(w));
 		    SW_HasHSB(w) = True;
@@ -2436,20 +2536,20 @@ _XmScrolledWConfigureChildren(Widget w, Widget child,
 	SW_CWWidth(w) = vals->ClipW;
 	SW_CWHeight(w) = vals->ClipH;
     }
-    if (SW_WorkWindow(w))
+    if (SW_WorkWindow(w) && XtIsManaged(SW_WorkWindow(w)))
     {
 	if (child == SW_WorkWindow(w))
 	{
 	    XtX(SW_WorkWindow(w)) = vals->WorkX;
 	    XtY(SW_WorkWindow(w)) = vals->WorkY;
-	    XtWidth(SW_WorkWindow(w)) = vals->WorkW;
-	    XtHeight(SW_WorkWindow(w)) = vals->WorkH;
+	    XtWidth(SW_WorkWindow(w)) = vals->WorkW - 2 * XtBorderWidth(SW_WorkWindow(w));
+	    XtHeight(SW_WorkWindow(w)) = vals->WorkH - 2 * XtBorderWidth(SW_WorkWindow(w));
 	}
 	else
 	{
 	    _XmConfigureObject(SW_WorkWindow(w),
 			       vals->WorkX, vals->WorkY,
-			       vals->WorkW, vals->WorkH,
+			       vals->WorkW - 2 * XtBorderWidth(SW_WorkWindow(w)), vals->WorkH - 2 * XtBorderWidth(SW_WorkWindow(w)),
 			       XtBorderWidth(SW_WorkWindow(w)));
 	}
 
@@ -2745,10 +2845,10 @@ XmScrolledWindowSetAreas(Widget widget,
 			 Widget v_scrollbar,
 			 Widget work_region)
 {
-    /* color computation */
-    XColor _widgetBackground;
-    XColor troughColor;
- 
+    DEBUGOUT(XdbDebug(__FILE__, widget, "XmScrolledWindowSetAreas() - %s %s %s\n",
+		      h_scrollbar ? XtName(h_scrollbar) : "NULL",
+		      v_scrollbar ? XtName(v_scrollbar) : "NULL",
+		      work_region ? XtName(work_region) : "NULL"));
     if (h_scrollbar)
     {
 	XtVaSetValues(widget,
@@ -2767,51 +2867,6 @@ XmScrolledWindowSetAreas(Widget widget,
 		  XmNworkWindow, work_region,
 		  NULL);
     }
-
-    if (h_scrollbar || v_scrollbar)
-    {
-	_widgetBackground.pixel = XtBackground(widget);
- 
-	XQueryColor(XtDisplay(widget),
-		    CoreColormap(widget),
-		    &_widgetBackground);
- 
-	troughColor.blue = _widgetBackground.blue * .80;
-	troughColor.green = _widgetBackground.green * .80;
-	troughColor.red = _widgetBackground.red * .80;
- 
-	if (!XAllocColor(XtDisplay(widget),
-			 CoreColormap(widget),
-			 &troughColor))
-	{
-	    troughColor.pixel = _XmWhitePixelOfObject(widget);
-	}
-    }
- 
-    if (h_scrollbar)
-    {
-	XtVaSetValues(h_scrollbar,
-		      XmNforeground, MGR_Foreground(widget),
-		      XmNbackground, XtBackground(widget),
-		      XmNtroughColor, troughColor.pixel,
-		      XmNtopShadowColor, MGR_TopShadowColor(widget),
-		      XmNtopShadowPixmap, MGR_TopShadowPixmap(widget),
-		      XmNbottomShadowColor, MGR_BottomShadowColor(widget),
-		      XmNbottomShadowPixmap, MGR_BottomShadowPixmap(widget),
-		      NULL);
-    }
-    if (v_scrollbar)
-    {
-	XtVaSetValues(v_scrollbar,
-		      XmNforeground,
-		      MGR_Foreground(widget),
-		      XmNbackground,
-		      XtBackground(widget),
-		      XmNtroughColor, troughColor.pixel,
-		      XmNtopShadowColor, MGR_TopShadowColor(widget),
-		      XmNtopShadowPixmap, MGR_TopShadowPixmap(widget),
-		      XmNbottomShadowColor, MGR_BottomShadowColor(widget),
-		      XmNbottomShadowPixmap, MGR_BottomShadowPixmap(widget),
-		      NULL);
-    }
 }
+
+
