@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/SelectionBox.c,v 1.28 1998/04/15 00:20:03 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/SelectionBox.c,v 1.37 1998/10/23 05:15:52 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,10 +22,11 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/SelectionBox.c,v 1.28 1998/04/15 00:20:03 rwscott Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/SelectionBox.c,v 1.37 1998/10/23 05:15:52 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
+#include <XmI/MacrosI.h>
 
 #include <Xm/XmP.h>
 #include <Xm/SelectioBP.h>
@@ -34,6 +35,7 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/SelectionBo
 #include <Xm/PushBP.h>
 #include <Xm/PushBGP.h>
 #include <Xm/RowColumnP.h>
+#include <Xm/TransltnsP.h>
 #ifdef USE_WIDGETS
 #include <Xm/Separator.h>
 #else
@@ -43,6 +45,8 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/SelectionBo
 #include <Xm/TextF.h>
 
 #include <XmI/DebugUtil.h>
+#include <stdio.h>
+#include <stdlib.h>
 
 /* Forward Declarations */
 
@@ -61,6 +65,10 @@ static Boolean set_values(Widget current, Widget request, Widget new_w,
 static void insert_child(Widget w);
 static void DeleteChild(Widget w);
 
+
+static void
+_XmDialogTypeDefault(Widget w, int offset, XrmValue *value);
+
 XmGeoMatrix _XmSelectionBoxGeoMatrixCreate(Widget _w, Widget _from,
 					   XtWidgetGeometry *_pref);
 
@@ -78,6 +86,7 @@ void _XmSelectionBoxGetTextColumns(Widget wid, int resource_offset,
 void _XmSelectionBoxGetTextString(Widget wid, int resource_offset,
 				  XtArgVal *value);
 
+static XtAccelerators text_accelerators_parsed = NULL;
 
 /*
  * Resources for the Selection Box class
@@ -178,12 +187,7 @@ static XtResource resources[] =
     {
 	XmNdialogType, XmCDialogType, XmRSelectionType,
 	sizeof(unsigned char), Offset(dialog_type),
-	XmRCallProc, (XtPointer)NULL	/* FIX ME: NEED PROC HERE */
-    },
-    {
-	XmNshadowThickness, XmCShadowThickness, XmRHorizontalDimension,
-	sizeof(Dimension), XtOffsetOf(XmSelectionBoxRec, manager.shadow_thickness),
-	XmRCallProc, (XtPointer)_XmPrimitiveShadowThicknessDefault
+	XmRCallProc, (XtPointer)_XmDialogTypeDefault
     },
     {
 	XmNchildPlacement, XmCChildPlacement, XmRChildPlacement,
@@ -250,6 +254,15 @@ static XmSyntheticResource syn_resources[] =
 	_XmExportXmString, NULL
     }
 };
+
+
+static XtActionsRec actions[] =
+{
+	{"UpOrDown", _XmSelectionBoxUpOrDown},
+	{"SelectionBoxUpOrDown", _XmSelectionBoxUpOrDown},
+	{"SelectionBoxRestore", _XmSelectionBoxRestore},
+};
+
 /* *INDENT-OFF* */
 #if 0
 static XmBaseClassExtRec _XmSelectionBCoreClassExtRec = {
@@ -299,8 +312,8 @@ XmSelectionBoxClassRec xmSelectionBoxClassRec = {
 	/* initialize            */ initialize,
 	/* initialize_hook       */ NULL,
 	/* realize               */ XtInheritRealize,
-	/* actions               */ NULL,
-	/* num_actions           */ 0,
+	/* actions               */ actions,
+	/* num_actions           */ XtNumber(actions),
 	/* resources             */ resources,
 	/* num_resources         */ XtNumber(resources),
 	/* xrm_class             */ NULLQUARK,
@@ -373,6 +386,11 @@ WidgetClass xmSelectionBoxWidgetClass = (WidgetClass)&xmSelectionBoxClassRec;
 static void
 class_initialize()
 {
+    if (text_accelerators_parsed == NULL)
+    {
+	text_accelerators_parsed = 
+	    XtParseAcceleratorTable(_XmSelectioB_defaultTextAccelerators);
+    }
 #if 0
     _XmSelectionBCoreClassExtRec.record_type = XmQmotif;
 #endif
@@ -491,7 +509,10 @@ _XmSbButton(Widget w, XtPointer client, XtPointer call)
     {
 	if (w == SB_OkButton(sb) || XmIsTextField(w))
 	{
-	    char *s = XmTextFieldGetString(SB_Text(sb));
+	    char *s = NULL;
+
+	    if (SB_Text(sb))
+		s = XmTextFieldGetString(SB_Text(sb));
 
 	    cbs.value = XmStringCreateSimple(s);
 	    XtFree(s);
@@ -538,7 +559,9 @@ _XmSbButton(Widget w, XtPointer client, XtPointer call)
 	/* rws 25 Apr 1997 value and length added 
 	   should a check for match be done??
 	 */
-	char *s = XmTextFieldGetString(SB_Text(sb));
+	char *s = NULL;
+	if (SB_Text(sb))
+		s = XmTextFieldGetString(SB_Text(sb));
 
 	cbs.value = XmStringCreateSimple(s);
 
@@ -557,14 +580,7 @@ _XmSbButton(Widget w, XtPointer client, XtPointer call)
 	XtCallCallbackList((Widget)sb, sb->manager.help_callback, &cbs);
     }
 
-#if 0
-/* I  don't know why the checks for TextField and List are here, they
- * break autounmanage. Danny 30/8/1997.
- */
-    if (BB_AutoUnmanage(sb) && au && (XmIsTextField(w) || XmIsList(w)))
-#else
     if (BB_AutoUnmanage(sb) && au)
-#endif
     {
 	Widget s = XtParent((Widget)sb);
 
@@ -610,16 +626,6 @@ initialize(Widget request,
 		      XtWidth(new_w), XtHeight(new_w)));
     DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
 
-    /*
-     * Dynamically determine XmNdialogType
-     */
-    if (SB_DialogType(sb) == 0)
-    {				/* its initializer ! */
-	if (XtIsSubclass(XtParent(sb), xmDialogShellWidgetClass))
-	    SB_DialogType(sb) = XmDIALOG_SELECTION;
-	else
-	    SB_DialogType(sb) = XmDIALOG_WORK_AREA;
-    }
 
     SB_List(sb) = NULL;
     SB_ListLabel(sb) = NULL;
@@ -678,6 +684,7 @@ initialize(Widget request,
 	XtVaSetValues(new_w, XmNdefaultButton, SB_OkButton(sb), NULL);
 #else
 	BB_DefaultButton(new_w) = SB_OkButton(sb);
+ 	_XmBulletinBoardSetDynDefaultButton(new_w, SB_OkButton(sb));
 #endif
     }
     if (SB_TextString(sb) == (XmString)XmUNSPECIFIED)
@@ -688,7 +695,17 @@ initialize(Widget request,
     {
     	SB_TextString(sb) = XmStringCopy(SB_TextString(sb));
     }
+    if (SB_TextString(sb))
+    {
+    String text;
 
+	if (XmStringGetLtoR(SB_TextString(sb), XmSTRING_DEFAULT_CHARSET, &text))
+	{
+	    XmTextFieldSetString(SB_Text(sb), text);
+	    XtFree(text);
+	}
+
+    }
     if (SB_List(sb) && SB_ListItems(sb) && SB_ListItemCount(sb) > 0)
     {
     	XtVaSetValues(SB_List(sb),
@@ -725,8 +742,22 @@ set_values(Widget old,
     Arg al[10];
     int ac;
 
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "set_values: %i args\n"
+		      "\t    old X %5i Y %5i W %5i H %5i\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      *num_args,
+		      XtX(old), XtY(old),
+		      XtWidth(old), XtHeight(old),
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+
     /* this is required for every BB subclass */
-    BB_InSetValues(new_w) = False;
+    BB_InSetValues(new_w) = True;
 
     /* XmList */
     ac = 0;
@@ -747,6 +778,7 @@ set_values(Widget old,
     {
 	String text;
 
+	SB_TextString(sb) = XmStringCopy(SB_TextString(sb));
 	if (XmStringGetLtoR(SB_TextString(sb), XmSTRING_DEFAULT_CHARSET, &text))
 	{
 	    XmTextFieldSetString(SB_Text(sb), text);
@@ -897,8 +929,10 @@ _XmSelectionBoxCreateList(XmSelectionBoxWidget sb)
     SB_List(sb) = XmCreateScrolledList((Widget)sb, "ItemsList", al, ac);
     XtManageChild(SB_List(sb));
 
+    /*
     XtAddCallback(SB_List(sb),
 		  XmNdefaultActionCallback, _XmSbButton, NULL);
+		  */
     XtAddCallback(SB_List(sb),
 		  XmNsingleSelectionCallback, ListSingleSelect, sb);
     XtAddCallback(SB_List(sb),
@@ -910,10 +944,22 @@ _XmSelectionBoxCreateText(XmSelectionBoxWidget sb)
 {
     int ac;
     Arg al[2];
+    XtTranslations save_acc;
 
     ac = 0;
     XtSetArg(al[0], XmNnavigationType, XmSTICKY_TAB_GROUP); ac++;
     SB_Text(sb) = XmCreateTextField((Widget)sb, "Text", al, ac);
+    save_acc = CoreAccelerators(sb);
+    if (SB_TextAccelerators(sb) == NULL)
+    {
+	CoreAccelerators(sb) = text_accelerators_parsed;
+    }
+    else
+    {
+	CoreAccelerators(sb) = SB_TextAccelerators(sb);
+    }
+    XtInstallAccelerators(SB_Text(sb), (Widget) sb);
+    CoreAccelerators(sb) = save_acc;
     XtManageChild(SB_Text(sb));
 }
 
@@ -1108,12 +1154,131 @@ void
 _XmSelectionBoxUpOrDown(Widget wid, XEvent *event,
 			String *argv, Cardinal *argc)
 {
+    Arg args[4];
+    Widget text = SB_Text(wid), list = SB_List(wid);
+    int type, top_item_position, visible_item_count, item_count;
+    XmStringTable items;
+    int *position_list, position_count, new_position, old_position;
+    /* char *selected_text; */
+
+    DEBUGOUT(XdbDebug(__FILE__, wid, "_XmSelectionBoxUpOrDown %s\n",
+		      argv ? *argv: "NULL"));
+    if (list != NULL && text != NULL)
+    {
+	if (*argc != 1)
+	{
+	    return;
+	}
+	type = atoi(*argv);
+	XtSetArg(args[0], XmNtopItemPosition, &top_item_position);
+	XtSetArg(args[1], XmNvisibleItemCount, &visible_item_count);
+	XtSetArg(args[2], XmNitemCount, &item_count);
+	XtSetArg(args[3], XmNitems, &items);
+	XtGetValues(list, args, 4);
+	if (item_count == 0)
+	{
+	    return;
+	}
+	if (XmListGetSelectedPos(list, &position_list, &position_count))
+	{
+	    old_position = *position_list;
+	    XtFree((char *) position_list);
+	}
+	else
+	{
+	    old_position = 0;
+	}
+	
+	switch (type)
+	{
+	case 0:
+	    /* move up */
+	    if (old_position > 1)
+	    {
+		new_position = old_position - 1;
+	    }
+	    else
+	    {
+		new_position = 1;
+	    }
+	    break;
+	case 1:
+	    /* move down */
+	    if (old_position < item_count)
+	    {
+		new_position = old_position + 1;
+	    }
+	    else
+	    {
+		new_position = item_count;
+	    }
+	    break;
+	case 2:
+	    /* move to first item */
+	    new_position = 1;
+	    break;
+	case 3:
+	    /* move to last item */
+	    new_position = item_count;
+	    break;
+	default:
+	    new_position = old_position;
+	    break;
+	}
+	if (new_position != old_position)
+	{
+	    if (new_position < top_item_position)
+	    {
+		XmListSetPos(list, new_position);
+	    }
+	    else if (new_position >= top_item_position + visible_item_count)
+	    {
+		XmListSetBottomPos(list, new_position);
+	    }
+	    XmListSelectPos(list, new_position, True);
+	}
+    }
 }
 
 void
 _XmSelectionBoxRestore(Widget wid, XEvent *event,
 		       String *argv, Cardinal *argc)
 {
+    Arg args[2];
+    Widget text = SB_Text(wid), list = SB_List(wid);
+    XmStringTable items;
+    int item_count, *position_list, position_count;
+    char *selected_text;
+
+    DEBUGOUT(XdbDebug(__FILE__, wid, "_XmSelectionBoxRestore\n"));
+
+    if (text != NULL && list != NULL)
+    {
+	XtSetArg(args[0], XmNitemCount, &item_count);
+	XtSetArg(args[1], XmNitems, &items);
+	XtGetValues(list, args, 2);
+	if (item_count == 0 || 
+	    !XmListGetSelectedPos(list, &position_list, &position_count))
+	{
+	  selected_text = XtMalloc(sizeof(char));
+	  *selected_text = '\0';
+	}
+	else
+	{
+	    if (!XmStringGetLtoR(items[*position_list - 1],
+				 XmFONTLIST_DEFAULT_TAG,
+				 &selected_text))
+	    {
+		selected_text = XtMalloc(sizeof(char));
+		*selected_text = '\0';
+	    }
+	    XtFree((char *) position_list);
+	}
+	XmTextFieldSetString(text, selected_text);
+	XtFree(selected_text);
+	XmTextFieldSetInsertionPosition(text,
+					XmTextFieldGetLastPosition(text));
+    }
 }
 
 XmGeoMatrix
@@ -1525,7 +1690,11 @@ XmCreateSelectionDialog(Widget parent, char *name,
 	XtSetArg(al[ac], argList[i].name, argList[i].value); ac++;
     }
 
+#if 0
     d = XtCreateWidget(s, xmDialogShellWidgetClass, parent, al, ac);
+#else
+    d = XmCreateDialogShell(parent, s, al, ac);
+#endif
     XtFree(s);
 
     r = XtCreateWidget(name, xmSelectionBoxWidgetClass, d, al, ac);
@@ -1676,3 +1845,21 @@ DeleteChild(Widget w)
 	if (w == SB_ListLabel(sb))
 		SB_ListLabel(sb) = NULL;
 }
+
+static void
+_XmDialogTypeDefault(Widget w, int offset, XrmValue *value)
+{
+    static unsigned char dialog_type;
+    
+    DEBUGOUT(XdbDebug(__FILE__, w, "_XmDialogTypeDefault\n"));
+    
+    dialog_type = XmDIALOG_WORK_AREA;
+    
+    value->addr = (char *) &dialog_type;
+    value->size = sizeof (unsigned char);
+    
+    if (XmIsDialogShell(XtParent(w)))
+    {
+	dialog_type = XmDIALOG_SELECTION;
+    }
+}
