@@ -1,6 +1,6 @@
 /**
  *
- * $Id: SeparatoG.c,v 1.7 1998/02/22 01:00:55 rwscott Exp $
+ * $Id: SeparatoG.c,v 1.11 1998/09/13 20:17:56 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: SeparatoG.c,v 1.7 1998/02/22 01:00:55 rwscott Exp $";
+static const char rcsid[] = "$Id: SeparatoG.c,v 1.11 1998/09/13 20:17:56 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -32,6 +32,7 @@ static char rcsid[] = "$Id: SeparatoG.c,v 1.7 1998/02/22 01:00:55 rwscott Exp $"
 #include <Xm/SeparatoGP.h>
 #include <Xm/ExtObjectP.h>
 #include <Xm/CacheP.h>
+#include <Xm/RepType.h>
 #include <X11/Xfuncs.h>
 
 #include <XmI/DebugUtil.h>
@@ -49,9 +50,11 @@ static void destroy(Widget w);
 
 static void expose(Widget w, XEvent *event, Region region);
 
+#if 0
 static XtGeometryResult query_geometry(Widget w,
 				       XtWidgetGeometry *proposed,
 				       XtWidgetGeometry *answer);
+#endif
 
 static Boolean set_values(Widget current, Widget request, Widget new_w,
 			  ArgList args, Cardinal *num_args);
@@ -256,7 +259,7 @@ XmSeparatorGadgetClassRec xmSeparatorGadgetClassRec = {
 	/* version               */ XtVersion,
 	/* callback offsets      */ NULL,
 	/* tm_table              */ NULL,
-	/* query_geometry        */ query_geometry,
+	/* query_geometry        */ NULL /* query_geometry */,
 	/* display_accelerator   */ NULL,
 	/* extension             */ (XtPointer)&_XmSeparatoGRectClassExtRec
     },
@@ -418,6 +421,14 @@ CreateSeparatorGC(Widget w)
 
     switch (SEPG_SeparatorType(w))
     {
+    case XmNO_LINE:
+	values.line_style = LineSolid;
+	values.join_style = JoinMiter;
+	values.cap_style = CapButt;
+	values.foreground = XmParentForeground(w);
+	values.background = XmParentBackground(w);
+	break;
+
     case XmSINGLE_LINE:
 	values.line_style = LineSolid;
 	values.join_style = JoinMiter;
@@ -492,13 +503,52 @@ initialize(Widget request, Widget new_w,
 	_XmError(new_w, "parent should be manager.");
     }
 
+    /* XmSHADOW_ETCHED_IN_DASH  and XmSHADOW_ETCHED_OUT_DASH 
+     * is a version 2.0 feature
+     */
+
+    if (SEPG_SeparatorType(new_w) == XmINVALID_SEPARATOR_TYPE ||
+	SEPG_SeparatorType(new_w) == XmSHADOW_ETCHED_IN_DASH ||
+	SEPG_SeparatorType(new_w) == XmSHADOW_ETCHED_OUT_DASH ||
+	!XmRepTypeValidValue(XmRepTypeGetId(XmRSeparatorType),
+			     SEPG_SeparatorType(new_w),
+			     new_w))
+    {
+	SEPG_SeparatorType(new_w) = XmSHADOW_ETCHED_IN;
+    }
+
     if (SEPG_Orientation(new_w) == XmVERTICAL)
     {
-	XtWidth(new_w) = G_ShadowThickness(new_w);
+        switch(SEPG_SeparatorType(new_w))
+        {
+            case XmSINGLE_LINE:
+            case XmSINGLE_DASHED_LINE:
+                XtWidth(new_w) = 1;
+                break;
+            case XmDOUBLE_LINE:
+            case XmDOUBLE_DASHED_LINE:
+                XtWidth(new_w) = 3;
+                break;
+            default:
+                XtWidth(new_w) = G_ShadowThickness(new_w);
+        }
     }
+
     if (SEPG_Orientation(new_w) == XmHORIZONTAL)
     {
-	XtHeight(new_w) = G_ShadowThickness(new_w);
+        switch(SEPG_SeparatorType(new_w))
+        {
+            case XmSINGLE_LINE:
+            case XmSINGLE_DASHED_LINE:
+                XtHeight(new_w) = 1;
+                break;
+            case XmDOUBLE_LINE:
+            case XmDOUBLE_DASHED_LINE:
+                XtHeight(new_w) = 3;
+                break;
+            default:
+                XtHeight(new_w) = G_ShadowThickness(new_w);
+        }
     }
 
     CreateSeparatorGC(new_w);
@@ -604,8 +654,17 @@ set_values(Widget old, Widget request, Widget new_w,
 {
     Boolean refresh_needed = False;
 
+    /* XmSHADOW_ETCHED_IN_DASH  and XmSHADOW_ETCHED_OUT_DASH 
+     * is a version 2.0 feature
+     */
     if (XmParentForeground(new_w) != XmParentForeground(old) ||
-	SEPG_SeparatorType(new_w) != SEPG_SeparatorType(old))
+	(SEPG_SeparatorType(new_w) != SEPG_SeparatorType(old) &&
+	 SEPG_SeparatorType(new_w) != XmINVALID_SEPARATOR_TYPE &&
+	 SEPG_SeparatorType(new_w) != XmSHADOW_ETCHED_IN_DASH &&
+	 SEPG_SeparatorType(new_w) != XmSHADOW_ETCHED_OUT_DASH &&
+	 XmRepTypeValidValue(XmRepTypeGetId(XmRSeparatorType),
+			     SEPG_SeparatorType(new_w),
+			     new_w)))
     {
 	XtReleaseGC(new_w, SEPG_SeparatorGC(new_w));
 	CreateSeparatorGC(new_w);
@@ -690,6 +749,7 @@ expose(Widget w, XEvent *event, Region region)
 		     SEPG_SeparatorType(w));
 }
 
+#if 0
 static XtGeometryResult
 query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 {
@@ -732,6 +792,7 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 	return XtGeometryAlmost;
     }
 }
+#endif
 
 static Cardinal
 get_sec_res_data(WidgetClass wc, XmSecondaryResourceData **data)
