@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Simple.c,v 1.13 1997/12/07 13:55:29 rwscott Exp $
+ * $Id: Simple.c,v 1.15 1998/10/04 07:23:16 gritton Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: Simple.c,v 1.13 1997/12/07 13:55:29 rwscott Exp $";
+static const char rcsid[] = "$Id: Simple.c,v 1.15 1998/10/04 07:23:16 gritton Exp $";
 
 #include <stdio.h>
 
@@ -85,7 +85,7 @@ static XtResource simple_resources[] =
 	XmRImmediate, (XtPointer)-1
     },
     {
-	XmNbuttonType, XmCButtonType, XmRPointer,
+	XmNbuttonType, XmCButtonType, XmRButtonType,
 	sizeof(XtPointer), Offset(button_type),
 	XmRImmediate, (XtPointer)NULL
     },
