@@ -1,6 +1,6 @@
 /**
  *
- * $Id: TearOff.c,v 1.24 1998/03/17 19:59:32 u27113 Exp $
+ * $Id: TearOff.c,v 1.27 1998/09/12 15:05:44 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: TearOff.c,v 1.24 1998/03/17 19:59:32 u27113 Exp $";
+static const char rcsid[] = "$Id: TearOff.c,v 1.27 1998/09/12 15:05:44 rwscott Exp $";
 
 #include <stdio.h>
 
@@ -247,7 +247,7 @@ _XmTearOffInitiate(Widget w, XEvent *event)
     /* FIX ME: PopdownEveryone */
     {
     Widget cb;
-    Widget posted;
+    Widget posted = NULL;
 
 	if (RC_CascadeBtn(menu))
 	{
@@ -386,12 +386,12 @@ CreateTopLevelShell(Widget w)
 	    }
 	    else
 	    {
-		to_shell_title = strdup("");
+		to_shell_title = XtNewString("");
 	    }
     	}
     	else
     	{
-    		to_shell_title = strdup("");
+    		to_shell_title = XtNewString("");
     	}
     }
 
@@ -437,7 +437,6 @@ _XmRestoreTearOffToToplevelShell(Widget w, XEvent *event)
 {
     /* w represents the row column */
     Widget menu_shell = XtParent(w);
-    int i;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmRestoreTearOffToToplevelShell()\n"));
 
@@ -529,7 +528,6 @@ _XmRestoreTearOffToMenuShell(Widget w,
     /* w represents the row column */
     Widget transient = XtParent(w);
     Widget menu_shell;
-    Cardinal numparams = 0;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmRestoreTearOffToMenuShell()\n"));
 
