@@ -1,6 +1,6 @@
 /**
  *
- * $Id: TearOffB.c,v 1.14 1998/03/01 16:44:20 rwscott Exp $
+ * $Id: TearOffB.c,v 1.16 1998/09/13 20:17:56 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: TearOffB.c,v 1.14 1998/03/01 16:44:20 rwscott Exp $";
+static const char rcsid[] = "$Id: TearOffB.c,v 1.16 1998/09/13 20:17:56 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -51,9 +51,11 @@ static void destroy(Widget w);
 
 static void expose(Widget w, XEvent *event, Region region);
 
+#if 0
 static XtGeometryResult query_geometry(Widget w,
 				       XtWidgetGeometry *proposed,
 				       XtWidgetGeometry *answer);
+#endif
 
 static Boolean set_values(Widget current, Widget request, Widget new_w,
 			  ArgList args, Cardinal *num_args);
@@ -156,7 +158,7 @@ XmTearOffButtonClassRec xmTearOffButtonClassRec = {
 	/* version               */ XtVersion,
 	/* callback offsets      */ NULL,
 	/* tm_table              */ XtInheritTranslations,
-	/* query_geometry        */ query_geometry,
+	/* query_geometry        */ XtInheritQueryGeometry /* query_geometry */,
 	/* display_accelerator   */ NULL,
 	/* extension             */ NULL
     },
@@ -461,6 +463,7 @@ expose(Widget w, XEvent *event, Region region)
 #undef superclass
 }
 
+#if 0
 static XtGeometryResult
 query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 {
@@ -528,6 +531,7 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 	return XtGeometryAlmost;
     }
 }
+#endif
 
 /*
  * move/draw a window outline
