@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/Text.c,v 1.57 1998/06/03 19:27:49 danny Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/Text.c,v 1.72 1998/10/24 03:18:42 rwscott Exp $
  *
  * Copyright (C) 1997 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: Text.c,v 1.57 1998/06/03 19:27:49 danny Exp $";
+static const char rcsid[] = "$Id: Text.c,v 1.72 1998/10/24 03:18:42 rwscott Exp $";
 
 #undef VERBOSE
 
@@ -74,7 +74,7 @@ static void Realize(Widget w, XtValueMask *value_mask,
 
 static void DoExpose(Widget w, XEvent *event, Region region);
 
-static XtGeometryResult QueryGeometry(Widget w,
+static XtGeometryResult query_geometry(Widget w,
 				      XtWidgetGeometry *proposed,
 				      XtWidgetGeometry *answer);
 
@@ -349,7 +349,7 @@ XmTextClassRec xmTextClassRec =
 	/* version               */ XtVersion,
 	/* callback offsets      */ NULL,
 	/* tm_table              */ NULL,
-	/* query_geometry        */ QueryGeometry,
+	/* query_geometry        */ query_geometry,
 	/* display_accelerator   */ NULL,
 	/* extension             */ (XtPointer) & textBaseClassExtRec
   },
@@ -404,7 +404,7 @@ InnerDestroy(Widget w)
 {
     /* Destroy Output and Input */
     /* Nah - already done in XmText's destroy method (Destroy). */
-    DEBUGOUT(XdbDebug(__FILE__, w, "InnerDestroy"));
+    DEBUGOUT(XdbDebug(__FILE__, w, "InnerDestroy\n"));
 }
 
 /*
@@ -486,9 +486,6 @@ WidgetClass xmTextInnerObjectClass = (WidgetClass)&xmTextInnerClassRec;
 
 
 
-extern Boolean _XmCvtStringToXmString(Display *, XrmValue *, Cardinal *,
-				      XrmValue *, XrmValue *, XtPointer *);
-
 extern XmFontList _XmFontListCreateDefault(Display *);
 
 static void
@@ -598,6 +595,10 @@ static void
 Destroy(Widget w)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "Destroy\n"));
+    /* rws 2 Oct 1998
+       This is odd, I thought Xt should be taking care of this
+     */
+    XtDestroyWidget(((XmTextWidget)w)->text.inner_widget);
     (*Text_Output(w)->destroy) (w);
 
     (*Text_Input(w)->destroy) (w);
@@ -717,7 +718,10 @@ Realize(Widget aw, XtValueMask *value_mask, XSetWindowAttributes *attributes)
 {
     XmTextWidget w = (XmTextWidget)aw;
 
-    DEBUGOUT(XdbDebug(__FILE__, aw, "Realize\n"));
+    DEBUGOUT(XdbDebug(__FILE__, aw, "%s:Realize(%d) - %dx%d\n",
+    		__FILE__, __LINE__,
+    		XtWidth(aw), XtHeight(aw)));
+
 #define superclass (&xmPrimitiveClassRec)
     (*superclass->core_class.realize) (aw, value_mask, attributes);
 #undef superclass
@@ -759,7 +763,9 @@ Resize(Widget aw)
 {
     XmTextWidget w = (XmTextWidget)aw;
 
-    DEBUGOUT(XdbDebug(__FILE__, aw, "resize"));
+    DEBUGOUT(XdbDebug(__FILE__, aw, "%s:resize(%d) - %dx%d\n",
+    		__FILE__, __LINE__,
+    		XtWidth(aw), XtHeight(aw)));
 
     /* Resize can get called from XmScrolledWindowSetAreas before we actually
      * made it through our initialize method.  Make sure not to die because
@@ -776,14 +782,15 @@ Resize(Widget aw)
 }
 
 static XtGeometryResult
-QueryGeometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
+query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 {
+#if 0
     XmTextWidget tw = (XmTextWidget)w;
     XtWidgetGeometry a;
 
 #define Wants(x)   (proposed->request_mode & x)
     DEBUGOUT(XdbDebug(__FILE__, w,
-		      "QueryGeometry proposed width=%d height=%d mode=%X\n",
+		      "query_geometry proposed width=%d height=%d mode=%X\n",
 		   proposed->width, proposed->height, proposed->request_mode));
 
     if (proposed->request_mode != 0)
@@ -832,6 +839,16 @@ QueryGeometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
     {
 	return XtGeometryAlmost;
     }
+#else
+    DEBUGOUT(XdbDebug(__FILE__, w,
+		      "query_geometry proposed width=%d height=%d mode=%X\n",
+		   proposed->width, proposed->height, proposed->request_mode));
+
+    answer->width = XtWidth(w);
+    answer->height = XtHeight(w);
+
+    return _XmGMReplyToQueryGeometry(w, proposed, answer);
+#endif
 }
 
 
@@ -1583,7 +1600,7 @@ XmCreateScrolledText(Widget parent, char *name, Arg *arglist, Cardinal argcount)
     i = argcount;
     XtSetArg(al[i], XmNeditMode, XmMULTI_LINE_EDIT); i++;
 
-    w = XtCreateWidget(name, xmTextWidgetClass, sw, al, i);
+    w = XtCreateManagedWidget(name, xmTextWidgetClass, sw, al, i);
     XtAddCallback(w, XmNdestroyCallback,
 		  _XmDestroyParentCallback,
 		  (XtPointer)w);
@@ -1598,14 +1615,21 @@ XmTextClearSelection(Widget w, Time time)
 {
     Boolean sel;
     XmTextPosition left, right;
-    XmTextBlockRec block;
 
     if (XtIsSubclass(w, xmTextWidgetClass))
     {
 	sel = (*Text_Source(w)->GetSelection) (Text_Source(w), &left, &right);
 	if (sel) 
 	{
+          /* SG 22/08/1998 this is wrong, it should deselect the text
+             for this widget only, not effecting the primary or selection
+             properties (according to O'Reilly), it certainly shouldn't
+             delete any text.
 		_XmTextDelete((XmTextWidget)w, NULL, left, right);
+             give this a go instead,based on the same thing in XmTextField,
+             however there is a complication, this clears the selection for
+             all XmText widgets sharing this source - is this right ??? */
+	    (*Text_Source(w)->SetSelection) (Text_Source(w), -1, -1, time);
 	}
         return;
     }   
@@ -1625,16 +1649,21 @@ XmTextCopy(Widget w, Time time)
     Display		*dpy;
     Window		win;
     int			r;
-    long		item_id, data_id;
+    long		item_id;
     XmString		lb = NULL;
-    char		*fmt = NULL;
     Boolean		sel;
     XmTextPosition	left, right;
     XmTextBlockRec	block;
 
-    if (!XmIsText(w))
+    if (XtIsSubclass(w, xmTextFieldWidgetClass))
+        return XmTextFieldCopy(w,time);
+    else if (!XmIsText(w))
+    {
+	_XmWarning(w, "XmTextCopy: widget has invalid class");
 	return False;
-    if (w == NULL || XtWindow(w) == NULL)
+    }
+
+    if (w == NULL || XtWindow(w) == None)
 	return False;
 
     dpy = XtDisplay(w);
@@ -1685,8 +1714,12 @@ XmTextCut(Widget w, Time time)
     Boolean		sel, r = False;
     XmTextPosition	left, right;
 
-    if (!XmIsText(w)) {
-        return False;   
+    if (XtIsSubclass(w, xmTextFieldWidgetClass))
+        return XmTextFieldCut(w,time);
+    else if (!XmIsText(w))
+    {
+	_XmWarning(w, "XmTextCut: widget has invalid class");
+	return False;
     }
 
     if ((r = XmTextCopy(w, time)) == False)
@@ -1741,8 +1774,19 @@ XmTextFindString(Widget w, XmTextPosition start, char *string,
 int
 XmTextGetBaseline(Widget w)
 {
-    _XmWarning(w, "XmTextGetBaseline: not implemented");
-    return 0;			/* FIX ME */
+    if (XtIsSubclass(w, xmTextWidgetClass))
+    {
+        _XmWarning(w, "XmTextGetBaseline: not implemented");
+	return 0;
+    }
+    else if (XtIsSubclass(w, xmTextFieldWidgetClass))
+    {
+	return XmTextFieldGetBaseline(w);
+    }
+
+    _XmWarning(w, "XmTextGetCursorPosition: widget has invalid class");
+
+    return 0;
 }
 
 
@@ -2069,7 +2113,7 @@ XmTextPaste(Widget w)
 	unsigned long	private_id, len, nbytes;
 	XmTextPosition	pos;
 
-	if (win == NULL || !Text_Editable(w))
+	if (win == None || !Text_Editable(w))
 	    return False;
 
 	while ((r = XmClipboardStartRetrieve(XtDisplay(w), XtWindow(w),
@@ -2518,7 +2562,9 @@ XmTextScroll(Widget aw, int n)
     else
     {
 	index = _XmMin(n + top_index,
-		       Text_TotalLines(w) - 1) - Text_LineCount(w) + 1;
+/* SG (15/08/1998)  Text_TotalLines(w) - 1) - Text_LineCount(w) + 1;
+   we would rather like to be able to scroll down to the last line */
+		       Text_TotalLines(w) ) - Text_LineCount(w) + 1;
     }
     DEBUGOUT(XdbDebug(__FILE__, aw, "XmTextScroll index=%d n=%d\n", index, n));
 
