@@ -1,7 +1,7 @@
 /*                   -*-Mode: c; c-file-style: "GNU"-*-              */
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.29 1998/05/28 22:14:24 u27113 Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.34 1998/08/27 17:39:48 pgw Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.29 1998/05/28 22:14:24 u27113 Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.34 1998/08/27 17:39:48 pgw Exp $";
 
 #include <LTconfig.h>
 #include <Xm/XmP.h>
@@ -539,6 +539,8 @@ DoInsert(XmTextWidget w, XEvent *ev, char *buf, int len)
     Widget aw = (Widget)w;
     XmTextVerifyCallbackStruct cbs;
     XmTextBlockRec blockrec;
+    Boolean sel;
+    XmTextPosition left, right;
 
     if (len <= 0)
     {
@@ -567,13 +569,25 @@ DoInsert(XmTextWidget w, XEvent *ev, char *buf, int len)
 
     cbs.reason = XmCR_MODIFYING_TEXT_VALUE;
     cbs.event = ev;
-    cbs.currInsert = cbs.startPos = cbs.endPos = Text_CursorPos(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
-		      "DoInsert before add: CursorPos %08x\n",
-		      Text_CursorPos(w)));
+    /* SG 23/08/1998, replace primary selection if exists, pending delete
+       is true and the insertion cursor is inside the selection.
+     */
+    sel = (*Text_Source(w)->GetSelection) (Text_Source(w), &left, &right);
 
-    cbs.newInsert = (Text_CursorPos(w) += blockrec.length);
+    if (sel && In_PendingDelete(Text_Input(w)->data) &&
+        Text_CursorPos(w)>=left && Text_CursorPos(w)<=right)
+    {
+        cbs.currInsert = Text_CursorPos(w);
+        cbs.startPos = left;
+        cbs.endPos = right;
+        cbs.newInsert = left + blockrec.length;
+    }
+    else
+    {
+        cbs.currInsert = cbs.startPos = cbs.endPos = Text_CursorPos(w);
+        cbs.newInsert = cbs.currInsert + blockrec.length;
+    }
 
     DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
 		      "DoInsert before CB: CursorPos %08x len %08x\n",
@@ -648,6 +662,7 @@ DoScanType(XmTextWidget w, XEvent *ev, XmTextPosition pos)
 					 XmsdRight, -1, False);
 	left = (*Text_Source(w)->Scan) (Text_Source(w), right, st,
 					XmsdLeft, -1, False);
+        In_HighlightPivot(i) = left;
 
 	DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
 		       "DoScanType: XmSELECT_WORD (left %d pos %d right %d)\n",
@@ -662,6 +677,7 @@ DoScanType(XmTextWidget w, XEvent *ev, XmTextPosition pos)
 					 XmsdRight, -1, False);
 	left = (*Text_Source(w)->Scan) (Text_Source(w), right, st,
 					XmsdLeft, -1, False);
+        In_HighlightPivot(i) = left;
 
 	DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
 		       "DoScanType: XmSELECT_LINE (left %d pos %d right %d)\n",
@@ -678,7 +694,7 @@ DoScanType(XmTextWidget w, XEvent *ev, XmTextPosition pos)
 #endif
 
     case XmSELECT_ALL:
-	left = 0;
+	In_HighlightPivot(i) = left = 0;
 	right = XmTextGetLastPosition((Widget)w);
 
 	DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
@@ -793,6 +809,7 @@ MoveBackwardWord(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 static void
 RingBell(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
+    XBell(XtDisplay((Widget)w), 50);
 }
 
 static void
@@ -801,11 +818,6 @@ MoveBeginningOfFile(Widget w, XEvent *ev, String *params,
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "MoveBeginningOfFile"));
 
-    if (!Text_Editable(w))
-    {
-	return;
-    }
-
     if (Text_CursorPos(w) > 0)
     {
 	(*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
@@ -854,6 +866,9 @@ CopyPrimary(Widget w, XEvent *ev, String *params, Cardinal *num_params)
  * _XmTextGetSelection is the function that's called when we receive the
  * string from a copy/paste operation.
  * Now we need to insert this text into our widget.
+ *
+ * It seems that if the widget is read only then we never get here
+ * so there is no need to check this (SG 18/08/1998)
  */
 static void
 _XmTextGetSelection(Widget w, XtPointer client, Atom *selection, Atom *type,
@@ -940,27 +955,26 @@ DeleteForwardChar(Widget w, XEvent *ev, String *params,
     if (!Text_Editable(w))
     {
 	VerifyBell((XmTextWidget)w);
-
 	return;
     }
 
     sel = (*Text_Source(w)->GetSelection) (Text_Source(w), &left, &right);
 
-    if (Text_AddMode(w))
+    /* SG 22/08/1998 - O'Reilly, and experience says that Motif 1.2 deletes
+       the selection as per the "add mode" sction above. However add mode
+       is not mentioned at all as far as I can tell in Motif 1.2, is this
+       a Motif 2.0 thing ? If so it doesn't belong here, except possibly
+       ifdef'ed .
+     */
+    if (sel && In_PendingDelete(Text_Input(w)->data) &&
+        Text_CursorPos(w)>=left && Text_CursorPos(w)<=right)
     {
-	/* FIX ME */
+        _XmTextDelete((XmTextWidget)w, ev, left, right);
     }
-    else
+    else if (Text_CursorPos(w) < Text_LastPos(w))
     {
-	if (sel)
-	{
-	    _XmTextDelete((XmTextWidget)w, ev, left, right);
-	}
-	else if (Text_CursorPos(w) < Text_LastPos(w))
-	{
-	    _XmTextDelete((XmTextWidget)w, ev, Text_CursorPos(w),
-		     Text_CursorPos(w) + 1);
-	}
+        _XmTextDelete((XmTextWidget)w, ev, Text_CursorPos(w),
+    	     Text_CursorPos(w) + 1);
     }
 }
 
@@ -968,14 +982,24 @@ static void
 DeleteBackwardChar(Widget w, XEvent *ev, String *params,
 		   Cardinal *num_params)
 {
+    Boolean sel;
+    XmTextPosition left, right;
     DEBUGOUT(XdbDebug(__FILE__, w, "DeleteBackardChar"));
 
     if (!Text_Editable(w))
     {
 	VerifyBell((XmTextWidget)w);
+	return;
     }
 
-    if (Text_CursorPos(w) > 0)
+    sel = (*Text_Source(w)->GetSelection) (Text_Source(w), &left, &right);
+
+    if(sel && In_PendingDelete(Text_Input(w)->data) &&
+       Text_CursorPos(w)>=left && Text_CursorPos(w)<=right)
+    {
+      _XmTextDelete((XmTextWidget)w,ev,left,right);
+    }
+    else if (Text_CursorPos(w) > 0)
     {
 	_XmTextDelete((XmTextWidget)w, ev, Text_CursorPos(w) - 1, Text_CursorPos(w));
     }
@@ -984,38 +1008,132 @@ DeleteBackwardChar(Widget w, XEvent *ev, String *params,
 static void
 DeleteForwardWord(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
+    Boolean sel;
+    XmTextPosition left, right;
     DEBUGOUT(XdbDebug(__FILE__, w, "DeleteForwardWord\n"));
+
+    if (!Text_Editable(w))
+    {
+	VerifyBell((XmTextWidget)w);
+	return;
+    }
+
+    sel = (*Text_Source(w)->GetSelection) (Text_Source(w), &left, &right);
+
+    if(sel && In_PendingDelete(Text_Input(w)->data) &&
+       Text_CursorPos(w)>=left && Text_CursorPos(w)<=right)
+    {
+      _XmTextDelete((XmTextWidget)w,ev,left,right);
+    }
+    else
+    {
+    XmTextPosition pos;
+
+        pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
+                                       XmSELECT_WORD, XmsdRight, -1, False);
+
+        pos = (*Text_Source(w)->Scan) (Text_Source(w), pos,
+                                       XmSELECT_WHITESPACE, XmsdRight, -1, False);
+        _XmTextDelete((XmTextWidget)w, ev,Text_CursorPos(w),pos);
+    }
 }
 
 static void
 DeleteBackwardWord(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
+    Boolean sel;
+    XmTextPosition left, right;
     DEBUGOUT(XdbDebug(__FILE__, w, "DeleteBackwardWord\n"));
+
+    if (!Text_Editable(w))
+    {
+	VerifyBell((XmTextWidget)w);
+	return;
+    }
+
+    sel = (*Text_Source(w)->GetSelection) (Text_Source(w), &left, &right);
+
+    if(sel && In_PendingDelete(Text_Input(w)->data) &&
+       Text_CursorPos(w)>=left && Text_CursorPos(w)<=right)
+    {
+      _XmTextDelete((XmTextWidget)w,ev,left,right);
+    }
+    else if (Text_CursorPos(w) > 0)
+    {
+        XmTextPosition pos;
+
+        pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
+                                     XmSELECT_WHITESPACE, XmsdLeft, -1, False);
+
+        pos = (*Text_Source(w)->Scan) (Text_Source(w), pos,
+                                       XmSELECT_WORD, XmsdLeft, -1, False);
+
+        _XmTextDelete((XmTextWidget)w,ev,pos,Text_CursorPos(w));
+    }
 }
 
 static void
 DeleteCurrentSelection(Widget w, XEvent *ev,
 		       String *params, Cardinal *num_params)
 {
+    Boolean sel;
+    XmTextPosition left, right;
+
     DEBUGOUT(XdbDebug(__FILE__, w, "DeleteCurrentSelection\n"));
+
+    if (!Text_Editable(w))
+    {
+        VerifyBell((XmTextWidget)w);
+        return;
+    }
+
+    sel = (*Text_Source(w)->GetSelection) (Text_Source(w), &left, &right);
+
+    if(sel)
+    {
+      _XmTextDelete((XmTextWidget)w,ev,left,right);
+    }
 }
 
 static void
 DeleteToEndOfLine(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
+    XmTextPosition pos;
     DEBUGOUT(XdbDebug(__FILE__, w, "DeleteToEndOfLine\n"));
+
+    if (!Text_Editable(w))
+    {
+	VerifyBell((XmTextWidget)w);
+	return;
+    }
+
+    pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
+				   XmSELECT_LINE, XmsdRight, -1, False);
+    _XmTextDelete((XmTextWidget)w,ev,Text_CursorPos(w),pos);
 }
 
 static void
 DeleteToStartOfLine(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
+    XmTextPosition pos;
     DEBUGOUT(XdbDebug(__FILE__, w, "DeleteToStartOfLine\n"));
+
+    if (!Text_Editable(w))
+    {
+	VerifyBell((XmTextWidget)w);
+	return;
+    }
+
+    pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
+				   XmSELECT_LINE, XmsdLeft, -1, False);
+    _XmTextDelete((XmTextWidget)w,ev,pos,Text_CursorPos(w));
 }
 
 static void
 DeselectAll(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "DeselectAll\n"));
+    (*Text_Source(w)->SetSelection)(Text_Source(w),-1,-1,ev->xkey.time);
 }
 
 
@@ -1069,36 +1187,13 @@ static void
 ExtendEnd(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     InputData i = Text_InputData(w);
-    XmTextPosition pos, left, right;
-    Boolean sel;
 
     if (!i->extending)
     {
 	return;
     }
 
-    pos = (*Text_Output(w)->XYToPos) ((XmTextWidget)w,
-				      ev->xbutton.x, ev->xbutton.y);
-
-    sel = (*Text_Source(w)->GetSelection) (Text_Source(w), &left, &right);
-
-    if (sel)
-    {
-	DEBUGOUT(XdbDebug(__FILE__, w,
-		 "ExtendEnd (pos %d, origleft %d), existing selection %d-%d\n",
-			  pos, In_HighlightStart(i), left, right));
-    }
-    else
-    {
-	DEBUGOUT(XdbDebug(__FILE__, w, "ExtendEnd (pos %d, origleft %d)\n",
-			  pos, In_HighlightStart(i)));
-    }
-
-    if (sel && In_HighlightStart(i) >= 0 && pos > In_HighlightStart(i))
-    {
-	XmTextSetSelection(w, In_HighlightStart(i), pos,
-			   ev->xbutton.time);
-    }
+    DEBUGOUT(XdbDebug(__FILE__, w, "ExtendEnd"));
 
     /*
      * Set a flag for ExtendSelection() to know that when it happens,
@@ -1115,39 +1210,68 @@ static void
 ExtendSelection(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     XmTextPosition pos, left, right;
+    XmTextScanType st;
     InputData i = Text_InputData(w);
     Boolean sel;
 
     pos = (*Text_Output(w)->XYToPos) ((XmTextWidget)w,
 				      ev->xbutton.x, ev->xbutton.y);
 
-    if (!i->extending)
-    {
-	sel = False;
-	In_HighlightStart(i) = pos;
-    }
-
     sel = (*Text_Source(w)->GetSelection) (Text_Source(w), &left, &right);
 
     if (sel)
     {
 	DEBUGOUT(XdbDebug(__FILE__, w,
 		 "ExtendSelection (pos %d, origleft %d) existing sel. %d-%d\n",
-			  pos, In_HighlightStart(i), left, right));
+			  pos, In_HighlightPivot(i), left, right));
     }
     else
     {
 	DEBUGOUT(XdbDebug(__FILE__, w,
 	    "ExtendSelection (pos %d, origleft %d) (x %d y %d)\n",
-	    pos, In_HighlightStart(i), ev->xbutton.x, ev->xbutton.y));
+	    pos, In_HighlightPivot(i), ev->xbutton.x, ev->xbutton.y));
     }
 
-    if (In_HighlightStart(i) < 0)
+    if (In_HighlightPivot(i) < 0)
     {
-	In_HighlightStart(i) = pos;
+	In_HighlightPivot(i) = pos;
     }
 
-    XmTextSetSelection(w, In_HighlightStart(i), pos, ev->xbutton.time);
+    st = ScanType((XmTextWidget)w);
+
+    if(pos >= In_HighlightPivot(i))
+    {
+        if(st == XmSELECT_POSITION)
+        {
+            left = In_HighlightPivot(i);
+            right = pos;
+        }
+        else
+        {
+            right = (*Text_Source(w)->Scan)(Text_Source(w), pos, st,
+                                            XmsdRight, -1, False);
+            left = In_HighlightPivot(i);
+        }
+        XmTextSetCursorPosition(w,right);
+    }
+    else
+    {
+        if(st == XmSELECT_POSITION)
+        {
+            left = pos;
+            right = In_HighlightPivot(i);
+        }
+        else
+        {
+            left = (*Text_Source(w)->Scan)(Text_Source(w), pos, st,
+                                           XmsdLeft, -1, False);
+            right = (*Text_Source(w)->Scan)(Text_Source(w),In_HighlightPivot(i),
+                                            st, XmsdRight, -1, False);
+        }
+        XmTextSetCursorPosition(w,left);
+    }
+
+    XmTextSetSelection(w,left,right,ev->xbutton.time);
 
     i->extending = True;
 }
@@ -1162,18 +1286,12 @@ MoveForwardChar(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "MoveForwardChar"));
 
-    if (!Text_Editable(w))
-    {
-	return;
-    }
-
     if (Text_CursorPos(w) < Text_LastPos(w))
     {
 	(*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
 					       Text_CursorPos(w), off);
+        _XmTextSetCursorPosition(w, Text_CursorPos(w) + 1);
     }
-
-    _XmTextSetCursorPosition(w, Text_CursorPos(w) + 1);
 }
 
 static void
@@ -1286,18 +1404,15 @@ _KeySelection(Widget w, XEvent *ev, int flag, String *params, Cardinal *np)
 	    {
 		right = Text_CursorPos(w);
 	    }
-
-	    (*Text_Source(w)->SetSelection) (Text_Source(w),
-					     left, right, ev->xkey.time);
 	}
 	else
 	{			/* New selection */
 	    left = Text_CursorPos(w);
 	    right = old;
-
-	    (*Text_Source(w)->SetSelection) (Text_Source(w),
-					     left, right, ev->xkey.time);
 	}
+
+	(*Text_Source(w)->SetSelection) (Text_Source(w),
+				         left, right, ev->xkey.time);
 	break;
 
     case 2:			/* right */
@@ -1324,18 +1439,14 @@ _KeySelection(Widget w, XEvent *ev, int flag, String *params, Cardinal *np)
 	    {
 		right = Text_CursorPos(w);
 	    }
-
-	    (*Text_Source(w)->SetSelection) (Text_Source(w),
-					     left, right, ev->xkey.time);
 	}
 	else
 	{			/* New selection */
 	    left = old;
 	    right = Text_CursorPos(w);
-
-	    (*Text_Source(w)->SetSelection) (Text_Source(w),
-					     left, right, ev->xkey.time);
 	}
+	(*Text_Source(w)->SetSelection) (Text_Source(w),
+					 left, right, ev->xkey.time);
 	break;
 
     case 3:			/* up */
@@ -1362,18 +1473,14 @@ _KeySelection(Widget w, XEvent *ev, int flag, String *params, Cardinal *np)
 	    {
 		right = Text_CursorPos(w);
 	    }
-
-	    (*Text_Source(w)->SetSelection) (Text_Source(w),
-					     left, right, ev->xkey.time);
 	}
 	else
 	{			/* New selection */
 	    left = Text_CursorPos(w);
 	    right = old;
-
-	    (*Text_Source(w)->SetSelection) (Text_Source(w),
-					     left, right, ev->xkey.time);
 	}
+	(*Text_Source(w)->SetSelection) (Text_Source(w),
+					 left, right, ev->xkey.time);
 	break;
 
     case 4:			/* down */
@@ -1399,18 +1506,14 @@ _KeySelection(Widget w, XEvent *ev, int flag, String *params, Cardinal *np)
 	    {
 		right = Text_CursorPos(w);
 	    }
-
-	    (*Text_Source(w)->SetSelection) (Text_Source(w),
-					     left, right, ev->xkey.time);
 	}
 	else
 	{			/* New selection */
 	    left = old;
 	    right = Text_CursorPos(w);
-
-	    (*Text_Source(w)->SetSelection) (Text_Source(w),
-					     left, right, ev->xkey.time);
 	}
+	(*Text_Source(w)->SetSelection) (Text_Source(w),
+					 left, right, ev->xkey.time);
 	break;
     }
 }
@@ -1564,16 +1667,15 @@ static void
 MoveNextLine(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
 {
     XmTextWidget w = (XmTextWidget)aw;
+/* SG (15/08/1998) having these as unsigned gives erroneous results
+   in the test index <= maxindex-2 when maxindex is 1
     unsigned int index, maxindex;
+*/
+    int index, maxindex;
     XmTextPosition pos = Text_CursorPos(w);
 
     DEBUGOUT(XdbDebug(__FILE__, aw, "MoveNextLine"));
 
-    if (!Text_Editable(w))
-    {
-	return;
-    }
-
     index = _XmTextGetTableIndex(w, pos);
     if (index == (maxindex = Text_TotalLines(w) - 1))
     {
@@ -1614,11 +1716,6 @@ MoveNextPage(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
 
     DEBUGOUT(XdbDebug(__FILE__, aw, "MoveNextPage"));
 
-    if (!Text_Editable(w))
-    {
-	return;
-    }
-
     index = _XmTextGetTableIndex(w, pos);
     maxindex = Text_TotalLines(w) - 1;
 
@@ -1733,11 +1830,6 @@ MovePreviousLine(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
 
     DEBUGOUT(XdbDebug(__FILE__, aw, "MovePreviousLine"));
 
-    if (!Text_Editable(w))
-    {
-	return;
-    }
-
     index = _XmTextGetTableIndex(w, pos);
     if (index == 0)
     {
@@ -1772,11 +1864,6 @@ MovePreviousPage(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
 
     DEBUGOUT(XdbDebug(__FILE__, aw, "MovePreviousPage"));
 
-    if (!Text_Editable(w))
-    {
-	return;
-    }
-
     index = _XmTextGetTableIndex(w, pos);
     if (index < delta)
     {
@@ -2033,11 +2120,15 @@ TextFocusIn(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	    XtCallCallbackList((Widget)w, Text_FocusCallback(w), &cbs);
 	}
 
-	XtCallActionProc((Widget)w, "PrimitiveFocusIn", ev,
-			 params, *num_params);
-
 	Out_HasFocus(o) = True;
     }
+
+    /*
+     * 980827 - pgw@hungry.com: Fix for highlight defect.
+     */
+    XtCallActionProc((Widget)w, "PrimitiveFocusIn", ev,
+			 params, *num_params);
+
 }
 
 static void
@@ -2066,11 +2157,14 @@ TextFocusOut(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	    XtCallCallbackList((Widget)w, Text_LosingFocusCallback(w), &cbs);
 	}
 
-	XtCallActionProc((Widget)w, "PrimitiveFocusOut", ev,
-			 params, *num_params);
-
 	Out_HasFocus(o) = False;
     }
+    /*
+     * 980827 - pgw@hungry.com: Fix for highlight defect.
+     */
+    XtCallActionProc((Widget)w, "PrimitiveFocusOut", ev,
+			 params, *num_params);
+
 }
 
 static void
