@@ -1,7 +1,7 @@
 /*                   -*-Mode: c; c-file-style: "GNU"-*-              */
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v 1.42 1998/05/06 21:32:11 u27113 Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v 1.48 1998/10/24 03:18:44 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v 1.42 1998/05/06 21:32:11 u27113 Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v 1.48 1998/10/24 03:18:44 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <Xm/XmP.h>
@@ -497,7 +497,7 @@ SizeRecalc(XmTextWidget w)
 static void
 DrawAll(XmTextWidget w)
 {
-    int i, hi, mh;
+    int i, hi, mh, resty;
     int highlight = XmHIGHLIGHT_NORMAL;		/* initial value */
     XmTextPosition np, first;
 
@@ -544,6 +544,15 @@ DrawAll(XmTextWidget w)
 			  i, line->start, next->start - 1, np));
 #endif
 
+       /* Cleaning part of widget */
+    XClearArea(XtDisplay(w), XtWindow(w),
+              Prim_HighlightThickness(w) + Prim_ShadowThickness(w),
+               Out_YDraw(o) + i * Out_FontHeight(o) - Out_FontAscent(o) + 1,
+              XtWidth(w) -
+              2 * (Prim_HighlightThickness(w) + Prim_ShadowThickness(w)),               
+               Out_FontHeight(o),
+              False);
+
 	/* Does highlighting change in this range of text ?? */
 	if (np >= line->start && np <= next->start - 1)
 	{			/* Yes */
@@ -607,6 +616,17 @@ DrawAll(XmTextWidget w)
 	    }
 	}
     }
+
+       /* Cleaning rest of widget */
+    resty = Out_YDraw(o) + i * Out_FontHeight(o) - Out_FontAscent(o) + 1;
+    XClearArea(XtDisplay(w), XtWindow(w),
+              Prim_HighlightThickness(w) + Prim_ShadowThickness(w),
+               resty,
+              XtWidth(w) -
+              2 * (Prim_HighlightThickness(w) + Prim_ShadowThickness(w)),               
+              XtHeight(w) - resty - 
+              (Prim_HighlightThickness(w) + Prim_ShadowThickness(w)),           
+              False);
 }
 
 
@@ -710,7 +730,7 @@ XToPos(XmTextWidget w, LineNum row, int x)
 		      row, x, Text_TotalLines(w)));
 
     /* Just guessing - Is this right ?? Danny */
-    if (row >= Text_TotalLines(w))
+    if (row >= Text_TotalLines(w) - 0)
     {
 	next = &Text_Line(w)[Text_TotalLines(w) - 1];
 	end = next->start - 1;
@@ -1012,7 +1032,12 @@ _XmChangeVSB(XmTextWidget w, XmTextPosition pos)
 		      NULL);
     }
 
-    newMax = _XmMax(Text_TotalLines(w), Out_Rows(o));
+/* SG (15/08/1998)    newMax = _XmMax(Text_TotalLines(w) - 1, Out_Rows(o));
+   with the original code it was not possible to use the scrollbar to scroll
+   down to the last line. Most obviously if out_rows is 1 (hence the slider
+   size is 1) and there are 2 lines of text, setting scrollbar max to 1
+   with a slider size of 1 means you cannot move the scroll bar at all ! */
+    newMax = _XmMax(Text_TotalLines(w) , Out_Rows(o));
 
     if (newMax == 0)
     {
@@ -1314,7 +1339,7 @@ HandleVBar(Widget sw, XtPointer client_data, XtPointer call_data)
     XmTextWidget w = (XmTextWidget)client_data;
     OutputData o = Text_OutputData(w);
     XmScrollBarCallbackStruct *cbs = (XmScrollBarCallbackStruct *)call_data;
-    Dimension pos = 0;
+    Dimension index;
 
     switch (cbs->reason)
     {
@@ -1323,7 +1348,6 @@ HandleVBar(Widget sw, XtPointer client_data, XtPointer call_data)
 			  "HandleVbar reason=XmCR_DECREMENT\n"));
 
 	XmTextScroll((Widget)w, -1);
-	pos = _XmTextGetTableIndex(w, Text_TopPos(w));
 	break;
 
     case XmCR_DRAG:
@@ -1331,15 +1355,14 @@ HandleVBar(Widget sw, XtPointer client_data, XtPointer call_data)
 			  "HandleVbar reason=XmCR_DRAG value=%d\n",
 			  cbs->value));
 
-	pos = cbs->value;
-	Text_TopPos(w) = Text_LineTable(w)[pos].start_pos;
+	index = cbs->value;
+	Text_TopPos(w) = Text_LineTable(w)[index].start_pos;
 	break;
 
     case XmCR_INCREMENT:
 	DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
 			  "HandleVbar reason=XmCR_INCREMENT\n"));
 	XmTextScroll((Widget)w, 1);
-	pos = _XmTextGetTableIndex(w, Text_TopPos(w));
 	break;
 
     case XmCR_PAGE_DECREMENT:
@@ -1347,7 +1370,6 @@ HandleVBar(Widget sw, XtPointer client_data, XtPointer call_data)
 			  "HandleVbar reason=XmCR_PAGE_DECREMENT\n"));
 
 	XmTextScroll((Widget)w, -Out_Rows(o));
-	pos = _XmTextGetTableIndex(w, Text_TopPos(w));
 	break;
 
     case XmCR_PAGE_INCREMENT:
@@ -1355,29 +1377,29 @@ HandleVBar(Widget sw, XtPointer client_data, XtPointer call_data)
 			  "HandleVbar reason=XmCR_PAGE_INCREMENT\n"));
 
 	XmTextScroll((Widget)w, Out_Rows(o));
-	pos = _XmTextGetTableIndex(w, Text_TopPos(w));
 	break;
 
     case XmCR_TO_BOTTOM:
 	DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
 			  "HandleVbar reason=XmCR_TO_BOTTOM\n"));
-	pos = 0;
-	Text_TopPos(w) = Text_FirstPos(w);
+
+	index = Text_TotalLines(w) - 1 - Out_Rows(o);
+	Text_TopPos(w) = Text_LineTable(w)[index].start_pos;
 	break;
 
     case XmCR_TO_TOP:
 	DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
 			  "HandleVbar reason=XmCR_TO_TOP\n"));
 
-	pos = _XmTextGetTableIndex(w, Text_TopPos(w)) - Text_LineCount(w) + 1;
+	Text_TopPos(w) = Text_FirstPos(w);
 	break;
 
     case XmCR_VALUE_CHANGED:
 	DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
 			  "HandleVbar reason=XmVALUE_CHANGED\n"));
 
-	pos = cbs->value;
-	Text_TopPos(w) = Text_LineTable(w)[pos].start_pos;
+	index = cbs->value;
+	Text_TopPos(w) = Text_LineTable(w)[index].start_pos;
 	break;
 
     default:
@@ -1386,11 +1408,6 @@ HandleVBar(Widget sw, XtPointer client_data, XtPointer call_data)
 	break;
     }
 
-#if 0
-    /* Peter says don't do this */
-    XtVaSetValues(Out_Vbar(o), XmNvalue, pos, NULL);
-#endif
-
     _XmRedisplayHBar(w, Out_XOffset(o));
 
     Text_NeedsRefigureLines(w) = True;
@@ -2016,7 +2033,7 @@ destroy(Widget aw)
     XmTextWidget w = (XmTextWidget)aw;
     OutputData o = Text_OutputData(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, aw, "destroy"));
+    DEBUGOUT(XdbDebug(__FILE__, aw, "destroy\n"));
 
     if (o && Out_TimerId(o))
     {
@@ -2056,6 +2073,7 @@ OutputExpose(Widget aw, XEvent *event, Region region)
 	return;
     }
 
+/*
     XClearArea(XtDisplay(w), XtWindow(w),
 	       Prim_HighlightThickness(w) + Prim_ShadowThickness(w),
 	       Prim_HighlightThickness(w) + Prim_ShadowThickness(w),
@@ -2064,6 +2082,7 @@ OutputExpose(Widget aw, XEvent *event, Region region)
 	       XtHeight(w) -
 	       2 * (Prim_HighlightThickness(w) + Prim_ShadowThickness(w)),
 	       False);
+*/
 
     DrawAll(w);
 
@@ -2105,6 +2124,18 @@ resize(Widget aw, Boolean state)
 
     SizeRecalc(w);
 
+    /* SG (15/08/1998) Update scrollbars immediately on resize
+       to give the correct visual feedback to the user and keep
+       scroll bars in sync with any keyboard actions after this resize */
+
+    _XmChangeVSB(w, Text_TopPos(w));
+    {
+    OutputData o = Text_OutputData(w);
+
+        _XmRedisplayHBar(w, Out_XOffset(o));
+    }
+
+
     /* Only adjust the GCs if they have already been created! */
     if (Out_DrawGC(o))
     {
@@ -2470,7 +2501,7 @@ _XmTextOutputGetSecResData(XmSecondaryResourceData *secResDataRtn)
 int
 _XmTextGetNumberLines(XmTextWidget w)
 {
-    return w->text.total_lines;
+    return w->text.total_lines - 1;
 }
 
 Boolean
