@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextStrSo.c,v 1.17 1998/05/02 21:01:08 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextStrSo.c,v 1.19 1998/08/24 21:37:02 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: TextStrSo.c,v 1.17 1998/05/02 21:01:08 rwscott Exp $";
+static const char rcsid[] = "$Id: TextStrSo.c,v 1.19 1998/08/24 21:37:02 rwscott Exp $";
 
 /*
  * Empirical evidence (fooling around with the structures found in Motif
@@ -69,6 +69,7 @@ StrPtr(XmSourceData d, XmTextPosition pos)
     return ptr;
 }
 
+#if 0
 static char *
 StrNext(XmSourceData d, XmTextPosition pos)
 {
@@ -87,6 +88,7 @@ StrPrev(XmSourceData d, XmTextPosition pos)
 	return NULL;
     }
 }
+#endif
 
 /* AddWidget ----------------------------------------------------------------
  *
@@ -440,7 +442,9 @@ Scan(XmTextSource source, XmTextPosition pos, XmTextScanType type,
     XmSourceData d = source->data;
     Boolean found = False;
     char *ptr;
+#ifdef VERBOSE_XdbDebug
     XmTextPosition ipos = pos;
+#endif
 
     if (pos > d->length)
     {
@@ -702,9 +706,14 @@ SetSelection(XmTextSource source, XmTextPosition left,
 
     if (left >= right)
     {				/* No decent selection */
+        /* SG 23/08/1998, just bring into line with changes in TextF.c */
+        if(!source->data->hasselection)
+        {
+            return; /* Already no selection, don't do unnecessary work */
+        }
 	source->data->hasselection = False;
-	source->data->left = left;
-	source->data->right = right;
+	source->data->left = -1;
+	source->data->right = -1;
 	source->data->prim_time = time;
 
 	XtDisownSelection((Widget)d->widgets[0], XA_PRIMARY, time);
