@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ToggleB.c,v 1.29 1998/03/16 21:52:18 rwscott Exp $
+ * $Id: ToggleB.c,v 1.35 1998/10/10 07:37:36 gritton Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ToggleB.c,v 1.29 1998/03/16 21:52:18 rwscott Exp $";
+static const char rcsid[] = "$Id: ToggleB.c,v 1.35 1998/10/10 07:37:36 gritton Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -37,7 +37,8 @@ static const char rcsid[] = "$Id: ToggleB.c,v 1.29 1998/03/16 21:52:18 rwscott E
 #include <Xm/TransltnsP.h>
 #include <X11/ShellP.h>
 
-extern void _XmRadioCallback(Widget, XtPointer, XtPointer);
+extern void _XmRadioHandler(Widget, Widget, XmToggleButtonCallbackStruct *,
+                Boolean);
 
 #ifndef __STDC__
 #include <varargs.h>
@@ -464,8 +465,11 @@ initialize(Widget request, Widget new_w,
 	}
 	else
 	{
-	TB_IndicatorDim(new_w) -= TB_Spacing(new_w);
-	TB_IndicatorDim(new_w) = TB_IndicatorDim(new_w) < 2 * Xm3D_ENHANCE_PIXEL ? 2 * Xm3D_ENHANCE_PIXEL : TB_IndicatorDim(new_w);
+	    TB_IndicatorDim(new_w) -= TB_Spacing(new_w);
+	    TB_IndicatorDim(new_w) = TB_IndicatorDim(new_w) <
+					2 * Xm3D_ENHANCE_PIXEL
+						? 2 * Xm3D_ENHANCE_PIXEL
+						: TB_IndicatorDim(new_w);
 	}
     }
     else
@@ -489,9 +493,16 @@ initialize(Widget request, Widget new_w,
 	}
     }
 
+#if 0
+    /* rws 26 Sep 1998
+       rowcolumn/test19 show this should not be here
+     */
     if (TB_IndType(new_w) == XmONE_OF_MANY &&
 	TB_IndicatorDim(new_w) > Xm3D_ENHANCE_PIXEL)
+    {
 	TB_IndicatorDim(new_w) -= Xm3D_ENHANCE_PIXEL;
+    }
+#endif
 
     /* allocate the normal and insensitive GC's */
     Lab_MarginLeft(new_w) = TB_IndicatorDim(new_w) + (TB_IndOn(new_w)
@@ -539,11 +550,14 @@ initialize(Widget request, Widget new_w,
 	   a form
 	 */
 	{
-	XmString string;
+	    XmString string;
 
-		string = XmStringCreateSimple("M");
-		if (XtHeight(new_w) < XmStringHeight(Lab_Font(new_w), string)) XtHeight(new_w) = XmStringHeight(Lab_Font(new_w), string);
-		XmStringFree(string);
+	    string = XmStringCreateSimple("M");
+	    if (XtHeight(new_w) < XmStringHeight(Lab_Font(new_w), string))
+            {
+	        XtHeight(new_w) = XmStringHeight(Lab_Font(new_w), string);
+            }
+	    XmStringFree(string);
 	}
 
 	(*xmLabelClassRec.core_class.resize) (new_w);
@@ -615,11 +629,14 @@ set_values(Widget old, Widget request, Widget new_w,
 static void
 expose(Widget w, XEvent *event, Region region)
 {
+    Dimension fill;
     Boolean State = TB_VisualSet(w);
     Pixmap tmp_pix = XmUNSPECIFIED_PIXMAP, tmp2_pix = XmUNSPECIFIED_PIXMAP;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "ToggleB Expose\n"));
 
+    fill = Prim_TopShadowColor(w) != TB_SelectColor(w) &&
+	Prim_BottomShadowColor(w) != TB_SelectColor(w);
     if (!TB_IndOn(w))
     {
 	if (TB_FillOnSelect(w))
@@ -694,10 +711,11 @@ expose(Widget w, XEvent *event, Region region)
 
 	XFillRectangle(XtDisplay(w), XtWindow(w),
 		       State ? TB_SelectGC(w) : TB_BackgroundGC(w),
-		       TB_Spacing(w) + Lab_Highlight(w) + Lab_Shadow(w),
-		       (XtHeight(w) - TB_IndicatorDim(w)) / 2,
-		       TB_IndicatorDim(w),
-		       TB_IndicatorDim(w));
+		       TB_Spacing(w) + Lab_Highlight(w) + Lab_Shadow(w) + 2 +
+		       (1 - fill),
+		       (XtHeight(w) - TB_IndicatorDim(w)) / 2 + 2 + (1 - fill),
+		       TB_IndicatorDim(w) - 4 - ((1 - fill) << 1),
+		       TB_IndicatorDim(w) - 4 - ((1 - fill) << 1));
 
 	if (TB_Visible(w) || State)
 	{
@@ -764,7 +782,7 @@ expose(Widget w, XEvent *event, Region region)
 			   TB_Spacing(w) + Lab_Highlight(w),
 			   (XtHeight(w) - TB_IndicatorDim(w)) / 2,
 			   TB_IndicatorDim(w), TB_IndicatorDim(w),
-			   2, True);
+			   2, fill);
 	}
 	if (!IN_MENU(w))
 	{
@@ -831,15 +849,18 @@ Select(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
 	if (TB_VisualSet(w) == TB_IndicatorSet(w))
 	{
-
-	    TB_Set(w) = TB_VisualSet(w);
-
 	    cbs.reason = XmCR_VALUE_CHANGED;
 	    cbs.event = event;
-	    cbs.set = TB_Set(w);
+	    cbs.set = TB_VisualSet(w);
 
+	    /*
+	     * Call radio handler *before* setting TB_Set
+	     *	so XmToggleButtonGetState still works.
+	     */
 	    if (XmIsRowColumn(XtParent(w)) && RC_RadioBehavior(XtParent(w)))
-		_XmRadioCallback(w, XtParent(w), (XtPointer)&cbs);
+		_XmRadioHandler(w, XtParent(w), &cbs, True);
+
+	    TB_Set(w) = TB_VisualSet(w);
 
 	    if (!Lab_SkipCallback(w) && TB_ValueChangedCallback(w))
 	    {
@@ -935,7 +956,7 @@ ButtonUp(Widget w, XEvent *event, String *params, Cardinal *num_params)
     cbs.set = TB_Set(w);
 
     if (XmIsRowColumn(XtParent(w)) && RC_RadioBehavior(XtParent(w)))
-	_XmRadioCallback(w, XtParent(w), (XtPointer)&cbs);
+	_XmRadioHandler(w, XtParent(w), &cbs, True);
 
     if (!Lab_SkipCallback(w) && TB_ValueChangedCallback(w))
     {
@@ -1106,7 +1127,7 @@ static void
 MenuProcEntry(int proc, Widget w,...)
 {
     va_list arg_list;
-    XEvent *event;
+    XEvent *event = NULL;
     extern void _XmTearOffArm();
  
     va_start(arg_list, w);
@@ -1117,7 +1138,7 @@ MenuProcEntry(proc, w, va_alist)
      va_dcl
 {
     va_list arglist;
-    XEvent *event;
+    XEvent *event = NULL;
     extern void _XmTearOffArm();
  
     va_start(arglist);
@@ -1190,22 +1211,14 @@ XmToggleButtonSetState(Widget w, Boolean state, Boolean notify)
 	return;
     }
 
-    /*
-     * Radio behaviour is disturbed by using this with notify == False.
-     *	The radio callback won't run.
-     *	Therefore, a hack ... a better solution might be not to use a
-     *	callback for this at all... we have a dependency between toggles
-     *	and RC anyway.
-     *	Danny 28/8/97
-     */
-    cbs.reason = XmCR_VALUE_CHANGED;
-    cbs.event = NULL;
-    cbs.set = state;
-
     if ((TB_Set(w) && !state) || (!TB_Set(w) && state))
     {
+	cbs.reason = XmCR_VALUE_CHANGED;
+	cbs.event = NULL;
+	cbs.set = state;
+
 	if (XmIsRowColumn(XtParent(w)) && RC_RadioBehavior(XtParent(w)))
-	    _XmRadioCallback(w, XtParent(w), &cbs);
+	    _XmRadioHandler(w, XtParent(w), &cbs, notify);
 
 	TB_VisualSet(w) = TB_Set(w) = state ? True : False;
 
@@ -1225,6 +1238,7 @@ XmToggleButtonSetState(Widget w, Boolean state, Boolean notify)
     }
 }
 
+#if 0
 /*
  * A simpler version, only to be used by _XmRadioCallback.
  * This one is only concerned with the widget itself no callbacks.
@@ -1247,3 +1261,36 @@ _XmToggleButtonSetState(Widget w, Boolean state)
 	}
     }
 }
+#else
+/*
+ * Redefine this sucker
+ * It should do callbacks, but it shouldn't think about it.
+ * That is: only _XmRadioCallback knows what's going on.
+ * Trust it and just do as it says.
+ * The notify parameter has a different meaning than.
+ */
+void
+_XmToggleButtonSetState(Widget w, Boolean state, Boolean notify)
+{
+    XmToggleButtonCallbackStruct	cbs;
+    XtExposeProc			exp = XtClass(w)->core_class.expose;
+
+    DEBUGOUT(XdbDebug(__FILE__, w, "_XmToggleButtonSetState(%s)\n",
+		XdbBoolean2String(state)));
+
+    TB_VisualSet(w) = TB_Set(w) = state;
+
+    if (notify) {
+	if (XtIsRealized(w))
+	    (*exp) (w, NULL, (Region)NULL);
+
+	cbs.reason = XmCR_VALUE_CHANGED;
+	cbs.event = NULL;
+	cbs.set = state;
+
+	XtCallCallbackList(w,
+		TB_ValueChangedCallback(w),
+		(XtPointer)&cbs);
+    }
+}
+#endif
