@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ToggleBG.c,v 1.28 1998/03/18 20:33:56 rwscott Exp $
+ * $Id: ToggleBG.c,v 1.35 1998/10/10 07:37:36 gritton Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ToggleBG.c,v 1.28 1998/03/18 20:33:56 rwscott Exp $";
+static const char rcsid[] = "$Id: ToggleBG.c,v 1.35 1998/10/10 07:37:36 gritton Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -47,7 +47,8 @@ static const char rcsid[] = "$Id: ToggleBG.c,v 1.28 1998/03/18 20:33:56 rwscott
 
 #include <XmI/DebugUtil.h>
 
-extern void _XmRadioCallback(Widget, XtPointer, XtPointer);
+extern void _XmRadioHandler(Widget, Widget, XmToggleButtonCallbackStruct *,
+                Boolean notify);
 
 /* Forward Declarations */
 
@@ -152,11 +153,28 @@ static XtResource cache_resources[] =
 	sizeof(Pixmap), Offset(insen_pixmap),
 	XmRImmediate, (XtPointer)XmUNSPECIFIED_PIXMAP
     },
+#if XmVERSION > 1
+#if LESSTIF_BACK_COMPAT
     {
 	XmNindicatorOn, XmCIndicatorOn, XmRBoolean,
 	sizeof(Boolean), Offset(ind_on),
 	XmRImmediate, (XtPointer)True
     },
+#else
+    /* The type changed to an unsigned char in 2.0..oh just lovely. */
+    {
+	XmNindicatorOn, XmCIndicatorOn, XmRIndicatorOn,
+        sizeof(unsigned char), Offset(ind_on),
+	XmRImmediate, (XtPointer)True
+    },
+#endif
+#else
+    {
+	XmNindicatorOn, XmCIndicatorOn, XmRBoolean,
+	sizeof(Boolean), Offset(ind_on),
+	XmRImmediate, (XtPointer)True
+    },
+#endif
     {
 	XmNfillOnSelect, XmCFillOnSelect, XmRBoolean,
 	sizeof(Boolean), Offset(fill_on_select),
@@ -605,40 +623,33 @@ initialize(Widget request, Widget new_w,
 	{
 	    if (IN_MENU(new_w) && LabG_TextRect_height(new_w) != 0 &&
 		LabG_TextRect_height(new_w) > 2 * Xm3D_ENHANCE_PIXEL)
+            {
 		TBG_IndicatorDim(new_w) = LabG_TextRect_height(new_w) -
 		    2 * Xm3D_ENHANCE_PIXEL;
+            }
 	    else
+            {
 		TBG_IndicatorDim(new_w) = XmDEFAULT_INDICATOR_DIM;
+            }
 	}
+        else
+        {
+            /* MLM: 6/11/98: OK, but which of these is right? */
+            TBG_IndicatorDim(new_w) -= TBG_Spacing(new_w);
+            TBG_IndicatorDim(new_w) = TBG_IndicatorDim(new_w) <
+					2 * Xm3D_ENHANCE_PIXEL
+						? 2 * Xm3D_ENHANCE_PIXEL
+						: TBG_IndicatorDim(new_w);
+        }
     }
     else
     {
 	TBG_IndicatorDim(new_w) = 0;
-        TBG_IndicatorDim(new_w) -= TBG_Spacing(new_w);
-        TBG_IndicatorDim(new_w) = TBG_IndicatorDim(new_w) < 2 * Xm3D_ENHANCE_PIXEL
- ? 2 * Xm3D_ENHANCE_PIXEL : TBG_IndicatorDim(new_w);
     }
 
     CreateSelectGC(new_w);
     CreateBackgroundGC(new_w);
 
-    /*
-     * have to check request since new may have been polluted by a
-     * superclass
-     */
-    if (XtWidth(request) == (Dimension)0 || XtHeight(request) == (Dimension)0)
-    {
-	/* call this to determine the real dimensions of the label 
-	 * since we don't want to use 0
-	 */
-	XtWidth(new_w) = 0;
-	XtHeight(new_w) = 0;
-
-	_XmCalcLabelGDimensions(new_w);
-
-	(*xmLabelGadgetClassRec.rect_class.resize) (new_w);
-    }
-
     if (TBG_IndType(new_w) == (unsigned char)XmUNSPECIFIED)
     {
 	if (XmIsRowColumn(XtParent(new_w)) &&
@@ -651,18 +662,28 @@ initialize(Widget request, Widget new_w,
 	    TBG_IndType(new_w) = XmN_OF_MANY;
 	}
     }
-
+#if 0
+    /* rws 26 Sep 1998
+       rowcolumn/test9 and 35 show this should not be here
+       test9 good
+       test8 35 good
+     */
     if (TBG_IndType(new_w) == XmONE_OF_MANY &&
 	TBG_IndicatorDim(new_w) > Xm3D_ENHANCE_PIXEL)
     {
 	TBG_IndicatorDim(new_w) -= Xm3D_ENHANCE_PIXEL;
     }
+#endif
 
     if (!XtIsSubclass(XtParent(new_w), xmManagerWidgetClass))
     {
 	_XmError(new_w, "parent should be manager.");
     }
 
+    LabG_MarginLeft(new_w) = TBG_IndicatorDim(new_w) + (TBG_IndOn(new_w)
+							? 2 * TBG_Spacing(new_w)
+							: 0);
+
     if (!IN_MENU(new_w))
     {
 	if (G_HighlightThickness(new_w) != 0)
@@ -684,10 +705,6 @@ initialize(Widget request, Widget new_w,
 	}
     }
 
-    LabG_MarginLeft(new_w) = TBG_IndicatorDim(new_w) + (TBG_IndOn(new_w)
-							? 2 * TBG_Spacing(new_w)
-							: 0);
-
     if (LabG_MarginLeft(new_w) != LabG_MarginLeft(request))
     {
 
@@ -701,6 +718,23 @@ initialize(Widget request, Widget new_w,
 	}
 
 	_XmCalcLabelGDimensions(new_w);
+        /* rws 16 Aug 1997
+           Do not let the toggle button be any shorter than the height of
+           a character. This is to make sure the pixmap can be drawn
+           correctly. This shows in Mosaic with radio or toggle buttons in
+           a form
+         */
+        {
+            XmString string;
+
+            string = XmStringCreateSimple("M");
+            if (XtHeight(new_w) < XmStringHeight(LabG_Font(new_w), string))
+            {
+                XtHeight(new_w) = XmStringHeight(LabG_Font(new_w), string);
+            }
+            XmStringFree(string);
+        }
+
 
 	(*xmLabelGadgetClassRec.rect_class.resize) (new_w);
     }
@@ -943,9 +977,12 @@ get_values_posthook(Widget new_w, ArgList args, Cardinal *num_args)
 static void
 expose(Widget w, XEvent *event, Region region)
 {
+    Dimension fill;
     Boolean State = TBG_VisualSet(w);
     Pixmap tmp_pix = XmUNSPECIFIED_PIXMAP, tmp2_pix = XmUNSPECIFIED_PIXMAP;
 
+    fill = XmParentTopShadowColor(w) != TBG_SelectColor(w) &&
+	XmParentBottomShadowColor(w) != TBG_SelectColor(w);
     if (!TBG_IndOn(w))
     {
 	if (TBG_FillOnSelect(w))
@@ -1022,10 +1059,11 @@ expose(Widget w, XEvent *event, Region region)
 	XFillRectangle(XtDisplayOfObject(w), XtWindowOfObject(w),
 		       State ? TBG_SelectGC(w) : TBG_BackgroundGC(w),
 		       TBG_Spacing(w) + LabG_Highlight(w)
-		       + LabG_Shadow(w) + XtX(w),
-		       (XtHeight(w) - TBG_IndicatorDim(w)) / 2 + XtY(w),
-		       TBG_IndicatorDim(w),
-		       TBG_IndicatorDim(w));
+		       + LabG_Shadow(w) + XtX(w) + 2 + (1 - fill),
+		       (XtHeight(w) - TBG_IndicatorDim(w)) / 2 + XtY(w) + 2 +
+		       (1 - fill),
+		       TBG_IndicatorDim(w) - 4 - ((1 - fill) << 1),
+		       TBG_IndicatorDim(w) - 4 - ((1 - fill) << 1));
 	if (TBG_Visible(w) || State)
 	{
 	    _XmDrawShadows(XtDisplayOfObject(w), XtWindowOfObject(w),
@@ -1094,7 +1132,7 @@ expose(Widget w, XEvent *event, Region region)
 			   (XtHeight(w) - TBG_IndicatorDim(w)) / 2 + XtY(w),
 			   TBG_IndicatorDim(w), TBG_IndicatorDim(w),
 			   2,
-			   True);
+			   fill);
 	}
 	if (!IN_MENU(w))
 	{
@@ -1168,15 +1206,18 @@ Select(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
 	if (TBG_VisualSet(w) == TBG_IndicatorSet(w))
 	{
-
-	    TBG_Set(w) = TBG_VisualSet(w);
-
 	    cbs.reason = XmCR_VALUE_CHANGED;
 	    cbs.event = event;
-	    cbs.set = TBG_Set(w);
+	    cbs.set = TBG_VisualSet(w);
 
+	    /*
+	     * Call to radio handler precedes TBG_Set
+	     *	so calling XmToggleButtonGadgetGetState still works.
+	     */
 	    if (XmIsRowColumn(XtParent(w)) && RC_RadioBehavior(XtParent(w)))
-		_XmRadioCallback(w, XtParent(w), (XtPointer)&cbs);
+		_XmRadioHandler(w, XtParent(w), &cbs, True);
+
+	    TBG_Set(w) = TBG_VisualSet(w);
 
 	    if (!LabG_SkipCallback(w) && TBG_ValueChangedCallback(w))
 	    {
@@ -1284,7 +1325,7 @@ ButtonUp(Widget w, XEvent *event, String *params, Cardinal *num_params)
     cbs.set = TBG_Set(w);
 
     if (XmIsRowColumn(XtParent(w)) && RC_RadioBehavior(XtParent(w)))
-	_XmRadioCallback(w, XtParent(w), (XtPointer)&cbs);
+	_XmRadioHandler(w, XtParent(w), &cbs, True);
 
     if (!LabG_SkipCallback(w) && TBG_ValueChangedCallback(w))
     {
@@ -1508,7 +1549,7 @@ static void
 MenuProcEntry(int proc, Widget w,...)
 {
     va_list arg_list;
-    XEvent *event;
+    XEvent *event = NULL;
     extern void _XmTearOffArm();
  
     va_start(arg_list, w);
@@ -1519,7 +1560,7 @@ MenuProcEntry(proc, w, va_alist)
      va_dcl
 {
     va_list arglist;
-    XEvent *event;
+    XEvent *event = NULL;
     extern void _XmTearOffArm();
  
     va_start(arglist);
@@ -1597,7 +1638,7 @@ XmToggleButtonGadgetSetState(Widget w, Boolean state, Boolean notify)
     {
 
 	if (XmIsRowColumn(XtParent(w)) && RC_RadioBehavior(XtParent(w)))
-	    _XmRadioCallback(w, XtParent(w), &cbs);
+	    _XmRadioHandler(w, XtParent(w), &cbs, notify);
 
 	TBG_VisualSet(w) = TBG_Set(w) = state ? True : False;
 
@@ -1616,6 +1657,7 @@ XmToggleButtonGadgetSetState(Widget w, Boolean state, Boolean notify)
     }
 }
 
+#if 0
 /*
  * Simpler version of the above, for use by _XmRadioCallback only.
  */
@@ -1638,3 +1680,29 @@ _XmToggleButtonGadgetSetState(Widget w, Boolean state)
 	}
     }
 }
+#else
+void
+_XmToggleButtonGadgetSetState(Widget w, Boolean state, Boolean notify)
+{
+    XmToggleButtonCallbackStruct cbs;
+    XtExposeProc exp = XtClass(w)->core_class.expose;
+
+    DEBUGOUT(XdbDebug(__FILE__, w, "_XmToggleButtonGadgetSetState(%s)\n",
+		XdbBoolean2String(state)));
+
+    TBG_VisualSet(w) = TBG_Set(w) = state;
+
+    if (notify) {
+	if (XtIsRealized(w) && XtIsManaged(w))
+	    (*exp) (w, NULL, (Region)NULL);
+
+	cbs.reason = XmCR_VALUE_CHANGED;
+	cbs.event = NULL;
+	cbs.set = state;
+
+	XtCallCallbackList(w,
+		TBG_ValueChangedCallback(w),
+		(XtPointer)&cbs);
+    }
+}
+#endif
