@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Traversal.c,v 1.23 1998/02/15 15:07:33 rwscott Exp $
+ * $Id: Traversal.c,v 1.28 1998/10/01 02:23:09 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: Traversal.c,v 1.23 1998/02/15 15:07:33 rwscott Exp $";
+static const char rcsid[] = "$Id: Traversal.c,v 1.28 1998/10/01 02:23:09 rwscott Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -445,48 +445,62 @@ int
 CompareNodesVert(XmConst void *a, XmConst void *b)
 {
     XmTravTreeNode nodea = *(XmTravTreeNode *)a, nodeb = *(XmTravTreeNode *)b;
+    Position centera = nodea->rect.x + nodea->rect.width / 2;
+    Position centerb = nodeb->rect.x + nodeb->rect.width / 2;
 
-    if (nodea->rect.x == nodeb->rect.x)
+    if (centera > nodeb->rect.x + nodeb->rect.width &&
+	centerb < nodea->rect.x)
     {
-	if (nodea->rect.y < nodeb->rect.y)
-	    return -1;
-	else if (nodea->rect.y > nodeb->rect.y)
-	    return 1;
-	else
-	    return 0;
+	return 1;
     }
-    else if (nodea->rect.x < nodeb->rect.x)
+    else if (centerb > nodea->rect.x + nodea->rect.width &&
+	centera < nodeb->rect.x)
     {
 	return -1;
     }
-    else
+    else if (nodea->rect.y < nodeb->rect.y)
+    {
+	return -1;
+    }
+    else if (nodea->rect.y > nodeb->rect.y)
     {
 	return 1;
     }
+    else
+    {
+	return 0;
+    }
 }
 
 int
 CompareNodesHoriz(XmConst void *a, XmConst void *b)
 {
     XmTravTreeNode nodea = *(XmTravTreeNode *)a, nodeb = *(XmTravTreeNode *)b;
+    Position centera = nodea->rect.y + nodea->rect.height / 2;
+    Position centerb = nodeb->rect.y + nodeb->rect.height / 2;
 
-    if (nodea->rect.y == nodeb->rect.y)
+    if (centera > nodeb->rect.y + nodeb->rect.height &&
+	centerb < nodea->rect.y)
     {
-	if (nodea->rect.x < nodeb->rect.x)
-	    return -1;
-	else if (nodea->rect.x > nodeb->rect.x)
-	    return 1;
-	else
-	    return 0;
+	return 1;
     }
-    else if (nodea->rect.y < nodeb->rect.y)
+    else if (centerb > nodea->rect.y + nodea->rect.height &&
+	centera < nodeb->rect.y)
     {
 	return -1;
     }
-    else
+    else if (nodea->rect.x < nodeb->rect.x)
+    {
+	return -1;
+    }
+    else if (nodea->rect.x > nodeb->rect.x)
     {
 	return 1;
     }
+    else
+    {
+	return 0;
+    }
 }
 
 void
@@ -523,7 +537,7 @@ SortGraph(XmTravTreeNode node, Boolean have_excls)
     {
 	if (have_excls)
 	{
-	    qsort(block, sizeof(XmTravTreeNode), cnt, CompareExcls);
+	    qsort(block, cnt, sizeof(XmTravTreeNode), CompareExcls);
 	}
 	else
 	{
@@ -1330,7 +1344,7 @@ DeleteFromTabList(XmTravTree tree, int node)
 	return;
     }
 
-    bcopy(&tree->excl_tabs[node], &tree->excl_tabs[node + 1],
+    bcopy(&tree->excl_tabs[node + 1], &tree->excl_tabs[node],
 	  (tree->num_tab_entries - node - 1) * sizeof(Widget));
 
     tree->num_tab_entries--;
@@ -2410,10 +2424,19 @@ _XmIsScrollableClipWidget(Widget w, XRectangle *rect)
     return XtParent(w);
 }
 
+/*
+ * Function Name: _XmCreateVisibilityRect
+ *
+ * If the widget isnt visible the function returns False, else it
+ * creates the intersection between the widget and all its parents execpt
+ * the shell and returns True.
+ */
+
 Boolean
 _XmCreateVisibilityRect(Widget w,
 			XRectangle *rectPtr)
 {
+    Widget cw;
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmCreateVisibilityRect\n"));
 
     if (!_XmIsViewable(w))
@@ -2421,27 +2444,32 @@ _XmCreateVisibilityRect(Widget w,
 	_XmClearRect(rectPtr);
 	return False;
     }
-
-    if (w && XtParent(w) && _XmIsScrollableClipWidget(XtParent(w), rectPtr))
+    if (w && XtParent(w) &&
+	(cw =_XmIsScrollableClipWidget(XtParent(w), rectPtr)))
     {
-	while (w && !XtIsShell(w))
+	w = cw;
+	if (!_XmIsViewable(w))
 	{
-	    if (_XmIsViewable(w) && _XmIntersectRect(rectPtr, w, rectPtr))
-	    {
-		w = XtParent(w);
-	    }
-	    else
-	    {
-		_XmClearRect(rectPtr);
-		return False;
-	    }
+	    _XmClearRect(rectPtr);
+	    return False;
 	}
-	return True;
     }
 
-    _XmClearRect(rectPtr);
+    _XmSetRect(rectPtr, w);
 
-    return False;
+    while (w && !XtIsShell(w))
+    {
+	if (_XmIsViewable(w) && _XmIntersectRect(rectPtr, w, rectPtr))
+	{
+	    w = XtParent(w);
+	}
+	else
+	{
+	    _XmClearRect(rectPtr);
+	    return False;
+	}
+    }
+    return True;
 }
 
 Boolean
@@ -2484,7 +2512,7 @@ _XmCallFocusMoved(Widget old,
 
     cbs.reason = XmCR_FOCUS;
     cbs.event = event;
-    cbs.cont = old ? False : True;
+    cbs.cont = True;
     cbs.old_focus = old;
     cbs.new_focus = new_wid;
     cbs.focus_policy = VSEP_FocusPolicy(ve);
@@ -2870,7 +2898,7 @@ _XmMgrTraversal(Widget widget, XmTraversalDirection direction)
     {
 
 	if (widget != NULL && ((tg = XmGetTabGroup(widget)) != NULL) &&
-	    _XmCallFocusMoved(cur_focus, widget, False) &&
+	    _XmCallFocusMoved(cur_focus, widget, NULL) &&
 	    _XmCallTraverseObscured(widget, direction))
 	{
 
@@ -3617,7 +3645,7 @@ XmGetVisibility(Widget widget)
 	return XmVISIBILITY_FULLY_OBSCURED;
     }
 
-    if (_XmCreateVisibilityRect(widget, &rect))
+    if (!_XmCreateVisibilityRect(widget, &rect))
     {
 	return XmVISIBILITY_FULLY_OBSCURED;
     }
