@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Vendor.c,v 1.33 1998/03/13 22:42:14 u27113 Exp $
+ * $Id: Vendor.c,v 1.38 1998/04/07 20:38:18 u27113 Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: Vendor.c,v 1.33 1998/03/13 22:42:14 u27113 Exp $";
+static const char rcsid[] = "$Id: Vendor.c,v 1.38 1998/04/07 20:38:18 u27113 Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -965,14 +965,12 @@ initialize(Widget req, Widget new_w,
 
     if (!XmIsDisplay(new_w))
     {
-#ifndef LESSTIF_PRODUCTION
 	extern void _XmNSEEditResCheckMessages(Widget w, XtPointer data,
 					       XEvent *event, Boolean *cont);
 
 	/* add the handler for editres messages */
 	XtAddEventHandler(new_w, (EventMask)0, True,
 			  (XtEventHandler)_XmNSEEditResCheckMessages, NULL);
-#endif
 
 	DEBUGOUT(XdbDebug(__FILE__, new_w,
 			  "Setting up virtual key bindings\n"));
@@ -1036,6 +1034,32 @@ realize(Widget w,
 #define superclass (&wmShellClassRec)
     (*superclass->core_class.realize) (w, value_mask, attributes);
 #undef superclass
+#if 0
+    {
+    int i;
+
+    	printf("%s %s %d %d %s\n",
+    		XtName(w),
+    		XtClass(w)->core_class.class_name,
+    		CoreNumPopups(w),
+    		MGR_NumChildren(w),
+    		XtParent(w) ? XtName(XtParent(w)) : "NULL"
+    		);
+    	for (i = 0; i < MGR_NumChildren(w); i++)
+    	{
+	    printf("\t%s %s\n",
+	    	XtName(MGR_Children(w)[i]),
+	    	XtClass(MGR_Children(w)[i])->core_class.class_name
+	    	);
+	    if (XtIsTransientShell(MGR_Children(w)[i]))
+	    {
+		XSetTransientForHint(XtDisplay(w),
+				     XtWindow(MGR_Children(w)[i]),
+				     XtWindow(w));
+	    }
+    	}
+    }
+#endif
 }
 
 static void
@@ -2719,12 +2743,12 @@ geometry_manager(Widget wid,
 	req.border_width = request->border_width;
 	req.request_mode |= CWBorderWidth;
     }
-    if (request->request_mode & CWX)
+    if ((request->request_mode & CWX) && request->x != 0)
     {
 	req.x = request->x;
 	req.request_mode |= CWX;
     }
-    if (request->request_mode & CWY)
+    if ((request->request_mode & CWY) && request->y != 0)
     {
 	req.y = request->y;
 	req.request_mode |= CWY;
@@ -2868,6 +2892,7 @@ _XmVendorExtSetValues(Widget cw, Widget rw, Widget nw,
 		      ArgList args, Cardinal *nargs)
 {
     Atom at;
+    long prop32[5];
 
     DEBUGOUT(XdbDebug(__FILE__, nw,
 		      "VendorShellExt %s SetValues\n", XtName(nw)));
@@ -2907,10 +2932,15 @@ _XmVendorExtSetValues(Widget cw, Widget rw, Widget nw,
 	bcmp(&VSEP_MwmHints(nw), &VSEP_MwmHints(cw), sizeof(MwmHints)) != 0)
     {
 	at = XmInternAtom(XtDisplay(nw), _XA_MWM_HINTS, False);
-	/* note the 32 format */
+	/* note the 32 format = long on client side */
+	prop32[0]=VSEP_MwmHints(nw).flags;
+	prop32[1]=VSEP_MwmHints(nw).functions;
+	prop32[2]=VSEP_MwmHints(nw).decorations;
+	prop32[3]=VSEP_MwmHints(nw).input_mode;
+	prop32[4]=VSEP_MwmHints(nw).status;
 	XChangeProperty(XtDisplay(nw), XtWindow(nw), at, at, 32,
 			PropModeReplace,
-			(unsigned char *)&VSEP_MwmHints(nw),
+			(unsigned char *)prop32,
 			PROP_MOTIF_WM_HINTS_ELEMENTS);
     }
 
@@ -2948,6 +2978,7 @@ _XmVendorExtRealize(Widget w, XtPointer closure, XtPointer call_data)
     Atom at;
     Widget par;
     int i;
+    long prop32[5];
 
     DEBUGOUT(XdbDebug(__FILE__, w, "XmVendorExtRealize\n"));
 
@@ -2973,10 +3004,16 @@ _XmVendorExtRealize(Widget w, XtPointer closure, XtPointer call_data)
      * The XChangeProperty will set them all in one go.
      * note the 32 format for HINTS, the 8 format for MENU
      */
+    /* format 32 = long on client side */
+    prop32[0]=VSEP_MwmHints(w).flags;
+    prop32[1]=VSEP_MwmHints(w).functions;
+    prop32[2]=VSEP_MwmHints(w).decorations;
+    prop32[3]=VSEP_MwmHints(w).input_mode;
+    prop32[4]=VSEP_MwmHints(w).status;
     at = XmInternAtom(XtDisplay(par), _XA_MWM_HINTS, False);
     XChangeProperty(XtDisplay(par), XtWindow(par), at, at, 32,
 		    PropModeReplace,
-		    (unsigned char *)&VSEP_MwmHints(w),
+		    (unsigned char *)prop32,
 		    PROP_MOTIF_WM_HINTS_ELEMENTS);
 
     at = XmInternAtom(XtDisplay(par), _XA_MWM_MENU, False);
