@@ -1,6 +1,6 @@
 /*
 
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.19 1998/04/16 19:19:46 u27113 Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.32 1998/10/01 21:10:10 danny Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.19 1998/04/16 19:19:46 u27113 Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.32 1998/10/01 21:10:10 danny Exp $";
 
 /*
  * ** Danny had added a flag here, but I think the behavior he added is always
@@ -46,6 +46,7 @@ static char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.19
 
 #include <limits.h>
 #include <XmI/XmI.h>
+#include <XmI/MacrosI.h>
 
 #include <Xm/XmP.h>
 #include <X11/IntrinsicP.h>
@@ -62,6 +63,8 @@ static char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.19
 
 #include <XmI/DebugUtil.h>
 
+#include "LTmisc.h"
+
 /*
  * for information on the external encoding, see doc/XmStrings.txt
  */
@@ -418,14 +421,7 @@ __XmStringComponentCopy(struct __XmStringComponentRec *dest,
 {
     dest->type = src->type;
     dest->length = src->length;
-    if (src->data && src->length)
-    {
-	dest->data = XtNewString(src->data);
-    }
-    else
-    {
-	dest->data = src->data;
-    }
+    dest->data = XtNewString(src->data);
 }
 
 static _XmString
@@ -997,11 +993,11 @@ __XmStringSegmentExtent(XmFontList flist, _XmStringComponent comp,
 
 	XmbTextExtents((XFontSet)flist[comp->font].font,
 		       comp->data, comp->length, &ink, &log);
-	/* maybe? */
-	*height = ink.height;
-	*width = ink.width;
-	*ascent = ink.height - 1;
-	*descent = 1;
+
+	*height = log.height;
+	*width = log.width;
+	*ascent = -log.y;
+	*descent = log.height+log.y;
 	break;
     }
 
@@ -1229,8 +1225,13 @@ _XmStringFree(_XmString string)
 
     for (i = 0; i < string->number_of_components; i++)
     {
-	if (string->components[i]->data && string->components[i]->length)
+	if (string->components[i]->data /*&& string->components[i]->length*/)
 	{
+	    /* rws 30 Aug 1998
+	       If we only free this if the length is greater than 0 we end
+	       up not freeing the space from the string "" which has a length
+	       of 0 but there has been space allocated for it.
+	     */
 	    XtFree((char *)string->components[i]->data);
 	}
 
@@ -1558,12 +1559,53 @@ _XmStringSingleSegment(XmString str, char **pTextOut,
     return ret;
 }
 
+/*
+ * In Motif 2.x this becomes XmeSetWMShellTitle and even gets documented!
+ * (this function, apparently, has also been name XmeStringUpdateWMShellTitle
+ * as well, sigh).
+ *
+ * If "xmstr" consists of a single segment with the "ISO8859-1" tag, then
+ * the title and icon name will be set to the text of that segment and
+ * the encoding will be set to STRING; If "xmstr" consists of a single
+ * segment with the default font tag, then we do the same except the
+ * encoding is set to None.  Otherwise, we convert to compound text and
+ * set the encoding to COMPOUND_TEXT.
+ *
+ * FIX ME - deal with the font tags as per the above spec.
+ */
 void
 _XmStringUpdateWMShellTitle(XmString xmstr, Widget shell)
 {
-    DEBUGOUT(XdbDebug(__FILE__, shell,
-		      "_XmStringUpdateWMShellTitle is not implemented yet\n"));
-    /* FIX ME */
+    String title = NULL;
+    Atom encoding;
+    Boolean freeit;
+    Arg a[4];
+    int n;
+
+    if (XmStringGetLtoR(xmstr, XmFONTLIST_DEFAULT_TAG, &title)
+    &&  title != NULL)
+    {
+	freeit   = True;
+	encoding = None;
+    }
+    else
+    {
+	title    = "";
+	freeit   = False;
+	encoding = None;
+    }
+
+    n = 0;
+    XtSetArg(a[n], XmNtitle,            title);    ++n;
+    XtSetArg(a[n], XmNtitleEncoding,    encoding); ++n;
+    XtSetArg(a[n], XmNiconName,         title);    ++n;
+    XtSetArg(a[n], XmNiconNameEncoding, encoding); ++n;
+    XtSetValues(shell, a, n);
+
+    if (freeit)
+    {
+        XtFree(title);
+    }
 }
 
 Boolean
@@ -1611,8 +1653,8 @@ _XmStringGetNextSegment(_XmStringContext context,
 	*text = NULL;
     }
     if (direction)
-    {
-	*direction = XmSTRING_DIRECTION_DEFAULT;
+    { /* like in original motif */
+	*direction = XmSTRING_DIRECTION_L_TO_R;
     }
     if (separator)
     {
@@ -2123,7 +2165,7 @@ _XmStringDraw(Display *d, Window w, XmFontList fontlist, _XmString string,
 			       comp->data, comp->length, &ink, &log);
 
 		/* need to do something with the extents of the font set here. */
-		current_x += ink.width;
+		current_x += log.width;
 		break;
 	    }
 	}
@@ -2308,6 +2350,7 @@ _XmStringDrawImage(Display *d, Window w,
 			       comp->data, comp->length, &ink, &log);
 
 		/* need to do something with the extents of the font set here. */
+                current_x += log.width;
 		break;
 	    }
 	}
@@ -2585,10 +2628,40 @@ _XmStringDrawUnderline(Display *d, Window w, XmFontList fontlist,
 				   gc, current_x, current_y,
 				   comp->data, comp->length);
 
+                /* Added by SG 08/08/1998
+                 * Now *first* underline, then update current_x
+                 */
+
+                if (u && (p = strstr(comp->data, u)))
+                {           /* Underline */
+                int ww, llen = p - comp->data;
+
+		    XmbTextExtents((XFontSet)fontlist[comp->font].font,
+			       comp->data, llen, &ink, &log);
+		    xo = log.width;
+
+                    /* ww = width of underlined text */
+		    XmbTextExtents((XFontSet)fontlist[comp->font].font,
+			       u, strlen(u), &ink, &log);
+                    ww = log.width;
+
+                    /*
+                     * Subtracted 1 from the width to take the space
+                     * between characters into account. Is there a
+                     * decent way to do this ??  Danny 17/4/96
+                     */
+                    XDrawLine(d, w, gc,
+                              current_x + xo,
+                              current_y + line_descent,
+                              current_x + xo + ww - 1,
+                              current_y + line_descent);
+                }
+
+		
 		XmbTextExtents((XFontSet)fontlist[comp->font].font,
 			       comp->data, comp->length, &ink, &log);
-
 		/* need to do something with the extents of the font set here. */
+		current_x += log.width; /* Try this ! */
 		break;
 	    }
 	}
@@ -2682,15 +2755,50 @@ XmStringByteCompare(XmString s1,
     return ret;
 }
 
-Boolean
+Boolean 
 XmStringCompare(XmString s1, XmString s2)
 {
-    if (!_XmStringIsXmString(s1) || !_XmStringIsXmString(s2))
-    {
-	return False;
-    }
+ XmStringContext context1 = NULL, context2 = NULL;
+ char *text1, *text2, *tag1, *tag2;
+ Boolean separator1, separator2, compare_tags;
+ XmStringDirection direction1, direction2;
+ 
+ if (!_XmStringIsXmString(s1) || !_XmStringIsXmString(s2))
+ 	return False;
+ 
+ if (!XmStringInitContext(&context1, s1))
+ 	return False;
+ if (!XmStringInitContext(&context2, s2))
+ 	{ XmStringFreeContext(context1); return False; }
 
-    return XmStringByteCompare(s1, s2);
+ while (XmStringGetNextSegment(context1, 
+         &text1, &tag1, &direction1, &separator1)) {
+ 	if (!XmStringGetNextSegment(context2, 
+              &text2, &tag2, &direction2, &separator2)) {
+        	XtFree(text1);
+        	XmStringFreeContext(context1);
+        	return False;      
+        }
+        compare_tags = (strcmp(tag1, XmFONTLIST_DEFAULT_TAG) != 0 &&
+                        strcmp(tag2, XmFONTLIST_DEFAULT_TAG) != 0);
+         /* Don't compare tags when one of them = XmFONTLIST_DEFAULT_TAG */
+        if (strcmp(text1, text2) != 0 /* diferent text */
+            || direction1 != direction2
+            || (compare_tags && strcmp(tag1, tag2) != 0)
+            || separator1 != separator2) {
+        	XtFree(text1);
+		XtFree(text2);
+        	XmStringFreeContext(context1);
+        	XmStringFreeContext(context2);
+        	return False;
+        }
+ }
+
+ XtFree(text1);
+ XtFree(text2);
+ XmStringFreeContext(context1);
+ XmStringFreeContext(context2);
+ return True;  
 }
 
 XmString
@@ -3542,7 +3650,7 @@ XmStringGetLtoR(XmString string,
 void
 XmStringFree(XmString string)
 {
-    if (string == (XmString)XmUNSPECIFIED || string == NULL)
+    if (!_XmStringIsSpecified(string))
     {
 	return;
     }
@@ -3810,3 +3918,22 @@ XmStringLineCount(XmString string)
 
     return lc;
 }
+
+/*
+ * This sucker appears to be undocumented
+ */
+XmFontList
+XmStringCreateFontList( XFontStruct *font, XmStringCharSet charset)
+{
+	return XmFontListCreate(font, charset);
+}
+
+XmString
+XmStringConcatAndFree(XmString str1, XmString str2)
+{
+  XmString tmp;
+  tmp = XmStringConcat(str1, str2);
+  XmStringFree(str1);
+  XmStringFree(str2);
+  return tmp;
+}
