@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Xmos.c,v 1.17 1998/04/16 19:19:47 u27113 Exp $
+ * $Id: Xmos.c,v 1.23 1998/10/17 23:40:38 gritton Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,13 +22,27 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Xmos.c,v 1.17 1998/04/16 19:19:47 u27113 Exp $";
+static const char rcsid[] = "$Id: Xmos.c,v 1.23 1998/10/17 23:40:38 gritton Exp $";
 
 #include <LTconfig.h>
 
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
+#endif
+#endif
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
+#ifdef HAVE_PWD_H
+#include <pwd.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
 #include <ctype.h>
 #include <X11/Xfuncproto.h>
@@ -42,6 +56,8 @@ static const char rcsid[] = "$Id: Xmos.c,v 1.17 1998/04/16 19:19:47 u27113 Exp $
 
 #include <XmI/DebugUtil.h>
 
+#include "LTmisc.h"
+
 /*
  * this is OS dependent, but this should catch most OS's.
  */
@@ -54,21 +70,30 @@ static const char rcsid[] = "$Id: Xmos.c,v 1.17 1998/04/16 19:19:47 u27113 Exp $
  *
  * BTW - Does anyone knows whether we continue sleeping after a
  * signal was raised? I think so, but I'm not sure... --aldi
+ *
+ * Not so.  There are actually two functions, one for seconds and one
+ * for microseconds.  The semantics are identical to the well-known
+ * sleep() and usleep().  Neither continues after a signal.  - Jamie
  */
+
+int
+_XmSleep(unsigned int secs)
+{
+    return sleep(secs);
+}
+
 int
 _XmMicroSleep(long secs)
 {
-    unsigned int zzz;
+    struct timeval tv;
 
-    if (secs > 0)
-    {
-	zzz = (unsigned int)secs;
-	while ((zzz = sleep(zzz)) > 0)
-	{
-	    /* continue sleeping */
-	}
-    }
-    return 0;
+    /* usleep() isn't as common as it could be,
+     * so this isn't quite as pointless as _XmSleep.
+     */
+
+    tv.tv_sec = 0;
+    tv.tv_usec = secs;
+    return select(0, NULL, NULL, NULL, &tv);
 }
 
 XmString
@@ -95,6 +120,16 @@ _XmOSFindPatternPart(String fileSpec)
     static char *wildcards = "*?[]+/";
     String ret, tmp;
 
+    /*
+     * Wendell Duncan <duncan@mercury.chem.utah.edu> says stripping off the
+     * directory information helps : now directories with some of the
+     * traditional wildcard characters in their names will behave as expected.
+     */
+    if (strlen(fileSpec)) {
+	if ((ret = strrchr(fileSpec, '/')))
+		fileSpec = ret + 1;
+    }
+
     for (;;)
     {
 	tmp = fileSpec;
@@ -136,21 +171,33 @@ retry:
 
 }
 
- /*
-  * this is used if the cwd is unreadable
-  */
 String
 _XmOSGetHomeDirName(void)
 {
-    char *ret;
+    static char *home;
 
-    if ((ret = getenv("HOME")) != NULL)
+    /* Only actually check for the directory the first time.
+     * We just keep it around for future calls.
+     */
+
+    if (!home)
     {
-	return ret;
-    }
+	char *estr;
+	struct passwd *pw;
 
-    /* FIX ME -- check password file */
-    return NULL;
+	/* First, try the environment.
+	 * If that fails, look in the passwd file.
+	 * If nothing there, return NULL - the caller must handle this.
+	 */
+
+	home = (estr = getenv("HOME"))
+	    ? XtNewString(estr)
+	    : ((estr = getenv("LOGNAME")) && (pw = getpwuid(getuid()))) ||
+	      ((estr = getenv("USER")) && (pw = getpwuid(getuid()))) ||
+	      (pw = getpwuid(getuid()))
+	    ? XtNewString(pw->pw_dir) : "";
+    }
+    return home;
 }
 
 static Boolean
@@ -176,11 +223,7 @@ convertTwiddle(char *dir)
     }
     ptr++;			/* to get by twiddle */
 
-    if ((home = _XmOSGetHomeDirName()) == NULL)
-    {
-	return NULL;
-    }
-
+    home = _XmOSGetHomeDirName();
     newdir = XtMalloc(strlen(home) + strlen(ptr) + 1);
     strcpy(newdir, home);
     strcat(newdir, ptr);
@@ -211,16 +254,7 @@ _XmOSQualifyFileSpec(String dirSpec,
 	strncmp(dirSpec, ".", 1) == 0 || strncmp(dirSpec, "..", 2) == 0)
     {
 	if ((tmp = (char *)getcwd(NULL, MAX_PATH_LEN)) == NULL)
-	{
-	    if ((tmp = _XmOSGetHomeDirName()) == NULL)
-	    {
-		abort();	/* we've gotta give up sometime */
-	    }
-	    if ((tmp = XtNewString(tmp)) == NULL)
-	    {
-		abort();	/* we've gotta give up sometime */
-	    }
-	}
+	    tmp = XtNewString(_XmOSGetHomeDirName());
 
 	dir = (String)XtMalloc(strlen(tmp) + 1);
 	strcpy(dir, tmp);
@@ -272,16 +306,8 @@ _XmOSQualifyFileSpec(String dirSpec,
     String tmp, tmp1;
 
 	if ((tmp = (char *)getcwd(NULL, MAX_PATH_LEN)) == NULL)
-	{
-	    if ((tmp = _XmOSGetHomeDirName()) == NULL)
-	    {
-		abort();	/* we've gotta give up sometime */
-	    }
-	    if ((tmp = XtNewString(tmp)) == NULL)
-	    {
-		abort();	/* we've gotta give up sometime */
-	    }
-	}
+	    tmp = XtNewString(_XmOSGetHomeDirName());
+
 	tmp1 = XtMalloc(strlen(tmp) + strlen(dir) + 2);
 	strcpy(tmp1, tmp);
 	strcat(tmp1,"/");
@@ -294,19 +320,13 @@ _XmOSQualifyFileSpec(String dirSpec,
     if (*filterSpec == '/')
     {
 	XtFree(dir);
-	if ((dir = XtNewString(filterSpec)) == NULL)
-	{
-	    abort();		/* oh, well */
-	}
+	dir = XtNewString(filterSpec);
     }
     else if (startsWithTwiddle(dir))
     {
 	char *newdir;
 
-	if ((newdir = convertTwiddle(dir)) == NULL)
-	{
-	    abort();		/* oh, well */
-	}
+	newdir = convertTwiddle(dir);
 	XtFree(dir);
 	dir = newdir;
 	dir = XtRealloc(dir, strlen(dir) + strlen(filterSpec) + 1);
@@ -390,24 +410,16 @@ _XmOSQualifyFileSpec(String dirSpec,
 	if (strlen(tmp) != 0)
 	{
 	    tmp++;
-	    if ((filt = XtNewString(tmp)) == NULL)
-	    {
-		abort();
-	    }
+	    filt = XtNewString(tmp);
 	    *tmp = 0;
 	}
-	else if ((filt = XtNewString("*")) == NULL)
-	{
-	    abort();
-	}
+	else
+	    filt = XtNewString("*");
     }
     else
     {
 	filt = dir;
-	if ((dir = XtNewString("/")) == NULL)
-	{
-	    abort();
-	}
+	dir = XtNewString("/");
     }
 
     *pQualifiedDir = dir;
@@ -777,6 +789,9 @@ _XmOSBuildFileList(String dirPath,
     {
 	*pNumEntries = cnt;
 	*pEntries = ret;
+	/*
+	qsort((void *)ret, cnt, sizeof(String), vstrcmp);
+	*/
     }
 
     if (XdbInDebug(__FILE__, NULL))
@@ -795,10 +810,15 @@ _XmOSBuildFileList(String dirPath,
 /*
  * a sort function, perhaps?
  */
+/* rws 3 Jul 1998
+   Mozilla shows us what this is.  It is the comparison function used in
+   qsort.
+ */
+
 int
 _XmOSFileCompare(XmConst void *sp1, XmConst void *sp2)
 {
-    return strcmp(sp1, sp2);
+    return strcmp(*((char **)sp1), *((char **)sp2));
 }
 
 char _XmSDEFAULT_FONT[] = "fixed";
