@@ -1,6 +1,6 @@
 /**
  *
- * $Id: misc.c,v 1.7 1998/04/09 12:23:49 u27113 Exp $
+ * $Id: misc.c,v 1.10 1998/10/21 22:46:34 danny Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -29,12 +29,16 @@
 #include <LTconfig.h>
 #include <XmI/XmI.h>
 
+#ifdef	HAVE_CTYPE_H
+#include <ctype.h>
+#endif
+
 #include <Xm/XmP.h>
 #if XtSpecificationRelease < 6
 #include <X11/IntrinsicP.h>
 #endif
 
-static char *VersionString = XmVERSION_STRING;
+static const char *VersionString = XmVERSION_STRING;
 
 #ifndef HAVE_STRCASECMP
 int
@@ -196,3 +200,74 @@ _XmInstallStippleImages(Widget w)
 		   odd_stipple_width, odd_stipple_height, LSBFirst);
     XmInstallImage(odd_stipple_image, XmODD_STIPPLE_IMAGE);
 }
+
+/*
+ * Below are some functions to deal with a multi-threaded environment.
+ * Use these instead of the Xt stuff, to ensure that we can still cope
+ * with X11r5 where this stuff didn't exist.
+ */
+#ifdef	XtSpecificationRelease
+#if	XtSpecificationRelease > 5
+#define	R6plus
+#endif
+#endif
+
+void
+_XmAppLock(XtAppContext appc)
+{
+#ifdef	R6plus
+	XtAppLock(appc);
+#endif
+}
+
+void
+_XmAppUnlock(XtAppContext appc)
+{
+#ifdef	R6plus
+	XtAppUnlock(appc);
+#endif
+}
+
+void
+_XmProcessLock(void)
+{
+#ifdef	R6plus
+	XtProcessLock();
+#endif
+}
+
+void
+_XmProcessUnlock(void)
+{
+#ifdef	R6plus
+	XtProcessUnlock();
+#endif
+}
+
+/*
+ * These are utility functions so you don't need to type so much.
+ * Just pass a widget or gadget.
+ */
+void
+_XmObjectLock(Widget w)
+{
+#ifdef	R6plus
+	if (XmIsGadget(w))
+		XtAppLock(XtWidgetToApplicationContext(
+			XtParent(w)));
+	else
+		XtAppLock(XtWidgetToApplicationContext(w));
+#endif
+}
+
+void
+_XmObjectUnlock(Widget w)
+{
+#ifdef	R6plus
+	if (XmIsGadget(w))
+		XtAppUnlock(XtWidgetToApplicationContext(
+			XtParent(w)));
+	else
+		XtAppUnlock(XtWidgetToApplicationContext(w));
+#endif
+}
