@@ -26,7 +26,14 @@ main(int argc, char **argv)
 
   XtRealizeWidget(toplevel);
 
-  XtAppMainLoop(theApp);
+  
+{
+    static XtWidgetGeometry Expected[] = {
+   CWWidth | CWHeight            ,   50,   50,   50,   50, 0,0,0, /* Button1 */ 
+    };
+    PrintDetails(toplevel,Expected);
+};
+  LessTifTestMainLoop(toplevel);
 
   exit(0);
 }
