@@ -21,7 +21,9 @@ main(int argc, char **argv)
   printf("XmQmotif: %s\n", XrmQuarkToString(XmQmotif));
 
   XtRealizeWidget(toplevel);
-  XtAppMainLoop(app);
+  
+  /* NO GEOMETRY */
+  LessTifTestMainLoop(  toplevel );
 
   exit(0);
 }
