@@ -25,7 +25,14 @@ main(int argc, char **argv)
 	printf("IS NOT LABEL\n");
 
   XtRealizeWidget(toplevel);
-  XtAppMainLoop(app);
+  
+{
+    static XtWidgetGeometry Expected[] = {
+   CWWidth | CWHeight            ,   50,   50,   30,   25, 0,0,0, /* One */ 
+    };
+    PrintDetails(  toplevel ,Expected);
+};
+   LessTifTestMainLoop(  toplevel );
 
   exit(0);
 }
