@@ -30,7 +30,20 @@ main(int argc, char **argv) {
 				   bb, wargs,n);
 
     XtRealizeWidget(toplevel);
-    XtMainLoop();
+
+
+/* Note: the following values are the result of
+ * querying the current geometry.
+ */
+{
+static XtWidgetGeometry Expected[] = {
+   CWWidth | CWHeight            ,    0,    0,  500,  500, 0,0,0, /* board */
+   CWWidth | CWHeight | CWX | CWY,  200,  100,  500,  500, 0,0,0, /* canvas */
+};
+/* toplevel should be replaced with to correct applicationShell */
+PrintDetails(toplevel, Expected);
+}
+LessTifTestMainLoop(toplevel);
 }
 
 
