@@ -5,11 +5,25 @@ Widget GXTopWidget, GXClipBox;
 int main(int argc, char **argv)
 {
   XtAppContext theApp;
+  Widget label;
 
+  make_tmp_dir_tree();
   GXTopWidget= XtVaAppInitialize(&theApp, "test1", NULL, 0, &argc, argv,
 				 NULL, NULL); 
+  label = XmCreateLabel(GXTopWidget, "Label", NULL, 0);
+  XtManageChild(label);
   XtManageChild(GXCreateFileBox(GXTopWidget));
   XtRealizeWidget(GXTopWidget);
+
+  
+{
+    static XtWidgetGeometry Expected[] = {
+   CWWidth | CWHeight            ,   57,   73,   34,   17, 0,0,0, /* Label */ 
+    };
+    PrintDetails(GXTopWidget,Expected);
+};
+  LessTifTestMainLoop(GXTopWidget);
+
   XtAppMainLoop(theApp);
   exit(0);
 }
@@ -28,7 +42,7 @@ GXCreateFileBox(Widget parent)
     n = 0;
     /* XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL);  n++; */
     filew = XmCreateFileSelectionDialog(parent, "FileBox", args, n);
-
+    set_path(filew);
     /* these calls are here to force the menu bar to the top, but it fails
        under lesstif */
     work = XmCreateRowColumn(filew, "FileBoxWork", NULL, 0);
