@@ -23,7 +23,6 @@ AppResources_t AppResources;
 static String FallBack[] = {
 	"*geometry: +500+500",
 	"*borderWidth: 1",
-	"*autoExit: False",
 NULL};
 
 static XtResource resources[] = {
@@ -751,7 +750,10 @@ main(int argc, char **argv)
   XtAppAddTimeOut(app, AppResources.time,
 		  (XtTimerCallbackProc)DoAllTests, Form);
 
-  XtAppMainLoop(app);
+      XtAppMainLoop(app);
+      /*
+      LessTifTestMainLoop(toplevel);
+      */
 
   exit(0);
 }
