@@ -49,14 +49,21 @@ main(int argc, char **argv)
 
   XtRealizeWidget(Shell);
   {
-  XtWidgetGeometry Expected[] = {
+  static XtWidgetGeometry Expected[] = {
   	CWWidth | CWHeight,		0,	0,	72,	41,	0,0,0,	/* Form */
   	CWWidth | CWHeight | CWX | CWY,	0,	0,	52,	17,	0,0,0,	/* two */
   	CWWidth | CWHeight | CWX | CWY,	0,	20,	70,	17,	0,0,0,	/* two */
+
+  	CWWidth | CWHeight,		0,	0,	144,	76,	0,0,0,	/* Form */
+  	CWWidth | CWHeight | CWX | CWY,	0,	0,	52,	17,	0,0,0,	/* two */
+  	CWWidth | CWHeight | CWX | CWY,	0,	20,	70,	52,	0,0,0,	/* two */
 };
 
+  PrintDetails(Shell, Expected);
+      LessTifTestWaitForIt(Shell);
+      LessTifTestResizeWidget(Shell, 144, 76);
   PrintDetails(Shell, Expected);
   }
-  XtAppMainLoop(app);
+      LessTifTestMainLoop(Shell);
   exit(0);
 }
