@@ -11,6 +11,7 @@
 
 static char *FallBack[] = {
 		"*.borderWidth: 1",
+		"*.geometrySlop: 1",
 		NULL
 };
 
@@ -39,13 +40,19 @@ main(int argc, char **argv)
 
   XtRealizeWidget(Shell);
   {
-  XtWidgetGeometry Expected[] = {
+  static XtWidgetGeometry Expected[] = {
   	CWWidth | CWHeight,		0,	0,	72,	37,	0,0,0,	/* Form */
   	CWWidth | CWHeight | CWX | CWY,	0,	0,	70,	17,	0,0,0,	/* two */
+
+  	CWWidth | CWHeight,		0,	0,	144,	76,	0,0,0,	/* Form */
+  	CWWidth | CWHeight | CWX | CWY,	0,	19,	70,	17,	0,0,0,	/* two */
 };
 
+  PrintDetails(Shell, Expected);
+      LessTifTestWaitForIt(Shell);
+      LessTifTestResizeWidget(Shell, 144, 76);
   PrintDetails(Shell, Expected);
   }
-  XtAppMainLoop(app);
+      LessTifTestMainLoop(Shell);
   exit(0);
 }
