@@ -47,14 +47,17 @@ main(int argc, char **argv)
   XtRealizeWidget(toplevel);
 
   {
-  XtWidgetGeometry Expected[] = {
+  static XtWidgetGeometry Expected[] = {
   	CWWidth | CWHeight,              0,  0,120,100, 0,0,0,	/* Form */
   	CWWidth | CWHeight | CWX | CWY, 45,25, 30, 25, 0,0,0,	/* two */
 };
 
   PrintDetails(toplevel, Expected);
   }
+  LessTifTestMainLoop(toplevel);
+  /*
   XtAppMainLoop(app);
+  */
 
   exit(0);
 }
