@@ -51,14 +51,21 @@ main(int argc, char **argv)
 
   XtRealizeWidget(Shell);
   {
-  XtWidgetGeometry Expected[] = {
+  static XtWidgetGeometry Expected[] = {
   	CWWidth | CWHeight,		0,	0,	126,	19,	0,0,0,	/* Form */
   	CWWidth | CWHeight | CWX | CWY,	0,	0,	52,	17,	0,0,0,	/* two */
   	CWWidth | CWHeight | CWX | CWY,	54,	0,	70,	17,	0,0,0,	/* two */
+
+  	CWWidth | CWHeight,		0,	0,	144,	76,	0,0,0,	/* Form */
+  	CWWidth | CWHeight | CWX | CWY,	0,	0,	52,	74,	0,0,0,	/* two */
+  	CWWidth | CWHeight | CWX | CWY,	54,	0,	70,	74,	0,0,0,	/* two */
 };
 
+  PrintDetails(Shell, Expected);
+      LessTifTestWaitForIt(Shell);
+      LessTifTestResizeWidget(Shell, 144, 76);
   PrintDetails(Shell, Expected);
   }
-  XtAppMainLoop(app);
+      LessTifTestMainLoop(Shell);
   exit(0);
 }
