@@ -44,13 +44,13 @@ main(int argc, char **argv)
 
   XtRealizeWidget(Shell);
   {
-  XtWidgetGeometry Expected[] = {
+  static XtWidgetGeometry Expected[] = {
   	CWWidth | CWHeight,		0,	0,	72,	92,	0,0,0,	/* Form */
   	CWWidth | CWHeight | CWX | CWY,	0,	23,	70,	17,	0,0,0,	/* two */
 };
 
   PrintDetails(Shell, Expected);
   }
-  XtAppMainLoop(app);
+      LessTifTestMainLoop(Shell);
   exit(0);
 }
