@@ -43,7 +43,7 @@ main(int argc, char **argv)
   XtRealizeWidget(toplevel);
 
   {
-  XtWidgetGeometry Expected[] = {
+  static XtWidgetGeometry Expected[] = {
   	CWWidth | CWHeight,              0,  0, 72, 25, 0,0,0,	/* Form */
   	CWWidth | CWHeight | CWX | CWY, 42,  0, 30, 25, 0,0,0,	/* two */
   	CWWidth | CWHeight | CWX | CWY,  0,  0, 42, 25, 0,0,0,	/* two */
@@ -51,7 +51,7 @@ main(int argc, char **argv)
 
   PrintDetails(toplevel, Expected);
   }
-  XtAppMainLoop(app);
+      LessTifTestMainLoop(toplevel);
 
   exit(0);
 }
