@@ -11,6 +11,7 @@
 
 static char *FallBack[] = {
 		"*.borderWidth: 1",
+		"*.geometrySlop: 1",
 		NULL
 };
 
@@ -51,7 +52,7 @@ main(int argc, char **argv)
 
   XtRealizeWidget(Shell);
   {
-  XtWidgetGeometry Expected[] = {
+  static XtWidgetGeometry Expected[] = {
   	CWWidth | CWHeight,		0,	0,	72,	62,	0,0,0,	/* Form */
   	CWWidth | CWHeight | CWX | CWY,	0,	12,	52,	17,	0,0,0,	/* two */
   	CWWidth | CWHeight | CWX | CWY,	0,	31,	70,	17,	0,0,0,	/* two */
@@ -59,6 +60,6 @@ main(int argc, char **argv)
 
   PrintDetails(Shell, Expected);
   }
-  XtAppMainLoop(app);
+      LessTifTestMainLoop(Shell);
   exit(0);
 }
