@@ -41,13 +41,13 @@ main(int argc, char **argv)
 
   XtRealizeWidget(Shell);
   {
-  XtWidgetGeometry Expected[] = {
+  static XtWidgetGeometry Expected[] = {
   	CWWidth | CWHeight,		0,	0,	82,	19,	0,0,0,	/* Form */
   	CWWidth | CWHeight | CWX | CWY,	10,	0,	70,	17,	0,0,0,	/* two */
 };
 
   PrintDetails(Shell, Expected);
   }
-  XtAppMainLoop(app);
+      LessTifTestMainLoop(Shell);
   exit(0);
 }
