@@ -735,7 +735,7 @@ int main(int argc, char **argv)
 	XtRealizeWidget(toplevel);
 
   {
-  XtWidgetGeometry Expected[] = {
+  static XtWidgetGeometry Expected[] = {
   	CWWidth | CWHeight,              0,  	0,	500,	504, 0,0,0,	/* ConfigureWindow */
   	CWWidth | CWHeight | CWX | CWY,383,	160,	99,	30, 0,0,0,	/* moveDownButton */
   	CWWidth | CWHeight | CWX | CWY,259,	160,	99,	30, 0,0,0,	/* moveUpButton */
@@ -762,6 +762,6 @@ int main(int argc, char **argv)
 
   PrintDetails(toplevel, Expected);
   }
-	XtAppMainLoop(app);
+	    LessTifTestMainLoop(toplevel);
 	exit(0);
 }
