@@ -46,14 +46,14 @@ main(int argc, char **argv)
   XtRealizeWidget(toplevel);
 
   {
-  XtWidgetGeometry Expected[] = {
+  static XtWidgetGeometry Expected[] = {
   	CWWidth | CWHeight,		0,	0,	30,	25,	0,0,0,	/* Form */
   	CWWidth | CWHeight | CWX | CWY,	0,	0,	30,	25,	0,0,0,	/* two */
 };
 
   PrintDetails(toplevel, Expected);
   }
-  XtAppMainLoop(app);
+      LessTifTestMainLoop(toplevel);
 
   exit(0);
 }
