@@ -6,6 +6,7 @@
 #include <Xm/MessageBP.h>
 
 char *fallback[] = {
+   "*.geometrySlop: 1",
    "*Box*fontList:      -*-times-bold-i-*--24-240-*=chset1,"
    "                    -*-times-bold-i-*--12-120-*=chset2,"
    "                    -misc-fixed-medium-r-*--10-100-*=chset3,"
@@ -20,6 +21,20 @@ void Push(Widget w, XtPointer client, XtPointer call)
 	Widget	dialog = (Widget)client;
 
 	XtManageChild(dialog);
+	LessTifTestWaitForIt(dialog);
+{
+static XtWidgetGeometry Expected[] = {
+   CWWidth | CWHeight            ,    6,   39,  253,  138, 0,0,0, /* Box */
+   CWWidth | CWHeight | CWX | CWY,   11,   24,   26,   26, 0,0,0, /* Symbol */
+   CWWidth | CWHeight | CWX | CWY,   47,   11,  195,   53, 0,0,0, /* Message */
+   CWWidth | CWHeight | CWX | CWY,    0,   74,  253,    2, 0,0,0, /* Separator */
+   CWWidth | CWHeight | CWX | CWY,   11,   86,   64,   41, 0,0,0, /* OK */
+   CWWidth | CWHeight | CWX | CWY,   94,   86,   64,   41, 0,0,0, /* Cancel */
+   CWWidth | CWHeight | CWX | CWY,  177,   86,   64,   41, 0,0,0, /* Help */
+};
+/* toplevel should be replaced with to correct applicationShell */
+PrintDetails(XtParent(dialog), Expected);
+}
 }
 
 int
@@ -54,6 +69,10 @@ main(int argc, char **argv)
 	XtRealizeWidget(toplevel);
 
 	/*XdbPrintTree(toplevel);*/
-	XtAppMainLoop(app);
+
+  LessTifTestWaitForIt(toplevel);
+  LessTifTestPushButton(push);
+  LessTifTestMainLoop(toplevel);
+
 	exit(0);
 }
