@@ -4,6 +4,11 @@
 #include <Xm/PushB.h>
 #include <stdio.h>
 
+static char *FallBack[] = {
+		"*.geometrySlop: 1",
+		NULL
+};
+
 void
 cb(Widget w, XtPointer data, XtPointer cbs)
 {
@@ -20,7 +25,7 @@ main(int argc, char **argv)
 
   XtSetLanguageProc(NULL, NULL, NULL);
 
-  toplevel = XtVaAppInitialize(&app, "Label", NULL, 0, &argc, argv, NULL, NULL);
+  toplevel = XtVaAppInitialize(&app, "Label", NULL, 0, &argc, argv, FallBack, NULL);
 
   fontlist = XmFontListAppendEntry(NULL,
 			   XmFontListEntryCreate("MY_FONT",
@@ -42,7 +47,18 @@ main(int argc, char **argv)
 
   XtRealizeWidget(toplevel);
 
+  {
+  static XtWidgetGeometry Expected[] = {
+  	CWWidth | CWHeight,		0,	0,	144,	50,	0,0,0,	/* Form */
+  	CWWidth | CWHeight | CWX | CWY,	45,	30,	30,	25,	0,0,0,	/* two */
+};
+
+  PrintDetails(toplevel, Expected);
+  }
+  LessTifTestMainLoop(toplevel);
+  /*
   XtAppMainLoop(app);
+  */
 
   exit(0);
 }
