@@ -36,7 +36,18 @@ main(int argc, char **argv)
 				XmNlabelString, xmstr, NULL);
 
   XtRealizeWidget(toplevel);
+  {
+  static XtWidgetGeometry Expected[] = {
+  	CWWidth | CWHeight,		0,	0,	185,	30,	0,0,0,	/* Form */
+  	CWWidth | CWHeight | CWX | CWY,	45,	30,	30,	25,	0,0,0,	/* two */
+};
+
+  PrintDetails(toplevel, Expected);
+  }
+  LessTifTestMainLoop(toplevel);
+  /*
   XtAppMainLoop(app);
+  */
 
   exit(0);
 }
