@@ -38,7 +38,15 @@ main(int argc, char **argv)
 
   XtRealizeWidget(toplevel);
 
-  XtAppMainLoop(app);
+  
+{
+    static XtWidgetGeometry Expected[] = {
+   CWWidth | CWHeight            ,   50,   50, 1987, 1987, 0,0,0, /* Two */
+   CWWidth | CWHeight | CWX | CWY,    0,    0,  987,  987, 0,0,0, /* One */ 
+    };
+    PrintDetails(  toplevel ,Expected);
+};
+   LessTifTestMainLoop(  toplevel );
 
   exit(0);
 }
