@@ -17,14 +17,24 @@ main(int argc, char **argv)
   toplevel = XtVaAppInitialize(&theApp, "toggle1", NULL, 0,
 			       &argc, argv, NULL, NULL);
 
-  butt= XtVaCreateManagedWidget("Button1", xmToggleButtonWidgetClass, toplevel, 
+  butt= XtVaCreateManagedWidget("Button1Button1", xmToggleButtonWidgetClass, toplevel, 
 				NULL);
 
   XtAddCallback(butt,XmNvalueChangedCallback,HiCB,NULL);
 
   XtRealizeWidget(toplevel);
 
-  XtAppMainLoop(theApp);
+/* Note: the following values are the result of
+ * querying the current geometry.
+ */
+{
+static XtWidgetGeometry Expected[] = {
+   CWWidth | CWHeight            ,    0,    0,  109,   25, 0,0,0, /* Button1Button1 */
+};
+/* toplevel should be replaced with to correct applicationShell */
+PrintDetails(toplevel, Expected);
+}
+LessTifTestMainLoop(toplevel);
 
   exit(0);
 }
