@@ -30,7 +30,16 @@ main(int argc, char **argv)
   XtManageChild(rowcol);
 
   XtRealizeWidget(toplevel);
-  XtAppMainLoop(app);
+  
+{
+    static XtWidgetGeometry Expected[] = {
+   CWWidth | CWHeight            ,   50,   50,   65,   56, 0,0,0, /* checkBox */
+   CWWidth | CWHeight | CWX | CWY,    3,    4,   59,   23, 0,0,0, /* button_0 */
+   CWWidth | CWHeight | CWX | CWY,    3,   29,   59,   23, 0,0,0, /* button_1 */ 
+    };
+    PrintDetails(  toplevel ,Expected);
+};
+   LessTifTestMainLoop(  toplevel );
 
   exit(0);
 }
