@@ -36,7 +36,16 @@ main(int argc, char **argv)
   XtManageChild(rowcol);
 
   XtRealizeWidget(toplevel);
-  XtAppMainLoop(app);
+  
+{
+    static XtWidgetGeometry Expected[] = {
+   CWWidth | CWHeight            ,   50,   50,   79,   23, 0,0,0, /* optionMenu */
+   CWWidth | CWHeight | CWX | CWY,    3,    3,   40,   17, 0,0,0, /* OptionLabel */
+   CWWidth | CWHeight | CWX | CWY,   46,    3,   30,    2, 0,0,0, /* OptionButton */ 
+    };
+    PrintDetails(  toplevel ,Expected);
+};
+   LessTifTestMainLoop(  toplevel );
 
   exit(0);
 }
