@@ -45,7 +45,20 @@ main(int argc, char **argv)
   XtManageChild(rc2);
 
   XtRealizeWidget(toplevel);
-  XtAppMainLoop(app);
+  
+{
+    static XtWidgetGeometry Expected[] = {
+   CWWidth | CWHeight            ,   50,   50,  130,   56, 0,0,0, /* form */
+   CWWidth | CWHeight | CWX | CWY,    0,    0,   65,   56, 0,0,0, /* checkBox1 */
+   CWWidth | CWHeight | CWX | CWY,    3,    4,   59,   23, 0,0,0, /* button_0 */
+   CWWidth | CWHeight | CWX | CWY,    3,   29,   59,   23, 0,0,0, /* button_1 */
+   CWWidth | CWHeight | CWX | CWY,   65,    0,   65,   56, 0,0,0, /* checkBox2 */
+   CWWidth | CWHeight | CWX | CWY,    3,    4,   59,   23, 0,0,0, /* button_0 */
+   CWWidth | CWHeight | CWX | CWY,    3,   29,   59,   23, 0,0,0, /* button_1 */ 
+    };
+    PrintDetails(  toplevel ,Expected);
+};
+   LessTifTestMainLoop(  toplevel );
 
   exit(0);
 }
