@@ -25,6 +25,13 @@ main(int argc,
     XmStringFree(xmstr);
 
     XtRealizeWidget(toplevel);
-    XtAppMainLoop(app);
+    
+{
+    static XtWidgetGeometry Expected[] = {
+   CWWidth | CWHeight            ,   50,   50,   70,   17, 0,0,0, /* label */ 
+    };
+    PrintDetails(    toplevel ,Expected);
+};
+   LessTifTestMainLoop(    toplevel );
     exit(0);
 }
